/* -*- linux-c -*- (Autoset linux indent style when using emacs) */

/*
 * Copyright 1999 Digi International (www.digi.com)
 *    Jeff Randall <Jeff_Randall@digi.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED; without even the 
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

/*
 * $Id: dgdm_mgmt.c,v 1.13 2000/09/21 16:47:02 randall Exp $
 *
 * Management device interface functions. Mostly just glue between
 * dxb_read/write() and dxb.
 */
#define __NO_VERSION__
#include "dgdm_driver.h"
#include "mgmtioc1.h"
#include "mgmtinternal.h"

#define MGMTBUF_SIZE 16384


/*
 * externals
 */
extern int		dgdm_debug;
extern BOARD_T		*dgdm_Board[MAXBOARDS];
extern uchar		dgdm_NumBoards;
extern char		*dgdm_version;
extern unsigned char	dgdm_bitbucket[];



/* prototypes for this file */
STATIC int		mgmt_openack_cb(DXB *dxb, int channel, int error_code,
                                        DXB_ARG arg);
STATIC int		mgmt_closeack_cb(DXB *dxb, int channel, DXB_ARG arg);
STATIC void		*mgmt_copyout(void *dst, uint8 *src, int len,
                                      DXB_ARG arg);
STATIC int		mgmt_data_cb(DXB *dxb, int channel, int data_len,
                                     DXB_ARG arg);
STATIC int		mgmt_setdebug(int *arg);
STATIC int		mgmt_getdd(struct digi_dinfo *arg);
STATIC int		mgmt_getbd(struct digi_info *arg);
STATIC int		mgmt_getstat(struct digi_stat *arg);
STATIC int		mgmt_execute(BOARD_T *brd, int cmd, int send_len,
                                     int expect_len);
STATIC int		mgmt_ioctlack_cb(DXB *dxb, int channel, int command,
                                         int data_len, int error_code,
                                         DXB_ARG arg);





/**************************************************************************
 *
 * The following table and macros are used to map the user level
 * ioctl API into the payload for DXB messages.
 *
 **************************************************************************/

typedef struct
{
	int	cmd;
	int	send_len;
	int	send_offset;
	int	expect_len;
	ulong	expect_offset;
} MGMTIOC;

#define	_GET(CMD, PORT, CHAN, TYPE, MEMBER)				\
		{							\
			GET_##CMD,					\
			sizeof(dxb_uint32) * (PORT+CHAN), 0,		\
			sizeof(TYPE##_t), offsetof(get_##TYPE##_t, MEMBER) \
		}

#define	_SET(CMD, PORT, CHAN, TYPE, MEMBER) \
		{							\
			SET_##CMD, sizeof(set_##TYPE##_t), 0, 0, 0	\
		}

#define	_GSET(CMD, PORT, CHAN, TYPE, MEMBER) \
		{							\
			GET_##CMD, 					\
			sizeof(dxb_uint32) * (PORT+CHAN), 0,		\
			sizeof(TYPE##_t), offsetof(getset_##TYPE##_t, MEMBER) \
		},							\
		{							\
			SET_##CMD, sizeof(getset_##TYPE##_t), 0, 0, 0	\
		}

#define	_TRC(CMD, PORT, CHAN, TYPE, MEMBER) \
		{							\
			GET_##CMD##_TRC,				\
			sizeof(dxb_uint32) * (PORT+CHAN), 0,		\
			sizeof(TYPE##_t),				\
			offsetof(getset_##TYPE##_trc_t, MEMBER)		\
		},							\
		{							\
			SET_##CMD##_TRC,				\
			sizeof(getset_##TYPE##_trc_t), 0, 0, 0		\
		}

#define	_GST(CMD, PORT, CHAN, TYPE, MEMBER) \
		_GSET(##CMD, PORT, CHAN, TYPE, MEMBER), \
		_TRC(##CMD, PORT, CHAN, TYPE, MEMBER)

#define	_GT(CMD, PORT, CHAN, TYPE, MEMBER) \
		_GET(##CMD, PORT, CHAN, TYPE, MEMBER), \
		_TRC(##CMD, PORT, CHAN, TYPE, MEMBER)


static MGMTIOC IocTbl[] =
{

/*	COMMAND			P? C?	PAYLOAD TYPE		MEMBER*/

_GET(	MGMT_PROTOCOL_VERSION,	0, 0,	mgmt_protocol_version, version),
_GET(	ADAPTER_INFO,		0, 0,	adapter_info,		info	),
_GET(	ADAPTER_EXT_INFO,	0, 0,	adapter_ext_info,	info	),
_GET(	ADAPTER_STATE,		0, 0,	adapter_state,		state	),
_GET(	ADAPTER_EXT_STATE,	0, 0,	adapter_ext_state,	state	),
_GSET(	MODEM_NATIONALITY,	0, 0,	modem_nationality,	nation	),

_GSET(	NVRAM_CONFIG,		0, 0,	nvram_config,		config	),

_GST(	DS1_CONFIG,		1, 0,	ds1_config,		config	),
_GT(	DS1_STATUS,		1, 0,	ds1_status,		status	),
_GST(	DS1_EXT_CONFIG,		1, 0,	ds1_ext_config,		config	),
_GT(	DS1_EXT_STATUS,		1, 0,	ds1_ext_status,		status	),
_GET(	DS1_CT1_SIG_STATUS,	1, 1,	ds1_ct1_sig_status,	status	),
_GET(	DS1_ALL_CT1_SIG_STATUS,	1, 0,	ds1_all_ct1_sig_status, status	),
_GST(	DS1_E1R2_CONFIG,	1, 0,	ds1_e1r2_config,	config	),

_GSET(	BRI_CONFIG,		1, 0,	bri_config,		config	),
_GET(	BRI_STATUS,		1, 0,	bri_status,		status	),

_GT(	CALL_STATUS,		1, 1,	call_status,		status	),
_GET(	ALL_CALL_STATUS,	1, 0,	all_call_status,	status	),

_GSET(	DCHAN_CONFIG,		1, 0,	dchan_config,		config	),
_GST(	DCHAN_EXT_CONFIG,	1, 0,	dchan_ext_config,	config	),
_GT(	DCHAN_EXT_STATUS,	1, 0,	dchan_ext_status,	status	),

_GST(	SERIAL_CONFIG,		0, 1,	serial_config,		config	),
_GT(	SERIAL_STATUS,		0, 1,	serial_status,		status	),
_GST(	SERIAL_EXT_CONFIG,	0, 1,	serial_ext_config,	config	),
_GT(	SERIAL_EXT_STATUS,	0, 1,	serial_ext_status,	status	),
#if defined(SERIAL_RS485_CONFIG)	/* temp -- headers */
_GSET(	SERIAL_RS485_CONFIG,	0, 1,	serial_rs485_config,	config	),
#endif

_GET(	SERIAL_DXB_CONFIG,	0, 1,	serial_dxb_config,	config	),
_GSET(	SERIAL_CHANNEL_MODE,	0, 1,	serial_channel_test,	mode	),

_GST(	MODEM_CONFIG,		0, 1,	modem_config,		config	),
_GT(	MODEM_STATUS,		0, 1,	modem_status,		status	),
_GST(	MODEM_EXT_CONFIG,	0, 1,	modem_ext_config,	config	),
_GT(	MODEM_EXT_STATUS,	0, 1,	modem_ext_status,	status	),

_GT(	BCHAN_STATUS,		0, 1,	bchan_status,		status	),
_GST(	BCHAN_EXT_CONFIG,	0, 1,	bchan_ext_config,	config	),
_GT(	BCHAN_EXT_STATUS,	0, 1,	bchan_ext_status,	status	),

_GSET(	DCHAN_DATA_TRC,		0, 1,	channel_data_trc,	channel),
_GSET(	BCHAN_TX_DATA_TRC,	0, 1,	channel_data_trc,	channel),
_GSET(	BCHAN_RX_DATA_TRC,	0, 1,	channel_data_trc,	channel),
_SET(	BCHAN_RXTX_DATA_TRC,	0, 1,	channel_data_trc,	channel),
_GSET(	SERIAL_TX_DATA_TRC,	0, 1,	channel_data_trc,	channel),
_GSET(	SERIAL_RX_DATA_TRC,	0, 1,	channel_data_trc,	channel),
_SET(	SERIAL_RXTX_DATA_TRC,	0, 1,	channel_data_trc,	channel),
_GSET(	SERIAL_STATUS_TRC,	0, 1,	channel_data_trc,	channel),

_SET(	BLACKBOX_DATA_TRC,	0, 0,	blackbox_data_trc,	0),
_GSET(	DAUGHTERCARD_SERIALID,	0, 0,	daughter_serialid,	serialnum),

_GSET(	SERIAL_TEST_STATUS,	0, 1,	serial_test_status,	status	),
_GSET(	SERIAL_TEST_CONFIG,	0, 1,	serial_test_config,	config	),
{0}

};







/**************************************************************************
 *
 * Functions
 *
 **************************************************************************/
 
/*
 * dgdm_mgmt_open()
 */
GLOBAL int dgdm_mgmt_open(BOARD_T *brd)
{
        if(!brd)
                return(-ENODEV);

	brd->mgmt.eof = 0;
	if (brd->mgmt.opencnt++)
		return (0);

	if ((brd->mgmt.buf = kmalloc(PAGE_SIZE, GFP_KERNEL)) == NULL) {
		return (-ENOMEM) ;
	}

	MOD_INC_USE_COUNT;

	for (;;)
	{
		dxb_lock(brd);

		brd->mgmt.dxbchan = dxb_rxbuf_open(brd->dxb, "MGMT", 0, NULL, 
			0, NULL, MGMTBUF_SIZE) ;

		if (brd->mgmt.dxbchan >= 0)
			break;

		dxb_unlock(brd);

		if (dgdm_sleep(1))
		{
			kfree(brd->mgmt.buf) ;
			MOD_DEC_USE_COUNT;
			return (-EINTR);
		}
	}

	dxb_return_credits(brd->dxb, brd->mgmt.dxbchan, MGMTBUF_SIZE - 1);
	dxb_set_openack_cb(brd->dxb, brd->mgmt.dxbchan,
		mgmt_openack_cb, (DXB_ARG) brd);

	dxb_set_data_cb(brd->dxb, brd->mgmt.dxbchan,
		mgmt_data_cb, (DXB_ARG) brd);

        dxb_set_closeack_cb(brd->dxb, brd->mgmt.dxbchan,
		mgmt_closeack_cb, (DXB_ARG) brd);

	dxb_set_ioctlack_cb(brd->dxb, brd->mgmt.dxbchan,
		mgmt_ioctlack_cb, (DXB_ARG)brd) ;

	dxb_unlock(brd);

	/*
	 *      Wait for open acknowledgement to come back,
	 *      then check its error code.
	 */
	interruptible_sleep_on(&brd->mgmt.openack_wait);
	if (current_got_fatal_signal()) {
		kfree(brd->mgmt.buf) ;
		MOD_DEC_USE_COUNT;
		return (-EINTR);
	}

	if (brd->mgmt.error_code) {
		DPR_BASIC(("dgdm_mgmt_open() DXB open failed %d\n",
                           brd->mgmt.error_code));
		kfree(brd->mgmt.buf) ;
		MOD_DEC_USE_COUNT;
		return (-ENODEV);
	}

	return (0);
}



/*
 * mgmt_openack_cb()
 *
 * callback we get on the open acknowledgement
 */
STATIC int mgmt_openack_cb(DXB *dxb, int channel, int error_code, DXB_ARG arg)
{
        BOARD_T *brd = (BOARD_T *) arg;

        if(!brd)
                return(-ENODEV);

        brd->mgmt.error_code = error_code;

        wake_up_interruptible(&brd->mgmt.openack_wait);

        return (0);
}



/*
 * dgdm_mgmt_close()
 */
GLOBAL void dgdm_mgmt_close(BOARD_T *brd)
{
	int	i;
	int	rc;

        if(!brd)
                return;

        if (--brd->mgmt.opencnt > 0)
                return;

        /*
         *      If the DXB open never completed, we are done
         */
        if (brd->mgmt.dxbchan < 0) {
		kfree(brd->mgmt.buf);
                MOD_DEC_USE_COUNT;
                return;
        }

        /*
         *      Attempt to queue close message up to 100 times
         *      If we can't get it thru by then, just give up
         *      because we don't want to hang the user.
         */
        for (i = 0; i < 100; ++i) {
                dxb_lock(brd);
                rc = dxb_close(brd->dxb, brd->mgmt.dxbchan);
                dxb_unlock(brd);

                /*
                 * If close message was queued, wait for ACK and
                 * then blow out of here.
                 */
                if (rc == 0) {
                        interruptible_sleep_on(&brd->mgmt.closeack_wait);
                        break;
                }

                /*
                 * Wait 1 tick and try again.  If user mashes
                 * interrupt key, blow out of here
                 */
                if (dgdm_sleep(1))
                        break;
        }

	kfree(brd->mgmt.buf);
	MOD_DEC_USE_COUNT;
}



/*
 * mgmt_closeack_cb()
 */
STATIC int mgmt_closeack_cb(DXB *dxb, int channel, DXB_ARG arg)
{
        BOARD_T *brd = (BOARD_T *) arg;

        if(!brd)
                return(-ENODEV);

        wake_up_interruptible(&brd->mgmt.closeack_wait);
        return (0);
}



/*
 * dgdm_mgmt_read()
 */
GLOBAL int dgdm_mgmt_read(BOARD_T *brd, READ_PROTO)
{
	int	avail;

        if(!brd)
                return(-ENODEV);

	/*
	 * Nothing yet.  This is where trace messages will appear.
	 */

	if (count == 0)
		return (0);

	/*
	 * Wait for desired amount of data to become available
	 */
	for (;;) {
		avail = dxb_rxbuf_count(brd->dxb, brd->mgmt.dxbchan);
		if (avail)
			break;
		interruptible_sleep_on(&brd->mgmt.read_wait);
		if (current_got_fatal_signal())
			return (-EINTR);
	}

	count = MIN(count, avail);

	/*
	 * Set copyout function and move the data from the
	 * "rxbuf" to user space.  Then return the DXB credits.
	 */
	dxb_rxbuf_set_copyout(brd->dxb, brd->mgmt.dxbchan,
                              mgmt_copyout, (DXB_ARG) 0);

	count = dxb_rxbuf_read(brd->dxb, brd->mgmt.dxbchan, buf, count);
	dxb_return_credits(brd->dxb, brd->mgmt.dxbchan, count);
	ADD_POS(count);
	return (count);
}



/*
 * mgmt_data_cb()
 */
STATIC int mgmt_data_cb(DXB *dxb, int channel, int data_len, DXB_ARG arg)
{
	BOARD_T	*brd = (BOARD_T *) arg;
	int rc;

        if(!brd)
                return(-ENODEV);

	if (data_len)
		rc = dxb_rxbuf_copyin(dxb, channel, data_len);
	else
		brd->mgmt.eof = 1;

	dxb_read_done(dxb);
        wake_up_interruptible(&brd->mgmt.read_wait);
	return (0);
}



/*
 * dgdm_mgmt_ioctl()
 */
GLOBAL int dgdm_mgmt_ioctl(unsigned int minor, FILE *file, unsigned int cmd,
                           unsigned long arg)
{
	BOARD_T	*brd = dgdm_Board[minor2board(minor)] ;
	MGMTIOC	*tbl;
	int	error = 0 ;
	void	*src;
	void	*dst;

        if(!brd)
                return(-ENODEV);

	switch(cmd) {
        case DIGI_SETDEBUG:
                return(mgmt_setdebug((int *) arg));
	case DIGI_GETDD:
                return(mgmt_getdd((struct digi_dinfo *) arg));
        case DIGI_GETBD:
                return(mgmt_getbd((struct digi_info *) arg));
        case DIGI_GETSTAT:
                return(mgmt_getstat((struct digi_stat *) arg));
	default:
                /*
                 * These are done completely by the FEP.  Pass it
                 * along as appropritate.
                 *
                 * Find command in table.  If table gets large, it
                 * will be better to sort it at init time, then do
                 * a binary search.
                 */
                for (tbl = IocTbl; tbl->cmd; ++tbl)
                        if (tbl->cmd == cmd)
                                break;

                if (tbl->cmd == 0)
                        return (-ENXIO);

                src = (void *) (arg + tbl->send_offset);
                dst = (void *) (arg + tbl->expect_offset);

                /*
                 * Enforce the max buffer size that we use as a bounce
                 * buffer between memcpy_*fs and dxb.
                 */ 
                if (MAX(tbl->send_len, tbl->expect_len) > PAGE_SIZE) {
                        return(-EFBIG) ;
                }

                /* Verify that the memory we will read/write is really there */
                error = verify_area(VERIFY_READ, src, tbl->send_len);
                if (error) {
                        return(error) ;
                }

                if (tbl->expect_len) {
                        error = verify_area(VERIFY_WRITE, dst, tbl->expect_len);
                        if (error) {
                                return(error) ;
                        }
                }

                /* Copy data in, if any. */
                if (tbl->send_len) {
                        COPY_FROM_USER(brd->mgmt.buf, src, tbl->send_len) ;
                }

                /* Send the request, wait for the reply. */
                error = mgmt_execute(brd, cmd, tbl->send_len, tbl->expect_len);
                if (error) {
                        return(error) ;
                }

                /*
                 * If we got here, and the favour of a reply was
                 * requested, then copy it out to user space.
                 */
                if (tbl->expect_len) {
                        COPY_TO_USER(dst, brd->mgmt.buf, tbl->expect_len) ;
                }
                break;
	}
			
	return(0) ;
}



/*
 * mgmt_setdebug()
 *
 * Set the driver debug level.
 *
 * NOTE: this has no effect unless DGDM_TRACER is defined at compile time.
 */
STATIC int mgmt_setdebug(int *arg)
{
        int sdebug;

        DPR_IOCTL(("mgmt_setdebug()\n"));
                        
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)
        if (copy_from_user(&sdebug, arg, sizeof(sdebug))) {
                DPR_IOCTL(("dgdm_setdebug() - EFAULT\n"));
                return(-EFAULT);
        }
#else
        memcpy_fromfs(&sdebug, arg, sizeof(sdebug));
#endif

        dgdm_debug = sdebug;
                        
        APR(("driver debug level set to 0x%x\n", dgdm_debug));
        return(0);
}



/*
 * mgmt_getdd()
 *
 *
 */
STATIC int mgmt_getdd(struct digi_dinfo *arg)
{
        struct digi_dinfo dinfo;

        DPR_IOCTL(("dgdm_mgmt_ioctl() - DIGI_GETDD\n"));

        if (!arg) {
                DPR_IOCTL(("mgmt_getdd() - EFAULT #1\n"));
                return(-EFAULT);
        }

        memset(&dinfo, 0, sizeof(struct digi_dinfo));
        dinfo.dinfo_nboards = dgdm_NumBoards;

        /* 21 is the cvs offset that is the version number */
        strncpy(dinfo.dinfo_version, (dgdm_version + 21), 16);

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)
        if (copy_to_user(arg, &dinfo, sizeof(struct digi_dinfo))) {
                DPR_IOCTL(("mgmt_getdd() - EFAULT #2\n"));
                return(-EFAULT);
        }
#else
        memcpy_tofs(arg, &dinfo, sizeof(struct digi_dinfo));
#endif
        return(0);
}




/*
 * mgmt_getbd()
 *
 *
 */
STATIC int mgmt_getbd(struct digi_info *arg)
{
	BOARD_T		 *brd;
        int		 bdnum;
        struct digi_info info;

        DPR_IOCTL(("mgmt_getbd()\n"));

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)
        if (copy_from_user(&info, arg, sizeof(struct digi_info))) {
                DPR_IOCTL(("mgmt_getbd() - EFAULT\n"));
                return(-EFAULT);
        }
#else
        memcpy_fromfs(&info, arg, sizeof(struct digi_info));
#endif

        /* get the board reqested from the struct */
        bdnum = info.info_bdnum;

        if(bdnum >= dgdm_NumBoards) {
                DPR_IOCTL(("mgmt_getbd() - invalid board %d\n", bdnum));
                return(-EINVAL);
        }
                        
        /* reset the board to be the one requested */
        brd = dgdm_Board[bdnum];

        if(!brd)
                return(-EINVAL);

        info.info_ioport   = brd->reg;
        info.info_physaddr = brd->ram;
        info.info_physsize = brd->ramlen;
        info.info_memsize  = brd->memsize;
        info.info_nports   = brd->nttychans;

        /* make the translation from POST_TYPE */
        info.info_bdtype   = (T_PCIBUS | T_DXB);
        switch(brd->board_type) {
        case POST_TYPE_HOUSTON_T1:
        case POST_TYPE_HOUSTON_E1:
        case POST_TYPE_CAMPINAS_T1:
        case POST_TYPE_CAMPINAS_E1:
                info.info_bdtype |= T_HOU;
                break;
        case POST_TYPE_LONDON_U:
        case POST_TYPE_LONDON_ST:
                info.info_bdtype |= T_LON;
                break;
        case POST_TYPE_CHASKA:
                info.info_bdtype |= T_CHA;
                break;
        case POST_TYPE_AP2K2_232:
        case POST_TYPE_AP2K4_232:
        case POST_TYPE_AP2K8_232:
        case POST_TYPE_AP2K16_232:
#if defined(POST_TYPE_AP2K4_MPI)	/* temp - until headers updated */
        case POST_TYPE_AP2K32_232:
        case POST_TYPE_AP2K4_MPI:
        case POST_TYPE_AP2K8_MPI:
        case POST_TYPE_AP2K16_MPI:
#endif
                info.info_bdtype |= T_HERC;
                break;
        default:
                APR(("mgmt_getbd() - Unknown Board type 0x%x\n",
                     brd->board_type));
                break;
        }

#if 0
        /* stolen from the UW7 driver, not ready yet */
        if ( brd->state == BRD_RUNNING ) {
                info.info_bdstate = BD_RUNNING;
        } else if ( brd->state == BRD_RESET ) {
                info.info_bdstate = BD_NOFEP;
        } else {
                info.info_bdstate = 0;
        }
#else
        /* lie and say we're always running */
        info.info_bdstate = 0;
#endif

        memset(info.info_reserved, 0, sizeof(info.info_reserved));

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)
        if (copy_to_user(arg, &info, sizeof(struct digi_info))) {
                DPR_IOCTL(("mgmt_getbd() - EFAULT #2\n"));
                return(-EFAULT);
        }
#else
        memcpy_tofs(arg, &info, sizeof(struct digi_info));
#endif
        return(0);
}



/*
 * mgmt_getstat()
 *
 *
 */
STATIC int mgmt_getstat(struct digi_stat *arg)
{
	BOARD_T		*brd;
        int		 bdnum;
        int		 dcnum;
        DRVCHAN		*dc;
        struct digi_stat dstat;

        DPR_IOCTL(("mgmt_getstat()\n"));

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)
        if (copy_from_user(&dstat, arg, sizeof(struct digi_stat))) {
                DPR_IOCTL(("mgmt_getstat() EFAULT\n"));
                return(-EFAULT);
        }
#else
        memcpy_fromfs(&dstat, arg, sizeof(struct digi_stat));
#endif

        /* get the board & chan reqested from the struct */
        bdnum = dstat.info_brd;
        dcnum = dstat.info_chan;

        DPR_IOCTL(("mgmt_getstat() - board %d channel %d\n", bdnum, dcnum));

        /* range check & reset board to be the one requested */
        if(bdnum >= dgdm_NumBoards) {
                DPR_IOCTL(("mgmt_getstat() - invalid board %d\n", bdnum));
                return(-EINVAL);
        }
        brd = dgdm_Board[bdnum];

        if(!brd) {
                DPR_IOCTL(("mgmt_getstat() - null board pointer\n"));
                return(-ENODEV);
        }

        /* range check & set channel to be the one requested */
        if(dcnum >= brd->nttychans) {
                DPR_IOCTL(("mgmt_getstat() - invalid channel %d\n", dcnum));
                return(-EINVAL);
        }
        dc = &brd->dc[dcnum];
                        
        if(!dc) {
                DPR_IOCTL(("mgmt_getstat() - null channel pointer\n"));
                return(-ENODEV);
        }

        /*
         * be sure we've cleared the structure and set the
         * channel and board numbers back into it
         */
        memset(&dstat, 0, sizeof(struct digi_stat));
        dstat.info_chan = dcnum;
        dstat.info_brd  = bdnum;

	if (dc->opencnt > 0 && dc->tty && dc->tty->termios) {
                dstat.info_cflag = dc->tty->termios->c_cflag;
                dstat.info_iflag = dc->tty->termios->c_iflag;
                dstat.info_oflag = dc->user_oflag;
                dstat.info_hflow = ((dc->tty->termios->c_cflag & CRTSCTS) ?
                                    (SDXB_RTS | SDXB_CTS) : 0);
        } else {
                dstat.info_cflag = DEFAULT_CFLAGS;
                dstat.info_iflag = DEFAULT_IFLAGS;
                dstat.info_oflag = DEFAULT_OFLAGS;
                dstat.info_hflow = 0;
        }

        /* fill out the rest of the structure */
        dstat.info_mstat   = dc->mstat;
        dstat.info_tx_data = (long) dc->stat[TxSTAT].cnt;
        dstat.info_rx_data = (long) dc->stat[RxSTAT].cnt;
                        
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)
        if (copy_to_user(arg, &dstat, sizeof(struct digi_stat))) {
                DPR_IOCTL(("mgmt_getstat() - EFAULT\n"));
                return(-EFAULT);
        }
#else
        memcpy_tofs(arg, &dstat, sizeof(struct digi_stat));
#endif
        return(0);
}










/*
 * mgmt_copyout()
 */
STATIC void *mgmt_copyout(void *dst, uint8 *src, int len, DXB_ARG arg)
{
        char    *cdst = (char *) dst;

        COPY_TO_USER(cdst, src, len);
        return (cdst + len);
}



/*
 * mgmt_execute()
 *
 * Formulate message to board and wait for reply.
 *
 * Expects send_len bytes of data to be present in *(brd->mgmt.buf),
 * and copies expect_len bytes of data into *(brd->mgmt.buf).
 */
STATIC int mgmt_execute(BOARD_T *brd, int cmd, int send_len, int expect_len)
{

        if(!brd)
                return(-ENODEV);

	/*
	 * Check for space in the buffer to send the ioctl and
	 * any associated baggage..
	 */
	for (;;) {
		int space ;

		dxb_lock(brd) ;
		space = dxb_ioctl_space_avail(brd->dxb, brd->mgmt.dxbchan,
								send_len) ;
		if (space == DXB_OK) {
			/*
			 * All is well, proceed to send the command.
			 */
			break ;
		} else if (space == DXB_ENOSPACE) {
			/*
			 * Temporary (hopefully) problem with space
			 * in the queue. Spin (interruptably) for a
			 * second until we try again.
			 */
			dxb_unlock(brd) ;
			if (dgdm_sleep(1)) {
				return(-EINTR) ;
			}
		} else {
			/*
			 * We got a bad reply, things are very,
			 * very sick. Bail now.
			 */
			dxb_unlock(brd) ;
			return(-EINVAL) ;
		}
	}

	/*
	 * Send the command, followed by the data payload (if any).
	 */
	dxb_ioctl(brd->dxb, brd->mgmt.dxbchan, cmd, send_len) ;
	if (send_len) {
		dxb_write_ioctl_block(brd->dxb, brd->mgmt.buf, send_len) ;
	}
	dxb_write_done(brd->dxb) ;
	dxb_unlock(brd) ;

	for (;;) {
		/*
		 * Sleep waiting for the reply (or user interrupt).
		 */
		interruptible_sleep_on(&brd->mgmt.ioctlack_wait) ;
		if (current_got_fatal_signal()) {
			return(-EINTR) ;
		} else {
			break ;
		}
	}


	if (brd->mgmt.error_code) {
		return(brd->mgmt.error_code) ;
	}
	if (brd->mgmt.reply_length != expect_len) {
		return(-EIO) ;
	}

	return(0) ;
}



/*
 * mgmt_ioctlack_cb()
 */
STATIC int mgmt_ioctlack_cb(DXB *dxb, int channel, int command, int data_len,
                            int error_code, DXB_ARG arg)
{
	BOARD_T	*brd = (BOARD_T *) arg;

        if(!brd)
                return(-ENODEV);

	brd->mgmt.error_code = error_code ;
	brd->mgmt.reply_length = data_len ;
	
	/*
	 * If there is a payload, be sure to prune it to fit our
	 * bufffer, then copy it in.
	 */
	if (data_len) {
                dxb_read_data_block(dxb, (brd->mgmt.buf ? brd->mgmt.buf : 
                                          dgdm_bitbucket), PAGE_SIZE,
                                    data_len);
                dxb_read_done(dxb) ;
	}

        if(!brd->mgmt.buf) {
                DPR(("mgmt_ioctlack_cb() - mgmt.buf = NULL\n"));
                brd->mgmt.error_code = -EIO;
        }

	wake_up_interruptible(&brd->mgmt.ioctlack_wait);
	return(0);
}
