#include <sys/types.h>
#include <fcntl.h>
#include <stdio.h>

#if !defined(LINUX) && !defined(SOLARIS) && !defined(HPUX)
# include <macros.h>
#endif

#if !defined(UW7)
# include <stdlib.h>
#endif

#include "dxbtypes.h"
#include "mgmthlc.h"
#include "mgmttigger.h"
#include "mgmtshim.h"

#if !defined(UW7)
	#if defined(SOLARIS)
#include "../dgdxb/dxbresolv.h" /* This is temporary until mgmthlc.h is fixed */
	#else
#include "resolv.h" /* This is temporary until mgmthlc.h is fixed */
	#endif
#endif

#ifdef AIX
#include "./aix/odm.h"
#endif

#ifdef SCO
#include <sys/digi.h>
#include "../version.h"
#define DOWNLOAD_DEVICE		"/dev/dxbdl"
#define MANAGEMENT_DEVICE	"/dev/dxb%dmgmt1"
#endif

#if defined(LINUX)
# include "rascon.h"
# include "../dgdm_version.h"
#define DOWNLOAD_DEVICE		"/dev/dg/dgdm/dxbdl"
#define MANAGEMENT_DEVICE	"/dev/dg/dgdm/mgmt%d"
#endif

#if defined(SOLARIS)
#  include <unistd.h>	/* for close() */
/* #  include <strings.h> */	/* for bcopy() */
#  include "stdigi.h"
#  include "shim.h"
#define DOWNLOAD_DEVICE "/dev/DIGIras/dgdxb_mgmt0"
#  include "solshim.c"
#  include "../include/version.h"
#endif

#if defined(HPUX)
#  include <sys/ioctl.h>
#  include "trace_os_dep.h"
#  include "dgdxb_digi.h"
#  define DOWNLOAD_DEVICE "/dev/dgdxb/dl"
#endif

#ifdef UW7
#include <unistd.h>
#include "../xxy/stdigi.h"

/* Are the following headers necessary? */
#include <errno.h>
//#include <string.h>
#include <sys/conf.h>
#include <sys/types.h>
#include <sys/stream.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/stropts.h>
#include "../dgtypes.h"
#include "../dxbtypes.h"
#include "mgmthlc.h"
#include "../dgdxb/rasmon.h"
#include <stdarg.h>
#endif /* UW7 */

void print_serial_channel_mode( const serial_channel_test_t	*mode);
char rev_info[]="$Revision: 1.45 $";

struct MGMT_IOCTL_ENTRY
{
	int cmd;	/* the mgmt command */
	int req_len;	/* the request buffer length */
	int rsp_len;	/* the response buffer length */
};

/* 'cmd' is the managment command value, 'req' is the request buffer size, and
	'rsp' is the response buffer size */

#define	MGMT_COMMAND( cmd, req, rsp ) (cmd), (req), (rsp)

struct MGMT_IOCTL_ENTRY mgmtTbl[ ] =
{

	/**** adapter commands ****/

	MGMT_COMMAND( GET_ADAPTER_INFO,
		0,
		sizeof( get_adapter_info_response_t ) ),

	MGMT_COMMAND( GET_ADAPTER_STATE,
		0,
		sizeof( get_adapter_state_response_t ) ),

	MGMT_COMMAND( GET_MODEM_NATIONALITY,
		0,
		sizeof( get_modem_nationality_response_t ) ),

	MGMT_COMMAND( SET_MODEM_NATIONALITY,
		sizeof( set_modem_nationality_request_t ),
		0 ),

	MGMT_COMMAND( GET_MGMT_PROTOCOL_VERSION,
		0,
		sizeof( get_mgmt_protocol_ver_response_t ) ),

	MGMT_COMMAND( GET_NVRAM_CONFIG,
		0,
		0 ),

	MGMT_COMMAND( SET_NVRAM_CONFIG,
		0,
		0 ),

	/**** ds1 commands ****/

	MGMT_COMMAND( GET_DS1_CONFIG,
		sizeof( get_ds1_config_request_t ),
		sizeof( get_ds1_config_response_t ) ),

	MGMT_COMMAND( SET_DS1_CONFIG,
		sizeof( set_ds1_config_request_t ),
		0 ),

	MGMT_COMMAND( GET_DS1_EXT_CONFIG,
		sizeof( get_ds1_ext_config_request_t ),
		sizeof( get_ds1_ext_config_response_t ) ),

	MGMT_COMMAND( SET_DS1_EXT_CONFIG,
		sizeof( set_ds1_ext_config_request_t ),
		0 ),

	MGMT_COMMAND( GET_DS1_STATUS,
		sizeof( get_ds1_status_request_t ),
		sizeof( get_ds1_status_response_t ) ),

	MGMT_COMMAND( GET_DS1_EXT_STATUS,
		sizeof( get_ds1_ext_status_request_t ),
		sizeof( get_ds1_ext_status_response_t ) ),

	MGMT_COMMAND( GET_DS1_CT1_SIG_STATUS,
		sizeof( get_ds1_ct1_sig_status_request_t ),
		sizeof( get_ds1_ct1_sig_status_response_t ) ),

	MGMT_COMMAND( GET_DS1_ALL_CT1_SIG_STATUS,
		sizeof( get_ds1_ct1_sig_status_request_t ),
		sizeof( get_ds1_ct1_sig_status_response_t ) ),

	/**** bri commands ****/

	MGMT_COMMAND( GET_BRI_CONFIG,
		sizeof( get_bri_config_request_t ),
		sizeof( get_bri_config_response_t ) ),

	MGMT_COMMAND( SET_BRI_CONFIG,
		sizeof( set_bri_config_request_t ),
		0 ),

	MGMT_COMMAND( GET_BRI_STATUS,
		sizeof( get_bri_status_request_t ),
		sizeof( get_bri_status_response_t ) ),

	/**** serial commands ****/

	MGMT_COMMAND( GET_SERIAL_CONFIG,
		sizeof( get_serial_config_request_t ),
		sizeof( get_serial_config_response_t ) ),
		

	MGMT_COMMAND( SET_SERIAL_CONFIG,
		sizeof( set_serial_config_request_t ),
		0 ),

	MGMT_COMMAND( GET_SERIAL_STATUS,
		sizeof( get_serial_status_request_t ),
		sizeof( get_serial_status_response_t ) ),

	MGMT_COMMAND( GET_SERIAL_EXT_CONFIG,
		sizeof( get_serial_ext_config_request_t ),
		sizeof( get_serial_ext_config_response_t ) ),

	MGMT_COMMAND( SET_SERIAL_EXT_CONFIG,
		sizeof( set_serial_ext_config_request_t ),
		0 ),

	MGMT_COMMAND( GET_SERIAL_EXT_STATUS,
		sizeof( get_serial_ext_status_request_t ),
		sizeof( get_serial_ext_status_response_t ) ),

	/**** call commands ****/

	MGMT_COMMAND( GET_CALL_STATUS,
		sizeof( get_call_status_request_t ),
		sizeof( get_call_status_response_t ) ),
		
	MGMT_COMMAND( GET_ALL_CALL_STATUS,
		sizeof( get_all_call_status_request_t ),
		sizeof( all_call_status_t ) ),

	/**** modem commands ****/

	MGMT_COMMAND( GET_MODEM_CONFIG,
		sizeof( get_modem_config_request_t ),
		sizeof( get_modem_config_response_t ) ),

	MGMT_COMMAND( SET_MODEM_CONFIG,
		sizeof( set_modem_config_request_t ),
		0 ),

	MGMT_COMMAND( GET_MODEM_STATUS,
		sizeof( get_modem_status_request_t ),
		sizeof( get_modem_status_response_t ) ),

	MGMT_COMMAND( GET_MODEM_EXT_CONFIG,
		sizeof( get_modem_ext_config_request_t ),
		sizeof( get_modem_ext_config_response_t ) ),

	MGMT_COMMAND( SET_MODEM_EXT_CONFIG,
		sizeof( set_modem_ext_config_request_t ),
		0 ),

	MGMT_COMMAND( GET_MODEM_EXT_STATUS,
		sizeof( get_modem_ext_status_request_t ),
		sizeof( get_modem_ext_status_response_t ) ),

	/**** d-channel commands ****/

	MGMT_COMMAND( GET_DCHAN_CONFIG,
		sizeof( get_dchan_config_request_t ),
		sizeof( get_dchan_config_response_t ) ),

	MGMT_COMMAND( SET_DCHAN_CONFIG,
		sizeof( set_dchan_config_request_t ),
		0 ),

	MGMT_COMMAND( GET_DCHAN_EXT_CONFIG,
		sizeof( get_dchan_ext_config_request_t ),
		sizeof( get_dchan_ext_config_response_t ) ),

	MGMT_COMMAND( SET_DCHAN_EXT_CONFIG,
		sizeof( set_dchan_ext_config_request_t ),
		0 ),

	MGMT_COMMAND( GET_DCHAN_EXT_STATUS,
		sizeof( get_dchan_ext_status_request_t ),
		sizeof( get_dchan_ext_status_response_t ) ),

	/**** b_channel commands ****/

	MGMT_COMMAND( GET_BCHAN_STATUS,
		sizeof( get_bchan_status_request_t ),
		sizeof( get_bchan_status_response_t ) ),

	MGMT_COMMAND( GET_BCHAN_EXT_CONFIG,
		sizeof( get_bchan_ext_config_request_t ),
		sizeof( get_bchan_ext_config_response_t ) ),

	MGMT_COMMAND( SET_BCHAN_EXT_CONFIG,
		sizeof( set_bchan_ext_config_request_t ),
		0 ),

	MGMT_COMMAND( GET_BCHAN_EXT_STATUS,
		sizeof( get_bchan_ext_status_request_t ),
		sizeof( get_bchan_ext_status_response_t ) ),

	/* Hercules specific commands */

	MGMT_COMMAND( GET_SERIAL_CHANNEL_MODE,
		sizeof( get_serial_channel_test_mode_request_t ),
		sizeof( get_serial_channel_test_mode_response_t ) ),

	MGMT_COMMAND( SET_SERIAL_CHANNEL_MODE,
		sizeof( set_serial_channel_test_mode_request_t ),
		0 ),

	MGMT_COMMAND( GET_SERIAL_TEST_CONFIG,
		sizeof( get_serial_test_config_request_t ),
		sizeof( get_serial_test_config_response_t ) ),

	MGMT_COMMAND( SET_SERIAL_TEST_CONFIG,
		sizeof( set_serial_test_config_request_t ),
		0 ),

	MGMT_COMMAND( GET_SERIAL_TEST_STATUS,
		sizeof( get_serial_test_status_request_t ),
		sizeof( get_serial_test_status_response_t ) ),

	MGMT_COMMAND( SET_SERIAL_TEST_STATUS,
		sizeof( set_serial_test_status_request_t ),
		0 ),

	MGMT_COMMAND( GET_ADAPTER_EXT_STATE,
		0,
		sizeof( get_adapter_ext_state_response_t) ),

	MGMT_COMMAND( GET_ADAPTER_EXT_INFO,
		0,
		sizeof( get_adapter_ext_info_response_t) ),

	MGMT_COMMAND( GET_SERIAL_DXB_CONFIG,
		sizeof( get_serial_dxb_config_request_t ),
		sizeof( get_serial_dxb_config_response_t) )


};

#define	MGMT_MAX	sizeof(mgmtTbl) / sizeof(struct MGMT_IOCTL_ENTRY)

int
mgmt_shim_command( uint32 board_number, uint16 command,
	void *request_buffer, uint16 request_length,
	void *response_buffer, uint16 response_length )
{
	int status = MGMT_OK;

	char devicename[24];

#ifdef UW7
	char dxbdevice[24];
	int i, j, board_offset;
#if DBUG
	printf( "In mgmt_shim_command (command=%x, board_number=%d)\n", command, board_number );
#endif
	if( board_number == -1 ) {
		board_number = 0;
	}
#endif /* UW7 */

#if defined(HPUX)
	int i;

	if( board_number == -1 ) {
		board_number = 0;
	}
#endif /* HPUX */

	switch( command )
	{

	case GET_MGMT_BOARD_NO:
	{
#if defined(SCO) || defined(LINUX) || defined(UW7) || defined(SOLARIS) || defined(HPUX)
		int dd;
		struct digi_dinfo ddi;
#endif

		get_mgmt_board_no_response_t *mgmt_response
			= (get_mgmt_board_no_response_t *) response_buffer;

		if ( response_length < sizeof(get_mgmt_board_no_response_t) )
		{
			status = MGMT_SHIM_ERROR_BAD_PACKET_SIZE;
			break;
		}

#ifdef AIX
		/* determine the number of 'Available' boards via the ODM database */
		if ( create_the_board_table( ) != 0 )
		{
			status = MGMT_SHIM_ERROR_INTERNAL;
			break;
		}
		mgmt_response->NumberOfBoards = board_count;
#endif

#if defined(SCO) || defined(LINUX) || defined(SOLARIS) || defined(HPUX)


		dd = open(DOWNLOAD_DEVICE, O_RDWR | O_NONBLOCK);

		if (dd < 0) {
		  perror("open(" DOWNLOAD_DEVICE ")");
		  status = MGMT_SHIM_ERROR_INTERNAL;
		  break;
		}

 #if defined(OLD_SOLARIS)
		if (sol_ioctl(dd, DIGI_GETDD, &ddi, sizeof(ddi)) != 0)
 #else
		if (ioctl(dd, DIGI_GETDD, &ddi) != 0)
 #endif
		  {
			perror("ioctl(DIGI_GETDD)");
			mgmt_response->NumberOfBoards = 0; 
		  }
		else
			mgmt_response->NumberOfBoards = ddi.dinfo_nboards;
		close( dd );
#endif

#if defined(UW7)
		for( i=0; i < 8; i++ ) {	/* for dxb0 to dxb7 device prefix */
			board_offset = board_number - 1 + i;
			for( j=1; j < 7; j++ ) {	/* for rasmon1 to rasmon 6 */
				sprintf( dxbdevice, "/dev/dxb%drasmon%d", board_offset, j );
				if( !access( dxbdevice, F_OK ) )	/* File Exists */
				{
					if( ( dd = open( dxbdevice, O_RDWR | O_NONBLOCK ) ) > 0 ) {
						i = 1000;
						break;
					}
				}
			}
		}

#if DBUG
		printf( "mgmt_shim_command: dd=%d =open(%s)\n",
			dd, dxbdevice );
		printf( "mgmt_shim_command: ioctl(dd, DIGI_GETDD, &ddi) \n" );
#endif
		if( ioctl( dd, DIGI_GETDD, &ddi ) != 0 )
			mgmt_response->NumberOfBoards = 0;
		else
			mgmt_response->NumberOfBoards = ddi.dinfo_nboards;
		close( dd );
#endif /* defined(UW7) */
	}
		break;

	case GET_MGMT_BOARD_INFO:
	{
	  adapter_info_t adapter_info;
	  get_mgmt_board_info_request_t *mgmt_request
		= (get_mgmt_board_info_request_t *) request_buffer;
	  get_mgmt_board_info_response_t *mgmt_response
		= (get_mgmt_board_info_response_t *) response_buffer;

#if defined(SOLARIS)
	  char *emptybuf="";
	  get_adapter_info_response_t adapter_info_response;

	  status=MgmtCommand(board_number, GET_ADAPTER_INFO,
						 emptybuf, 0,
						 &adapter_info_response,
						 sizeof(adapter_info_response));
	  if (status!=MGMT_OK)
		break;

	  adapter_info=adapter_info_response.info;

#else
		int dd;
		char devicename[24];
	
#ifdef AIX
		BOARD *board;
#endif

#ifdef AIX
		if ( ( board = get_a_board( mgmt_request->BoardNo ) ) == NULL )
		{
			status = MGMT_SHIM_ERROR_BAD_DEVICE;
			break;
		} 

		sprintf( devicename, "/dev/%s/mgmt", board->name );
#endif /* defined(AIX) */


#if defined(SCO) || defined(LINUX)
		sprintf( devicename, MANAGEMENT_DEVICE,
			 mgmt_request->BoardNo-1);
#endif
#if defined(UW7)
		for( i=0; i < 8; i++ ) {
			for( j=1; j < 7; j++ ) {
				sprintf( devicename, "/dev/dxb%drasmon%d", mgmt_request->BoardNo - 1 + i, j );
				if( !access( devicename, F_OK ) )	/* File Exists */
				{
					if( ( dd = open( devicename, O_RDWR ) ) > 0 ) {
						i = 1000;
						break;
					}
				}
			}
		}
#endif /* defined(UW7) */
#if defined(HPUX)
		for( i=0; i<TRACE_CHAN_COUNT; i++ )
		{
			MK_DEV_NAME( devicename, DEVDIR, FIRST_TRACE_CHAN + i,
				      mgmt_request->BoardNo - 1 );
			if( ( dd = open( devicename, O_RDWR ) ) >= 0 )
				break;
		}
#endif

#if !defined(UW7) && !defined(HPUX)
		dd = open( devicename, O_RDWR );
#endif
		if ( dd == -1 )
		{
			status = MGMT_SHIM_ERROR_INTERNAL;
			break;
		}

		if ( ioctl( dd, GET_ADAPTER_INFO, &adapter_info ) == -1 )
		{
			status = MGMT_SHIM_ERROR_INTERNAL;
			close( dd );
			break;
		}

		close( dd );

#endif /* !defined(SOLARIS) */


		mgmt_response->BoardInfo.BoardNo = mgmt_request->BoardNo;
#ifdef AIX
		mgmt_response->BoardInfo.BrdId = atoi( &board->name[2] ); 
#endif
#if defined(SCO) || defined(LINUX) || defined(UW7) || defined(SOLARIS) || defined(HPUX)
		mgmt_response->BoardInfo.BrdId = mgmt_request->BoardNo; 
#endif

		mgmt_response->BoardInfo.BoardType = adapter_info.board_type;
		mgmt_response->BoardInfo.TotalChannels = adapter_info.port_count;
		mgmt_response->BoardInfo.TotalPorts = adapter_info.port_count
			* adapter_info.channels_per_port;
		
	}
		break;

	case GET_MGMT_SHIM_VER:
	{
		get_mgmt_shim_ver_response_t *mgmt_response
			= (get_mgmt_shim_ver_response_t *) response_buffer;
	
		if ( response_length < sizeof(get_mgmt_shim_ver_response_t) )
		{
			status = MGMT_SHIM_ERROR_BAD_PACKET_SIZE;
			break;
		}
		/* the version is defined in mgmtshim.h */
		mgmt_response->ver = MGMT_SHIM_VERSION;
	}
		break;

	case SET_MGMT_STOP_TRC:
		/* not implemented */
		status = MGMT_SHIM_ERROR_BAD_COMMAND;
		break;

	case GET_MGMT_DRV_VERS:
		{
#if defined(SCO)
		get_mgmt_drv_vers_response_t *mgmt_response =
		  (get_mgmt_drv_vers_response_t *) response_buffer;
	
		if ( response_length < sizeof(get_mgmt_drv_vers_response_t) )
		{
			status = MGMT_SHIM_ERROR_BAD_PACKET_SIZE;
			break;
		}

		mgmt_response->versions.driverCount = 2;
        	mgmt_response->versions.drivers[1].type = MGMT_SER_DRV_VER;
		sprintf((char *)mgmt_response->versions.drivers[0].name, "Driver Part Number");
		sprintf((char *)mgmt_response->versions.drivers[0].version, DIGI_PARTNO);
        	mgmt_response->versions.drivers[0].type = MGMT_SER_DRV_VER;
		sprintf((char *)mgmt_response->versions.drivers[1].name, "Driver Version");
		sprintf((char *)mgmt_response->versions.drivers[1].version, "DGAP V"DGASYN_VERSION);

#elif defined(LINUX)
		get_mgmt_drv_vers_response_t *mgmt_response = 
		  (get_mgmt_drv_vers_response_t *) response_buffer;
	
		if (response_length < sizeof(get_mgmt_drv_vers_response_t)) {
			status = MGMT_SHIM_ERROR_BAD_PACKET_SIZE;
			break;
		}
		mgmt_response->versions.driverCount = 2;
		sprintf((char *)mgmt_response->versions.drivers[0].name, 
				"Driver Part Number");
		sprintf((char *)mgmt_response->versions.drivers[0].version, DG_PART);
		sprintf((char *)mgmt_response->versions.drivers[1].name, 
				"Driver Version");
		sprintf((char *)mgmt_response->versions.drivers[1].version, DG_NAME);
#elif defined(SOLARIS)
		get_mgmt_drv_vers_response_t *mgmt_response = 
		  (get_mgmt_drv_vers_response_t *) response_buffer;
	
		if (response_length < sizeof(get_mgmt_drv_vers_response_t)) {
		  status = MGMT_SHIM_ERROR_BAD_PACKET_SIZE;
		  break;
		}

		/* Note: The DGDXB driver can also handle GET_MGMT_DRV_VERS
		   requests itself and will report the same as below (if the
		   drivers and shim are compiled at the same time). */

		mgmt_response->versions.driverCount = 3;		

		sprintf((char *)mgmt_response->versions.drivers[0].name, 
				"Driver Part Number");
		sprintf((char *)mgmt_response->versions.drivers[0].version,
				DIGI_PARTNO);
		mgmt_response->versions.drivers[0].type = MGMT_DXB_DRV_VER;

		sprintf((char *)mgmt_response->versions.drivers[1].name, 
				"Driver Version");
		sprintf((char *)mgmt_response->versions.drivers[1].version,
				"DGDXB ver. " DGDXB_VERSION);
		mgmt_response->versions.drivers[1].type = MGMT_DXB_DRV_VER;

		sprintf((char *)mgmt_response->versions.drivers[2].name, 
				"Driver Version");
		sprintf((char *)mgmt_response->versions.drivers[2].version,
				"DGASY ver. " DGASY_VERSION);
		mgmt_response->versions.drivers[2].type = MGMT_SER_DRV_VER;

#elif defined(UW7)
 		get_mgmt_drv_vers_response_t *mgmt_response = 
			(get_mgmt_drv_vers_response_t *) response_buffer;
		int dd;

		for( i=0; i < 8; i++ ) { /* for dxb0 to dxb7 device prefix */
			board_offset = board_number - 1 + i;
			for( j=1; j < 7; j++ ) { /* for rasmon1 to rasmon 6 */
				sprintf( dxbdevice, "/dev/dxb%drasmon%d", board_offset, j );
				if( !access( dxbdevice, F_OK ) )/*File Exists*/
				{
					if( ( dd = open( dxbdevice, O_RDWR | O_NONBLOCK ) ) > 0 ) {
						i = 1000;
						break;
					}
				}
			}
		}

		if ( response_length < sizeof(get_mgmt_drv_vers_response_t) )
		{
			status = MGMT_SHIM_ERROR_BAD_PACKET_SIZE;
			break;
		}

		if ( ioctl( dd, GET_MGMT_DRV_VERS , mgmt_response ) == -1 )
		{
			status = MGMT_SHIM_ERROR_INTERNAL;
			close( dd );
			break;
		}

		close( dd );
		status = MGMT_OK;
		break;
#elif defined( AIX )
		get_mgmt_drv_vers_response_t *mgmt_response
		    = (get_mgmt_drv_vers_response_t *) response_buffer;

		if ( response_length < sizeof(get_mgmt_drv_vers_response_t) )
		{
		    status = MGMT_SHIM_ERROR_BAD_PACKET_SIZE;
		    break;
		}
		mgmt_response->versions.driverCount = 2;
		mgmt_response->versions.drivers[0].type = MGMT_DXB_DRV_VER;
		sprintf((char *)mgmt_response->versions.drivers[0].name, "Driver Part Number");
		sprintf((char *)mgmt_response->versions.drivers[0].version, VRMF);
		mgmt_response->versions.drivers[1].type = MGMT_DXB_DRV_VER;
		sprintf((char *)mgmt_response->versions.drivers[1].name, "Driver Version");
		sprintf((char *)mgmt_response->versions.drivers[1].version, "digidm V" DIGIDM_PARTNO);
#elif defined( HPUX )
	get_mgmt_drv_vers_response_t *mgmt_response
	    = (get_mgmt_drv_vers_response_t *) response_buffer;

	if( response_length < sizeof(get_mgmt_drv_vers_response_t) )
	{
		status = MGMT_SHIM_ERROR_BAD_PACKET_SIZE;
		break;
	}
	mgmt_response->versions.driverCount = 2;
	mgmt_response->versions.drivers[0].type = MGMT_DXB_DRV_VER;
	sprintf((char *)mgmt_response->versions.drivers[0].name, "Driver Part Number");
	sprintf((char *)mgmt_response->versions.drivers[0].version, DIGI_PARTNO);
	mgmt_response->versions.drivers[1].type = MGMT_DXB_DRV_VER;
	sprintf((char *)mgmt_response->versions.drivers[1].name, "Driver Version");
	sprintf((char *)mgmt_response->versions.drivers[1].version, "DGDXB V" DIGI_VERSION);
#else
		/* not implmented */
		status = MGMT_SHIM_ERROR_BAD_COMMAND;
#endif
		}
		break;

	default:
		/* unknown command */
		status = MGMT_SHIM_ERROR_BAD_COMMAND;
		break;

	} /* end switch */

	return status;
}

int
mgmt_channel_command( uint32 board_number, uint16 command,
	void *request_buffer, uint16 request_length,
	void *response_buffer, uint16 response_length )
{
	int dd, i;
	char devicename[32];

#ifdef AIX
	BOARD *board;
#endif
#if defined(UW7)
	int board_offset;
	int j;
#endif
	int status = MGMT_SHIM_ERROR_BAD_COMMAND;

	void *mgmtdata = NULL;

#if defined(SOLARIS)
 #if defined(OLD_SOLARIS)
	dd=sol_open_mgmt_dev(board_number);
 #else
	int board_offset;
	board_offset = board_number - 1;
	sprintf( devicename, "/dev/DIGIras/dgdxb_mgmt%d", board_offset);
	dd = open( devicename, O_RDWR );

 #endif

#else

#ifdef AIX
	/* (board_number is 1-based) */
	if ( ( board = get_a_board( board_number - 1 ) ) == NULL )
	{
		status = MGMT_SHIM_ERROR_BAD_DEVICE;
		return status;
	} 

	sprintf( devicename, "/dev/%s/mgmt", board->name );
#endif

#if defined(SCO) || defined(LINUX)
	sprintf( devicename, MANAGEMENT_DEVICE, board_number - 1);
#endif

#if defined(UW7)
	for( i=0; i < 8; i++ ) {
		for( j=1; j < 7; j++ ) {
			board_offset = board_number - 1 + i;
			sprintf( devicename, "/dev/dxb%drasmon%d", board_offset, j );
			if( !access( devicename, F_OK ) )	/* File Exists */
			{
				if( ( dd = open( devicename, O_RDWR ) ) > 0 ) {
					i=1000;
					break;
				}
			}
		}
	}

#endif /* defined(UW7) */
#if defined(HPUX)
	for( i=0; i<TRACE_CHAN_COUNT; i++ )
	{
		MK_DEV_NAME( devicename, DEVDIR, FIRST_TRACE_CHAN + i,
			      board_number - 1 );
		if( ( dd = open( devicename, O_RDWR ) ) >= 0 )
			break;
	}
#endif
	
#if !defined(UW7) && !defined(HPUX)
	dd = open( devicename, O_RDWR );
#endif

#endif /* !defined(SOLARIS) */

	if ( dd == -1 )
	{
#if 1
		perror( "open failed" );
#endif
		status = MGMT_SHIM_ERROR_INTERNAL;
		return status;
	}

	for ( i = 0; i < MGMT_MAX; i++ )
	{

		if ( mgmtTbl[i].cmd == command )
		{
			status = MGMT_OK;	/* indicate valid command */

			/* if a request buffer is required, check the length */
			if ( mgmtTbl[i].req_len != 0 )
			{
				if ( request_length < mgmtTbl[i].req_len )
				{
					status = MGMT_SHIM_ERROR_BAD_PACKET_SIZE;
					break;
				}
			}

			/* if a response buffer is required, check the length */
			if ( mgmtTbl[i].rsp_len != 0 )
			{
				if ( response_length < mgmtTbl[i].rsp_len )
				{
					status = MGMT_SHIM_ERROR_BAD_PACKET_SIZE;
					break;
				}
			}

			/* allocate a working buffer */
			mgmtdata = 
			  (void *)malloc( mgmtTbl[i].req_len + mgmtTbl[i].rsp_len + 8 );

			if ( mgmtdata == NULL )
			{
				status = MGMT_SHIM_ERROR_OUT_OF_MEMORY;
				break;
			}

			/* copy request data area to buffer, if required */
			if ( mgmtTbl[i].req_len != 0 )
				memcpy( mgmtdata, request_buffer, mgmtTbl[i].req_len );

#if defined(SOLARIS)

 #if defined(OLD_SOLARIS)
			status=sol_do_mgmt_cmd_with_fd(dd, command, board_number, request_buffer,  request_length, request_buffer,  request_length);

 #else

			if ((status = ioctl( dd, command, mgmtdata )) == -1) {
				break;
			}

  	        	if ( mgmtTbl[i].rsp_len != 0 )
            		{
	        		switch(command ) {
                		case GET_SERIAL_STATUS:
                		{
		    			memcpy( response_buffer,
		    			(int) mgmtdata+4,
		    			mgmtTbl[i].rsp_len ); /*sizeof(get_serial_status_t);*/
                    			break;
                		}

                		case GET_MGMT_PROTOCOL_VERSION:
                		{
		    			memcpy( response_buffer,
		    			(int) mgmtdata+4,
		    			mgmtTbl[i].rsp_len ); /*sizeof(get_serial_status_t);*/
                    			break;
                		}

				case GET_SERIAL_CHANNEL_MODE:
				{
					getset_serial_channel_test_t * getserialmode;
					getserialmode = ( getset_serial_channel_test_t * ) mgmtdata;
#if YP_DEBUG
	                		printf( "\t GET_SERIAL_CHANNEL_MODE: channel = %d\n\n", getserialmode->channel);
#endif
					memcpy( response_buffer, (int )&getserialmode->mode, response_length);
#if YP_DEBUG
					print_serial_channel_mode( &getserialmode->mode);
#endif
					break;
				}

                		default:
		        	{
		   	        	memcpy( response_buffer,
		    			(int) mgmtdata + mgmtTbl[i].req_len,
		    			mgmtTbl[i].rsp_len );
                    			break;
		        	}

             		   } //switch

           		} //IF
			else {
	        		switch(command ) {
                		case SET_SERIAL_TEST_STATUS:
                		{
                    			break;
                		}

				//case GET_SERIAL_CHANNEL_MODE:
				case SET_SERIAL_CHANNEL_MODE:
				{
					getset_serial_channel_test_t * getserialmode;
					getserialmode = ( getset_serial_channel_test_t * ) mgmtdata;
					memcpy( response_buffer, (int )&getserialmode->mode, response_length);
					break;
				}

                		default:
					memcpy( response_buffer, (int )mgmtdata, response_length);
                    			break;
                		}
			}
 #endif

#else /* !defined(SOLARIS) */
			if ( ioctl( dd, command, mgmtdata ) == -1 )
			{
#if 1
				perror( "ioctl failure" );
#endif
				status = MGMT_SHIM_ERROR_INTERNAL;
				break;
			}

			/* copy buffer to response data area, if required */
#if !defined(LINUX) && !defined(HPUX) && !defined(AIX)
			/*
			 * Process GET_ALL_CALL_STATUS seperately.
			 */
			if (command == GET_ALL_CALL_STATUS) {
				memcpy( response_buffer, (int )mgmtdata + 8, mgmtTbl[i].rsp_len - 8);

#if defined(UW7)
				memcpy( response_buffer, (int )mgmtdata, sizeof(all_call_status_t ));
#endif

			} else
#endif /* !defined(LINUX) && !defined(HPUX) && !defined(AIX) */
			{
				if ( mgmtTbl[i].rsp_len != 0 )
#if !defined(LINUX)

#if defined(UW7)
					switch( command) {
					case GET_BRI_CONFIG:
					{
						getset_bri_config_t *bri;
						bri=(getset_bri_config_t*) mgmtdata;
						memcpy( response_buffer, (int )&bri->config, sizeof(get_bri_config_response_t));
						break;
					}

					case GET_SERIAL_CONFIG:
					{
						getset_serial_config_t * getserialcfg;
						getserialcfg = (getset_serial_config_t *) mgmtdata;

						memcpy( response_buffer, (int )&getserialcfg->config, response_length);
						break;
					}

					case GET_SERIAL_STATUS:
					{
						get_serial_status_t * getserialstatus;
						getserialstatus = ( get_serial_status_t * ) mgmtdata;
						memcpy( response_buffer, (int )&getserialstatus->status, response_length);
						break;
					}

					case GET_SERIAL_EXT_STATUS:
					{
						get_serial_ext_status_t * getserialextstatus;
						getserialextstatus = ( get_serial_ext_status_t * ) mgmtdata;
						memcpy( response_buffer, (int )&getserialextstatus->status, response_length);
						break;
					}

					case GET_SERIAL_CHANNEL_MODE:
					{
						getset_serial_channel_test_t * getserialmode;
						getserialmode = ( getset_serial_channel_test_t * ) mgmtdata;
						memcpy( response_buffer, (int )&getserialmode->mode, response_length);
						break;
					}

	                case GET_DS1_CONFIG:
                    {
						getset_ds1_config_t * getds1cfg;
						getds1cfg = ( getset_ds1_config_t * ) mgmtdata;
						memcpy( response_buffer, (int )&getds1cfg->config, response_length);
						break;
                    }

					default:
						memcpy( response_buffer, (int )mgmtdata, response_length);
						break;

					}

#else
					memcpy( response_buffer,
						(int) mgmtdata + mgmtTbl[i].req_len,
						mgmtTbl[i].rsp_len );
#endif

#else
					memcpy( response_buffer,
						mgmtdata + mgmtTbl[i].req_len,
						mgmtTbl[i].rsp_len );

#endif
	}

#endif /* !defined(SOLARIS) */

			break;
		} /* if ( mgmtTbl[i].cmd == command )... */
	} /* for ( i = 0; i < MGMT_MAX; i++ )... */



	if ( mgmtdata != NULL )
		free( mgmtdata );

	close( dd );

	return status;

}

/* ----------------------------------------------------------------------------

  MgmtCommand( )

---------------------------------------------------------------------------- */

#if defined(LINUX) || defined(SOLARIS) || defined(HPUX)
uint32 MGMT_DECL
#else
unsigned long
#endif
MgmtCommand( uint32 board_number, uint16 command,
	void *request_buffer, uint16 request_length,
	void *response_buffer, uint16 response_length )
{
	int status;

#if 0
	printf( "MgmtCommand\n" );
	printf( "board_number: %d\n", board_number );
	printf( "command: %d\n", command );
	printf( "request_buffer: %x\n", request_buffer );
	printf( "request_length: %d\n", request_length );
	printf( "response_buffer: %x\n", response_buffer );
	printf( "response_length: %d\n", response_length );
#endif

#if defined(UW7)
#if DBUG
	printf( "MgmtCommand: command: 0x%x\n", command );
#endif
#endif

	if ( command & RESERVED_BIT ){
		status = mgmt_shim_command( board_number, command,
			request_buffer, request_length,
			response_buffer, response_length );
	} else {
		status = mgmt_channel_command( board_number, command,
			request_buffer, request_length,
			response_buffer, response_length );
	}
#if 0
	fprintf( stderr, "status: %d\n", status );
#endif

	return status;
}

/* ----------------------------------------------------------------------------

  MgmtTrace( )

---------------------------------------------------------------------------- */

#if defined(LINUX) || defined(SOLARIS) || defined(HPUX)
uint32 MGMT_DECL
#else
unsigned long
#endif
MgmtTrace( uint32 board_number, uint16 command,
	void *request_buffer, uint16 request_length,
	void *response_buffer, uint16 response_length,
	TRC_DATACOPY_CB callback_function, TRCARG trace_handle )
{
	int status;

	switch( command )
	{

	default:
		/* unknown command */
		status = MGMT_SHIM_ERROR_BAD_COMMAND;
		break;

	} /* end switch */

	return status;
}

void
print_serial_channel_mode( const serial_channel_test_t	*mode)
{
	//printf( " mode  \t\t: %d \n",mode->mode);
	switch( mode->mode )
	{
		case TEST_MODE:
			printf( " mode  \t\t: TEST_MODE \n");
			break;
		case EXT_LOOP_AUTO:
			printf( " mode  \t\t: EXT_LOOP_AUTO \n");
			break;
		case EXT_LOOP_MANUAL:
			printf( " mode  \t\t: EXT_LOOP_MANAUL\n");
			break;
		case ECHO_TEST:
			printf( " mode  \t\t: EXHO_TEST \n");
			break;
		case REPORT_RESULTS:
			printf( " mode  \t\t: REPORT_RESULTS \n");
			break;
		case STOP_TEST:
			printf( " mode  \t\t: STOP_TEST \n");
			break;
		case TEST_EXIT:
			printf( " mode  \t\t: TEST_EXIT \n");
			break;
		default:
			printf( " mode  \t\t: %d \n",mode->mode);
			break;
		}
	printf( " mode  \t\t: TEST_MODE=%x \n", TEST_MODE);
	printf( " mode  \t\t: ECHO_TEST s=%x \n", ECHO_TEST);
	printf( " mode  \t\t: REPORT_RESULTS = %x \n", REPORT_RESULTS);
	printf( " mode  \t\t: EXT_LOOP_MANAUL=%x\n", EXT_LOOP_MANUAL);
	printf( " mode  \t\t: EXT_LOOP_AUTO=%x\n", EXT_LOOP_AUTO);
	printf( " mode  \t\t: STOP_TEST =%x\n", STOP_TEST);

	printf( " good  \t\t: %d \n",mode->good); 
	printf( " fail  \t\t: %d \n",mode->fail);
	printf( " *****  \t\t\n");
}
