
/*
 Copyright 1999 by Digi International (www.digi.com)

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED; without even the 
 implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
 * $Id: dm_admin.c,v 1.14 2000/11/01 06:25:02 jamesp Exp $
 *
 * Admin program for dense modem products using the /dev/dm/mgmt[0-n]
 * device.
 *
 *	$Log: dm_admin.c,v $
 *	Revision 1.14  2000/11/01 06:25:02  jamesp
 *	Removed the "dm_portreg -m" call and replaced it with a "dm_portreg -c"
 *	call... this fixes a problem where dm_admin has to know how to parse
 *	an RS485 configuration structure into the calling convention for
 *	dm_portreg -m.
 *	
 *	Revision 1.13  2000/10/03 20:19:42  jamesp
 *	Cleaned up the altpin handling in dm_admin.c, and added MEI support.
 *	
 *	To enable MEI support, XP_MEI and DM_PORTREG_PATH both have to be defined
 *	during the build.  The MEI implementation uses ioctls to set/get the
 *	values in the adapter, and an external application to maintain the
 *	backing store of settings.
 *	
 *	Revision 1.12  2000/09/15 11:57:51  dickl
 *	Changed displayed short haul line lengths in dm_gui and dm_admin to be
 *	consistent with industry standards
 *	
 *	Revision 1.11  2000/08/30 01:40:18  jordanh
 *		*	Added -r which applies a cold modem reset to the
 *			specified channel.
 *		*	Updated dm_admin man-page to reflect above change.
 *	
 *	Revision 1.10  2000/08/03 16:48:41  randall
 *	- Updates to allow for MAXBOARDS of 4 or 8.
 *	
 *	Revision 1.9  2000/05/25 22:27:09  chrisel
 *	handle the real geographic address if present on cPCI cards.
 *	
 *	Revision 1.8  2000/05/25 16:27:57  dickl
 *	Oops, need to allow 0 as a choice for --cd_bits
 *	
 *	Revision 1.7  2000/05/25 13:21:28  dickl
 *	more MSA and E1 R2 work
 *	
 *	Revision 1.6  2000/05/22 15:43:14  dickl
 *	Add MSA and E1 R2 switch type for E1 adapters
 *	
 *	Revision 1.5  2000/04/09 06:30:11  erics
 *	E1 R2 changes
 *	
 *	Revision 1.4  2000/03/16 18:32:57  steveb
 *	Added a couple of "#ifndef SCO" statements so that debug messages do
 *	not get printed out on device opens during program execution.
 *	
 *	Revision 1.3  2000/03/09 18:16:15  dickl
 *	Remove dupicate include for stdio
 *	
 *	Revision 1.1  2000/01/30 16:21:51  dickl
 *	Migrated various dgdm (linux) utilities to this cvs module
 *	to make it accessible to all Unix variants
 *	
 *	Revision 1.23  2000/01/05 13:39:36  dickl
 *	Corrected spelling of decibel
 *	
 *	Revision 1.22  1999/12/14 17:30:09  dickl
 *	Changed adapter info display to show fields based on adapter type
 *	
 *	Revision 1.21  1999/12/07 14:07:23  dickl
 *	Remove upper bit from all NFAS parameter tests
 *	
 *	Revision 1.20  1999/12/02 19:52:02  dickl
 *	Added checking and consistency checking to dm_admin
 *	Updated man page to refled dm_admin changes
 *	
 *	Revision 1.19  1999/11/04 13:10:28  dickl
 *	Reworked many of the query displays for consistency and greater readability
 *	Allowed multiple queries per dm_admin call
 *	Added a stderr message if config changes mixed with queries
 *	
 *	Revision 1.18  1999/10/13 16:17:20  dickl
 *	Added jitter attenuation to list of setable parameters
 *	Added D-channel address type, numbering plan and subaddress type to
 *	list of setable parameters
 *	
 */

#include <stdio.h>

#ifndef NEED_LONG_OPTIONS
#include <getopt.h>
#endif

#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <errno.h>

#ifndef MAXBOARDS
# define MAXBOARDS 4
#endif

/*
 * The "standard" Digi base objects (re)defined here.
 * It's only about the bazillionth place they are defined
 * in the dense tree..
 */


typedef	unsigned int	uint32 ;
typedef	unsigned short	uint16 ;
typedef	unsigned char	uint8 ;

typedef uint8	dxb_uint8;
typedef uint16	dxb_uint16;
typedef uint32	dxb_uint32;
typedef uint32	pad_32;


#include "mgmtioc1.h"
#include "trace_os_dep.h"
#include "mgmtcc.h"
#include "drivercc.h"

#ifdef NEED_LONG_OPTIONS

/* Describe the long-named options requested by the application.
   The LONG_OPTIONS argument to getopt_long or getopt_long_only is a vector
   of `struct option' terminated by an element containing a name which is
   zero.

   The field `has_arg' is:
   no_argument		(or 0) if the option does not take an argument,
   required_argument	(or 1) if the option requires an argument,
   optional_argument 	(or 2) if the option takes an optional argument.

   If the field `flag' is not NULL, it points to a variable that is set
   to the value given in the field `val' when the option is found, but
   left unchanged if the option is not found.

   To have a long-named option do something other than set an `int' to
   a compiled-in constant, such as set a value from `optarg', set the
   option's `flag' field to zero and its `val' field to a nonzero
   value (the equivalent single-letter option character, if there is
   one).  For long options that have a zero `flag' field, `getopt'
   returns the contents of the `val' field.  */

struct option
{
  const char *name;
  /* has_arg can't be an enum because some compilers complain about
     type mismatches in all the code that assumes it is an int.  */
  int has_arg;
  int *flag;
  int val;
};

/* Names for the values of the `has_arg' field of `struct option'.  */

# define no_argument		0
# define required_argument	1
# define optional_argument	2

static int _getopt_internal (int __argc, char *const *__argv,
 const char *__shortopts, const struct option *__longopts,
 int *__longind, int long_only);

#endif

#define	asizeof(x)	(sizeof(x) / sizeof(x[0]))

#define FAMILY_MASK 0xfff0

/*
 * Prototypes
 */

void validate_t1_settings (int fd);
void validate_e1_settings (int fd);

/*
 * Global variables for which adapter (default 0), port (no default)
 * and channel (no default)
 */

int board = 0 ;
int port = -1 ;
int channel = -1 ;
int flat_channel = -1 ;

/*
 * Global structures for configuration get/sets
 */

get_adapter_info_t info ;
getset_ds1_config_t	ds1_config ;
getset_ds1_ext_config_t	ds1_ext_config ;
getset_ds1_e1r2_config_t ds1_e1r2_config ;
getset_bri_config_t bri_config ;
getset_dchan_ext_config_t dchan_config ;

/*
 * Global structures for checking current switch type
 */

getset_ds1_config_t	cur_ds1_config ;

/*
 * Global variables for why we are being called:  query or config change
 * Error flag
 */

int query_call = 0 ;
int config_call = 0 ;

/*
 * Global variables for config change types
 */

int config_error = 0;	/* Invalid or incompatible option specified */
int config_xri = 0;		/* BRI/DS1 config change made */
int config_bri = 0;		/* BRI config change made */
int config_ds1 = 0;		/* DS1 config change made */
int config_ds1_r2 = 0;	/* DS1 E1R2 config change made */
int config_ds1x = 0;	/* DS1 extended config change made */
int config_dchanx = 0;	/* dchan extended config change made */
int config_t1 = 0;		/* T1/2 unique config change made */
int config_t2 = 0;		/* T2 unique config change made */
int config_e1 = 0;		/* E1 unique config change made */
int config_modem = 0;	/* Modem config change made */
int config_serial = 0;

/*
 * Global variables for which queries being made
 */

int query = 0 ;
uint32 bchan = 0 ;
uint32 dchan = 0 ;
uint32 serial = 0 ;
uint32 serial_config = 0 ;
uint32 rs485_config = 0 ;
uint32 modem_stat = 0 ;
uint32 e1r2_print_config = 0 ;

/*
 * Global variables for configuration changes
 */

struct str_change {
	int changed;	/* 0=> no change, 1 => change */
	char *new_str;	/* If changed == 1, pointer to new string */
};

struct str_change number[MSN_MAX] ;
struct str_change spid[MSN_MAX] ;
struct str_change sub_address[MSN_MAX] ;

uint32 back2back = 0 ;
uint32 address_type = 0 ;
uint32 subaddress_type = 0 ;
uint32 numbering_plan = 0 ;
uint32 country_code = 0 ;
int  dir_count = 0 ;
int switch_type = 0 ;
uint32 leased_mask = 0 ;
uint32 modem_lock = 0 ;
int bri_msn_option = 0 ;
int bri_alert_option = 0 ;
int jitter_atten = 0 ;
int long_haul_lbo = 0 ;
int short_haul_len = 0 ;
int framing = 0 ;
int eq_gain_limit = 0 ;
int nfas_master_port = 0 ;
int nfas_interface_id = 0 ;
int if_type = 0 ;
int coding = 0 ;
int loopback = 0 ;
int clocking = 0 ;
int tx_special = 0 ;
int wink_digits = 0 ;

int port_type = 0 ;
int rx_enable = 0 ;
int tx_enable = 0 ;

int msa_switch = 0 ;
int e1r2_switch = 0 ;

/*
 * Do a hang-up on this channel
 */

int hangup = 0 ;

/*
 * Do a modem reset on this channel
 */

int reset = 0 ;


/*
 *	Modem Nationality tables
 */

#ifdef DFRAS
uint32 hou30_cc[] = HOUSTON_E1_DRV_CC_LIST_INIT;
uint32 hou60_cc[] = HOUSTON_E1_DRV_CC_LIST_INIT;
uint32 hou24_cc[] = HOUSTON_T1_DRV_CC_LIST_INIT;
uint32 hou48_cc[] = HOUSTON_T1_DRV_CC_LIST_INIT;
uint32 lonu_cc[] = LONDON_U_DRV_CC_LIST_INIT;
uint32 lonst_cc[] = LONDON_ST_DRV_CC_LIST_INIT;
#endif
uint32 chas40_cc[] = CHASKA_DAA0_DRV_CC_LIST_INIT;
uint32 chas41_cc[] = CHASKA_DAA1_DRV_CC_LIST_INIT;
uint32 chas42_cc[] = CHASKA_DAA2_DRV_CC_LIST_INIT;
uint32 chas43_cc[] = CHASKA_DAA3_DRV_CC_LIST_INIT;
uint32 chas80_cc[] = CHASKA_DAA0_DRV_CC_LIST_INIT;
uint32 chas81_cc[] = CHASKA_DAA1_DRV_CC_LIST_INIT;
uint32 chas82_cc[] = CHASKA_DAA2_DRV_CC_LIST_INIT;
uint32 chas83_cc[] = CHASKA_DAA3_DRV_CC_LIST_INIT;

typedef struct {
	dxb_uint32	board_type;
	char		*name;
	uint32		*cc_array;
	uint32		array_size;
} board_type_2_list_init_t;

board_type_2_list_init_t board2list[] = {
#ifdef DFRAS
	{ADAPTER_HOU24, DIGI_PCIHOUSTON24_NAME, hou24_cc, asizeof (hou24_cc)},
	{ADAPTER_HOU24cpci, DIGI_CPCIHOUSTON24_NAME, hou24_cc, asizeof (hou24_cc)},
	{ADAPTER_HOU48, DIGI_PCIHOUSTON48_NAME, hou48_cc, asizeof (hou48_cc)},
	{ADAPTER_HOU48cpci, DIGI_CPCIHOUSTON48_NAME, hou48_cc, asizeof (hou48_cc)},
	{ADAPTER_HOU30, DIGI_PCIHOUSTON30_NAME, hou30_cc, asizeof (hou30_cc)},
	{ADAPTER_HOU30cpci, DIGI_CPCIHOUSTON30_NAME, hou30_cc, asizeof (hou30_cc)},
	{ADAPTER_HOU60, DIGI_PCIHOUSTON60_NAME, hou60_cc, asizeof (hou60_cc)},
	{ADAPTER_HOU60cpci, DIGI_CPCIHOUSTON60_NAME, hou60_cc, asizeof (hou60_cc)},
	{ADAPTER_LONU, DIGI_LONDONU_NAME, lonu_cc, asizeof (lonu_cc)},
	{ADAPTER_LONST, DIGI_LONDONST_NAME, lonst_cc, asizeof (lonst_cc)},
#endif
	{ADAPTER_CHASKA40, DIGI_CHASKA40_NAME, chas40_cc, asizeof (chas40_cc)},
	{ADAPTER_CHASKA41, DIGI_CHASKA40_NAME, chas41_cc, asizeof (chas41_cc)},
	{ADAPTER_CHASKA42, DIGI_CHASKA40_NAME, chas42_cc, asizeof (chas42_cc)},
	{ADAPTER_CHASKA43, DIGI_CHASKA40_NAME, chas43_cc, asizeof (chas43_cc)},
	{ADAPTER_CHASKA80, DIGI_CHASKA80_NAME, chas80_cc, asizeof (chas80_cc)},
	{ADAPTER_CHASKA81, DIGI_CHASKA80_NAME, chas81_cc, asizeof (chas81_cc)},
	{ADAPTER_CHASKA82, DIGI_CHASKA80_NAME, chas82_cc, asizeof (chas82_cc)},
	{ADAPTER_CHASKA83, DIGI_CHASKA80_NAME, chas83_cc, asizeof (chas83_cc)}
};

CC_NAME_PAIR cc_2_name[] = ISO_CC_NAME_PAIR_INIT;

void
usage(void)
{
	fprintf(stdout,
"\nConfigure Digi International adapters, usage:\n\n"
"       dm_admin [--help] [-b adapter] [-p port] [-c channel]\n"
#ifdef XP_MEI
"       [-N] [-v] [-q] [-B] [-D] [-G] [-g] [-S] [-E] [-C modem_nat]\n"
#else
"       [-N] [-v] [-q] [-B] [-D] [-G] [-g] [-E] [-C modem_nat]\n"
#endif /* XP_MEI */
"       [-m busy_out] [-h] [-r] [-f framing] [-i if_type] [-l coding]\n"
"       [-s switch] [-t clocking] [-O decibels] [-o length]\n"
"       [-e eq_gain] [-w wink_digits] [-L loopback] [-M leased_mask]\n"
"       [-T tx_spec] [-I back2back] [-F nfas_port] [-R nfas_id]\n"
"       [-Y addr_type] [-y subaddr_type] [-P num_plan] [-j jitter]\n"
"       [--A1 dirnbr1] [--A2 dirnbr2] ... [--A8 dirnbr8]\n"
"       [--S1 spid1] [--S2 spid2]\n"
"       [--a1 subaddr1] [--a2 subaddr2] ... [--a8 subaddr8]\n"
"       [-x msn] [-X autoalert] [-z] [--e1r2_sw_type type]\n"
#ifdef XP_MEI
"       [--porttype type] [--tx_enable val] [--rx_enable val]\n"
#endif /* XP_MEI */
"       [--print_e1r2 ] [--cd_bits bits] [--r2_method method]\n"
"       [--max_dnis nbr ] [--min_dnis nbr] [--max_ani nbr]\n"
"       [--min_ani nbr ] [--send_ani send] [--ani_sig nbr]\n"
"       [--answer_sig nbr ] [--congest_sig nbr] [--end_dnis_sig nbr]\n"
"       [--busy_sig nbr ] [--sz_ack_delay mscs] [--answer_delay mscs]\n"
"       [--incoming_allow msk ] [--group2_type type] [--linesig_min mscs]\n"
"\nEnter \"dm_admin --help\" for the long version of usage\n"
"\nSee \"man dm_admin\" for the most complete information\n\n"
);
}

void
long_usage(void)
{
	fprintf(stdout,
"Usage:	dm_admin [options]\n"
"\n"
"Configure Digi Intl. adapters:\n"
#ifdef DFRAS
"\t" DIGI_PCIHOUSTON24_NAME ", " DIGI_CPCIHOUSTON24_NAME "\n"
"\t" DIGI_PCIHOUSTON48_NAME ", " DIGI_CPCIHOUSTON48_NAME "\n"
"\t" DIGI_PCIHOUSTON30_NAME ", " DIGI_CPCIHOUSTON30_NAME "\n"
"\t" DIGI_PCIHOUSTON60_NAME ", " DIGI_CPCIHOUSTON60_NAME "\n"
"\t" DIGI_LONDONU_NAME "\n"
"\t" DIGI_LONDONST_NAME "\n"
#endif
"\t" DIGI_CHASKA80_NAME "\n"
"\t" DIGI_CHASKA40_NAME "\n"
"\t" DIGI_AP2K_NAME "\n"
"\t" DIGI_AP4K_NAME "\n"
"\t" DIGI_AP8K_NAME "\n"
"\t" DIGI_AP16K_NAME "\n"
#ifdef DIGI_AP24K_NAME
"\t" DIGI_AP24K_NAME "\n"
#endif
#ifdef DIGI_AP24K_NAME
"\t" DIGI_AP32K_NAME "\n"
#endif
#ifdef XP_MEI /* AccelePort Xp multi-electrical interface */
"\t" DIGI_AP4K_MPI_NAME "\n"
"\t" DIGI_AP8K_MPI_NAME "\n"
"\t" DIGI_AP16K_MPI_NAME "\n"
#endif
"\n"
"Main Options:\n"
"--help\t\tDisplay long usage text\n"
"-b adapter\tAdapter number (0-3) [0]\n"
"-p port\t\tPort Number (1-32) [none]\n"
"-c channel\tChannel Number (1-32) [none]\n"
"-N\t\tDisplay all Modem Nationalities\n"
"\t\t\tIgnores -b, -p and -c\n"
"-q\t\tQuery adapter if -p not specified\n"
"\t\t\tQuery port if -p specified\n"
"\t\t\tIgnores -c\n"
#ifdef DFRAS
"-B\t\tQuery B-chan\n"
"\t\t\tRequires -p and -c\n"
"-D\t\tQuery D-chan\n"
"\t\t\tRequires -p\n"
"\t\t\tIgnores -c\n"
#endif
"-G\t\tQuery serial-chan status\n"
"\t\t\tRequires -p and -c\n"
"-g\t\tQuery serial-chan config\n"
"\t\t\tRequires -p and -c\n"
"-E\t\tQuery modem info\n"
"\t\t\tRequires -p and -c\n"
"-v\t\tPrint dm_admin revision only\n"
"\t\t\tIgnores -b, -p and -c\n"
"-C modem_nat\tSet Modem Nationality\n"
"\t\t\tIgnores -p and -c\n"
"-m\t\tSet modem busy-out (1=DISABLE, 2=IMMEDIATE, 3=DELAYED)\n"
"\t\t\tRequires -p and -c\n"
"-h\t\tForce modem hangup\n"
"\t\t\tRequires -p and -c\n"
"-r\t\tForce modem cold reset\n"
"\t\t\tRequires -p and -c\n"
#ifdef DEVELOPER
"-Z count\tRepeat config command count times (for testing only)\n"
#endif
"\n"
#ifdef DFRAS
DIGI_PCIHOUSTON24_NAME ", " DIGI_CPCIHOUSTON24_NAME ", " DIGI_PCIHOUSTON48_NAME
" and\n" DIGI_CPCIHOUSTON48_NAME " unique options:\n"
"\t[default] for all is [nochange], requires -p\n"
"-f framing\tSet framing type (1=ESF, 2=D4)\n"
"-i if_type\tSet IF type (1=LONG_HAUL, 2=SHORT_HAUL, 3=Odd/Even loopback)\n"
"-l coding\tSet coding (1=B8ZS, 2=AMI)\n"
"-s switch\tSet switch type (1=LOOP, 2=WINK, 3=GROUND, 4=IMMED,\n"
"\t\t\t32=PRI_NONE, 33=NI2, 34=5ESS, 35=DMS100,\n"
"\t\t\t37=INS-1500)\n"
"-t clocking\tSet clocking (1=LOOP, 2=LOCAL)\n"
"-O decibels\tSet long haul level (1=0.0db, 2=7.5db, 3=15.0db, 4=22.5db)\n"
"-o length\tSet short haul length (1=0/133ft, 2=134/266ft, 3=267/399ft,\n"
"\t\t\t4=400/533ft, 5=534/655ft)\n"
"-w wink_digits\tSet wink_digits (1-20)\t(WINK start only)\n"
"-j jitter\tSet jitter attenuation (1=TX, 2=RX, 3=AUTO, 4=NONE)\n"
"-e eq_gain\tSet equalization gain limit (1=LOW, 2=HIGH)\n"
"-Y addr_type\tSet D-channel address type (1=UNKN, 2=INTL, 3=NATL, 4=NET\n"
"\t\t\t5=SUBSCR, 6=ABBREV)\n"
"-y subaddr_type\tSet D-channel subaddress type (1=NSAP, 2=USER)\n"
"-P num_plan\tSet D-channel numbering plan (1=UNKN, 2=ISDN, 3=DATA,\n"
"\t\t\t4=TELEX, 5=NATL, 6=PRIVATE)\n"
"-L loopback\tSet network loopback (1=NO, 2=PAYLOAD, 3=LINE, 4=LOCAL)\n"
"-M leased_mask\tSet leased_mask (bitmap)\n"
"-T tx_special\tSet tx_special (1=NORMAL, 2=PRSS, 3=QRSS, 4=LLU,\n"
"\t\t\t5=LLD, 6=FDL_LLU, 7=FDL_LLD, 8=FDL_PLU, 9=FDL_PLD,\n"
"\t\t\t10=RFA, 11=Unframed AIS)\n"
"-I back2back\tSet back-to-back mode (1=DISABLED, 2=NT, 3=TE)\n"
"\n"
DIGI_PCIHOUSTON48_NAME " and " DIGI_CPCIHOUSTON48_NAME " unique options:\n"
"\t[default] for all is [nochange], requires -p\n"
"-F nfas_port\tSet NFAS master port (0=No NFAS, 1=Port 1, 2=Port 2)\n"
"-R nfas_id\tSet NFAS interface port identifier (0-127)\n"
"-M leased_mask\tSet leased_mask (bitmap)\n"
"\n"
DIGI_PCIHOUSTON30_NAME ", " DIGI_CPCIHOUSTON30_NAME ", " DIGI_PCIHOUSTON60_NAME
 " and\n" DIGI_CPCIHOUSTON60_NAME " unique options:\n"
"\t[default] for all is [nochange], requires -p\n"
"-f framing\tSet framing type (3=CEPT/CRC4, 4=CEPT)\n"
"-l coding\tSet coding (3=HDB3)\n"
"-s switch\tSet switch type (12=E1_R2, 32=PRI_NONE, 36=PRI_EURODSS1, 38=MSA)\n"
"-t clocking\tSet clocking (1=LOOP, 2=LOCAL)\n"
"-j jitter\tSet jitter attenuation (1=TX, 2=RX, 3=AUTO, 4=NONE)\n"
"-Y addr_type\tSet D-channel address type (1=UNKN, 2=INTL, 3=NATL, 4=NET\n"
"\t\t\t5=SUBSCR, 6=ABBREV)\n"
"-y subaddr_type\tSet D-channel subaddress type (1=NSAP, 2=USER)\n"
"-P num_plan\tSet D-channel numbering plan (1=UNKN, 2=ISDN, 3=DATA,\n"
"\t\t\t4=TELEX, 5=NATL, 6=PRIVATE)\n"
"-L loopback\tSet network loopback (1=NO, 2=PAYLOAD, 3=LINE, 4=LOCAL)\n"
"-M leased_mask\tSet leased_mask (bitmap)\n"
"-T tx_special\tSet tx_special (1=NORMAL, 2=PRSS, 3=QRSS, 4=LLU,\n"
"\t\t\t5=LLD, 6=FDL_LLU, 7=FDL_LLD, 8=FDL_PLU, 9=FDL_PLD,\n"
"\t\t\t10=RFA, 11=Unframed AIS)\n"
"-I back2back\tSet back-to-back mode (1=DISABLED, 2=NT, 3=TE)\n"
"\n E1 R2 signalling options (valid only with -s 12 and -f 3 or 4):\n"
" --print_e1r2\tPrint out the current options in command line format\n"
" --e1r2_sw_type\tSet E1 R2 switch type, many default to ITU behavior\n"
"\t\t\t(1=ITU, 2=ABACUS, 3=ARGENTINA, 4=AUSTRALIA,\n"
"\t\t\t5=BRAZIL, 6=CHINA, 7=COLUMBIA, 8=COSTA_RICA,\n"
"\t\t\t9=CZECH, 10=EAST_EUROPE, 11=ECUADOR_ITU,\n"
"\t\t\t12=ECUADOR_LME, 13=GREECE, 14=GUATEMALA,\n"
"\t\t\t15=HONG_KONG, 16=INDIA, 17=INDONESIA,\n"
"\t\t\t18=ISRAEL, 19=KOREA, 20=MALAYSIA,\n"
"\t\t\t21=NEW_ZEALAND, 22=PARAGUAY, 23=PERU,\n"
"\t\t\t24=PHILLIPPINES, 25=SAUDI_ARABIA, 26=SINGAPORE,\n"
"\t\t\t27=SOUTH_AFRICA_PANAFTEL, 28=MEXICO_TELMEX,\n"
"\t\t\t29=MEXICO_TELNOR, 30=THAILAND, 31=URUGUAY,\n"
"\t\t\t32=VENEZUELA, 33=VIETNAM)\n"
" --cd_bits\tSet the Transmitted C and D bits (0x0, 0x1, 0x2, 0x3)\n"
" --r2_method\tSet the Method (1=MF Compelled, 2=Inverted ABCD, 3=Metered)\n"
" --max_dnis\tSet the maximum number for incoming DNIS digits (0-64)\n"
" --min_dnis\tSet the minimum number for incoming DNIS digits (0-64)\n"
" --max_ani\tSet the maximum number for incoming ANI digits (0-64)\n"
" --min_ani\tSet the minimum number for incoming ANI digits (0-64)\n"
" --send_ani\tSet send ANI (Calling) digits (1=TRUE, 0=FALSE)\n"
" --ani_sig\tSet backward signal requesting another ANI digit (1-15)\n"
" --answer_sig\tSet backward signal to answer a call (1-15)\n"
" --congest_sig\tSet backward signal to advertise congestion (1-15)\n"
" --end_dnis_sig\tSet Group B signal noting complete DNIS (Called Party) (1-15)\n"
" --busy_sig\tSet Group B signal to advertise a busy signal (1-15)\n"
" --sz_ack_delay\tSet Time to Delay Seize Ack (in ms)\n"
" --answer_delay\tSet Time to Delay Answer Signal (in ms)\n"
" --incoming_allow Set Types of Incoming Calls to Accept (bitmap)\n"
" --group2_type\tSet Outgoing Call Type (1-9)\n"
" --linesig_min\tSet Minimum Line Signal Recognition time (in ms)\n"
"\n"
DIGI_LONDONU_NAME " and " DIGI_LONDONST_NAME " unique options:\n"
"\t[default] for all is [nochange], requires -p\n"
"-s switch\tSet switch type (16=BRI_NONE, 17=NI1, 18=5ESS,\n"
"\t\t\t19=DMS100, 20=BRI_EURODSS1, 21=INS-64)\n"
"--A<n> dirnbr (--A1 to --A8 can be specified)\n"
"\t\tSet a directory number\n"
"--S<n> spid (--S1 to --S2 can be specified)\n"
"\t\tSet a SPID sequence\t(non-EURODSS1 and non-INS-64 only)\n"
"--a<n> subaddr (--a1 to --a8 can be specified)\n"
"\t\tSet a sub-address sequence\t(EURODSS1 and INS-64 only)\n"
"-x msn\t\tSet multiple subscriber number (1 off, 2 on) (EURODSS1 only)\n"
"-X autoalert\tSet autoalerting (1 off, 2 on)\n"
"-Y addr_type\tSet D-channel address type (1=UNKN, 2=INTL, 3=NATL, 4=NET\n"
"\t\t\t5=SUBSCR, 6=ABBREV)\n"
"-y subaddr_type\tSet D-channel subaddress type (1=NSAP, 2=USER)\n"
"-P num_plan\tSet D-channel numbering plan (1=UNKN, 2=ISDN, 3=DATA,\n"
"\t\t\t4=TELEX, 5=NATL, 6=PRIVATE)\n"
"-z\t\tClear directory numbers.\n"
"-I back2back\tSet back-to-back mode (1=DISABLED, 2=NT, 3=TE)\n"
"-M leased_mask\tSet leased_mask (bitmap)\n"
"\n"
#endif
DIGI_CHASKA_FAMILY_NAME " family of adapters unique options:\n"
"\t[default] for all is [nochange], requires -p\n"
"\t[No unique options]\n"
"\n"
DIGI_AP2K_FAMILY_NAME " family of adapters unique options:\n"
"\t[default] for all is [nochange], requires -p and -c\n"
#ifdef XP_MEI
"-S\t\tQuery RS485 config\n"
"--porttype type\tSet port type\n"
"\t\t\t1 = RS232\n"
"\t\t\t2 = RS422\n"
"\t\t\t5 = RS485 2-wire, midpoint\n"
"\t\t\t6 = RS485 2-wire, endpoint\n"
"\t\t\t7 = RS485 4-wire, midpoint\n"
"\t\t\t8 = RS485 4-wire, endpoint\n"
"--tx_enable val\tSet transmitter enable\n"
"\t\t\t1 = Transmit only when RTS signal is asserted\n"
"\t\t\t2 = Automatically raise RTS when transmitting\n"
"--rx_enable val\tSet receiver enable\n"
"\t\t\t1 = Automatically disable receiver when transmitting\n"
"\t\t\t2 = Always have receiver enabled\n"
#else /* !XP_MEI */
"\t[No unique options]\n"
#endif /* XP_MEI */
"\nSee \"man dm_admin\" for additional information\n\n"
	);
}


#ifdef NEED_LONG_OPTIONS
/* Getopt for GNU.
   NOTE: getopt is now part of the C library, so if you don't know what
   "Keep this file name-space clean" means, talk to drepper@gnu.org
   before changing it!

   Copyright (C) 1987, 88, 89, 90, 91, 92, 93, 94, 95, 96, 97, 98, 99
   	Free Software Foundation, Inc.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the GNU C Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

/* Comment out all this code if we are using the GNU C Library, and are not
   actually compiling the library itself.  This code is part of the GNU C
   Library, but also included in many other GNU distributions.  Compiling
   and linking in this code is a waste when using the GNU C library
   (especially if it is a shared library).  Rather than having every GNU
   program understand `configure --with-gnu-libc' and omit the object files,
   it is simpler to just do this in the source for each such file.  */

#define GETOPT_INTERFACE_VERSION 2
#if !defined _LIBC && defined __GLIBC__ && __GLIBC__ >= 2
# include <gnu-versions.h>
# if _GNU_GETOPT_INTERFACE_VERSION == GETOPT_INTERFACE_VERSION
#  define ELIDE_CODE
# endif
#endif

#ifndef ELIDE_CODE


/* This needs to come after some library #include
   to get __GNU_LIBRARY__ defined.  */
#ifdef	__GNU_LIBRARY__
/* Don't include stdlib.h for non-GNU C libraries because some of them
   contain conflicting prototypes for getopt.  */
# include <stdlib.h>
# include <unistd.h>
#endif	/* GNU C library.  */

#ifndef _
/* This is for other GNU distributions with internationalized messages.
   When compiling libc, the _ macro is predefined.  */
# ifdef HAVE_LIBINTL_H
#  include <libintl.h>
#  define _(msgid)	gettext (msgid)
# else
#  define _(msgid)	(msgid)
# endif
#endif

/* This version of `getopt' appears to the caller like standard Unix `getopt'
   but it behaves differently for the user, since it allows the user
   to intersperse the options with the other arguments.

   As `getopt' works, it permutes the elements of ARGV so that,
   when it is done, all the options precede everything else.  Thus
   all application programs are extended to handle flexible argument order.

   Setting the environment variable POSIXLY_CORRECT disables permutation.
   Then the behavior is completely standard.

   GNU application programs can use a third alternative mode in which
   they can distinguish the relative order of options and other arguments.  */

/* For communication from `getopt' to the caller.
   When `getopt' finds an option that takes an argument,
   the argument value is returned here.
   Also, when `ordering' is RETURN_IN_ORDER,
   each non-option ARGV-element is returned here.  */

char *optarg;

/* Index in ARGV of the next element to be scanned.
   This is used for communication to and from the caller
   and for communication between successive calls to `getopt'.

   On entry to `getopt', zero means this is the first call; initialize.

   When `getopt' returns -1, this is the index of the first of the
   non-option elements that the caller should itself scan.

   Otherwise, `optind' communicates from one call to the next
   how much of ARGV has been scanned so far.  */

/* 1003.2 says this must be 1 before any call.  */
int optind = 1;

/* Formerly, initialization of getopt depended on optind==0, which
   causes problems with re-calling getopt as programs generally don't
   know that. */

int __getopt_initialized;

/* The next char to be scanned in the option-element
   in which the last option character we returned was found.
   This allows us to pick up the scan where we left off.

   If this is zero, or a null string, it means resume the scan
   by advancing to the next ARGV-element.  */

static char *nextchar;

/* Callers store zero here to inhibit the error message
   for unrecognized options.  */

int opterr = 1;

/* Set to an option character which was unrecognized.
   This must be initialized on some systems to avoid linking in the
   system's own getopt implementation.  */

int optopt = '?';

/* Describe how to deal with options that follow non-option ARGV-elements.

   If the caller did not specify anything,
   the default is REQUIRE_ORDER if the environment variable
   POSIXLY_CORRECT is defined, PERMUTE otherwise.

   REQUIRE_ORDER means don't recognize them as options;
   stop option processing when the first non-option is seen.
   This is what Unix does.
   This mode of operation is selected by either setting the environment
   variable POSIXLY_CORRECT, or using `+' as the first character
   of the list of option characters.

   PERMUTE is the default.  We permute the contents of ARGV as we scan,
   so that eventually all the non-options are at the end.  This allows options
   to be given in any order, even with programs that were not written to
   expect this.

   RETURN_IN_ORDER is an option available to programs that were written
   to expect options and other ARGV-elements in any order and that care about
   the ordering of the two.  We describe each non-option ARGV-element
   as if it were the argument of an option with character code 1.
   Using `-' as the first character of the list of option characters
   selects this mode of operation.

   The special argument `--' forces an end of option-scanning regardless
   of the value of `ordering'.  In the case of RETURN_IN_ORDER, only
   `--' can cause `getopt' to return -1 with `optind' != ARGC.  */

static enum
{
  REQUIRE_ORDER, PERMUTE, RETURN_IN_ORDER
} ordering;

/* Value of POSIXLY_CORRECT environment variable.  */
static char *posixly_correct;

#ifdef	__GNU_LIBRARY__
/* We want to avoid inclusion of string.h with non-GNU libraries
   because there are many ways it can cause trouble.
   On some systems, it contains special magic macros that don't work
   in GCC.  */
# include <string.h>
# define my_index	strchr
#else

# if HAVE_STRING_H
#  include <string.h>
# else
#  include <strings.h>
# endif

/* Avoid depending on library functions or files
   whose names are inconsistent.  */

#ifndef getenv
extern char *getenv ();
#endif

static char *
my_index (str, chr)
     const char *str;
     int chr;
{
  while (*str)
    {
      if (*str == chr)
	return (char *) str;
      str++;
    }
  return 0;
}

/* If using GCC, we can safely declare strlen this way.
   If not using GCC, it is ok not to declare it.  */
#ifdef __GNUC__
/* Note that Motorola Delta 68k R3V7 comes with GCC but not stddef.h.
   That was relevant to code that was here before.  */
# if (!defined __STDC__ || !__STDC__) && !defined strlen
/* gcc with -traditional declares the built-in strlen to return int,
   and has done so at least since version 2.4.5. -- rms.  */
extern int strlen (const char *);
# endif /* not __STDC__ */
#endif /* __GNUC__ */

#endif /* not __GNU_LIBRARY__ */

/* Handle permutation of arguments.  */

/* Describe the part of ARGV that contains non-options that have
   been skipped.  `first_nonopt' is the index in ARGV of the first of them;
   `last_nonopt' is the index after the last of them.  */

static int first_nonopt;
static int last_nonopt;

#ifdef _LIBC
/* Bash 2.0 gives us an environment variable containing flags
   indicating ARGV elements that should not be considered arguments.  */

/* Defined in getopt_init.c  */
extern char *__getopt_nonoption_flags;

static int nonoption_flags_max_len;
static int nonoption_flags_len;

static int original_argc;
static char *const *original_argv;

/* Make sure the environment variable bash 2.0 puts in the environment
   is valid for the getopt call we must make sure that the ARGV passed
   to getopt is that one passed to the process.  */
static void
__attribute__ ((unused))
store_args_and_env (int argc, char *const *argv)
{
  /* XXX This is no good solution.  We should rather copy the args so
     that we can compare them later.  But we must not use malloc(3).  */
  original_argc = argc;
  original_argv = argv;
}
# ifdef text_set_element
text_set_element (__libc_subinit, store_args_and_env);
# endif /* text_set_element */

# define SWAP_FLAGS(ch1, ch2) \
  if (nonoption_flags_len > 0)						      \
    {									      \
      char __tmp = __getopt_nonoption_flags[ch1];			      \
      __getopt_nonoption_flags[ch1] = __getopt_nonoption_flags[ch2];	      \
      __getopt_nonoption_flags[ch2] = __tmp;				      \
    }
#else	/* !_LIBC */
# define SWAP_FLAGS(ch1, ch2)
#endif	/* _LIBC */

/* Exchange two adjacent subsequences of ARGV.
   One subsequence is elements [first_nonopt,last_nonopt)
   which contains all the non-options that have been skipped so far.
   The other is elements [last_nonopt,optind), which contains all
   the options processed since those non-options were skipped.

   `first_nonopt' and `last_nonopt' are relocated so that they describe
   the new indices of the non-options in ARGV after they are moved.  */

#if defined __STDC__ && __STDC__
static void exchange (char **);
#endif

static void
exchange (argv)
     char **argv;
{
  int bottom = first_nonopt;
  int middle = last_nonopt;
  int top = optind;
  char *tem;

  /* Exchange the shorter segment with the far end of the longer segment.
     That puts the shorter segment into the right place.
     It leaves the longer segment in the right place overall,
     but it consists of two parts that need to be swapped next.  */

#ifdef _LIBC
  /* First make sure the handling of the `__getopt_nonoption_flags'
     string can work normally.  Our top argument must be in the range
     of the string.  */
  if (nonoption_flags_len > 0 && top >= nonoption_flags_max_len)
    {
      /* We must extend the array.  The user plays games with us and
	 presents new arguments.  */
      char *new_str = malloc (top + 1);
      if (new_str == NULL)
	nonoption_flags_len = nonoption_flags_max_len = 0;
      else
	{
	  memset (__mempcpy (new_str, __getopt_nonoption_flags,
			     nonoption_flags_max_len),
		  '\0', top + 1 - nonoption_flags_max_len);
	  nonoption_flags_max_len = top + 1;
	  __getopt_nonoption_flags = new_str;
	}
    }
#endif

  while (top > middle && middle > bottom)
    {
      if (top - middle > middle - bottom)
	{
	  /* Bottom segment is the short one.  */
	  int len = middle - bottom;
	  register int i;

	  /* Swap it with the top part of the top segment.  */
	  for (i = 0; i < len; i++)
	    {
	      tem = argv[bottom + i];
	      argv[bottom + i] = argv[top - (middle - bottom) + i];
	      argv[top - (middle - bottom) + i] = tem;
	      SWAP_FLAGS (bottom + i, top - (middle - bottom) + i);
	    }
	  /* Exclude the moved bottom segment from further swapping.  */
	  top -= len;
	}
      else
	{
	  /* Top segment is the short one.  */
	  int len = top - middle;
	  register int i;

	  /* Swap it with the bottom part of the bottom segment.  */
	  for (i = 0; i < len; i++)
	    {
	      tem = argv[bottom + i];
	      argv[bottom + i] = argv[middle + i];
	      argv[middle + i] = tem;
	      SWAP_FLAGS (bottom + i, middle + i);
	    }
	  /* Exclude the moved top segment from further swapping.  */
	  bottom += len;
	}
    }

  /* Update records for the slots the non-options now occupy.  */

  first_nonopt += (optind - last_nonopt);
  last_nonopt = optind;
}

/* Initialize the internal data when the first call is made.  */

#if defined __STDC__ && __STDC__
static const char *_getopt_initialize (int, char *const *, const char *);
#endif
static const char *
_getopt_initialize (argc, argv, optstring)
     int argc;
     char *const *argv;
     const char *optstring;
{
  /* Start processing options with ARGV-element 1 (since ARGV-element 0
     is the program name); the sequence of previously skipped
     non-option ARGV-elements is empty.  */

  first_nonopt = last_nonopt = optind;

  nextchar = NULL;

  posixly_correct = getenv ("POSIXLY_CORRECT");

  /* Determine how to handle the ordering of options and nonoptions.  */

  if (optstring[0] == '-')
    {
      ordering = RETURN_IN_ORDER;
      ++optstring;
    }
  else if (optstring[0] == '+')
    {
      ordering = REQUIRE_ORDER;
      ++optstring;
    }
  else if (posixly_correct != NULL)
    ordering = REQUIRE_ORDER;
  else
    ordering = PERMUTE;

#ifdef _LIBC
  if (posixly_correct == NULL
      && argc == original_argc && argv == original_argv)
    {
      if (nonoption_flags_max_len == 0)
	{
	  if (__getopt_nonoption_flags == NULL
	      || __getopt_nonoption_flags[0] == '\0')
	    nonoption_flags_max_len = -1;
	  else
	    {
	      const char *orig_str = __getopt_nonoption_flags;
	      int len = nonoption_flags_max_len = strlen (orig_str);
	      if (nonoption_flags_max_len < argc)
		nonoption_flags_max_len = argc;
	      __getopt_nonoption_flags =
		(char *) malloc (nonoption_flags_max_len);
	      if (__getopt_nonoption_flags == NULL)
		nonoption_flags_max_len = -1;
	      else
		memset (__mempcpy (__getopt_nonoption_flags, orig_str, len),
			'\0', nonoption_flags_max_len - len);
	    }
	}
      nonoption_flags_len = nonoption_flags_max_len;
    }
  else
    nonoption_flags_len = 0;
#endif

  return optstring;
}

/* Scan elements of ARGV (whose length is ARGC) for option characters
   given in OPTSTRING.

   If an element of ARGV starts with '-', and is not exactly "-" or "--",
   then it is an option element.  The characters of this element
   (aside from the initial '-') are option characters.  If `getopt'
   is called repeatedly, it returns successively each of the option characters
   from each of the option elements.

   If `getopt' finds another option character, it returns that character,
   updating `optind' and `nextchar' so that the next call to `getopt' can
   resume the scan with the following option character or ARGV-element.

   If there are no more option characters, `getopt' returns -1.
   Then `optind' is the index in ARGV of the first ARGV-element
   that is not an option.  (The ARGV-elements have been permuted
   so that those that are not options now come last.)

   OPTSTRING is a string containing the legitimate option characters.
   If an option character is seen that is not listed in OPTSTRING,
   return '?' after printing an error message.  If you set `opterr' to
   zero, the error message is suppressed but we still return '?'.

   If a char in OPTSTRING is followed by a colon, that means it wants an arg,
   so the following text in the same ARGV-element, or the text of the following
   ARGV-element, is returned in `optarg'.  Two colons mean an option that
   wants an optional arg; if there is text in the current ARGV-element,
   it is returned in `optarg', otherwise `optarg' is set to zero.

   If OPTSTRING starts with `-' or `+', it requests different methods of
   handling the non-option ARGV-elements.
   See the comments about RETURN_IN_ORDER and REQUIRE_ORDER, above.

   Long-named options begin with `--' instead of `-'.
   Their names may be abbreviated as long as the abbreviation is unique
   or is an exact match for some defined option.  If they have an
   argument, it follows the option name in the same ARGV-element, separated
   from the option name by a `=', or else the in next ARGV-element.
   When `getopt' finds a long-named option, it returns 0 if that option's
   `flag' field is nonzero, the value of the option's `val' field
   if the `flag' field is zero.

   The elements of ARGV aren't really const, because we permute them.
   But we pretend they're const in the prototype to be compatible
   with other systems.

   LONGOPTS is a vector of `struct option' terminated by an
   element containing a name which is zero.

   LONGIND returns the index in LONGOPT of the long-named option found.
   It is only valid when a long-named option has been found by the most
   recent call.

   If LONG_ONLY is nonzero, '-' as well as '--' can introduce
   long-named options.  */

int
_getopt_internal (argc, argv, optstring, longopts, longind, long_only)
     int argc;
     char *const *argv;
     const char *optstring;
     const struct option *longopts;
     int *longind;
     int long_only;
{
  optarg = NULL;

  if (optind == 0 || !__getopt_initialized)
    {
      if (optind == 0)
	optind = 1;	/* Don't scan ARGV[0], the program name.  */
      optstring = _getopt_initialize (argc, argv, optstring);
      __getopt_initialized = 1;
    }

  /* Test whether ARGV[optind] points to a non-option argument.
     Either it does not have option syntax, or there is an environment flag
     from the shell indicating it is not an option.  The later information
     is only used when the used in the GNU libc.  */
#ifdef _LIBC
# define NONOPTION_P (argv[optind][0] != '-' || argv[optind][1] == '\0'	      \
		      || (optind < nonoption_flags_len			      \
			  && __getopt_nonoption_flags[optind] == '1'))
#else
# define NONOPTION_P (argv[optind][0] != '-' || argv[optind][1] == '\0')
#endif

  if (nextchar == NULL || *nextchar == '\0')
    {
      /* Advance to the next ARGV-element.  */

      /* Give FIRST_NONOPT & LAST_NONOPT rational values if OPTIND has been
	 moved back by the user (who may also have changed the arguments).  */
      if (last_nonopt > optind)
	last_nonopt = optind;
      if (first_nonopt > optind)
	first_nonopt = optind;

      if (ordering == PERMUTE)
	{
	  /* If we have just processed some options following some non-options,
	     exchange them so that the options come first.  */

	  if (first_nonopt != last_nonopt && last_nonopt != optind)
	    exchange ((char **) argv);
	  else if (last_nonopt != optind)
	    first_nonopt = optind;

	  /* Skip any additional non-options
	     and extend the range of non-options previously skipped.  */

	  while (optind < argc && NONOPTION_P)
	    optind++;
	  last_nonopt = optind;
	}

      /* The special ARGV-element `--' means premature end of options.
	 Skip it like a null option,
	 then exchange with previous non-options as if it were an option,
	 then skip everything else like a non-option.  */

      if (optind != argc && !strcmp (argv[optind], "--"))
	{
	  optind++;

	  if (first_nonopt != last_nonopt && last_nonopt != optind)
	    exchange ((char **) argv);
	  else if (first_nonopt == last_nonopt)
	    first_nonopt = optind;
	  last_nonopt = argc;

	  optind = argc;
	}

      /* If we have done all the ARGV-elements, stop the scan
	 and back over any non-options that we skipped and permuted.  */

      if (optind == argc)
	{
	  /* Set the next-arg-index to point at the non-options
	     that we previously skipped, so the caller will digest them.  */
	  if (first_nonopt != last_nonopt)
	    optind = first_nonopt;
	  return -1;
	}

      /* If we have come to a non-option and did not permute it,
	 either stop the scan or describe it to the caller and pass it by.  */

      if (NONOPTION_P)
	{
	  if (ordering == REQUIRE_ORDER)
	    return -1;
	  optarg = argv[optind++];
	  return 1;
	}

      /* We have found another option-ARGV-element.
	 Skip the initial punctuation.  */

      nextchar = (argv[optind] + 1
		  + (longopts != NULL && argv[optind][1] == '-'));
    }

  /* Decode the current option-ARGV-element.  */

  /* Check whether the ARGV-element is a long option.

     If long_only and the ARGV-element has the form "-f", where f is
     a valid short option, don't consider it an abbreviated form of
     a long option that starts with f.  Otherwise there would be no
     way to give the -f short option.

     On the other hand, if there's a long option "fubar" and
     the ARGV-element is "-fu", do consider that an abbreviation of
     the long option, just like "--fu", and not "-f" with arg "u".

     This distinction seems to be the most useful approach.  */

  if (longopts != NULL
      && (argv[optind][1] == '-'
	  || (long_only && (argv[optind][2] || !my_index (optstring, argv[optind][1])))))
    {
      char *nameend;
      const struct option *p;
      const struct option *pfound = NULL;
      int exact = 0;
      int ambig = 0;
      int indfound = -1;
      int option_index;

      for (nameend = nextchar; *nameend && *nameend != '='; nameend++)
	/* Do nothing.  */ ;

      /* Test all long options for either exact match
	 or abbreviated matches.  */
      for (p = longopts, option_index = 0; p->name; p++, option_index++)
	if (!strncmp (p->name, nextchar, nameend - nextchar))
	  {
	    if ((unsigned int) (nameend - nextchar)
		== (unsigned int) strlen (p->name))
	      {
		/* Exact match found.  */
		pfound = p;
		indfound = option_index;
		exact = 1;
		break;
	      }
	    else if (pfound == NULL)
	      {
		/* First nonexact match found.  */
		pfound = p;
		indfound = option_index;
	      }
	    else
	      /* Second or later nonexact match found.  */
	      ambig = 1;
	  }

      if (ambig && !exact)
	{
	  if (opterr)
	    fprintf (stderr, _("%s: option `%s' is ambiguous\n"),
		     argv[0], argv[optind]);
	  nextchar += strlen (nextchar);
	  optind++;
	  optopt = 0;
	  return '?';
	}

      if (pfound != NULL)
	{
	  option_index = indfound;
	  optind++;
	  if (*nameend)
	    {
	      /* Don't test has_arg with >, because some C compilers don't
		 allow it to be used on enums.  */
	      if (pfound->has_arg)
		optarg = nameend + 1;
	      else
		{
		  if (opterr)
		    {
		      if (argv[optind - 1][1] == '-')
			/* --option */
			fprintf (stderr,
				 _("%s: option `--%s' doesn't allow an argument\n"),
				 argv[0], pfound->name);
		      else
			/* +option or -option */
			fprintf (stderr,
				 _("%s: option `%c%s' doesn't allow an argument\n"),
				 argv[0], argv[optind - 1][0], pfound->name);
		    }

		  nextchar += strlen (nextchar);

		  optopt = pfound->val;
		  return '?';
		}
	    }
	  else if (pfound->has_arg == 1)
	    {
	      if (optind < argc)
		optarg = argv[optind++];
	      else
		{
		  if (opterr)
		    fprintf (stderr,
			   _("%s: option `%s' requires an argument\n"),
			   argv[0], argv[optind - 1]);
		  nextchar += strlen (nextchar);
		  optopt = pfound->val;
		  return optstring[0] == ':' ? ':' : '?';
		}
	    }
	  nextchar += strlen (nextchar);
	  if (longind != NULL)
	    *longind = option_index;
	  if (pfound->flag)
	    {
	      *(pfound->flag) = pfound->val;
	      return 0;
	    }
	  return pfound->val;
	}

      /* Can't find it as a long option.  If this is not getopt_long_only,
	 or the option starts with '--' or is not a valid short
	 option, then it's an error.
	 Otherwise interpret it as a short option.  */
      if (!long_only || argv[optind][1] == '-'
	  || my_index (optstring, *nextchar) == NULL)
	{
	  if (opterr)
	    {
	      if (argv[optind][1] == '-')
		/* --option */
		fprintf (stderr, _("%s: unrecognized option `--%s'\n"),
			 argv[0], nextchar);
	      else
		/* +option or -option */
		fprintf (stderr, _("%s: unrecognized option `%c%s'\n"),
			 argv[0], argv[optind][0], nextchar);
	    }
	  nextchar = (char *) "";
	  optind++;
	  optopt = 0;
	  return '?';
	}
    }

  /* Look at and handle the next short option-character.  */

  {
    char c = *nextchar++;
    char *temp = my_index (optstring, c);

    /* Increment `optind' when we start to process its last character.  */
    if (*nextchar == '\0')
      ++optind;

    if (temp == NULL || c == ':')
      {
	if (opterr)
	  {
	    if (posixly_correct)
	      /* 1003.2 specifies the format of this message.  */
	      fprintf (stderr, _("%s: illegal option -- %c\n"),
		       argv[0], c);
	    else
	      fprintf (stderr, _("%s: invalid option -- %c\n"),
		       argv[0], c);
	  }
	optopt = c;
	return '?';
      }
    /* Convenience. Treat POSIX -W foo same as long option --foo */
    if (temp[0] == 'W' && temp[1] == ';')
      {
	char *nameend;
	const struct option *p;
	const struct option *pfound = NULL;
	int exact = 0;
	int ambig = 0;
	int indfound = 0;
	int option_index;

	/* This is an option that requires an argument.  */
	if (*nextchar != '\0')
	  {
	    optarg = nextchar;
	    /* If we end this ARGV-element by taking the rest as an arg,
	       we must advance to the next element now.  */
	    optind++;
	  }
	else if (optind == argc)
	  {
	    if (opterr)
	      {
		/* 1003.2 specifies the format of this message.  */
		fprintf (stderr, _("%s: option requires an argument -- %c\n"),
			 argv[0], c);
	      }
	    optopt = c;
	    if (optstring[0] == ':')
	      c = ':';
	    else
	      c = '?';
	    return c;
	  }
	else
	  /* We already incremented `optind' once;
	     increment it again when taking next ARGV-elt as argument.  */
	  optarg = argv[optind++];

	/* optarg is now the argument, see if it's in the
	   table of longopts.  */

	for (nextchar = nameend = optarg; *nameend && *nameend != '='; nameend++)
	  /* Do nothing.  */ ;

	/* Test all long options for either exact match
	   or abbreviated matches.  */
	for (p = longopts, option_index = 0; p->name; p++, option_index++)
	  if (!strncmp (p->name, nextchar, nameend - nextchar))
	    {
	      if ((unsigned int) (nameend - nextchar) == strlen (p->name))
		{
		  /* Exact match found.  */
		  pfound = p;
		  indfound = option_index;
		  exact = 1;
		  break;
		}
	      else if (pfound == NULL)
		{
		  /* First nonexact match found.  */
		  pfound = p;
		  indfound = option_index;
		}
	      else
		/* Second or later nonexact match found.  */
		ambig = 1;
	    }
	if (ambig && !exact)
	  {
	    if (opterr)
	      fprintf (stderr, _("%s: option `-W %s' is ambiguous\n"),
		       argv[0], argv[optind]);
	    nextchar += strlen (nextchar);
	    optind++;
	    return '?';
	  }
	if (pfound != NULL)
	  {
	    option_index = indfound;
	    if (*nameend)
	      {
		/* Don't test has_arg with >, because some C compilers don't
		   allow it to be used on enums.  */
		if (pfound->has_arg)
		  optarg = nameend + 1;
		else
		  {
		    if (opterr)
		      fprintf (stderr, _("\
%s: option `-W %s' doesn't allow an argument\n"),
			       argv[0], pfound->name);

		    nextchar += strlen (nextchar);
		    return '?';
		  }
	      }
	    else if (pfound->has_arg == 1)
	      {
		if (optind < argc)
		  optarg = argv[optind++];
		else
		  {
		    if (opterr)
		      fprintf (stderr,
			       _("%s: option `%s' requires an argument\n"),
			       argv[0], argv[optind - 1]);
		    nextchar += strlen (nextchar);
		    return optstring[0] == ':' ? ':' : '?';
		  }
	      }
	    nextchar += strlen (nextchar);
	    if (longind != NULL)
	      *longind = option_index;
	    if (pfound->flag)
	      {
		*(pfound->flag) = pfound->val;
		return 0;
	      }
	    return pfound->val;
	  }
	  nextchar = NULL;
	  return 'W';	/* Let the application handle it.   */
      }
    if (temp[1] == ':')
      {
	if (temp[2] == ':')
	  {
	    /* This is an option that accepts an argument optionally.  */
	    if (*nextchar != '\0')
	      {
		optarg = nextchar;
		optind++;
	      }
	    else
	      optarg = NULL;
	    nextchar = NULL;
	  }
	else
	  {
	    /* This is an option that requires an argument.  */
	    if (*nextchar != '\0')
	      {
		optarg = nextchar;
		/* If we end this ARGV-element by taking the rest as an arg,
		   we must advance to the next element now.  */
		optind++;
	      }
	    else if (optind == argc)
	      {
		if (opterr)
		  {
		    /* 1003.2 specifies the format of this message.  */
		    fprintf (stderr,
			   _("%s: option requires an argument -- %c\n"),
			   argv[0], c);
		  }
		optopt = c;
		if (optstring[0] == ':')
		  c = ':';
		else
		  c = '?';
	      }
	    else
	      /* We already incremented `optind' once;
		 increment it again when taking next ARGV-elt as argument.  */
	      optarg = argv[optind++];
	    nextchar = NULL;
	  }
      }
    return c;
  }
}

#endif	/* Not ELIDE_CODE.  */

#endif /* NEED_LONG_OPTIONS */


/*
 * Display Modem Nationality for all supported adapters
 */

void
display_cc (void)
{
	unsigned int i, j, k;

	printf(
"\nDigi Intl. Modem Nationalities on a per adapter basis\n");

	for (k = 0; k < asizeof (board2list); ++k) {
		printf("\n%s:\n", board2list[k].name);

		for (i = 0; i < board2list[k].array_size; ++i) {
			for (j = 0; j < asizeof (cc_2_name); ++j) {
				if (board2list[k].cc_array[i] == cc_2_name[j].country_code) {
					printf("\t%c%c, %s\n",
					 cc_2_name[j].country_code >> 8,
					 cc_2_name[j].country_code & 0x0ff,
					 cc_2_name[j].country_name);
					break;
				}
			}
		}
	}

	printf ("\n");

}

/*
 * Generic failure function to be called if an ioctl to
 * the board fails.
 */
void
dm_ioctl_fail(void)
{
	if (errno)
		fprintf(stderr, "\nioctl to adapter failed: %s\n",
		 strerror(errno)) ;
	else
		fprintf(stderr, "\nioctl to adapter failed\n") ;

	exit(2) ;
}

char *
strboard(int board_type)
{
	char *s ;

	switch(board_type) {
#ifdef DFRAS
	    /* Houston */
	    case ADAPTER_HOU30:
		s = DIGI_PCIHOUSTON30_NAME ;
		break ;

	    case ADAPTER_HOU60:
		s = DIGI_PCIHOUSTON60_NAME ;
		break ;

	    case ADAPTER_HOU24:
		s = DIGI_PCIHOUSTON24_NAME ;
		break ;

	    case ADAPTER_HOU48:
		s = DIGI_PCIHOUSTON48_NAME ;
		break ;

	    case ADAPTER_HOU30cpci:
		s = DIGI_CPCIHOUSTON30_NAME ;
		break ;

	    case ADAPTER_HOU60cpci:
		s = DIGI_CPCIHOUSTON60_NAME ;
		break ;

	    case ADAPTER_HOU24cpci:
		s = DIGI_CPCIHOUSTON24_NAME ;
		break ;

	    case ADAPTER_HOU48cpci:
		s = DIGI_CPCIHOUSTON48_NAME ;
		break ;

	    /* London */
	    case ADAPTER_LONU:
		s = DIGI_LONDONU_NAME ;
		break ;

	    case ADAPTER_LONST:
		s = DIGI_LONDONST_NAME ;
		break ;
#endif

	    /* Chaska */
	    case ADAPTER_CHASKA80:
		s = DIGI_CHASKA80_NAME ;
		break;

	    case ADAPTER_CHASKA81:
		s = DIGI_CHASKA81_NAME ;
		break;

	    case ADAPTER_CHASKA82:
		s = DIGI_CHASKA82_NAME ;
		break;

	    case ADAPTER_CHASKA83:
		s = DIGI_CHASKA83_NAME ;
		break;

	    case ADAPTER_CHASKA84:
		s = DIGI_CHASKA84_NAME ;
		break;

	    case ADAPTER_CHASKA85:
		s = DIGI_CHASKA85_NAME ;
		break;

	    case ADAPTER_CHASKA40:
		s = DIGI_CHASKA40_NAME ;
		break ;

	    case ADAPTER_CHASKA41:
		s = DIGI_CHASKA41_NAME ;
		break ;

	    case ADAPTER_CHASKA42:
		s = DIGI_CHASKA42_NAME ;
		break ;

	    case ADAPTER_CHASKA43:
		s = DIGI_CHASKA43_NAME ;
		break ;

	    case ADAPTER_CHASKA44:
		s = DIGI_CHASKA44_NAME ;
		break ;

	    case ADAPTER_CHASKA45:
		s = DIGI_CHASKA45_NAME ;
		break ;

	    case ADAPTER_AP2K2:
		s = DIGI_AP2K_NAME ;
		break ;

	    case ADAPTER_AP2K4:
		s = DIGI_AP4K_NAME ;
		break ;

	    case ADAPTER_AP2K8:
		s = DIGI_AP8K_NAME ;
		break ;

	    case ADAPTER_AP2K16:
		s = DIGI_AP16K_NAME ;
		break ;

#ifdef XP_MEI
	    case ADAPTER_AP2K4_MPI:
		s = DIGI_AP4K_MPI_NAME ;
		break ;

	    case ADAPTER_AP2K8_MPI:
		s = DIGI_AP8K_MPI_NAME ;
		break ;

	    case ADAPTER_AP2K16_MPI:
		s = DIGI_AP16K_MPI_NAME ;
		break ;
#endif

#ifdef DIGI_AP24K_NAME

	    case ADAPTER_AP2K24:
		s = DIGI_AP16K_NAME ;
		break ;
#endif

#ifdef DIGI_AP32K_NAME

	    case ADAPTER_AP2K32:
		s = DIGI_AP32K_NAME ;
		break ;
#endif

	    default:
		sprintf (s, "Unknown adapter type = 0x%x", board_type );
		break ;
	}

	return(s) ;
}

/*
 * Display the contents of a get_adapter_info_t
 * in human-readable form on stdout.
 */
void
print_adapter_info(const adapter_info_t * const infop)
{
	int daughter = 0;

	printf("Adapter Information:\n");
	printf("\tBoard type: %s\n", strboard(infop->board_type)) ;

	switch (infop->board_type) {

	case ADAPTER_HOU60cpci:
	case ADAPTER_HOU48cpci:
	case ADAPTER_HOU30cpci:
	case ADAPTER_HOU24cpci:

		/*
		 * Because we were prevented from modifying any of the
		 * structs used by the mgmt channel, and I just wanted to
		 * add one byte, I refused to create a whole new ioctl and
		 * supporting code just for this one byte.  Instead, I tacked
		 * it onto the end of the partnumber field and was done with
		 * the exercise in five minutes.  Please fire me...
		 */
		printf("\tGeographic Address: 0x%02x\n",
			infop->part_number[sizeof(infop->part_number)-1]);
	}

	printf("\tPorts: %d\n", infop->port_count) ;
	printf("\tChannels/port: %d\n", infop->channels_per_port) ;
	printf("\tMemory size: %d bytes\n", infop->memory_size) ;
	printf("\tFEP version: %x\n", infop->fep_version);

	switch (infop->board_type) {

	case ADAPTER_HOU60:
	case ADAPTER_HOU48:
	case ADAPTER_HOU60cpci:
	case ADAPTER_HOU48cpci:

		daughter = 1;

		/* Fall through */

	case ADAPTER_HOU30:
	case ADAPTER_HOU24:
	case ADAPTER_HOU30cpci:
	case ADAPTER_HOU24cpci:
	case ADAPTER_LONU:
	case ADAPTER_LONST:
	case ADAPTER_CHASKA80:
	case ADAPTER_CHASKA81:
	case ADAPTER_CHASKA82:
	case ADAPTER_CHASKA83:
	case ADAPTER_CHASKA84:
	case ADAPTER_CHASKA85:
	case ADAPTER_CHASKA40:
	case ADAPTER_CHASKA41:
	case ADAPTER_CHASKA42:
	case ADAPTER_CHASKA43:
	case ADAPTER_CHASKA44:
	case ADAPTER_CHASKA45:

		printf("\tCSC version: %c.%c%c%c\n",
		 (infop->csc_version >> 24) & 0xff,
		 (infop->csc_version >> 16) & 0xff ,
		 (infop->csc_version >> 8) & 0xff ,
		 infop->csc_version & 0xff) ;

		printf("\tPump version: %x.%x%c\n",
		 (infop->pump_version >> 24) & 0xff,
		 (infop->pump_version >> 16) & 0xff ,
		 infop->pump_version & 0xff) ;
	}
	printf("\tPart Number: %s\n", infop->part_number );
	printf("\tMain board serialid: %s\n", infop->main_board_serial );

	if (daughter)
		printf("\tDaughter card serialid: %s\n",
		 infop->daughter_card_serial);
}

/*
 * Return a textual representation of the coding member of
 * a get_ds1_config_t.
 */
char *
ds1_coding_string(uint32 const coding)
{
	char *s ;

	switch(coding) {
	    case LINE_CODE_B8ZS:
		s = "b8zs" ;
		break ;

	    case LINE_CODE_AMI:
		s = "ami" ;
		break ;

	    case LINE_CODE_HDB3:
		s = "hdb3" ;
		break ;

	    default:
		s = "Unknown" ;
		break ;
	}

	return (s) ;
}

/*
 * Return a textual representation of the framing member of
 * a get_ds1_config_t.
 */
char *
ds1_framing_string(uint32 const framing)
{
	char *s ;

	switch(framing) {
	    case FRAME_ESF:
		s = "esf" ;
		break ;

	    case FRAME_D4:
		s = "d4" ;
		break ;

	    case FRAME_CEPT_CRC4:
		s = "cept (with crc4)" ;
		break ;

	    case FRAME_CEPT_WO_CRC4:
		s = "cept (without crc4)" ;
		break ;

	    default:
		s = "Unknown" ;
		break ;
	}

	return (s) ;
}

/*
 * Return a textual representation of the long_haul_lbo member of
 * a get_ds1_config_t.
 */
char *
ds1_lbo_string(uint32 const lbo)
{
	char *s ;

	switch(lbo) {
	    case LBO_00_0_DB:
		s = "0" ;
		break ;

	    case LBO_07_5_DB:
		s = "7.5" ;
		break ;

	    case LBO_15_0_DB:
		s = "15" ;
		break ;

	    case LBO_22_5_DB:
		s = "22.5" ;
		break ;

	    default:
		s = "Unknown" ;
		break ;
	}

	return (s) ;
}

/*
 * Return a textual representation of the short_haul_len member of
 * a get_ds1_config_t.
 */
char *
ds1_length_string(uint32 const length)
{
	char *s ;

	switch(length) {
	    case SH_0_133:
		s = "0-133" ;
		break ;

	    case SH_133_266:
		s = "133-266" ;
		break ;

	    case SH_266_399:
		s = "266-399" ;
		break ;

	    case SH_399_533:
		s = "399-533" ;
		break ;

	    case SH_533_655:
		s = "533-655" ;
		break ;

	    default:
		s = "Unknown" ;
		break ;
	}

	return (s) ;
}


/*
 * Return a textual representation of the signalling member of
 * a get_ds1_config_t.
 */
char *
ds1_signalling_string(uint32 const signalling)
{
	char *s ;

	switch(signalling) {

	    case CT1_LOOP_START:
		s = "loop start" ;
		break ;

	    case CT1_WINK_START:
		s = "wink start" ;
		break ;

	    case CT1_GROUND_START:
		s = "ground start" ;
		break ;

	    case CT1_IMMED_START:
		s = "immediate start" ;
		break ;

	    case PRI_NI2:
		s = "pri (NI-2)" ;
		break ;

	    case PRI_5ESS:
		s = "pri (5ESS)" ;
		break ;

	    case PRI_DMS100:
		s = "pri (DMS-100)" ;
		break ;

	    case PRI_EURODSS1:
		s = "pri (EURODSS1)" ;
		break ;

	    case PRI_INS1500:
		s = "pri (INS-1500)" ;
		break ;

		case PRI_MSA:
		s = "MSA" ;
		break;

	    case PRI_NONE:
		s = "pri (No switch type selected)" ;
		break ;

	    case E1_R2:
		s = "E1 R2" ;
		break ;

	    default:
		s = "Unknown" ;
		break ;
	}

	return (s) ;
}

/*
 * Return a textual representation of the clocking member of
 * a get_ds1_config_t.
 */
char *
ds1_clocking_string(uint32 const clocking)
{
	char *s ;

	switch(clocking) {
	    case TIMING_LOOP:
		s = "loop" ;
		break ;

	    case TIMING_LOCAL:
		s = "local" ;
		break ;

	    case TIMING_MVIP:
		s = "mvip" ;
		break ;

	    default:
		s = "Unknown" ;
		break ;
	}

	return (s) ;
}

/*
 * Return a textual representation of the jitter_atten member of
 * a get_ds1_config_t.
 */
char *
ds1_jitter_string(uint32 const jitter)
{
	char *s ;

	switch(jitter) {
	    case JITTER_TX:
		s = "tx" ;
		break ;

	    case JITTER_RX:
		s = "rx" ;
		break ;

	    case JITTER_AUTO:
		s = "auto" ;
		break ;

	    case JITTER_NONE:
		s = "none" ;
		break ;

	    default:
		s = "Unknown" ;
		break ;
	}

	return (s) ;
}

/*
 * Return a textual representation of the eq_gain_limit member of
 * a get_ds1_config_t.
 */
char *
ds1_eq_gain_limit(uint32 const eq_gain_limit)
{
	char *s ;

	switch(eq_gain_limit) {
	    case EGL_LOW:
		s = "Low" ;
		break ;

	    case EGL_HIGH:
		s = "High" ;
		break ;

	    default:
		s = "Unknown" ;
		break ;
	}

	return (s) ;
}


/*
 * Return a textual representation of the switch_type member of
 * a get_bri_config_t.
 */
char *
bri_switch_type_string(dxb_uint32 switch_type)
{
	char *s ;

	switch(switch_type) {

	    case BRI_NI1:
		s = "bri (NI-1)" ;
		break ;

	    case BRI_5ESS:
		s = "bri (5ESS)" ;
		break ;

	    case BRI_DMS100:
		s = "bri (DMS-100)" ;
		break ;

	    case BRI_EURODSS1:
		s = "bri (EURODSS1)" ;
		break ;

	    case BRI_INS64:
		s = "bri (INS-64)" ;
		break ;

	    case BRI_NONE:
		s = "bri (No switch type selected)" ;
		break ;

	    default:
		s = "Unknown" ;
		break ;
	}

	return (s) ;
}

void
print_ds1_config(const ds1_config_t * const p)
{
	printf("Port Configuration/Information:\n");
	printf("\tSignalling: %s", ds1_signalling_string(p->signalling)) ;

	if (p->signalling == CT1_WINK_START) {
		printf(" (%d digits)", p->wink_digits) ;
	}
	printf("\n") ;

	if (!msa_switch) {
		printf("\tCoding: %s\n", ds1_coding_string(p->coding)) ;
		printf("\tFraming: %s\n", ds1_framing_string(p->framing)) ;
		printf("\tClock source: %s\n",
		 ds1_clocking_string(p->clocking)) ;
		printf("\tJitter attenuation: %s\n",
		 ds1_jitter_string(p->jitter_atten)) ;
	}

	if (!e1r2_switch)
		printf("\tleased_mask: 0x%08x\n", p->leased_mask & 0x7fffffff);

	switch (info.info.board_type) {

	case ADAPTER_HOU48:
	case ADAPTER_HOU48cpci:

		printf("\tNFAS port: %d\n", p->nfas_master_port) ;
		printf("\tNFAS ID: %d\n", p->nfas_interface_id) ;

		/* Falls through */

	case ADAPTER_HOU24:
	case ADAPTER_HOU24cpci:

		switch (p->if_type)
		{
		case LONG_HAUL:
			printf("\tInterface: Long Haul, LBO: %s\n",
						ds1_lbo_string(p->long_haul_lbo)) ;
			break;
		case SHORT_HAUL:
			printf("\tInterface: Short Haul, length: %s\n",
						ds1_length_string(p->short_haul_len)) ;
			break;
		case LOOP_HAUL:	/* developers mode only */
			printf("\tInterface: odd channel to even channel loopback\n");
			break;
		}
		printf("\tEqualization gain limit: %s\n",
		 ds1_eq_gain_limit(p->eq_gain_limit)) ;

		break;

	case ADAPTER_HOU30:
	case ADAPTER_HOU30cpci:
	case ADAPTER_HOU60:
	case ADAPTER_HOU60cpci:

		if (p->if_type == LOOP_HAUL)
			printf("\tInterface: odd channel to even channel loopback\n");

		break;
	}
}

void
print_ds1_status(const ds1_status_t * const p)
{
	char s_rx_special[160], s_alarm[80], s_loopback[80];

	/* rx_special is a bit mask */

	s_rx_special[0] = 0x00;

	if (p->rx_special & DS1_LOS)
		strcat (s_rx_special, "LOS ");

	if (p->rx_special & DS1_LFA)
		strcat (s_rx_special, "LFA ");

	if (p->rx_special & DS1_AIS)
		strcat (s_rx_special, "AIS ");

	if (p->rx_special & DS1_RFA)
		strcat (s_rx_special, "RFA ");

	if (p->rx_special & DS1_LTS0MFA)
		strcat (s_rx_special, "LTS0MFA ");

	if (p->rx_special & DS1_CREBIT)
		strcat (s_rx_special, "CREBIT ");

	if (p->rx_special & DS1_FBE)
		strcat (s_rx_special, "FBE ");

	if (p->rx_special & DS1_CRCE)
		strcat (s_rx_special, "CRCE ");

	if (p->rx_special & DS1_ECE)
		strcat (s_rx_special, "ECE ");

	if (p->rx_special & DS1_REBIT)
		strcat (s_rx_special, "REBIT ");

	if (p->rx_special & DS1_LCRCATMX)
		strcat (s_rx_special, "LCRCATMX ");

	if (p->rx_special & DS1_NFA)
		strcat (s_rx_special, "NFA ");

	if (p->rx_special & DS1_SSFA)
		strcat (s_rx_special, "SSFA ");

	if (p->rx_special & DS1_BFA)
		strcat (s_rx_special, "BFA ");

	if (p->rx_special & DS1_CMA)
		strcat (s_rx_special, "CMA ");

	if (p->rx_special & DS1_LBFA)
		strcat (s_rx_special, "LBFA ");

	if (p->rx_special & DS1_UNDEFINED)
		strcat (s_rx_special, "UNDEFINED");

	switch (p->alarm) {

	case DS1_NO_ALARM:
		sprintf(s_alarm, "No Alarm");
		break;

	case DS1_RED_ALARM:
		sprintf(s_alarm, "Red Alarm");
		break;

	case DS1_BLUE_ALARM:
		sprintf(s_alarm, "Blue Alarm");
		break;

	case DS1_YELLOW_ALARM:
		sprintf(s_alarm, "Yellow Alarm");
		break;

	default:
		sprintf(s_alarm, "Unknown Alarm (0x%x)", p->alarm);
		break;
	}

	switch (p->loopback) {

	case DS1_NO_LOOP:
		sprintf(s_loopback, "No Loopback");
		break;

	case DS1_PAYLOAD_LOOP:
		sprintf(s_loopback, "Payload Loopback");
		break;

	case DS1_LINE_LOOP:
		sprintf(s_loopback, "Line Loopback");
		break;

	case DS1_LOCAL_LOOP:
		sprintf(s_loopback, "Local Loopback");
		break;

	default:
		sprintf(s_loopback, "Unknown Loopback (0x%x)", p->loopback);
		break;

	}

	printf("\tLOS: %s\n", s_rx_special);

	if (e1r2_switch)
		printf("\tAlarm: %s\n", s_alarm);
	else
		printf("\tAlarm: %s, Loopback: %s\n", s_alarm, s_loopback);
}
void
print_ds1_ext_status(const ds1_ext_status_t * const p)
{
	printf("\terrsec: 0x%x, unavailsec: 0x%x, burstsec: 0x%x\n", 
	  p->errored_seconds, p->unavailable_seconds, p->bursty_seconds);
	printf("\tseversec: 0x%x, severfrsec: 0x%x, controlslipsec: 0x%x\n", 
	  p->severe_seconds, p->severe_framing_seconds, 
	    p->controlled_slip_seconds);
	printf("\tLOFcount: 0x%x, degradmin: 0x%x, lineerrsec: 0x%x\n", 
	  p->loss_of_frame_count, p->degraded_minutes, p->line_errored_seconds);
	printf("\tlinecodevio: 0x%x, crcerrcount: 0x%x\n", 
	  p->line_code_violations, p->crc_error_count);
}  

void
print_ds1_ext_config(const ds1_ext_config_t * const p)
{
	printf("\tTX special: %x, Loopback %x, Auto Config %x\n",
	 p->tx_special, p->loopback, p->auto_config) ;
}

char *
num_to_switch(int code)
{
	switch (code) {
		case R2_ITU: return "ITU";
		case R2_ABACUS: return "ABACUS";
		case R2_ARGENTINA: return "ARGENTINA";
		case R2_AUSTRALIA: return "AUSTRALIA";
		case R2_BRAZIL: return "BRAZIL";
		case R2_CHINA: return "CHINA";
		case R2_COLUMBIA: return "COLUMBIA";
		case R2_COSTA_RICA: return "COSTA_RICA";
		case R2_CZECH: return "CZECH";
		case R2_EAST_EUROPE: return "EAST_EUROPE";
		case R2_ECUADOR_ITU: return "ECUADOR_ITU";
		case R2_ECUADOR_LME: return "ECUADOR_LME";
		case R2_GREECE: return "GREECE";
		case R2_GUATEMALA: return "GUATEMALA";
		case R2_HONG_KONG: return "HONG_KONG";
		case R2_INDIA: return "INDIA";
		case R2_INDONESIA: return "INDONESIA";
		case R2_ISRAEL: return "ISRAEL";
		case R2_KOREA: return "KOREA";
		case R2_MALAYSIA: return "MALAYSIA";
		case R2_NEW_ZEALAND: return "NEW_ZEALAND";
		case R2_PARAGUAY: return "PARAGUAY";
		case R2_PERU: return "PERU";
		case R2_PHILLIPPINES: return "PHILLIPPINES";
		case R2_SAUDI_ARABIA: return "SAUDI_ARABIA";
		case R2_SINGAPORE: return "SINGAPORE";
		case R2_SOUTH_AFRICA_PANAFTEL: return "SOUTH_AFRICA_PANAFTEL";
		case R2_MEXICO_TELMEX: return "MEXICO_TELMEX";
		case R2_MEXICO_TELNOR: return "MEXICO_TELNOR";
		case R2_THAILAND: return "THAILAND";
		case R2_URUGUAY: return "URUGUAY";
		case R2_VENEZUELA: return "VENEZUELA";
		case R2_VIETNAM: return "VIETNAM";
		default: return "ITU";
	}
}

int 
switch_to_num(char *sw)
{
	char *ptr = sw;

	while (*ptr) {
		if (*ptr == ' ') 
			*ptr = '_';
		else
			*ptr = toupper(*ptr);
		ptr++;
	}

	switch (sw[0]) {
		case 'I': 
			if (!strcmp(sw, "ITU"))
				return R2_ITU;
			else if (!strcmp(sw, "INDIA"))
				return R2_INDIA;
			else if (!strcmp(sw, "INDONESIA"))
				return R2_INDONESIA;
			else if (!strcmp(sw, "ISRAEL"))
				return R2_ISRAEL;
			else
				return -1;
		case 'A': 
			if (!strcmp(sw, "ABACUS"))
				return R2_ABACUS;
			else if (!strcmp(sw, "ARGENTINA"))
				return R2_ARGENTINA;
			else if (!strcmp(sw, "AUSTRALIA"))
				return R2_AUSTRALIA;
			else
				return -1;
		case 'B': 
			if (!strcmp(sw, "BRAZIL"))
				return R2_BRAZIL;
			else
				return -1;
		case 'C': 
			if (!strcmp(sw, "CHINA"))
				return R2_CHINA;
			else if (!strcmp(sw, "COLUMBIA"))
				return R2_COLUMBIA;
			else if (!strcmp(sw, "COSTA_RICA"))
				return R2_COSTA_RICA;
			else if (!strncmp(sw, "CZECH", 5))
				return R2_COSTA_RICA;
			else
				return -1;
		case 'E': 
			if (!strcmp(sw, "EAST_EUROPE"))
				return R2_EAST_EUROPE;
			else if (!strcmp(sw, "ECUADOR_ITU"))
				return R2_ECUADOR_ITU;
			else if (!strcmp(sw, "ECUADOR_LME"))
				return R2_ECUADOR_LME;
			else
				return -1;
		case 'G': 
			if (!strcmp(sw, "GREECE"))
				return R2_GREECE;
			else if (!strcmp(sw, "GUATEMALA"))
				return R2_GUATEMALA;
			else
				return -1;
		case 'P': 
			if (!strcmp(sw, "PARAGUAY"))
				return R2_PARAGUAY;
			else if (!strcmp(sw, "PERU"))
				return R2_PERU;
			else if (!strcmp(sw, "PHILLIPPINES"))
				return R2_PHILLIPPINES;
			else
				return -1;
		case 'S': 
			if (!strcmp(sw, "SAUDI_ARABIA"))
				return R2_SAUDI_ARABIA;
			else if (!strcmp(sw, "SINGAPORE"))
				return R2_SINGAPORE;
			else if (!strncmp(sw, "SOUTH_AFRICA", 12))
				return R2_SOUTH_AFRICA_PANAFTEL;
			else
				return -1;
		case 'M': 
			if (!strcmp(sw, "MEXICO_TELMEX"))
				return R2_MEXICO_TELMEX;
			else if (!strcmp(sw, "MEXICO_TELNOR"))
				return R2_MEXICO_TELNOR;
			else
				return -1;
		case 'V': 
			if (!strcmp(sw, "VENEZUELA"))
				return R2_VENEZUELA;
			else if (!strcmp(sw, "VIETNAM"))
				return R2_VIETNAM;
			else
				return -1;
		default:
			if (!strcmp(sw, "THAILAND"))
				return R2_THAILAND;
			else if (!strcmp(sw, "URUGUAY"))
				return R2_URUGUAY;
			else if (!strcmp(sw, "KOREA"))
				return R2_KOREA;
			else if (!strcmp(sw, "MALAYSIA"))
				return R2_MALAYSIA;
			else if (!strcmp(sw, "NEW_ZEALAND"))
				return R2_NEW_ZEALAND;
			else if (!strcmp(sw, "HONG_KONG"))
				return R2_HONG_KONG;
			else
				return -1;
	}
}

char *
num_to_method(int code)
{
	switch (code) {
		case R2_DIGITAL_COMPELLED:  return "Multifrequency Compelled";
		case R2_DIGITAL_INVERTED:   return "Digital Inverted ABCD";
		case R2_DIGITAL_METERED:   return "Digital Metered";
		default:  return "Unknown";
	}
}

char *
sig_string(char *c, int code, char *buf)
{
	sprintf(buf, "%s-%d", c, code&0xf);
	return buf;
}

void
print_ds1_e1r2_config(const ds1_e1r2_config_t * const p)
{
	char buf1[8];
	char buf2[8];

	printf("\tE1 R2 configuration:\n");
	printf("\t\tType: %s, Method: %s\n", 
		num_to_switch(p->r2_switch_type), num_to_method(p->r2_method));
	printf("\t\tCD Bits: 0x%x, Min DNIS Digits: %d, Max DNIS Digits: %d\n", 
		p->r2_cd_bits, p->min_dnis_digits, p->max_dnis_digits);
	printf("\t\tMin ANI Digits: %d, Max ANI Digits: %d, Send ANI: %s\n", 
		p->min_ani_digits, p->max_ani_digits,
		p->send_ani_digits ? "TRUE":"FALSE"); 
	printf("\t\tANI Next Digit Signal: %s, Group B Answer Signal: %s\n", 
		sig_string("A", p->ani_digit_signal, buf1),
		sig_string("B", p->answer_signal, buf2));
	printf("\t\tCongestion Signal: %s, End of DNIS Digits Signal: %s\n", 
		sig_string("B", p->congestion_signal, buf1),
		sig_string("A", p->end_dnis_signal, buf2)),
	printf("\t\tBusy Signal: %s, Group II Outgoing Signal: %s\n", 
		sig_string("B", p->busy_signal, buf1),
		sig_string("II", p->outgoing_type, buf2));
	printf("\t\tSeize Ack Delay: %d ms, Answer Delay: %d ms\n", 
		p->seize_ack_delay, p->answer_delay);
	printf("\t\tIncoming Type Mask: 0x%x, Line Signal Min Time: %d ms\n",
		p->incoming_allow_mask, p->linesig_min);
}
void
print_ds1_e1r2_config_string(const ds1_e1r2_config_t * const p)
{
    printf("\n"
    "--e1r2_sw_type=%d --cd_bits=0x%x --r2_method=%d --min_dnis=%d \\\n"
    "--max_dnis=%d --min_ani=%d --max_ani=%d --send_ani=%d \\\n"
    "--ani_sig=%d --answer_sig=%d --congest_sig=%d --end_dnis_sig=%d\\\n"
    "--busy_sig=%d --sz_ack_delay=%d --answer_delay=%d \\\n"
    "--incoming_allow=0x%x --group2_type=%d --linesig_min=%d\n\n",
	p->r2_switch_type, p->r2_cd_bits, p->r2_method,
	p->min_dnis_digits, p->max_dnis_digits, p->min_ani_digits,
	p->max_ani_digits, p->send_ani_digits, p->ani_digit_signal&0xf,
	p->answer_signal&0xf, p->congestion_signal&0xf, p->end_dnis_signal&0xf,
	p->busy_signal&0xf, p->seize_ack_delay, p->answer_delay,
	p->incoming_allow_mask, p->outgoing_type&0xf, p->linesig_min);
}

void
print_bri_config(const bri_config_t * const bri)
{
	register unsigned int i ;

	printf("Port Configuration/Information:\n");
	printf("\tSwitch type: %s,  Autospid (off=%d, on=%d): %d\n",
	 bri_switch_type_string(bri->switch_type),
	 BRI_AUTOSPID_OFF, BRI_AUTOSPID_ON, bri->auto_spid) ;

	printf("\tLeased mask: 0x%x\n", bri->leased_mask & 0x7fffffff) ;

	printf("\tDirectory count: 0x%x\n", bri->dir_count & 0x7fffffff);

	printf("\tDirectory Numbers:\n") ;
	for(i = 0; i < (bri->dir_count & 0x7fffffff); i++) {
		printf("\t  %1d: %s\n", i + 1, bri->directory[i].number) ;
	}

	if ((bri->switch_type == BRI_EURODSS1) ||
	 (bri->switch_type == BRI_INS64)) {

		printf("\tSub-Addresses:\n") ;
		for(i = 0; i < (bri->dir_count & 0x7fffffff); i++) {
			printf("\t  %1d: %s\n", i + 1, bri->directory[i].sub_address) ;
		}
	}
	else {

		printf("\tSPIDs:\n") ;
		for(i = 0; i < (bri->dir_count & 0x7fffffff); i++) {
			printf("\t  %1d: %s\n", i + 1, bri->directory[i].spid) ;
		}
	}

    printf("\tMSN (off=%d, on=%d): %d\n", BRI_MSN_OFF, BRI_MSN_ON, bri->msn);
    printf("\tAUTOALERT (off=%d, on=%d): %d\n", BRI_AUTOALERT_OFF, BRI_AUTOALERT_ON,
	       bri->autoalert);
}

void
print_dchan_ext_config(const dchan_ext_config_t * const dchan)
{
	if (msa_switch)
		printf("\tDchannel Mode: %x\n", dchan->mode) ;
	else
		printf("\tDchannel Mode: %x, Addr Type: %x, Num plan: %x, "
		 "Subaddr type: %x\n", dchan->mode, dchan->address_type,
		 dchan->numbering_plan, dchan->subaddress_type) ;
}

void
print_modem_ext_config(const modem_ext_config_t * const modem)
{
	printf("Modem Information:\n") ;
	printf("\tBusy-out: %x, Leased line: %x, Tx Audio Level: %x, Rx Audio Level: %x\n\n", modem->busy_out, modem->leased_line, modem->audio_level_tx, modem->audio_level_rx) ;
}

void
print_hdlc_status(const hdlc_status_t * const p)
{
	printf("\tTX bytes: %d, RX bytes: %d, TX frames: %d, RX frames: %d\n",
		p->tx_bytes, p->rx_bytes, p->tx_frames, p->rx_frames);
	printf("\tRX bad frames: %d, RX short frames: %d, RX long frames: %d\n",
		p->rx_bad_frames, p->rx_short_frames, p->rx_long_frames);
	printf("\tRX aborts: %d, RX overruns: %d, RX crc err: %d\n",
		p->rx_aborts, p->rx_overruns, p->rx_crc_errors);
	printf("\tRX buffer overflows: %d, TX underruns: %d\n\n",
		p->rx_buffer_overflows, p->tx_underruns);
}

/*
 * Keep in mind that the DDWG's version of the TEI state is 1
 * less than the _real_ Q.921 value.
 */
#define TEI_HUMAN_READABLE(state)	\
	((state) < 1 || (state) > 9) ? "???" : \
	tei_readable[(state)-1]
	
char *tei_readable[] = {
	"TEI_OFF",
	"TEI_UNASSIGNED",
	"ASS_AWAITING_TEI",
	"EST_AWAITING_TEI",
	"TEI_ASSIGNED",
	"AWAITING_EST",
	"AWAITING_RELEASE",
	"MULTIPLE_FRAME",
	"TIMER_RECOVERY"
};

void
print_dchan_ext_status(const dchan_ext_status_t * const p)
{
	printf("D-Channel Information:\n");
	printf("\ttei state[0]: %d (%s)\n", 
		p->tei_state[0], 
		TEI_HUMAN_READABLE(p->tei_state[0]));
	printf("\ttei state[1]: %d (%s)\n", 
		p->tei_state[1],
		TEI_HUMAN_READABLE(p->tei_state[1]));
	/* print_lapd_status(p->lapd); */
	print_hdlc_status(&p->hdlc);
}

void
print_bchan_ext_status(const bchan_ext_status_t * const p)
{
	printf("B-Channel Information:\n");
	print_hdlc_status(&p->hdlc);

}

void
print_bchan_status(const bchan_status_t * const p)
{
	printf("\tTx rate: %d CPS\n", p->dte_tx_rate) ;
	printf("\tRx rate: %d CPS\n", p->dte_rx_rate) ;
	printf("\tConnect time: %d\n", p->connect_time) ;
	printf("\tConnect rate: %d\n\n", p->channel_rate) ;
}

void
print_adapter_state(const adapter_state_t * const s)
{
	printf("\tMemory usage: %d %%\n", s->memory_usage) ;
	printf("\tCPU usage: %d %%\n", s->cpu_usage) ;

#if 0
	/* Won't be implemented, will be replaced by something else
	   FIXME when done.
	 */
	printf("\tDXB to adapter: %d\n", s->dxb_h2a_count) ;
	printf("\tDXB from adapter: %d\n", s->dxb_a2h_count) ;
#endif

	printf("\tUptime: %d.%d secs\n", s->uptime.seconds,
						s->uptime.mseconds % 1000) ;
}

void
print_bri_status(const bri_status_t * const s)
{
	printf("\tPower: %x  activation state: %x\n", 
	  s->power, s->activation_state);
	printf("\trx_info_state: %x  tx_info_state : %x\n", 
	  s->rx_info_state, s->tx_info_state);
	printf("\tloss of sync: %d  collisions : %d\n\n", 
	  s->loss_of_sync, s->collisions);
}

void
print_modem_nationality(const getset_modem_nationality_t * const p)
{
	char *country_name = "Unknown Nationality";
	unsigned int i;

	for (i = 0; i < asizeof (cc_2_name) ; ++i)
		if (p->nation.country_code == cc_2_name[i].country_code)
			country_name = cc_2_name[i].country_name;

	printf("\tModem Nationality: %c%c (%s, 0x%x)\n\n",
	 p->nation.country_code >> 8, p->nation.country_code & 0xff, country_name,
	 p->nation.country_code) ;
}

void
dump_adapter_state(int const fd) 
{
	get_adapter_state_t state ;

	if (ioctl(fd, GET_ADAPTER_STATE, &state) != 0) {
		dm_ioctl_fail() ;
	}

	print_adapter_state(&state.state) ;
}

void
dump_houston_port_info(int const fd, int const port)
{
	getset_ds1_config_t	config;

	config.port = port;

	if (ioctl(fd, GET_DS1_CONFIG, &config) != 0) {
		dm_ioctl_fail() ;
	}

	print_ds1_config(&config.config) ;
}

void
dump_houston_port_status(int const fd, int const port)
{
	get_ds1_status_t	status;

	status.port = port;

	if (ioctl(fd, GET_DS1_STATUS, &status) != 0) {
		dm_ioctl_fail() ;
	}

	print_ds1_status(&status.status) ;
}

void
dump_london_port_bri_config(int const fd, int const port)
{
	getset_bri_config_t config ;

	config.port = port ;

	if (ioctl(fd, GET_BRI_CONFIG, &config) != 0) {
		dm_ioctl_fail() ;
	}

	print_bri_config(&config.config) ;
}

void
dump_dchan_ext_config(int const fd, int const port)
{
	getset_dchan_ext_config_t config ;

	config.port = port ;

	if (ioctl(fd, GET_DCHAN_EXT_CONFIG, &config) != 0) {
		dm_ioctl_fail() ;
	}

	print_dchan_ext_config(&config.config) ;
}

void
dump_dchan_info(int const fd, int const port)
{
	get_dchan_ext_status_t	status;

	status.port = port;

	if (ioctl(fd, GET_DCHAN_EXT_STATUS, &status) != 0) {
		dm_ioctl_fail() ;
	}

	print_dchan_ext_status(&status.status) ;
}

void
print_serial_config(const serial_config_t * const s)
{
	printf("Serial Configuration:\n");

 	switch (s->port_type) {
	case SERIAL_PORTTYPE_RS232:
		printf("\tLine Type:\tRS232\n");
		break;

	case SERIAL_PORTTYPE_RS422:
		printf("\tLine Type:\tRS422\n");
		break;

	case SERIAL_PORTTYPE_RS423:
		printf("\tLine Type:\tRS423\n");
		break;

#ifdef XP_MEI
	case SERIAL_PORTTYPE_RS485_2_E:
		printf("\tLine Type:\tRS485 2-wire, endpoint\n");
		break;
	case SERIAL_PORTTYPE_RS485_2_M:
		printf("\tLine Type:\tRS485 2-wire, midpoint\n");
		break;
	case SERIAL_PORTTYPE_RS485_4_E:
		printf("\tLine Type:\tRS485 4-wire, endpoint\n");
		break;
	case SERIAL_PORTTYPE_RS485_4_M:
		printf("\tLine Type:\tRS485 4-wire, midpoint\n");
		break;
#endif /* XP_MEI */

	case SERIAL_PORTTYPE_V35:
		printf("\tLine Type:\tV35\n");
		break;
	}

	/* clock_source */
 	switch (s->clock_source) {
	case SERIAL_CLOCKSOURCE_INT:
		printf("\tClock Source:\tInternal\n");
		break;

	case SERIAL_CLOCKSOURCE_EXT:
		printf("\tClock Source:\tExternal\n");
		break;

	case SERIAL_CLOCKSOURCE_SPLIT:
		printf("\tClock Source:\tSplit\n");
		break;

	}
	
	/* alt pin */
 	switch (s->alt_pin) {
	case SERIAL_ALTPIN_OFF:
		printf("\tAlt Pin:\tRJ-45 (disabled)\n");
		break;

	case SERIAL_ALTPIN_ON:
		printf("\tAlt Pin:\tSwap DCD & DSR (enabled)\n");
		break;
	default:
		printf("\tAlt Pin:\t%x\n", s->alt_pin);
		break;
	}

}

#ifdef XP_MEI
void
print_rs485_config(const serial_rs485_config_t * const s)
{
	printf("RS485 Configuration:\n");

	switch (s->rx_enable) {
	case RS485_RXENABLE_AUTO:
		printf("\tRX enable:\tAutomatic (echo cancel)\n");
		break;
	case RS485_RXENABLE_ALWAYS:
		printf("\tRX enable:\tAlways on\n");
		break;
	}

	/* tx_enable */
	switch (s->tx_enable) {
	case RS485_TXENABLE_AUTO:
		printf("\tTX enable:\tAutomatic\n\n");
		break;
	case RS485_TXENABLE_RTS:
		printf("\tTX enable:\tTransmit follows RTS\n\n");
		break;
	}

}
#endif /* XP_MEI */

void
print_serial_ext_status(const serial_ext_status_t * const status)
{
	printf( "Serial Extended Status:\n");
	printf( "\tTransmit Bytes:\t\t\t%d\n", status->tx_bytes);
	printf( "\tReceive Bytes:\t\t\t%d\n", status->rx_bytes);
	printf( "\tTransmit Frames:\t\t%d\n", status->tx_frames);
	printf( "\tReceive Frames:\t\t\t%d\n", status->rx_frames);
	printf( "\tReceive Bad Frames:\t\t%d\n", status->rx_bad_frames);
	printf( "\tReceive Short Frames:\t\t%d\n", status->rx_short_frames);
	printf( "\tReceive Long Frames:\t\t%d\n", status->rx_long_frames);
	printf( "\tReceive Aborts:\t\t\t%d\n", status->rx_aborts);
	printf( "\tReceive Overruns:\t\t%d\n", status->rx_overruns);
	printf( "\tReceive CRC Errors:\t\t%d\n", status->rx_crc_errors);
	printf( "\tReceive Buffer Overflows:\t%d\n", status->rx_buffer_overflows);
	printf( "\tTransmit Underruns:\t\t%d\n", status->tx_underruns);
	printf( "\tInterrupted Frames:\t\t%d\n", status->interrupted_frames);
	printf( "\tBreaks Recieved:\t\t%d\n", status->breaks_received);
	printf( "\tParity Errors:\t\t\t%d\n", status->parity_errs);
	printf( "\tFraming Errors:\t\t\t%d\n", status->framing_errs);
	printf( "\tOverrun Errors:\t\t\t%d\n", status->overrun_errs);
	printf( "\tTX Async Ctrl Char Map:\t\t%d\n", status->txaccm);
	printf( "\tRX Async Ctrl Char Map:\t\t%d\n\n", status->rxaccm);
}

void
print_serial_ext_config(const serial_ext_config_t * const s)
{
	printf("Serial Extended Configuration:\n");

 	switch (s->unpause_rx) {
	case 0:
		printf( "\tUNPAUSE_RX:\tXON/raised RTS = 0 \n");
		break;

	case 1:
		printf( "\tUNPAUSE_RX:\tXON/raised RTS = 1 \n");
		break;

	default:
		printf( "\tUNPAUSE_RX:\tUNKNOWN \n");
		break;
	}

 	switch (s->unpause_tx) {
	case 0:
		printf( "\tUNPAUSE_TX:\tXOFF = 0\n\n");
		break;

	case 1:
		printf( "\tUNPAUSE_TX:\tXOFF = 1\n\n");
		break;

	default:
		printf( "\tUNPAUSE_TX:\tUNKNOWN ?\n\n");
		break;
	}
}

void
print_serial_status(const serial_status_t * const status)
{
	int i;
	int tmp_hold;

	/* port_state */
	printf( "Serial Status:\n");
	switch(status->port_state) {
    	case SERIAL_CLOSED:
			printf( "\tLine State:\t\tClosed");
			break;

    	case SERIAL_OPEN:
			printf( "\tLine State:\t\tOpen");
			if (status->port_state & SERIAL_ACTIVE)
			{
				printf("/Active");
			}
			break;

    	case SERIAL_ACTIVE:
			printf( "\tLine State:\t\tActive");
			break;

		default:
			printf( "\tLine State:\t\t0x%x",status->port_state);
			break;
	}

	printf( "\n");

	/* modem signals and states */

	for (i=0;i<SERIAL_SIGNAL_MAX;i++) {

	  switch(i) {
	  case SERIAL_SIGNAL_RTS:
		printf( "\tRTS:\t\t\t%s\n",
		 (status->mdm_sig_state[i] & 2) ? "On" : "Off") ;
		break;

	  case SERIAL_SIGNAL_CTS:
		printf( "\tCTS:\t\t\t%s\n",
		 (status->mdm_sig_state[i] & 2) ? "On" : "Off") ;
		break;

	  case SERIAL_SIGNAL_DTR:
		printf( "\tDTR:\t\t\t%s\n",
		 (status->mdm_sig_state[i] & 2) ? "On" : "Off") ;
		break;

	  case SERIAL_SIGNAL_DSR:
		printf( "\tDSR:\t\t\t%s\n",
		 (status->mdm_sig_state[i] & 2) ? "On" : "Off") ;
		break;

	  case SERIAL_SIGNAL_RI:
		printf( "\tRI:\t\t\t%s\n",
		 (status->mdm_sig_state[i] & 2) ? "On" : "Off") ;
		break;

	  case SERIAL_SIGNAL_DCD:
		printf( "\tDCD:\t\t\t%s\n",
		 (status->mdm_sig_state[i] & 2) ? "On" : "Off") ;
		break;

	  default:
		break;
	 }
	}

	printf("\tData Bits:");
	/* data_bits */
	switch(status->data_bits) {
	case 1:
		printf( "\t\t8");
		break;

	case 2:
		printf( "\t\t7");
		break;

	case 3:
		printf( "\t\t6");
		break;

	case 4:
		printf( "\t\t5");
		break;

	default :
		printf( "\t\t%d",status->data_bits);
		break;
	}

	printf("\n");

	/* stop_bits */
	printf( "\tStop Bits:");
	switch( status->stop_bits) {
	case 1:
		printf( "\t\t1");
		break;

	case 2:
		printf( "\t\t2");
		break;

	case 3:
		printf( "\t\tOne and One Half");
		break;
	default:
		printf( "\t\t%d",status->stop_bits);
		break;
	}

	printf("\n");

	/* parity */
	printf( "\tParity:");
	switch( status->parity) {
	case 1:
		printf( "\t\t\tNone");
        	break;
	case 2:
		printf( "\t\t\tOdd");
          	break;
	case 3:
		printf( "\t\t\tEven");
        	break;

	case 4:
		printf( "\t\t\tSpace");
            	break;
	case 5:
		printf( "\t\t\tMark");
		break;

	default:
		printf( "\t\t\t%d",status->parity);
            	break;
	}

	printf( "\n");

		/* frame_mode */

	switch(status->frame_mode) {
	case SERIAL_FRAME_NONE:
		printf( "\tFrame Mode:\t\tNone");
		break;

	case SERIAL_FRAME_ASYNC_SLIP:
		printf( "\tFrame Mode:\t\tSLIP");
		break;

	case SERIAL_FRAME_ASYNC_PPP:
		printf( "\tFrame Mode:\t\tPPP");
		break;

	case SERIAL_FRAME_SYNC_HDLC:
		printf( "\tFrame Mode:\t\tHDLC");
		break;

	default:
		printf( "\tFrame Mode:\t\t0x%x",status->frame_mode);
		break;
	}

	printf( "\n");

	printf( "\tInput Flow Control:\t");
	tmp_hold = 0;
	for (i=0; i< SERIAL_IN_FLOW_MAX;i++) {
		switch(i) {
		case SERIAL_IN_FLOW_XONXOFF_INDEX:
			if (status->in_flow_type[i] & 2) {
				printf( "XON/XOFF ");
				tmp_hold++;
			}
			break;
	
		case SERIAL_IN_FLOW_RTS_INDEX:
			if (status->in_flow_type[i] & 2) {
				printf( "RTS ");
				tmp_hold++;
			}
			break;
	
		case SERIAL_IN_FLOW_DTR_INDEX:
			if (status->in_flow_type[i] & 2) {
				printf( "DTR ");
				tmp_hold++;
			}
			break;
	
		default :
			break;
		}
	}

	if (!tmp_hold) {
		printf( "None");
	}

	printf( "\n");

	printf( "\tOutput Flow Control:\t");
	tmp_hold = 0;
	for (i=0; i< SERIAL_OUT_FLOW_MAX;i++) {
		switch(i) {
		case SERIAL_OUT_FLOW_XONXOFF_INDEX:
			if (status->out_flow_type[i] & 2) {
				printf( "XON/XOFF ");
				tmp_hold++;
			}
			break;
	
		case SERIAL_OUT_FLOW_CTS_INDEX:
			if (status->out_flow_type[i] & 2) {
				printf( "CTS ");
				tmp_hold++;
			}
			break;
	
		case SERIAL_OUT_FLOW_DSR_INDEX:
			if (status->out_flow_type[i] & 2) {
				printf( "DSR ");
				tmp_hold++;
			}
			break;
	
		default :
			break;
		}
	}

	if (!tmp_hold) {
		printf( "None");
	}

	printf( "\n");

	tmp_hold=0;
	printf("\tInput Flow Asserted:\t");
	for (i=0;i<SERIAL_IN_FLOW_MAX;i++) {

		switch(i) {
		case SERIAL_IN_FLOW_XONXOFF_INDEX:
			if (status->in_flow_state[i] & SERIAL_FLOW_ASSERTED) {
				tmp_hold++;
				printf( "XON/XOFF ") ;
			}
			break;
		case SERIAL_IN_FLOW_RTS_INDEX:
			if (status->in_flow_state[i] & SERIAL_FLOW_ASSERTED) {
				tmp_hold++;
				printf( "RTS ") ;
			}
			break;
		case SERIAL_IN_FLOW_DTR_INDEX:
			if (status->in_flow_state[i] & SERIAL_FLOW_ASSERTED) {
				tmp_hold++;
				printf( "DTR ") ;
			}
			break;
		default:
			break;
		}
	}

	if (!tmp_hold)
		printf("None");

	printf( "\n");

	tmp_hold=0;
	printf("\tOutput Flow Asserted:\t");

	for (i=0;i<SERIAL_OUT_FLOW_MAX;i++) {

		switch(i) {
		case SERIAL_OUT_FLOW_XONXOFF_INDEX:
			if (status->out_flow_state[i] & SERIAL_FLOW_ASSERTED) {
				tmp_hold++;
				printf( "XON/XOFF ") ;
			}
			break;
		case SERIAL_OUT_FLOW_CTS_INDEX:
			if (status->out_flow_state[i] & SERIAL_FLOW_ASSERTED) {
				tmp_hold++;
				printf( "CTS ") ;
			}
			break;
		case SERIAL_OUT_FLOW_DSR_INDEX:
			if (status->out_flow_state[i] & SERIAL_FLOW_ASSERTED) {
				tmp_hold++;
				printf( "DSR ") ;
			}
			break;
		default:
			break;
		}
	}

	if (!tmp_hold)
		printf("None");

	printf( "\n");
	
	printf("\tInput/Output Speed:\t");
	if(status->in_speed == 0 ) {
		printf( "Unknown/");
	} else {
		printf( "%d/",status->in_speed);
	}
	if(status->out_speed == 0 ) {
		printf( "Unknown");
	} else {
		printf( "%d",status->out_speed);
	}

	printf( "\n");

	/* autobaud */

	printf("\tAutobaud:\t\t%s",
	 (status->autobaud & SERIAL_AUTOBAUD_ENABLED) ? "Enabled" : "Disabled");

	printf( "\n\n");
}

void
dump_serial_info(int const fd)
{
	get_serial_ext_status_t	ext_status;
	get_serial_status_t status ;

	status.channel = ext_status.channel = flat_channel;

	if (ioctl(fd, GET_SERIAL_EXT_STATUS, &ext_status) != 0) {
		dm_ioctl_fail() ;
	}

	print_serial_ext_status(&ext_status.status) ;

	if (ioctl(fd, GET_SERIAL_STATUS, &status) != 0) {
		dm_ioctl_fail() ;
	}

	print_serial_status(&status.status) ;
}

void
get_serial_config(int const fd, getset_serial_config_t *config)
{
	config->channel = flat_channel;

	if (ioctl(fd, GET_SERIAL_CONFIG, config) != 0) {
		dm_ioctl_fail() ;
	}
}

void
dump_serial_config(int const fd)
{
	getset_serial_ext_config_t	ext_config;
	getset_serial_config_t config ;

	get_serial_config(fd, &config);

	print_serial_config(&config.config) ;

	ext_config.channel = flat_channel;

	if (ioctl(fd, GET_SERIAL_EXT_CONFIG, &ext_config) != 0) {
		dm_ioctl_fail() ;
	}

	print_serial_ext_config(&ext_config.config) ;
}

#ifdef XP_MEI
void
get_rs485_config(int const fd, getset_serial_rs485_config_t *config)
{
	config->channel = flat_channel;

	if (ioctl(fd, GET_SERIAL_RS485_CONFIG, config) != 0) {
		dm_ioctl_fail() ;
	}
}

void
dump_rs485_config(int const fd)
{
	getset_serial_rs485_config_t	config;

	get_rs485_config(fd, &config);

	print_rs485_config(&config.config) ;
}

void
set_port_type(int const fd)
{
	getset_serial_config_t config ;
	memset(&config, 0, sizeof(config));

	config.channel  = flat_channel;

	config.config.port_type = port_type;

	if (ioctl(fd, SET_SERIAL_CONFIG, &config) != 0) {
		dm_ioctl_fail() ;
	}
}

void
set_rx_enable(int const fd)
{
	getset_serial_rs485_config_t config ;
	memset(&config, 0, sizeof(config));

	config.channel  = flat_channel;

	config.config.rx_enable = rx_enable;

	if (ioctl(fd, SET_SERIAL_RS485_CONFIG, &config) != 0) {
		dm_ioctl_fail() ;
	}
}

void
set_tx_enable(int const fd)
{
	getset_serial_rs485_config_t config ;
	memset(&config, 0, sizeof(config));

	config.channel  = flat_channel;

	config.config.tx_enable = tx_enable;

	if (ioctl(fd, SET_SERIAL_RS485_CONFIG, &config) != 0) {
		dm_ioctl_fail() ;
	}
}


/*
 *  The set_xp_persistent function knows how to call an external program
 *  (dm_portreg) to modify the persistent database with the current
 *  persistent settings (which are learned by querying the adapter).
 *
 *  We can use the "copy" feature of this program, because we've just
 *  sent ioctls to update the driver.
 */
void
set_xp_persistent(int const fd)
{
	char cmdbuf[1024];

	sprintf( cmdbuf, DM_PORTREG_PATH " -c %s %d",
	         info.info.main_board_serial, flat_channel );

	if( system( cmdbuf ) != 0 )
	{
		if (errno)
			fprintf(stderr, "\ndatabase storage failure: %s\n",
			        strerror(errno)) ;
		else
			fprintf(stderr, "\ndatabase storage failure\n") ;

		exit(2);
	}
}
#endif /* XP_MEI */


void
dump_bchan_info(int const fd)
{
	get_bchan_ext_status_t	ext_status;
	get_bchan_status_t status ;

	status.channel = 
	ext_status.channel = flat_channel;

	if (ioctl(fd, GET_BCHAN_EXT_STATUS, &ext_status) != 0) {
		dm_ioctl_fail() ;
	}

	print_bchan_ext_status(&ext_status.status) ;

	if (ioctl(fd, GET_BCHAN_STATUS, &status) != 0) {
		dm_ioctl_fail() ;
	}

	print_bchan_status(&status.status) ;
}

void
dump_modem_info(int const fd)
{
	getset_modem_ext_config_t	config;

	config.channel = flat_channel;

	if (ioctl(fd, GET_MODEM_EXT_CONFIG, &config) != 0) {
		dm_ioctl_fail() ;
	}

	print_modem_ext_config(&config.config) ;
}

void
dump_modem_nationality(int const fd)
{
	getset_modem_nationality_t modem_response ;

	/* modem_response.country_code = ; */

	if (ioctl(fd, GET_MODEM_NATIONALITY, &modem_response) != 0) {
		dm_ioctl_fail() ;
	}

	print_modem_nationality(&modem_response) ;
}

void
dump_houston_port_ext_status(int const fd, int const port)
{
	get_ds1_ext_status_t	status;

	status.port = port;

	if (ioctl(fd, GET_DS1_EXT_STATUS, &status) != 0) {
		dm_ioctl_fail() ;
	}

	print_ds1_ext_status(&status.status) ;
}

void
dump_london_bri_status(int const fd, int const port)
{
	get_bri_status_t	status;

	status.port = port;

	if (ioctl(fd, GET_BRI_STATUS, &status) != 0) {
		dm_ioctl_fail() ;
	}

	print_bri_status(&status.status) ;
}

void
dump_houston_port_ext_config(int const fd, int const port)
{
	getset_ds1_ext_config_t	config;

	config.port = port;

	if (ioctl(fd, GET_DS1_EXT_CONFIG, &config) != 0) {
		dm_ioctl_fail() ;
	}

	print_ds1_ext_config(&config.config) ;
}

void
dump_houston_port_e1r2_config_string(int const fd, int const port)
{

	getset_ds1_e1r2_config_t	config;

	config.port = port;

	if (ioctl(fd, GET_DS1_E1R2_CONFIG, &config) != 0) {
		dm_ioctl_fail() ;

		exit (1);
	}

	print_ds1_e1r2_config_string(&config.config) ;
}

void
dump_houston_port_e1r2_config(int const fd, int const port)
{

	getset_ds1_e1r2_config_t	config;

	config.port = port;

	if (ioctl(fd, GET_DS1_E1R2_CONFIG, &config) != 0) {
		dm_ioctl_fail() ;

		exit (1);
	}

	print_ds1_e1r2_config(&config.config) ;
}

void
validate_ds1x_config(void)
{

	if (msa_switch && loopback) {

		fprintf(stderr,
		 "dm_admin: loopback is not valid if switch type is MSA\n");

		usage();

		exit (1);
	}

	if (e1r2_switch && loopback) {

		fprintf(stderr,
		 "dm_admin: loopback is not valid if switch type is E1 R2\n");

		usage();

		exit (1);
	}

	if (loopback)
		ds1_ext_config.config.loopback = loopback;

	if (msa_switch && tx_special) {

		fprintf(stderr,
		 "dm_admin: tx_special is not valid if switch type is MSA\n");

		usage();

		exit (1);
	}

	if (tx_special)
		ds1_ext_config.config.tx_special = tx_special;

	return;

}

void
validate_dchan_config(void)
{
	memset(&dchan_config, 0, sizeof(getset_dchan_ext_config_t)) ;

	dchan_config.port = port ;

	/*
	 * Fill in the structure with user specified data
	 * Note that if a switched variable is zero, 
	 * no change is made to that field
	 */

	if (msa_switch && address_type) {

		fprintf(stderr,
		 "dm_admin: addr_type is not valid if switch type is MSA\n");

		usage();

		exit (1);
	}

	switch (address_type) {

	case 0:
		break;

	case 1:
		dchan_config.config.address_type = DCHAN_ADDRESS_TYPE_UNKNOWN;
		break;

	case 2:
		dchan_config.config.address_type = DCHAN_ADDRESS_TYPE_INTL;
		break;

	case 3:
		dchan_config.config.address_type = DCHAN_ADDRESS_TYPE_NATIONAL;
		break;

	case 4:
		dchan_config.config.address_type = DCHAN_ADDRESS_TYPE_NETWORK;
		break;

	case 5:
		dchan_config.config.address_type = DCHAN_ADDRESS_TYPE_SUBSCR;
		break;

	case 6:
		dchan_config.config.address_type = DCHAN_ADDRESS_TYPE_ABBREV;
		break;

	}

	if (msa_switch && numbering_plan) {

		fprintf(stderr,
		 "dm_admin: num_plan is not valid if switch type is MSA\n");

		usage();

		exit (1);
	}

	switch (numbering_plan) {

	case 0:
		break;

	case 1:
		dchan_config.config.numbering_plan = DCHAN_NUMBERING_PLAN_UNKNOWN;
		break;

	case 2:
		dchan_config.config.numbering_plan = DCHAN_NUMBERING_PLAN_ISDN;
		break;

	case 3:
		dchan_config.config.numbering_plan = DCHAN_NUMBERING_PLAN_DATA;
		break;

	case 4:
		dchan_config.config.numbering_plan = DCHAN_NUMBERING_PLAN_TELEX;
		break;

	case 5:
		dchan_config.config.numbering_plan = DCHAN_NUMBERING_PLAN_NATIONAL;
		break;

	case 6:
		dchan_config.config.numbering_plan = DCHAN_NUMBERING_PLAN_PRIVATE;
		break;

	}

	if (msa_switch && subaddress_type) {

		fprintf(stderr,
		 "dm_admin: subaddr_type is not valid if switch type is MSA\n");

		usage();

		exit (1);
	}

	switch (subaddress_type) {

	case 0:
		break;

	case 1:
		dchan_config.config.subaddress_type = DCHAN_SUBADDRESS_TYPE_NSAP;
		break;

	case 2:
		dchan_config.config.subaddress_type = DCHAN_SUBADDRESS_TYPE_USER;
		break;

	}

	switch (back2back) {

	case 0:
		break;

	case 1:
		dchan_config.config.mode = DCHAN_NORMAL ;
		break;

	case 2:
		dchan_config.config.mode = DCHAN_LOOPBACK_NT ;
		break;

	case 3:
		dchan_config.config.mode = DCHAN_LOOPBACK_TE ;
		break;

	}
}

void
PrepModemConfig(getset_modem_ext_config_t *modem_config)
{
	memset(modem_config, 0, sizeof(getset_modem_ext_config_t)) ;

	modem_config->channel = flat_channel ;
	modem_config->config.busy_out = modem_lock ;
}

void
ModemHangup(int const fd)
{
	getset_modem_config_t h ;

	memset(&h, 0, sizeof(h)) ;
	h.channel = flat_channel ;
	h.config.hangup = MODEM_HANGUP ;

	printf("Applying modem hangup...\n");

	if (ioctl(fd, SET_MODEM_CONFIG, &h) != 0) {
		dm_ioctl_fail() ;
	}
}

void
ModemReset(int const fd)
{
	getset_modem_config_t h ;
	memset(&h, 0, sizeof(h)) ;
	h.channel = flat_channel ;
	/* The combination of hangup and reset works well under dm_gui: */
	h.config.hangup = MODEM_HANGUP ;
	h.config.reset = MODEM_RESET_COLD ;

	printf("Applying cold modem reset...\n");

	if (ioctl(fd, SET_MODEM_CONFIG, &h) != 0) {
		dm_ioctl_fail() ;
	}
}

/*
 * Satisfy a query request. If no port is specified, then
 * return board level info. If port is specified,
 * then return port level info
 */
void
query_action(int const fd, const adapter_info_t * const infop)
{
	int family;

	if (port == -1) {
		print_adapter_info(infop) ;
		dump_adapter_state(fd) ;

		family = infop->board_type & FAMILY_MASK;

		switch (family) {
		case ADAPTER_HOUXX:
		case ADAPTER_HOUXXcpci:
		case ADAPTER_LONXX:
		case ADAPTER_CHASKA8X:
			dump_modem_nationality(fd) ;
		}
	} else {
		switch (infop->board_type) {
			case ADAPTER_HOU30:
			case ADAPTER_HOU60:
			case ADAPTER_HOU24:
			case ADAPTER_HOU48:
			case ADAPTER_HOU30cpci:
			case ADAPTER_HOU60cpci:
			case ADAPTER_HOU24cpci:
			case ADAPTER_HOU48cpci:
			/*
			 * Handle the various flavours of Houston.
			 */

				cur_ds1_config.port = port;

				if (ioctl(fd, GET_DS1_CONFIG, &cur_ds1_config)
				 != 0) {
					dm_ioctl_fail() ;

					exit (1);
				}

				if (cur_ds1_config.config.signalling == E1_R2)
					e1r2_switch = 1;

				if (cur_ds1_config.config.signalling == PRI_MSA)
					msa_switch = 1;

				dump_houston_port_info(fd, port) ;

				if (!msa_switch) {
					dump_houston_port_status(fd, port);
					dump_houston_port_ext_config(fd, port);
					dump_houston_port_ext_status(fd, port);
				}

				if (!e1r2_switch)
					dump_dchan_ext_config(fd, port) ;

				if (e1r2_switch) {
					dump_houston_port_e1r2_config(fd, port) ;
				}

				break ;

			/*
			 * London handled here.
			 */
			case ADAPTER_LONU:
			case ADAPTER_LONST:
				dump_london_port_bri_config(fd, port) ;
				dump_dchan_ext_config(fd, port) ;
				dump_london_bri_status(fd, port) ;
				break ;
			/*
			 * Chaska would be handled here.
			 */

			default:
				break ;
		}
	}
}

void
process_queries(int const fd, const adapter_info_t * const infop)
{

	if (config_call) {
		fprintf (stderr,
		 "\n\ndm_admin: Query-type commands "\
		 "(-q, B, D, E, g, G, --print_e1r2)\n"\
		 "cannot be mixed with configuration change options!\n\n"\
		 "Change options are being ignored\n\n");
	}

	if (query) {
		query_action(fd, infop) ;
	}

	if (dchan) {

		if (port == -1) {
			fprintf (stderr, "dm_admin: port (-p) required with -D\n");

			usage();

			return ;
		}

		dump_dchan_info(fd, port) ;

	}	

	if (bchan) {

		if ((channel < 1) || (port == -1)) {
			fprintf (stderr,
			 "dm_admin: port (-p) and channel (-c) required with -B\n");

			usage();

			return ;
		}

		dump_bchan_info(fd) ;
	}	

	if (modem_stat) {

		if ((channel < 1) || (port == -1)) {
			fprintf (stderr,
			 "dm_admin: port (-p) and channel (-c) required with -E\n");

			usage();

			return ;
		}

		dump_modem_info(fd) ;
	}	

	if (serial) {

		if ((channel < 1) || (port == -1)) {
			fprintf (stderr,
			 "dm_admin: port (-p) and channel (-c) required with -G\n");

			usage();

			return ;
		}

		dump_serial_info(fd) ;
	}	

	if (serial_config) {

		if ((channel < 1) || (port == -1)) {
			fprintf (stderr,
			 "dm_admin: port (-p) and channel (-c) required with -g\n");

			usage();

			return ;
		}

		dump_serial_config(fd) ;
	}

#ifdef XP_MEI	
	if (rs485_config) {
		
		if ((channel < 1) || (port == -1)) {
			fprintf (stderr,
				 "dm_admin: port (-p) and channel (-c) required with -S\n");
			
			usage();
			
			return;
		}
		
		dump_rs485_config(fd) ;
	}
#endif /* XP_MEI */

	
	if (e1r2_print_config) {

		if (port == -1) {
			fprintf (stderr,
			 "dm_admin: port (-p) required with --print_e1r2\n");

			usage();

			return ;
		}

		switch (info.info.board_type) {

		case ADAPTER_HOU30:
		case ADAPTER_HOU60:
		case ADAPTER_HOU30cpci:
		case ADAPTER_HOU60cpci:
			break;

		default:

			fprintf (stderr, "dm_admin: Attempting to display E1 only information on a non-E1 adapter, invalid\n\n");

			exit(1);

		}

		cur_ds1_config.port = port;

		if (ioctl(fd, GET_DS1_CONFIG, &cur_ds1_config) != 0) {
			dm_ioctl_fail() ;

			exit (1);
		}

		if (cur_ds1_config.config.signalling != E1_R2) {
			fprintf (stderr,
			 "dm_admin: --print_e1r2 valid only if switch type (-s) is E1 R2 (12)\n");

			exit (1);
		}

		dump_houston_port_e1r2_config_string(fd, port) ;
	}

	return;

}

char *
validate_number ( char *charp, int index)
{

	if (strlen(charp) > DN_LIMIT) {

		++config_error ;

		fprintf (stderr,
		 "dm_admin: directory number (--A%d %s) is too long, "
		 "max length is %d!\n\n", index, charp, DN_LIMIT);
	}

	return (charp) ;
}

char *
validate_sub_address ( char *charp, int index)
{

	if (strlen(charp) > DN_LIMIT) {

		++config_error ;

		fprintf (stderr,
		 "dm_admin: sub-address (--s%d %s) is too long, "
		 "max length is %d!\n\n", index, charp, DN_LIMIT);
	}

	return (charp) ;
}

char *
validate_spid ( char *charp, int index)
{

	if (strlen(charp) > DN_LIMIT) {

		++config_error ;

		fprintf (stderr,
		 "dm_admin: spid (--S%d %s) is too long, "
		 "max length is %d!\n\n", index, charp, DN_LIMIT);
	}

	return (charp) ;
}

uint32
validate_country_code ( char *charp)
{
	uint32 cc;
	unsigned int j;

	if (strlen(charp) != 2) {
		++config_error;

		fprintf (stderr,
		 "dm_admin: Modem nationality=\"%s\" invalid, must be 2 chars\n\n",
		 charp);

		return (0);
	}

	cc = MAKE_COUNTRY_CODE(charp[0], charp[1]) ;

	for (j = 0; j < asizeof (cc_2_name); ++j) {
		if (cc == cc_2_name[j].country_code)
			break;
	}

	if (j >= asizeof (cc_2_name)) {
		++config_error;

		fprintf (stderr,
		 "dm_admin: Modem nationality=\"%s\" is not suppoerted\n\n",
		 charp);

		return (0);
	}

	return (cc) ;

}

uint32
validate_adapter_country_code ( uint32 cc)
{
	unsigned int i, j;

	/* Validate modem nationality based on adapter type */

	for (j = 0; j < asizeof (board2list); ++j) {
		if (board2list[j].board_type == info.info.board_type) {
			break;
		}
	}

	if (j >= asizeof (board2list)) {

		++config_error ;

		fprintf (stderr,
		 "dm_admin: Unknkown adapter, unable to set modem nationality\n\n");

		usage () ;
		exit (1) ;
	}

	for (i = 0; i < board2list[j].array_size; ++i)
		if (cc == board2list[j].cc_array[i])
			break;

	if (i >= board2list[j].array_size) {

		++config_error ;

		fprintf (stderr,
		 "dm_admin: Modem Nationality=\"%c%c\" not supported by %s adapter\n\n",
		 cc >> 8, cc & 0xff, board2list[j].name);

		usage () ;
		exit (1) ;
	}

	return (cc) ;
}

int
validate_jitter_atten ( int value)
{

	switch (value) {

	case JITTER_TX:
	case JITTER_RX:
	case JITTER_AUTO:
	case JITTER_NONE:

		break;

	default:

		++config_error ;

		fprintf (stderr, "dm_admin: jitter (-j) = %d is invalid!\n\n",
		 value);

		return (0) ;

	}

	return (value) ;
}

int
validate_long_haul_lbo ( int value)
{

	switch (value) {

	case LBO_00_0_DB:
	case LBO_07_5_DB:
	case LBO_15_0_DB:
	case LBO_22_5_DB:

		break;

	default:

		++config_error ;

		fprintf (stderr, "dm_admin: long haul level (-O) = %d is invalid!\n\n",
		 value);

		return (0) ;

	}

	return (value) ;
}

int
validate_short_haul_len ( int value)
{

	switch (value) {

	case SH_0_133:
	case SH_133_266:
	case SH_266_399:
	case SH_399_533:
	case SH_533_655:

		break;

	default:

		++config_error ;

		fprintf (stderr,
		 "dm_admin: short haul length (-o) = %d is invalid!\n\n", value);

		return (0) ;

	}

	return (value) ;
}

int
validate_framing ( int value)
{

	switch (value) {

	case FRAME_ESF:
	case FRAME_D4:
	case FRAME_CEPT_CRC4:
	case FRAME_CEPT_WO_CRC4:

		break;

	default:

		++config_error ;

		fprintf (stderr, "dm_admin: framing (-f) = %d is invalid!\n\n",
		 value);

		return (0) ;

	}

	return (value) ;
}

int
validate_eq_gain_limit ( int value)
{

	switch (value) {

	case EGL_LOW:
	case EGL_HIGH:

		break;

	default:

		++config_error ;

		fprintf (stderr, "dm_admin: eq_gain (-e) = %d is invalid!\n\n",
		 value);

		return (0) ;

	}

	return (value) ;
}

int
validate_nfas_master_port ( int value)
{

	switch (value) {

	case NFAS_NONE:
	case NFAS_PORT0:
	case NFAS_PORT1:

		break;

	default:

		++config_error ;

		fprintf (stderr,
		 "dm_admin: NFAS master port (-F) = %d is invalid!\n\n", value);

		return (-1) ;

	}

	return (value | NFAS_MODIFY_BIT) ;
}

int
validate_nfas_interface_id ( int value)
{

	if ((value < 0) || (value > 127)) {

		++config_error ;

		fprintf (stderr,
		 "dm_admin: NFAS interface ID (-R) = %d is invalid! Must be 0 - 127\n\n",
		 value);

		return (-1) ;

	}

	return (value | NFAS_MODIFY_BIT) ;
}

int
validate_if_type ( int value)
{

	switch (value) {

	case LONG_HAUL:
	case SHORT_HAUL:
	case LOOP_HAUL:

		break;

	default:

		++config_error ;

		fprintf (stderr, "dm_admin: interface type (-i) = %d is invalid!\n\n",
		 value);

		return (0) ;

	}

	return (value) ;
}

int
validate_back2back ( int value)
{

	switch (value) {

	case DCHAN_NORMAL:
	case DCHAN_LOOPBACK_TE:
	case DCHAN_LOOPBACK_NT:

		break;

	default:

		++config_error ;

		fprintf (stderr, "dm_admin: back-to-back (-I) = %d is invalid!\n\n",
		 value);

		return (0) ;

	}

	return (value) ;
}

int
validate_address_type ( int value)
{

	switch (value) {

	case DCHAN_ADDRESS_TYPE_UNKNOWN:
	case DCHAN_ADDRESS_TYPE_INTL:
	case DCHAN_ADDRESS_TYPE_NATIONAL:
	case DCHAN_ADDRESS_TYPE_NETWORK:
	case DCHAN_ADDRESS_TYPE_SUBSCR:
	case DCHAN_ADDRESS_TYPE_ABBREV:

		break;

	default:

		++config_error ;

		fprintf (stderr, "dm_admin: address type (-Y) = %d is invalid!\n\n",
		 value);

		return (0) ;

	}

	return (value) ;
}

int
validate_subaddress_type ( int value)
{

	switch (value) {

	case DCHAN_SUBADDRESS_TYPE_NSAP:
	case DCHAN_SUBADDRESS_TYPE_USER:

		break;

	default:

		++config_error ;

		fprintf (stderr, "dm_admin: subaddress type (-y) = %d is invalid!\n\n",
		 value);

		return (0) ;

	}

	return (value) ;
}

int
validate_numbering_plan ( int value)
{

	switch (value) {

	case DCHAN_NUMBERING_PLAN_UNKNOWN:
	case DCHAN_NUMBERING_PLAN_ISDN:
	case DCHAN_NUMBERING_PLAN_DATA:
	case DCHAN_NUMBERING_PLAN_TELEX:
	case DCHAN_NUMBERING_PLAN_NATIONAL:
	case DCHAN_NUMBERING_PLAN_PRIVATE:

		break;

	default:

		++config_error ;

		fprintf (stderr, "dm_admin: numbering plan (-P) = %d is invalid!\n\n",
		 value);

		return (0) ;

	}

	return (value) ;
}

int
validate_coding ( int value)
{

	switch (value) {

	case LINE_CODE_B8ZS:
	case LINE_CODE_AMI:
	case LINE_CODE_HDB3:

		break;

	default:

		++config_error ;

		fprintf (stderr, "dm_admin: coding (-l) = %d is invalid!\n\n",
		 value);

		return (0) ;

	}

	return (value) ;
}

int
validate_loopback ( int value)
{

	switch (value) {

	case DS1_NO_LOOP:
	case DS1_PAYLOAD_LOOP:
	case DS1_LINE_LOOP:
	case DS1_LOCAL_LOOP:

		break;

	default:

		++config_error ;

		fprintf (stderr, "dm_admin: loopback (-L) = %d is invalid!\n\n",
		 value);

		return (0) ;

	}

	return (value) ;
}

int
validate_switch_type ( int value)
{

	switch (value) {

	case CT1_LOOP_START:
	case CT1_WINK_START:
	case CT1_GROUND_START:
	case CT1_IMMED_START:
	case E1_R2:
	case PRI_NONE:
	case PRI_NI2:
	case PRI_5ESS:
	case PRI_DMS100:
	case PRI_EURODSS1:
	case PRI_INS1500:
	case PRI_MSA:
	case BRI_NONE:
	case BRI_NI1:
	case BRI_5ESS:
	case BRI_DMS100:
	case BRI_EURODSS1:
	case BRI_INS64:

		break;

	default:

		++config_error ;

		fprintf (stderr, "dm_admin: switch (-s) = %d is invalid!\n\n",
		 value);

		return (0) ;

	}

	return (value) ;
}

int
validate_clocking ( int value)
{

	switch (value) {

	case TIMING_LOOP:
	case TIMING_LOCAL:

		break;

	default:

		++config_error ;

		fprintf (stderr, "dm_admin: clocking (-t) = %d is invalid!\n\n",
		 value);

		return (0) ;

	}

	return (value) ;
}

int
validate_tx_special ( int value)
{

	switch (value) {

	case DS1_TX_NORMAL:
	case DS1_TX_PRSS:
	case DS1_TX_QRSS:
	case DS1_TX_LLU:
	case DS1_TX_LLD:
	case DS1_TX_FDL_LLU:
	case DS1_TX_FDL_LLD:
	case DS1_TX_FDL_PLU:
	case DS1_TX_FDL_PLD:
	case DS1_TX_RFA:
	case DS1_TX_AIS:

		break;

	default:

		++config_error ;

		fprintf (stderr, "dm_admin: tx special (-T) = %d is invalid!\n\n",
		 value);

		return (0) ;

	}

	return (value) ;
}

int
validate_modem_lock ( int value)
{

	switch (value) {

	case BUSY_OUT_DISABLE:
	case BUSY_OUT_ENABLE_IMMED:
	case BUSY_OUT_ENABLE_DELAYED:

		break;

	default:

		++config_error ;

		fprintf (stderr, "dm_admin: modem busy-out (-m) = %d is invalid!\n\n",
		 value);

		return (0) ;

	}

	return (value) ;
}

int
validate_wink_digits ( int value)
{

	if ((value < 1) || (value > DN_LIMIT)) {

		++config_error ;

		fprintf (stderr,
		 "dm_admin: wink digits (-w) = %d is invalid! Must be 1 - %d\n\n",
		 value, DN_LIMIT);

		return (0) ;

	}

	return (value) ;
}

int
validate_bri_msn_option ( int value)
{

	switch (value) {

	case BRI_MSN_OFF:
	case BRI_MSN_ON:

		break;

	default:

		++config_error ;

		fprintf (stderr, "dm_admin: msn (-x) = %d is invalid!\n\n",
		 value);

		return (0) ;

	}

	return (value) ;
}

int
validate_bri_alert_option ( int value)
{

	switch (value) {

	case BRI_AUTOALERT_OFF:
	case BRI_AUTOALERT_ON:

		break;

	default:

		++config_error ;

		fprintf (stderr, "dm_admin: autoalert (-X) = %d is invalid!\n\n",
		 value);

		return (0) ;

	}

	return (value) ;
}

#ifdef XP_MEI
int
validate_port_type(int value)
{
	switch (value) {
	case SERIAL_PORTTYPE_RS232:
	case SERIAL_PORTTYPE_RS422:
	case SERIAL_PORTTYPE_RS485_2_E:
	case SERIAL_PORTTYPE_RS485_2_M:
	case SERIAL_PORTTYPE_RS485_4_E:
	case SERIAL_PORTTYPE_RS485_4_M:
		break;

	default:

		++config_error ;

		fprintf(stderr, "dm_admin: Invalid port type %d.\n\n", value);

		return 0;
	}
	
	return (value);
}

int
validate_rx_enable(int value)
{
	switch (value) {
	case RS485_RXENABLE_ALWAYS:
	case RS485_RXENABLE_AUTO:
		break;

	default:

		++config_error ;

		fprintf(stderr, "dm_admin: Invalid rx_enable value %d.\n\n", value);

		return 0;
	}
	
	return (value);
}

int
validate_tx_enable(int value)
{
	switch (value) {
	case RS485_TXENABLE_AUTO:
	case RS485_TXENABLE_RTS:
		break;

	default:

		++config_error ;

		fprintf(stderr, "dm_admin: Invalid tx_enable value %d.\n\n", value);

		return 0;
	}
	
	return (value);
}
#endif /* XP_MEI */

int
validate_e1r2_sw_type( char *cp)
{
	int i;

	if (isalpha(cp[0]))
		i = switch_to_num(cp);
	else
		i = strtoul(cp, NULL, 0);

	if ((i < 1) || (i > R2_LAST_SWITCH_TYPE)) {

		++config_error ;

		fprintf (stderr,
		 "dm_admin: --e1r2_sw_type = %s is invalid!\n\n", cp);

		return (0) ;

	}

	/*
	 * According to Eric S., to be added later
	 */

	// return (i | E1R2_MODIFY_BIT);

	return (i);

}

int
validate_cd_bits( int value)
{

	switch (value) {

	case 0:
	case E1_R2_C_BIT:
	case E1_R2_D_BIT:
	case E1_R2_C_BIT + E1_R2_D_BIT:

		break;

	default:

		++config_error ;

		fprintf (stderr, "dm_admin: --cd_bits = %d is invalid!\n\n",
		 value);

		return (0) ;

	}

	return (value | E1R2_MODIFY_BIT) ;
}

int
validate_r2_method( int value)
{

	switch (value) {

	case R2_DIGITAL_COMPELLED:
	case R2_DIGITAL_INVERTED:
	case R2_DIGITAL_METERED:

		break;

	default:

		++config_error ;

		fprintf (stderr, "dm_admin: --r2_method = %d is invalid!\n\n",
		 value);

		return (0) ;

	}

	return (value | E1R2_MODIFY_BIT) ;
}

int
validate_min_dnis( int value)
{


	if ((value < 0) || (value > 64)) {

		++config_error ;

		fprintf (stderr, "dm_admin: --min_dnis = %d is invalid!\n\n",
		 value);

		return (0) ;

	}

	return (value | E1R2_MODIFY_BIT) ;
}

int
validate_max_dnis( int value)
{


	if ((value < 0) || (value > 64)) {

		++config_error ;

		fprintf (stderr, "dm_admin: --max_dnis = %d is invalid!\n\n",
		 value);

		return (0) ;

	}

	return (value | E1R2_MODIFY_BIT) ;
}

int
validate_min_ani( int value)
{


	if ((value < 0) || (value > 64)) {

		++config_error ;

		fprintf (stderr, "dm_admin: --min_ani = %d is invalid!\n\n",
		 value);

		return (0) ;

	}

	return (value | E1R2_MODIFY_BIT) ;
}

int
validate_max_ani( int value)
{


	if ((value < 0) || (value > 64)) {

		++config_error ;

		fprintf (stderr, "dm_admin: --max_ani = %d is invalid!\n\n",
		 value);

		return (0) ;

	}

	return (value | E1R2_MODIFY_BIT) ;
}

int
validate_send_ani( int value)
{


	if ((value < 0) || (value > 1)) {

		++config_error ;

		fprintf (stderr, "dm_admin: --send_ani = %d is invalid!\n\n",
		 value);

		return (0) ;

	}

	return (value | E1R2_MODIFY_BIT) ;
}

int
validate_ani_sig( int value)
{


	if ((value < 1) || (value > 15)) {

		++config_error ;

		fprintf (stderr, "dm_admin: --ani_sig = %d is invalid!\n\n",
		 value);

		return (0) ;

	}

	return (value | MF_R2_BW_BASE | E1R2_MODIFY_BIT) ;
}

int
validate_answer_sig( int value)
{


	if ((value < 1) || (value > 15)) {

		++config_error ;

		fprintf (stderr, "dm_admin: --answer_sig = %d is invalid!\n\n",
		 value);

		return (0) ;

	}

	return (value | MF_R2_BW_BASE | E1R2_MODIFY_BIT) ;
}

int
validate_congest_sig( int value)
{


	if ((value < 1) || (value > 15)) {

		++config_error ;

		fprintf (stderr, "dm_admin: --congest_sig = %d is invalid!\n\n",
		 value);

		return (0) ;

	}

	return (value | MF_R2_BW_BASE | E1R2_MODIFY_BIT) ;
}

int
validate_end_dnis_sig( int value)
{


	if ((value < 1) || (value > 15)) {

		++config_error ;

		fprintf (stderr, "dm_admin: --end_dnis_sig = %d is invalid!\n\n",
		 value);

		return (0) ;

	}

	return (value | MF_R2_BW_BASE | E1R2_MODIFY_BIT) ;
}

int
validate_busy_sig( int value)
{


	if ((value < 1) || (value > 15)) {

		++config_error ;

		fprintf (stderr, "dm_admin: --busy_sig = %d is invalid!\n\n",
		 value);

		return (0) ;

	}

	return (value | MF_R2_BW_BASE | E1R2_MODIFY_BIT) ;
}

int
validate_incoming_allow( int value)
{


	if (value & ~R2_ALL_SUBSCRIBERS ) {

		++config_error ;

		fprintf (stderr, "dm_admin: --incoming_allow = 0%x is invalid!\n\n",
		 value);

		return (0) ;

	}

	return (value | E1R2_MODIFY_BIT) ;
}

int
validate_group2_type( int value)
{


	if ((value < 1) || (value > 9)) {

		++config_error ;

		fprintf (stderr, "dm_admin: --group2_type = %d is invalid!\n\n",
		 value);

		return (0) ;

	}

	return (value | E1R2_MODIFY_BIT) ;
}

void
validate_pri_settings(int fd)
{

	if (config_bri) {

		fprintf(stderr,
		 "dm_admin: Attempting to set BRI only parameters on a PRI port, invalid");

		usage() ;
		exit(1) ;
	}

	/* Get our current configuration */

	if (ioctl(fd, GET_DS1_CONFIG, &ds1_config) != 0) {
		dm_ioctl_fail() ;
	}

	/* Do ds1_config if any specified */

	if (config_ds1 || config_xri) {

		/* T1 or E1? */

		switch (info.info.board_type) {

		case ADAPTER_HOU24:
		case ADAPTER_HOU48:
		case ADAPTER_HOU24cpci:
		case ADAPTER_HOU48cpci:

			validate_t1_settings (fd);

			break;

		default:

			validate_e1_settings (fd);

			break;
		}
	}

	if (ds1_config.config.signalling == PRI_MSA)
		msa_switch = 1;

	if (ds1_config.config.signalling == E1_R2)
		e1r2_switch = 1;

	/* prepare for extended D-channel config if changes specified */

	if (config_dchanx) {
		validate_dchan_config() ;
	}

	/* Do extended DS1 if any specified */

	if (config_ds1x) {
		validate_ds1x_config() ;
	}

	if (!e1r2_switch && config_ds1_r2) {

		fprintf(stderr,
		 "dm_admin: Attempting to set E1 R2 only parameters when switch is not\n"
		 " set to E1 R2, invalid\n");

		usage() ;
		exit(1) ;
	}

	return ;

}

void
validate_t1_settings(int fd)
{
	getset_ds1_config_t other_ds1_config;
	int no_nfas;
	uint32	invalid_mask_bits;

	/* Signalling first */

	switch (switch_type) {

	case 0:

		/* nothing specified */
		break;

	case CT1_LOOP_START:
	case CT1_WINK_START:
	case CT1_GROUND_START:
	case CT1_IMMED_START:
	case PRI_NONE:
	case PRI_NI2:
	case PRI_5ESS:
	case PRI_DMS100:
	case PRI_INS1500:

		ds1_config.config.signalling = switch_type;
		break;

	case PRI_EURODSS1:
	case PRI_MSA:
	case E1_R2:

		fprintf(stderr,
		 "dm_admin: Attempting to set E1 only switch type (-s) on a T1 adapter, invalid\n\n");

		usage() ;
		exit(1) ;

	case BRI_NONE:
	case BRI_NI1:
	case BRI_5ESS:
	case BRI_DMS100:
	case BRI_EURODSS1:
	case BRI_INS64:

		fprintf(stderr,
		 "dm_admin: Attempting to set BRI only switch type (-s) on a T1 adapter, invalid\n\n");

		usage() ;
		exit(1) ;
	}

	/* Then interface type */

	if (if_type)
		ds1_config.config.if_type = if_type;

	/* Now the settings that depend on if_type */

	if (long_haul_lbo) {

		if (ds1_config.config.if_type == SHORT_HAUL) {

			fprintf(stderr,
			 "dm_admin: Long haul decibel level (-O) valid only with if_type=Long Haul\n\n");

			usage() ;
			exit(1) ;

		}

		ds1_config.config.long_haul_lbo = long_haul_lbo;
	}

	if (short_haul_len) {

		if (ds1_config.config.if_type == LONG_HAUL) {

			fprintf(stderr,
			 "dm_admin: Short haul length (-o) valid only with if_type=Short Haul\n\n");

			usage() ;
			exit(1) ;

		}

		ds1_config.config.short_haul_len = short_haul_len;
	}

	/* Allow wink digits only if wink start */

	if (wink_digits) {

		if (ds1_config.config.signalling != CT1_WINK_START) {

			fprintf(stderr,
			 "dm_admin: Wink digits (-w) valid only with switch type=Wink Start\n\n");

			usage() ;
			exit(1) ;

		}

		ds1_config.config.wink_digits = wink_digits;
	}

	switch (framing) {

	case 0:

		/* nothing specified */
		break;

	case FRAME_ESF:
	case FRAME_D4:
		ds1_config.config.framing = framing;
		break;

	case FRAME_CEPT_CRC4:
	case FRAME_CEPT_WO_CRC4:

		fprintf(stderr,
		 "dm_admin: Framing (-f) = %d valid only for E1 adapter\n\n", framing);

		usage() ;
		exit(1) ;
	}

	switch (coding) {

	case 0:

		/* nothing specified */
		break;

	case LINE_CODE_B8ZS:
	case LINE_CODE_AMI:
		ds1_config.config.coding = coding;
		break;

	case LINE_CODE_HDB3:

		fprintf(stderr,
		 "dm_admin: Coding (-l) = %d valid only for E1 adapter\n\n", coding);

		usage() ;
		exit(1) ;
	}

	if (clocking)
		ds1_config.config.clocking = clocking;

	if (jitter_atten)
		ds1_config.config.jitter_atten = jitter_atten;

	if (eq_gain_limit)
		ds1_config.config.eq_gain_limit = eq_gain_limit;

	/* These guys care whether or not this is a two port adapter */

	if ((info.info.board_type == ADAPTER_HOU24) || 
	 (info.info.board_type == ADAPTER_HOU24cpci)) {

		/* One port adapter */

		if (nfas_master_port) {

			fprintf(stderr,
			 "dm_admin: NFAS master port (-F) valid only for two port T1 adapter\n\n");

			usage() ;
			exit(1) ;
		}

		if (nfas_interface_id) {

			fprintf(stderr,
			 "dm_admin: NFAS interface ID (-R) valid only for two port T1 adapter\n\n");

			usage() ;
			exit(1) ;
		}

		if (leased_mask) {
			switch (switch_type) {

			case CT1_LOOP_START:
			case CT1_WINK_START:
			case CT1_GROUND_START:
			case CT1_IMMED_START:

				/* 24 time slots with channelized T1 */

				invalid_mask_bits = 0x7f000000;

				if (leased_mask & invalid_mask_bits) {

					fprintf(stderr,
					 "dm_admin: Bits 0x%08x invalid in leased mask (-M)\n\n",
					 leased_mask & invalid_mask_bits);

					usage() ;
					exit(1) ;
				}

				break;

			default:

				/* 23 time slots with PRI */

				invalid_mask_bits = 0x7f800000;

				if (leased_mask & invalid_mask_bits) {

					fprintf(stderr,
					 "dm_admin: Bits 0x%08x invalid in leased mask (-M)\n\n",
					 leased_mask & invalid_mask_bits);

					usage() ;
					exit(1) ;
				}

				break;

			}

			ds1_config.config.leased_mask = leased_mask;
		}

		switch (ds1_config.config.signalling) {

		case CT1_LOOP_START:
		case CT1_WINK_START:
		case CT1_GROUND_START:
		case CT1_IMMED_START:

			/* 24 time slots for channelized T1 */

			invalid_mask_bits = 0x7f000000;
			break;

		default:

			/* 23 time slots for PRI */

			invalid_mask_bits = 0x7f800000;
		}

		if (ds1_config.config.leased_mask & invalid_mask_bits) {

			fprintf(stderr,
			 "dm_admin: WARNING: Bits 0x%08x invalid in current leased mask, "
			 "ignored\n\n",
			 ds1_config.config.leased_mask & invalid_mask_bits);

		}

	} /* One port adapter */

	else {

		/* Two port adapter */

		/* Get the other port's current configuration */

		other_ds1_config.port = (ds1_config.port == 1) ? 2 : 1;

		if (ioctl(fd, GET_DS1_CONFIG, &other_ds1_config) != 0) {
			dm_ioctl_fail() ;
		}

		/* Don't be confused by upper bit */

		other_ds1_config.config.nfas_master_port =
		 other_ds1_config.config.nfas_master_port & 0x7fffffff;

		other_ds1_config.config.nfas_interface_id =
		 other_ds1_config.config.nfas_interface_id & 0x7fffffff;

		/* If either switch is channelized T1, NFAS is out */

		no_nfas = 0;

		switch (ds1_config.config.signalling) {

		case CT1_LOOP_START:
		case CT1_WINK_START:
		case CT1_GROUND_START:
		case CT1_IMMED_START:

			/* 24 time slots for channelized T1, no NFAS allowed */

			invalid_mask_bits = 0x7f000000;
			no_nfas = 1;
			break;

		default:

			/* 23 time slots for PRI */

			invalid_mask_bits = 0x7f800000;

			/* This port is PRI, check the other one */

			switch (other_ds1_config.config.signalling) {

			case CT1_LOOP_START:
			case CT1_WINK_START:
			case CT1_GROUND_START:
			case CT1_IMMED_START:

				/* Other port is channelized T1, no NFAS */

				no_nfas = 1;
				break;

			}

			break;

		}

		if (nfas_master_port) {
			if (no_nfas && (nfas_master_port & 0x7fffffff)) {

				fprintf(stderr,
				 "dm_admin: NFAS master port invalid if either of the T1 "
				 "ports on adapter is channelized T1\n\n");

				usage() ;
				exit(1) ;
			}

			ds1_config.config.nfas_master_port = nfas_master_port;
		}

		if (other_ds1_config.config.nfas_master_port != 
		 (ds1_config.config.nfas_master_port & 0x7fffffff)) {

			fprintf(stderr,
			 "dm_admin: WARNING: NFAS master port(%d) on this port(%d) is not "
			 "the same as NFAS master port(%d) on the other port(%d).\n"
			 "\tNFAS will not be enabled until they are configured to be the "
			 "same.\n\n",
			 ds1_config.config.nfas_master_port & 0x7fffffff, ds1_config.port,
			 other_ds1_config.config.nfas_master_port, other_ds1_config.port);

		}

		/* If NFAS enabled and we're NOT master port, we gain a B-channel */

		if ((ds1_config.config.nfas_master_port & 0x7fffffff) ==
		 other_ds1_config.port)
			invalid_mask_bits = 0x7f000000;

		if (nfas_interface_id) {
			if (no_nfas) {

				fprintf(stderr,
				 "dm_admin: NFAS interface id invalid if either of the T1 "
				 "ports on adapter is channelized T1\n\n");

				usage() ;
				exit(1) ;
			}

			if ((ds1_config.config.nfas_master_port & 0x7fffffff) ==
			 NFAS_NONE) {

				fprintf(stderr,
				 "dm_admin: NFAS interface id invalid if NFAS master port is "
				 "disabled\n\n");

				usage() ;
				exit(1) ;
			}

			ds1_config.config.nfas_interface_id = nfas_interface_id;
		}

		if (((other_ds1_config.config.nfas_master_port != NFAS_NONE) ||
		 ((ds1_config.config.nfas_master_port & 0x7fffffff) != NFAS_NONE)) &&
		 ((ds1_config.config.nfas_interface_id & 0x7fffffff) ==
		 other_ds1_config.config.nfas_interface_id)) {

			fprintf(stderr,
			 "dm_admin: WARNING: NFAS interface id(%d) on this port(%d) is "
			 "the same as NFAS interface id(%d) on the other port(%d).\n"
			 "\tNFAS will not function correctly until they are unique.\n\n",
			 ds1_config.config.nfas_interface_id & 0x7fffffff, ds1_config.port,
			 other_ds1_config.config.nfas_interface_id, other_ds1_config.port);

		}

		if (leased_mask) {
			if (leased_mask & invalid_mask_bits) {

				fprintf(stderr,
				 "dm_admin: Bits 0x%08x invalid in leased mask (-M)\n\n",
				 leased_mask & invalid_mask_bits);

				usage() ;
				exit(1) ;
			}

			ds1_config.config.leased_mask = leased_mask;
		}

		if (ds1_config.config.leased_mask & invalid_mask_bits) {

			fprintf(stderr,
			 "dm_admin: WARNING: Bits 0x%08x invalid in current leased mask, "
			 "ignored\n\n",
			 ds1_config.config.leased_mask & invalid_mask_bits);

		}

	} /* else two port adapter */
}

void
validate_e1_settings(int fd)
{

	switch (switch_type) {

	case 0:

		/* nothing specified */
		break;

	case PRI_MSA:

		switch (info.info.board_type) {

		case ADAPTER_HOU30cpci:
		case ADAPTER_HOU60cpci:
			break;

		default:

			fprintf(stderr,
			 "dm_admin: Attempting to set E1 cpci only switch type on a non-cpci adapter, invalid\n\n");

			usage() ;
			exit(1) ;

		}

		/* Fall through */

	case PRI_EURODSS1:
	case PRI_NONE:
	case E1_R2:
		ds1_config.config.signalling = switch_type;
		break;

	case CT1_LOOP_START:
	case CT1_WINK_START:
	case CT1_GROUND_START:
	case CT1_IMMED_START:
	case PRI_NI2:
	case PRI_5ESS:
	case PRI_DMS100:
	case PRI_INS1500:

		fprintf(stderr,
		 "dm_admin: Attempting to set T1 only switch type on an E1 adapter, invalid\n\n");

		usage() ;
		exit(1) ;

	case BRI_NONE:
	case BRI_NI1:
	case BRI_5ESS:
	case BRI_DMS100:
	case BRI_EURODSS1:
	case BRI_INS64:

		fprintf(stderr,
		 "dm_admin: Attempting to set BRI only switch type on an E1 adapter, invalid\n\n");

		usage() ;
		exit(1) ;
	}

	if (ds1_config.config.signalling == PRI_MSA)
		msa_switch = 1;

	if (ds1_config.config.signalling == E1_R2)
		e1r2_switch = 1;

	if (msa_switch && framing) {

		fprintf(stderr,
		 "dm_admin: framing is not valid if switch type is MSA\n");

		usage();

		exit (1);
	}

	switch (framing) {

	case 0:

		/* nothing specified */
		break;

	case FRAME_CEPT_CRC4:
	case FRAME_CEPT_WO_CRC4:
		ds1_config.config.framing = framing;
		break;

	case FRAME_ESF:
	case FRAME_D4:

		fprintf(stderr,
		 "dm_admin: Framing (-f) = %d valid only for T1 adapter\n\n", framing);

		usage() ;
		exit(1) ;
	}

	if (msa_switch && coding) {

		fprintf(stderr,
		 "dm_admin: coding is not valid if switch type is MSA\n");

		usage();

		exit (1);
	}

	switch (coding) {

	case 0:

		/* nothing specified */
		break;

	case LINE_CODE_HDB3:
		ds1_config.config.coding = coding;
		break;

	case LINE_CODE_B8ZS:
	case LINE_CODE_AMI:

		fprintf(stderr,
		 "dm_admin: Coding (-l) = %d valid only for T1 adapter\n\n", coding);

		usage() ;
		exit(1) ;
	}

	if (msa_switch && clocking) {

		fprintf(stderr,
		 "dm_admin: clocking is not valid if switch type is MSA\n");

		usage();

		exit (1);
	}

	if (clocking)
		ds1_config.config.clocking = clocking;

	if (msa_switch && jitter_atten) {

		fprintf(stderr,
		 "dm_admin: jitter is not valid if switch type is MSA\n");

		usage();

		exit (1);
	}

	if (jitter_atten)
		ds1_config.config.jitter_atten = jitter_atten;

	switch (if_type) {

	case LOOP_HAUL:
		/* allowed for E1? */

	case LONG_HAUL:
	case SHORT_HAUL:
		fprintf(stderr,
		 "dm_admin: if_type = %d (-i) valid only for T1 adapter\n\n", if_type);

		usage() ;
		exit(1) ;
	}

	if (long_haul_lbo) {

		fprintf(stderr,
		 "dm_admin: long_haul_lbo (-O) valid only for T1 adapter\n\n");

		usage() ;
		exit(1) ;
	}

	if (short_haul_len) {

		fprintf(stderr,
		 "dm_admin: Short haul length (-o) valid only for T1 adapter\n\n");

		usage() ;
		exit(1) ;
	}

	if (wink_digits) {

		fprintf(stderr,
		 "dm_admin: Wink digits (-w) valid only for T1 adapter\n\n");

		usage() ;
		exit(1) ;
	}

	if (eq_gain_limit) {

		fprintf(stderr,
		 "dm_admin: Equalization gain limit (-e) valid only for T1 adapter\n\n");

		usage() ;
		exit(1) ;
	}

	if (nfas_master_port) {

		fprintf(stderr,
		 "dm_admin: NFAS master port (-F) valid only for two port T1 adapter\n\n");

		usage() ;
		exit(1) ;
	}

	if (nfas_interface_id) {

		fprintf(stderr,
		 "dm_admin: NFAS interface ID (-R) valid only for two port T1 adapter\n\n");

		usage() ;
		exit(1) ;
	}

	if (e1r2_switch && leased_mask) {

		fprintf(stderr,
		 "dm_admin: leased_mask is not valid if switch type is E1 R2\n");

		usage();

		exit (1);
	}

	if (leased_mask) {

		/* 30 time slots with E1 */

		if (leased_mask & 0x00008000) {

			fprintf(stderr,
			 "dm_admin: Bits 0x%08x invalid in leased mask (-M)\n\n",
			 leased_mask & 0x00008000);

			usage() ;
			exit(1) ;
		}

		ds1_config.config.leased_mask = leased_mask;
	}

}

void
validate_bri_settings(int fd)
{
	unsigned int new_dir_count = 0, max_dir_count = 1 ;
	int dir_changed = 0, spid_only = 0, subaddr_only = 0, j ;
	unsigned int i ;

	if (config_ds1 || config_ds1x || config_ds1_r2) {

		fprintf(stderr,
		 "dm_admin: Attempting to set PRI only parameters on a BRI port, invalid");

		usage() ;
		exit(1) ;
	}

	/* prepare for extended D-channel config if changes specified */

	if (config_dchanx) {
		validate_dchan_config() ;
	}

	/* Don't bother with bri_config if nothing changed */

	if (!config_bri && !config_xri)
		return ;

	bri_config.port = port ;

	/* get current bri config settings, then validate and fill in changes */

	if (ioctl(fd, GET_BRI_CONFIG, &bri_config) != 0) {
		dm_ioctl_fail() ;
	}

	/* Clear upper bits for dir_count */

	bri_config.config.dir_count = bri_config.config.dir_count & 0x7fffffff ;

	switch (switch_type) {
	case 0: /* option not specified */
		break;

	case BRI_NONE:
	case BRI_NI1:
	case BRI_5ESS:
	case BRI_DMS100:
	case BRI_EURODSS1:
	case BRI_INS64:
		bri_config.config.switch_type = switch_type ;
		break;

	default:
		fprintf(stderr,
		 "PRI switch type (-s %d) not valid for BRI adapter\n", switch_type);

		usage() ;
		close(fd) ;
		exit(1) ;
	}

	if (leased_mask) {
		if (leased_mask & 0x7ffffffc) {
			fprintf(stderr,
			 "Invalid Leased Mask (-M 0x%08x), only bits 0x03 may be set\n",
			 leased_mask & 0x7fffffff);

			usage() ;
			close(fd) ;
			exit(1) ;
		}

		bri_config.config.leased_mask = leased_mask ;
	}

	if (bri_msn_option) {

		if (bri_config.config.switch_type != BRI_EURODSS1) {
			fprintf(stderr,
			 "msn option (-x %d) valid only for switch_type=EURODSS1 (20)\n",
			 bri_msn_option);

			usage() ;
			close(fd) ;
			exit(1) ;
		}

		bri_config.config.msn = bri_msn_option;
	}

	if (bri_alert_option)
		bri_config.config.autoalert = bri_alert_option;

	/* Determine max possible dir_count based on switch type and msn */

	switch (bri_config.config.switch_type) {
	case BRI_NONE:
	case BRI_NI1:
	case BRI_5ESS:
	case BRI_DMS100:
		max_dir_count = 2;
		++spid_only;
		break;

	case BRI_EURODSS1:
		max_dir_count = (bri_config.config.msn == BRI_MSN_ON) ? MSN_MAX : 1;
		++subaddr_only;
		break;

	case BRI_INS64:
		max_dir_count = 1;
		++subaddr_only;
		break;

	}

	/* Do a little repair if dir_count is already out-of-range */

	if (bri_config.config.dir_count > max_dir_count) {
		++dir_changed;
		bri_config.config.dir_count = max_dir_count;
	}

	/* Then insure all unused entries are zero length strings */

	for (i = bri_config.config.dir_count; i < MSN_MAX; ++i) {

		bri_config.config.directory[i].number[0] = 0x00 ;
		bri_config.config.directory[i].sub_address[0] = 0x00 ;
		bri_config.config.directory[i].spid[0] = 0x00 ;
	}

	/* Now we get the directory numbers right */

	for (i = 0; i < MSN_MAX; ++i) {

		if (number[i].changed) {

			if (i + 1 > max_dir_count) {
				fprintf(stderr,
				 "Directory number option (--A%d %s) is beyond max entries "
				 "(%d) for this switch type\n", i + 1, number[i].new_str,
				 max_dir_count);

				usage() ;
				close(fd) ;
				exit(1) ;
			}

			++dir_changed;

			strcpy((char *) bri_config.config.directory[i].number,
			 number[i].new_str);

			new_dir_count = i + 1;
		}

	}

	if (new_dir_count > bri_config.config.dir_count)
		bri_config.config.dir_count = new_dir_count;

	/* Now find the last entry that actually has text */

	for (i = bri_config.config.dir_count - 1; i > 0; --i)
		if (bri_config.config.directory[i].number[0] != 0x00)
			break ;

	if (i + 1 < bri_config.config.dir_count) {
		++dir_changed;
		bri_config.config.dir_count = i + 1;
	}

	for (i = 0; i < MSN_MAX; ++i) {

		if (sub_address[i].changed) {

			if (spid_only) {
				fprintf(stderr,
				 "Sub-address option (--a%d %s) invalid for this switch "
				 "type, spid only\n", i + 1, sub_address[i].new_str);

				usage() ;
				close(fd) ;
				exit(1) ;
			}

			if (i + 1 > max_dir_count) {
				fprintf(stderr,
				 "Sub-address option (--a%d %s) is beyond max entries "
				 "(%d) for this switch type\n", i + 1, sub_address[i].new_str,
				 max_dir_count);

				usage() ;
				close(fd) ;
				exit(1) ;
			}

			if ((sub_address[i].new_str[0] != 0x00) &&
			 (i + 1 > bri_config.config.dir_count)) {
				fprintf(stderr,
				 "Sub-address option (--a%d %s) has no directory "
				 "number, required\n", i + 1, sub_address[i].new_str);

				usage() ;
				close(fd) ;
				exit(1) ;
			}

			++dir_changed;

			strcpy((char *) &bri_config.config.directory[i].sub_address[0],
			 &sub_address[i].new_str[0]);
		}

		if (spid[i].changed) {

			if (subaddr_only) {
				fprintf(stderr,
				 "Spid option (--S%d %s) invalid for this switch "
				 "type, sub-address only\n", i + 1, spid[i].new_str);

				usage() ;
				close(fd) ;
				exit(1) ;
			}

			if (i + 1 > max_dir_count) {
				fprintf(stderr,
				 "Spid option (--S%d %s) is beyond max entries "
				 "(%d) for this switch type\n", i + 1, spid[i].new_str,
				 max_dir_count);

				usage() ;
				close(fd) ;
				exit(1) ;
			}

			if ((spid[i].new_str[0] != 0x00) &&
			 (i + 1 > bri_config.config.dir_count)) {
				fprintf(stderr,
				 "Spid option (--a%d %s) has no directory "
				 "number, required\n", i + 1, spid[i].new_str);

				usage() ;
				close(fd) ;
				exit(1) ;
			}

			++dir_changed;

			strcpy((char *) &bri_config.config.directory[i].spid[0],
			 &spid[i].new_str[0]);
		}

	}

	/* Now search for empty directory numbers and warn user */

	for (j = bri_config.config.dir_count - 1; j >= 0; --j)
		if (bri_config.config.directory[j].number[0] == 0x00)
			fprintf(stderr,
			 "dm_admin: Warning, Directory Number %d is empty\n\n", j + 1) ;

	if (dir_changed)
		bri_config.config.dir_count |= BRI_DIR_UPDATE;

	return ;

}

int
main(int argc, char * const argv[])
{
	extern char *optarg ;

	int c, n, i ;
	int fd ;
	char device[128] ;
	char mesg[128] ;
	int repeat = 1;

	/*
	 * getopt_long stuff
	 */

	int option_index = 0;

	enum {
		A1_OPT = 0, A2_OPT, A3_OPT, A4_OPT, A5_OPT, A6_OPT, A7_OPT, A8_OPT,
		a1_OPT, a2_OPT, a3_OPT, a4_OPT, a5_OPT, a6_OPT, a7_OPT, a8_OPT,
		S1_OPT, S2_OPT, S3_OPT, S4_OPT, S5_OPT, S6_OPT, S7_OPT, S8_OPT,
		R2_SW, R2_CD, R2_METH, R2_MND, R2_MXD, R2_MNA, R2_MXA, R2_SNDA,
		R2_ADG, R2_ANS, R2_CG, R2_EDN, R2_BSY, R2_SZA, R2_ADL, R2_MSK,
		R2_OUT, R2_LSM, E1R2_PRINT, HELP_OPT, PORT_OPT, RX_OPT, TX_OPT
	};

	static struct option long_options[] = {
		{"A1", 1, 0, 0}, {"A2", 1, 0, 0}, {"A3", 1, 0, 0}, {"A4", 1, 0, 0},
		{"A5", 1, 0, 0}, {"A6", 1, 0, 0}, {"A7", 1, 0, 0}, {"A8", 1, 0, 0},
		{"a1", 1, 0, 0}, {"a2", 1, 0, 0}, {"a3", 1, 0, 0}, {"a4", 1, 0, 0},
		{"a5", 1, 0, 0}, {"a6", 1, 0, 0}, {"a7", 1, 0, 0}, {"a8", 1, 0, 0},
		{"S1", 1, 0, 0}, {"S2", 1, 0, 0}, {"S3", 1, 0, 0}, {"S4", 1, 0, 0},
		{"S5", 1, 0, 0}, {"S6", 1, 0, 0}, {"S7", 1, 0, 0}, {"S8", 1, 0, 0},
		{"e1r2_sw_type", 1, 0, 0},  {"cd_bits", 1, 0, 0},
		{"r2_method", 1, 0, 0},     {"min_dnis", 1, 0, 0},
		{"max_dnis", 1, 0, 0},      {"min_ani", 1, 0, 0},
		{"max_ani", 1, 0, 0},       {"send_ani", 1, 0, 0},
		{"ani_sig", 1, 0, 0},       {"answer_sig", 1, 0, 0}, 
		{"congest_sig", 1, 0, 0},   {"end_dnis_sig", 1, 0, 0},
		{"busy_sig", 1, 0, 0},      {"sz_ack_delay", 1, 0, 0}, 
		{"answer_delay", 1, 0, 0},  {"incoming_allow", 1, 0, 0},
		{"group2_type", 1, 0, 0},   {"linesig_min", 1, 0, 0},
		{"print_e1r2", 0, 0, 0},    {"help", 0, 0, 0},
#ifdef XP_MEI
		{"porttype", 1, 0, 0},	    {"rx_enable", 1, 0, 0}, 
		{"tx_enable", 1, 0, 0},
#endif /* XP_MEI */
		{0, 0, 0, 0}
	};

	get_mgmt_protocol_version_t protocol ;
	getset_modem_ext_config_t modem_config ;
	getset_modem_nationality_t modem_set_nat ;

	/*
	 * Initialize number, sub_address and spid structs to "no change"
	 */

	for (i = 0; i < MSN_MAX; ++i) {

		number[i].changed = 0;
		number[i].new_str = NULL;

		sub_address[i].changed = 0;
		sub_address[i].new_str = NULL;

		spid[i].changed = 0;
		spid[i].new_str = NULL;

	}

	/*
	 * Crack the command line options.
	 */

#ifndef NEED_LONG_OPTIONS
	while((c = getopt_long(argc, argv,
	 "F:R:I:L:T:b:c:C:d:e:j:Y:y:P:o:O:f:hi:l:M:Nnp:qrs:t:vEDgSGBm:w:Z:x:X:z",
	 long_options, &option_index)) != EOF) {

#else
	while((c = _getopt_internal(argc, argv,
	 "F:R:I:L:T:b:c:C:d:e:j:Y:y:P:o:O:f:hi:l:M:Nnp:qrs:t:vEDgSGBm:w:Z:x:X:z",
	 long_options, &option_index, 0)) != EOF) {
#endif

		switch(c) {

		/* These calls exit directly out of the case statement */

		case 'N':
			display_cc();
			exit (0);

		case 'v':
			printf("dm_admin - $Revision: 1.14 $\n") ;
			exit(0) ;

		/* These specify adapter, port and channel */

		case 'b':
			board = strtol(optarg, NULL, 0) ;
			break ;

		case 'p':
			port = strtol(optarg, NULL, 0) ;
			ds1_config.port = port ;
			ds1_ext_config.port = port ;
			ds1_e1r2_config.port = port ;
			break ;

		case 'c':
			channel = strtol(optarg, NULL, 0) ;
			break ;

		/* These are the querey options that can be done in groups */

		case 'q':
			++query_call;
			query++ ;
			break ;

		case 'B':
			++query_call;
			bchan++;
			break ;

		case 'D':
			++query_call;
			dchan++;
			break ;

		case 'E':
			++query_call;
			modem_stat++;
			break ;

		case 'g':
			++query_call;
			serial_config++;
			break ;

		case 'G':
			++query_call;
			serial++;
			break ;

#ifdef XP_MEI
		case 'S':
			++query_call;
			rs485_config++;
			break;
#endif /* XP_MEI */

		/* Long option configuration changes */

		case 0:

			switch (option_index) {

			/* BRI directory number */

			case A1_OPT:
			case A2_OPT:
			case A3_OPT:
			case A4_OPT:
			case A5_OPT:
			case A6_OPT:
			case A7_OPT:
			case A8_OPT:

				++config_call ;
				++config_bri ;

				number[option_index - A1_OPT].new_str =
				 validate_number(optarg, option_index - A1_OPT);

				number[option_index - A1_OPT].changed = 1;

				break;

			/* BRI sub-address */

			case a1_OPT:
			case a2_OPT:
			case a3_OPT:
			case a4_OPT:
			case a5_OPT:
			case a6_OPT:
			case a7_OPT:
			case a8_OPT:

				++config_call ;
				++config_bri ;

				sub_address[option_index - a1_OPT].new_str =
				 validate_sub_address(optarg, option_index - a1_OPT);

				sub_address[option_index - a1_OPT].changed = 1;

				break;

			/* BRI spid */

			case S1_OPT:
			case S2_OPT:
			case S3_OPT:
			case S4_OPT:
			case S5_OPT:
			case S6_OPT:
			case S7_OPT:
			case S8_OPT:

				++config_call ;
				++config_bri ;

				spid[option_index - S1_OPT].new_str =
				 validate_spid(optarg, option_index - S1_OPT);

				spid[option_index - S1_OPT].changed = 1;

				break;

			case R2_SW:
				++config_ds1_r2;
				++config_call;

				ds1_e1r2_config.config.r2_switch_type = 
				 validate_e1r2_sw_type (optarg);

				break;

			case R2_CD:
				++config_ds1_r2;
				++config_call;
				ds1_e1r2_config.config.r2_cd_bits = 
				 validate_cd_bits (strtoul(optarg, NULL, 0));
				break;

			case R2_METH:
				++config_ds1_r2;
				++config_call;
				ds1_e1r2_config.config.r2_method = 
				 validate_r2_method (strtoul(optarg, NULL, 0));
				break;

			case R2_MND:
				++config_ds1_r2;
				++config_call;
				ds1_e1r2_config.config.min_dnis_digits = 
				 validate_min_dnis (strtoul(optarg, NULL, 0));
				break;

			case R2_MXD:
				++config_ds1_r2;
				++config_call;
				ds1_e1r2_config.config.max_dnis_digits = 
				 validate_max_dnis (strtoul(optarg, NULL, 0));
				break;

			case R2_MNA:
				++config_ds1_r2;
				++config_call;
				ds1_e1r2_config.config.min_ani_digits = 
				 validate_min_ani (strtoul(optarg, NULL, 0));
				break;

			case R2_MXA:
				++config_ds1_r2;
				++config_call;
				ds1_e1r2_config.config.max_ani_digits = 
				 validate_max_ani (strtoul(optarg, NULL, 0));
				break;

			case R2_SNDA:
				++config_ds1_r2;
				++config_call;
				ds1_e1r2_config.config.send_ani_digits =
				 validate_send_ani (strtoul(optarg, NULL, 0));
				break;

			case R2_ADG:
				++config_ds1_r2;
				++config_call;
				ds1_e1r2_config.config.ani_digit_signal = 
				 validate_ani_sig (strtoul(optarg, NULL, 0));
				break;

			case R2_ANS:
				++config_ds1_r2;
				++config_call;
				ds1_e1r2_config.config.answer_signal = 
				 validate_answer_sig (strtoul(optarg, NULL, 0));
				break;

			case R2_CG:
				++config_ds1_r2;
				++config_call;
				ds1_e1r2_config.config.congestion_signal = 
				 validate_congest_sig (strtoul(optarg, NULL, 0));
				break;

			case R2_EDN:
				++config_ds1_r2;
				++config_call;
				ds1_e1r2_config.config.end_dnis_signal = 
				 validate_end_dnis_sig (strtoul(optarg, NULL, 0));
				break;

			case R2_BSY:
				++config_ds1_r2;
				++config_call;
				ds1_e1r2_config.config.busy_signal = 
				 validate_busy_sig (strtoul(optarg, NULL, 0));
				break;

			case R2_SZA:
				++config_ds1_r2;
				++config_call;
				ds1_e1r2_config.config.seize_ack_delay = 
				 strtoul(optarg, NULL, 0) | E1R2_MODIFY_BIT;
				break;

			case R2_ADL:
				++config_ds1_r2;
				++config_call;
				ds1_e1r2_config.config.answer_delay = 
				 strtoul(optarg, NULL, 0) | E1R2_MODIFY_BIT;
				break;

			case R2_MSK:
				++config_ds1_r2;
				++config_call;
				ds1_e1r2_config.config.incoming_allow_mask = 
				 validate_incoming_allow (strtoul(optarg, NULL, 0));
				break;

			case R2_OUT:
				++config_ds1_r2;
				++config_call;
				ds1_e1r2_config.config.outgoing_type = 
				 validate_group2_type (strtoul(optarg, NULL, 0));
				break;

			case R2_LSM:
				++config_ds1_r2;
				++config_call;
				ds1_e1r2_config.config.linesig_min = 
				 strtoul(optarg, NULL, 0) | E1R2_MODIFY_BIT;
				break;

			case E1R2_PRINT:
				++query_call;
				++e1r2_print_config;
				break;
			
#ifdef XP_MEI	
			case PORT_OPT:
				++config_call;
				++config_serial;
				port_type = 
					validate_port_type(strtoul(optarg
								   , NULL, 0));
				break;
			case RX_OPT:
				++config_call;
				++config_serial;
				rx_enable = validate_rx_enable(strtoul(optarg
							       ,NULL,0));
				break;
			case TX_OPT:
				++config_call;
				++config_serial;
				tx_enable = validate_tx_enable(strtoul(optarg
							       ,NULL,0));
				break;
#endif /* XP_MEI */
				
			default:
			case HELP_OPT:
				long_usage() ;
				exit(0) ;
			}

			break;

		/* Short option configuration changes */

		case 'C':
			++config_call;

			/*
			 * It looks like we should do ++config_modem here but
			 * that flag is used to insure that hangup and modem_lock
			 * cannot be done in conjunction with other config
			 * changes, not to specify a country code change.
			 */

			country_code = validate_country_code (optarg);
			break ;

		case 'e':
			++config_call;
			++config_ds1;
			eq_gain_limit = validate_eq_gain_limit(strtol(optarg, NULL, 0)) ;
			break ;

		case 'j':
			++config_call;
			++config_ds1;
			jitter_atten = validate_jitter_atten(strtol(optarg, NULL, 0)) ;
			break ;

		case 'O':
			++config_call;
			++config_ds1;
			++config_t1;
			long_haul_lbo = validate_long_haul_lbo(strtol(optarg, NULL, 0)) ;
			break ;

		case 'o':
			++config_call;
			++config_ds1;
			++config_t1;
			short_haul_len = validate_short_haul_len(strtol(optarg, NULL, 0)) ;
			break ;

		case 'f':
			++config_call;
			++config_ds1;
			framing = validate_framing(strtol(optarg, NULL, 0)) ;
			break ;

		case 'h':
			++config_modem;
			hangup++ ;
			break ;
		case 'r':
			++config_modem;
			reset++;
			break;
		case 'F':
			++config_call;
			++config_ds1;
			++config_t2;
			nfas_master_port =
			 validate_nfas_master_port(strtol(optarg, NULL, 0)) ;
			break ;

		case 'R':
			++config_call;
			++config_ds1;
			++config_t2;
			nfas_interface_id =
			 validate_nfas_interface_id(strtol(optarg, NULL, 0)) ;
			break ;

		case 'i':
			++config_call;
			++config_ds1;
			++config_t1;
			if_type = validate_if_type(strtol(optarg, NULL, 0)) ;
			break ;

		case 'I':
			++config_call;
			++config_dchanx;
			back2back = validate_back2back(strtol(optarg, NULL, 0)) ;
			break ;

		case 'Y':
			++config_call;
			++config_dchanx;
			address_type = validate_address_type(strtol(optarg, NULL, 0)) ;
			break ;

		case 'y':
			++config_call;
			++config_dchanx;
			subaddress_type =
			 validate_subaddress_type(strtol(optarg, NULL, 0)) ;
			break ;

		case 'P':
			++config_call;
			++config_dchanx;
			numbering_plan = validate_numbering_plan(strtol(optarg, NULL, 0)) ;
			break ;

		case 'l':
			++config_call;
			++config_ds1;
			coding = validate_coding(strtol(optarg, NULL, 0)) ;
			break ;

		case 'L':
			++config_call;
			++config_ds1x;
			loopback = validate_loopback(strtol(optarg, NULL,0)) ;
			break ;

		case 's':
			++config_call;
			++config_xri;
			switch_type = validate_switch_type(strtol(optarg, NULL, 0)) ;
			break ;

		case 't':
			++config_call;
			++config_ds1;
			clocking = validate_clocking(strtol(optarg, NULL, 0)) ;
			break ;

		case 'M':
			++config_call;
			++config_xri;
			leased_mask = strtol(optarg, NULL, 0) ;
			if (!leased_mask)
				leased_mask |= 0x80000000 ;
			break ;

		case 'T':
			++config_call;
			++config_ds1x;
			tx_special = validate_tx_special(strtol(optarg, NULL,0)) ;
			break ;

		case 'm':
			++config_call;
			++config_modem;
			modem_lock = validate_modem_lock(strtol(optarg, NULL, 0)) ;
			break ;

		case 'Z':
#ifdef DEVELOPER
			repeat = strtol(optarg, NULL, 0) ;
#endif
			break ;

		case 'w':
			++config_call;
			++config_ds1;
			++config_t1;
			wink_digits = validate_wink_digits(strtol(optarg, NULL, 0)) ;
			break ;

		case 'x':
			++config_call;
			++config_bri;
			bri_msn_option = validate_bri_msn_option(strtol(optarg, NULL, 0)) ;
			break ;

		case 'X':
			++config_call;
			++config_bri;
			bri_alert_option =
			 validate_bri_alert_option(strtol(optarg, NULL, 0)) ;
			break ;

		case 'z':
			++config_call;
			++config_bri;

			/*
			 * This bit of magic doesn't really set dir_count to zero,
			 * it actually leaves dir_count unchanged.
			 * It clears all address, sub-address and spid strings
			 * to 0x00 */

			dir_count = BRI_DIR_UPDATE;
			break ;

		case '?':
		case ':':
			usage() ;
			exit (1) ;

		default:
			exit(1) ;
		}
	}

	/*
	 * Range check the board argument, if any.
	 */
	if ((board < 0) || (board >= MAXBOARDS)) {
		fprintf(stderr, "dm_admin: adapter %d out of range\n", board) ;
		usage();
		exit(1) ;
	}

	if ((channel != -1) && (port == -1)) {
		fprintf(stderr, "dm_admin: no port specified for channel %d\n",
							channel) ;
		usage();
		exit(1) ;
	}

	for (n = FIRST_TRACE_CHAN; n < TRACE_CHAN_COUNT + FIRST_TRACE_CHAN;
	 n++ ) {

		/* try all devices */

		MK_DEV_NAME (device, DEVDIR, n, board);

		if((fd=open(device, O_RDWR)) != -1) {
			break;
		}	
	}

	if (n >= TRACE_CHAN_COUNT + FIRST_TRACE_CHAN) {
		sprintf(mesg, "Cannot open device %s", device);
		perror(mesg);
		exit(1);
	}

	if (ioctl(fd, GET_MGMT_PROTOCOL_VERSION, &protocol) != 0) {
		dm_ioctl_fail() ;
	}

	/*
	 * Make a GET_ADAPTER_INFO ioctl to the board.
	 *
	 * We will need this information for many things, not
	 * the least of which will be sanity checks for things
	 * like port/channel #.
	 */
	if (ioctl(fd, GET_ADAPTER_INFO, &info) != 0) {
		dm_ioctl_fail() ;
	}

	if ((port == 0) || (port > (int)info.info.port_count)) {
		fprintf(stderr, "dm_admin: port %d out of range (1 - %d)\n",
					port, info.info.port_count) ;
		usage();
		exit(1) ;
	}

	if ((channel == 0) || (channel > (int)info.info.channels_per_port)) {
		fprintf(stderr, "dm_admin: channel %d out of range  (1 - %d)\n",
					channel, info.info.channels_per_port) ;
		usage();
		exit(1) ;
	}

	flat_channel = (info.info.channels_per_port * (port - 1)) + channel - 1 ;

	if (query_call) {

		process_queries (fd, &info.info);

		close (fd) ;
		return(0) ;
	}

	if (config_error > 0) {

		usage() ;
		close (fd) ;
		exit (1) ;
	}

	if (config_call < 1 && config_modem < 1) {

		fprintf(stderr,
		 "dm_admin: No query options or configuration change options specified, Required\n");

		usage() ;
		close (fd) ;
		exit (1) ;
	}

	if (country_code) {

		modem_set_nat.nation.country_code =
		 validate_adapter_country_code(country_code);

		print_modem_nationality(&modem_set_nat);

		if (ioctl(fd, SET_MODEM_NATIONALITY, &modem_set_nat) != 0) {
			dm_ioctl_fail() ;
		}
	
		/*
		 * Set country_code back to zero
		 * Determine if other config changes made
		 */
		country_code = 0 ;
		--config_call;

		if (config_call < 1) {

			port = -1;
			query_action(fd, &info.info) ;

			close(fd) ;
			return(0) ;
		}
	}

	if (port == -1) {
		fprintf(stderr, "dm_admin: port (-p) must be specified.\n\n") ;
		usage();
		exit (1) ;
	}

	if (config_modem) {

		if (channel == -1) {
			fprintf(stderr,
			 "dm_admin: channel (-c) must be specified with -m, -h, or -r.\n\n") ;
			usage();
			exit (1) ;
		}

		if (modem_lock) {
			PrepModemConfig(&modem_config) ;
			if (ioctl(fd, SET_MODEM_EXT_CONFIG, &modem_config) != 0) {
				dm_ioctl_fail() ;
			}
		}

		if (hangup) {
			ModemHangup(fd) ;
		}

		if (reset) {
			ModemReset(fd) ;
		}

		dump_modem_info(fd) ;

		close(fd) ;
		return(0) ;
	}

	switch (info.info.board_type)
	{

	default:

		fprintf (stderr, "dm_admin: Unknown adapter type") ;

		close(fd) ;
		exit(1) ;

	case ADAPTER_AP2K2:
	case ADAPTER_AP2K4:
	case ADAPTER_AP2K8:
	case ADAPTER_AP2K16:
#ifdef ADAPTER_AP2K32
	case ADAPTER_AP2K32:
#endif
#ifdef XP_MEI
	case ADAPTER_AP2K4_MPI:
	case ADAPTER_AP2K8_MPI:
	case ADAPTER_AP2K16_MPI:
#endif
#ifdef XP_MEI
		if (port_type) {
			if ((channel < 1) || (port == -1)) {
				fprintf (stderr,
				         "dm_admin: port (-p) and channel (-c)"
				         " required with --porttype\n");
		    
				usage();
		    
				exit(1);
			}
		  

			set_port_type(fd);
		}

		if (rx_enable) {
			if ((channel < 1) || (port == -1)) {
				fprintf (stderr,
				         "dm_admin: port (-p) and channel (-c)"
				         " required with --rx_enable\n");
		    
				usage();
		    
				exit(1);
			}
			
			set_rx_enable(fd);
		}

		if (tx_enable) {
			if ((channel < 1) || (port == -1)) {
				fprintf (stderr,
				         "dm_admin: port (-p) and channel (-c)"
				         " required with --tx_enable\n");
		    
				usage();
		    
				exit(1);
			}

			set_tx_enable(fd);
		}

		/*
		 *  Some of the Xp settings need to be made persistent.
		 *  To do so, a helper application must be used.  After
		 *  testing whether a new persistent setting has been
		 *  requested, call a function which calls the helper
		 *  application.
		 */
		if (port_type || rx_enable || tx_enable)
		{
			set_xp_persistent(fd);
		}
#endif /* XP_MEI */

		break;

    /* Chaska */
    case ADAPTER_CHASKA80:
    case ADAPTER_CHASKA81:
    case ADAPTER_CHASKA82:
    case ADAPTER_CHASKA83:
    case ADAPTER_CHASKA84:
    case ADAPTER_CHASKA85:
    case ADAPTER_CHASKA40:
    case ADAPTER_CHASKA41:
    case ADAPTER_CHASKA42:
    case ADAPTER_CHASKA43:
    case ADAPTER_CHASKA44:
    case ADAPTER_CHASKA45:

#ifdef DEVELOPER

		/*
		 * Set special loop back mode
		 * (ds1_config isn't normally used with this adapter type)
		 */

		if (if_type == LOOP_HAUL)
		{
			/* Special loopback mode */

			ds1_config.config.if_type = LOOP_HAUL;
			ds1_config.config.leased_mask = leased_mask | 0x80000000;

			if (ioctl(fd, SET_DS1_CONFIG, &ds1_config) != 0) {
				dm_ioctl_fail() ;
			}
		}

		break;
#else

		fprintf (stderr,
		 "dm_admin: Invalid configuration option for this adapter type") ;

		close(fd) ;
		exit(1) ;

#endif

	/* Houston */
	case ADAPTER_HOU30:
	case ADAPTER_HOU60:
	case ADAPTER_HOU24:
	case ADAPTER_HOU48:
	case ADAPTER_HOU30cpci:
	case ADAPTER_HOU60cpci:
	case ADAPTER_HOU24cpci:
	case ADAPTER_HOU48cpci:

		validate_pri_settings(fd) ;

		/*
		 *	We now set the configuration.  This repeat loop
		 *	is here purely for the purposes of being able
		 *	to hammer on the FEP and its NVRAM for testing.
		 */
		while (repeat--) {

			if (config_ds1 || config_xri) {
				if (ioctl(fd, SET_DS1_CONFIG, &ds1_config) != 0) {
					dm_ioctl_fail() ;
				}
			}

			if (config_ds1x) {
				if (ioctl(fd, SET_DS1_EXT_CONFIG, &ds1_ext_config) != 0) {
					dm_ioctl_fail() ;
				}
			}
			if (config_ds1_r2) {
				if (ioctl(fd, SET_DS1_E1R2_CONFIG, &ds1_e1r2_config) != 0) {
					dm_ioctl_fail() ;
				}
			}

			if (config_dchanx) {
				if (ioctl(fd, SET_DCHAN_EXT_CONFIG, &dchan_config) != 0) {
					dm_ioctl_fail() ;
				}
			}

		}
		break;

	/* London */
    case ADAPTER_LONU:
    case ADAPTER_LONST:

		validate_bri_settings(fd) ;

		/*
		 *	We now set the configuration.  This repeat loop
		 *	is here purely for the purposes of being able
		 *	to hammer on the FEP and its NVRAM for testing.
		 */

		while (repeat--) {

			if (config_bri || config_xri) {
				if (ioctl(fd, SET_BRI_CONFIG, &bri_config) != 0) {
					dm_ioctl_fail() ;
				}
			}

			if (config_dchanx) {
				if (ioctl(fd, SET_DCHAN_EXT_CONFIG, &dchan_config) != 0) {
					dm_ioctl_fail() ;
				}
			}
		}
		break;
	}

	query_action(fd, &info.info) ;

	close(fd) ;

	return(0) ;
}
