
#
# Copyright 1999 by Digi International (www.digi.com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED; without even the 
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

#
#	$Id: dm_config.tcl,v 1.7 2000/11/10 15:15:01 jamesp Exp $
#
package provide dm_gui 1.0

#
#	Globals
#
#	$mgmt($bn)	open fd to /dev/dg/dgdm/mgmt0
#	$om()		associative mapping arrary for option menus
#	ioc_XXX($bn,$pn,$cn)	Ptr to get[set]_XXX_t structure
#	XXX($bn,$pn,$cn)	Ptr to XXX_t structure
#	

#
#       Create the notebook page for a single board
#
proc create_config_board {w bn} {
	global  mgmt          #(bn)
	global  nboards
	global	adapter_info
	global	port_count
	global	board_type
	global	board_family
	global	no_port

	#
	# Init no_port to an impossibly high value.  When $pn == $no_port,
	# it means port is not to be used by a proc callee
	#

	set no_port 1024

	#
	#	Open the management channel and get the adapter info
	#
	open_mgmt $bn
	if { $mgmt($bn) <= 0 } { return; }

	get_adapter_info $bn

	set port_count($bn) [adapter_info_t_port_count_get $adapter_info($bn) ]

	#
	# Add the board page
	#
	$w add board$bn -label "Adapter $bn" -underline 0

	#
	# Create the Adapter Info/Adapter/Ports/Channels notebook
	#
	set f [$w subwidget board$bn]
	tixNoteBook $f.config_nb -ipadx 6 -ipady 6
	$f.config_nb subwidget nbframe config -backpagecolor gray

	#
	# Create the pages of the notebook
	#
	create_adapter_info $f.config_nb $bn

	#
	#	Init the associative array for option menus
	#
	init_om $bn

	#
	# Create additional pages as appropriate per adapter type
	#

	if { ($board_family($bn) == "dfrp") || \
	 ($board_family($bn) == "dfdp") || \
	 ($board_family($bn) == "dfrb") || \
	 ($board_family($bn) == "apr") } {

		create_adapter $f.config_nb $bn
	}

	if { ($board_family($bn) == "dfrp") || \
	 ($board_family($bn) == "dfdp") || \
	 ($board_family($bn) == "dfrb") } {

		for {set pn 0} {$pn < $port_count($bn)} {incr pn} {
			create_config_port $f.config_nb $bn $pn
		}

	}

	if { ($board_family($bn) == "dfrp") || \
		($board_family($bn) == "dfdp") || \
		($board_family($bn) == "dfrb") } {
	    create_config_channels $f.config_nb $bn
	}
	    
	if { ($board_family($bn) == "ap2k") } {
		create_config_serial_ports $f.config_nb $bn
	}

	#
	#	Ready to accept changes
	#


	for {set pn 0} {$pn < $port_count($bn)} {incr pn} {
		global	port_change_${bn}_${pn}
		set port_change_${bn}_${pn} 1
	}


	pack $f.config_nb -expand yes -fill both -padx 5 -pady 5 -side top

	incr nboards
}

#
#	Create an info label
#
proc create_info_label {w bn label varname } {
	global	adapter_info

	set ai $adapter_info($bn)
	
	label $w.label_$varname -text "$label:"
	label $w.value_$varname -text \
		"[adapter_info_t_${varname}_get $ai]"
	grid $w.label_$varname $w.value_$varname
	grid $w.label_$varname -sticky e -padx 10
	grid $w.value_$varname -sticky w
}

#
#	Enter an item into the om (optionmap) associative array
#
#	create the following mappings:
#		lbl,...		number to label
#		num,...		label to number
#		lbllbl,...	label to label
#
proc om_item { bn struct member label valname } {
	global om
	upvar #0 $valname val
	
	set om($bn,num,$struct,$member,$label)	$val
	set om($bn,lbl,$struct,$member,$val)	$label
	set om($bn,lbllbl,$struct,$member,$label)	$label
}

#
#	Same as om_item without the upvar on valname
#
proc om_item2 { bn struct member label val } {
	global om

	set om($bn,num,$struct,$member,$label)	$val
	set om($bn,lbl,$struct,$member,$val)	$label
	set om($bn,lbllbl,$struct,$member,$label)	$label
}

#
#	Initialize the associative array with all of the
#	option names and values
#
proc init_om { bn } {
	global	board_family
	global board_type

	#
	# Init the om arrays based on adapter type
	#

	if { ($board_type($bn) == "dfrpt1") || ($board_type($bn) == "dfdpt1") || \
	 ($board_type($bn) == "dfrpt2") || ($board_type($bn) == "dfdpt2") } {

		#
		# Init for T1
		#

		init_t1_om $bn
	}

	if { ($board_type($bn) == "dfrpe1") || ($board_type($bn) == "dfdpe1") || \
	 ($board_type($bn) == "dfrpe2") || ($board_type($bn) == "dfdpe2") } {

		#
		# Init for E1
		#

		init_e1_om $bn
	}

	if { $board_family($bn) == "dfrb" } {

		#
		# Init for BRI
		#

		init_bri_om $bn
	}

	if { ($board_family($bn) == "dfrp") || \
	 ($board_family($bn) == "dfdp") || \
	 ($board_family($bn) == "dfrb") || \
	 ($board_family($bn) == "apr") } {

		#
		# Init modem nationality
		#

		init_mn_om $bn
	}

	if { ($board_family($bn) == "ap2k") } {

	    # 
	    # Serial Config
	    #

	    init_serial_config_om $bn
	}

}

#
#	Initialize the associative array with all of the
#	option names and values for T1 adapters
#
proc init_t1_om { bn } {
	global board_type

	set ds1 ds1_config

	#
	#	struct	member		label		value_variable
	#
	om_item $bn	$ds1	framing		ESF		FRAME_ESF
	om_item $bn	$ds1	framing		D4		FRAME_D4

	om_item $bn	$ds1	coding		B8ZS		LINE_CODE_B8ZS
	om_item $bn	$ds1	coding		AMI		LINE_CODE_AMI

	om_item $bn	$ds1	long_haul_lbo	"0.0 db"	LBO_00_0_DB
	om_item $bn	$ds1	long_haul_lbo	"7.5 db"	LBO_07_5_DB
	om_item $bn	$ds1	long_haul_lbo	"15.0 db"	LBO_15_0_DB
	om_item $bn	$ds1	long_haul_lbo	"22.5 db"	LBO_22_5_DB

	om_item $bn	$ds1	short_haul_len	"0 - 133"	SH_0_133
	om_item $bn	$ds1	short_haul_len	"134 - 266"	SH_133_266
	om_item $bn	$ds1	short_haul_len	"267 - 399"	SH_266_399
	om_item $bn	$ds1	short_haul_len	"400 - 533"	SH_399_533
	om_item $bn	$ds1	short_haul_len	"534 - 655"	SH_533_655

	om_item $bn	$ds1	if_type		"Short Haul"	SHORT_HAUL
	om_item $bn	$ds1	if_type		"Long Haul"	LONG_HAUL

	switch -glob -- [info hostname] {
		*.digi.com {
			om_item $bn	$ds1	if_type		"Loop Haul"	LOOP_HAUL
		}
	}

	om_item $bn	$ds1	eq_gain_limit	"Low"		EGL_LOW
	om_item $bn	$ds1	eq_gain_limit	"High"		EGL_HIGH

	if { ($board_type($bn) == "dfrpt2") || ($board_type($bn) == "dfdpt2") } {
		om_item $bn	$ds1	nfas_master_port	"None"		NFAS_NONE
		om_item $bn	$ds1	nfas_master_port	"Port 1"	NFAS_PORT0
		om_item $bn	$ds1	nfas_master_port	"Port 2"	NFAS_PORT1
	}

	om_item $bn	$ds1	signalling	"Loop Start"	CT1_LOOP_START
	om_item $bn	$ds1	signalling	"Wink Start"	CT1_WINK_START
	om_item $bn	$ds1	signalling	"Ground Start"	CT1_GROUND_START
	om_item $bn	$ds1	signalling	"Immed Start"	CT1_IMMED_START

	om_item $bn	$ds1	signalling	"PRI (none)"	PRI_NONE
	om_item $bn	$ds1	signalling	"PRI (NI2)"	PRI_NI2
	om_item $bn	$ds1	signalling	"PRI (5ESS)"	PRI_5ESS
	om_item $bn	$ds1	signalling	"PRI (DMS100)"	PRI_DMS100
	om_item $bn	$ds1	signalling	"PRI (INS-1500)"	PRI_JAPAN
	om_item $bn	$ds1	signalling	"PRI (MSA)"	PRI_MSA

	om_item $bn	$ds1	clocking	"Loop"		TIMING_LOOP

	om_item $bn	$ds1	jitter_atten	"Tx"		JITTER_TX
	om_item $bn	$ds1	jitter_atten	"Rx"		JITTER_RX
	om_item $bn	$ds1	jitter_atten	"Auto"		JITTER_AUTO
	om_item $bn	$ds1	jitter_atten	"None"		JITTER_NONE
}

#
#	Initialize the associative array with all of the
#	option names and values for E1 adapters
#
proc init_e1_om { bn } {
	global board_type

	set ds1 ds1_config

	#
	#	struct	member		label		value_variable
	#
	om_item $bn	$ds1	framing		"CEPT CRC4"	FRAME_CEPT_CRC4
	om_item $bn	$ds1	framing		"CEPT w/o CRC4"	FRAME_CEPT_WO_CRC4

	om_item $bn	$ds1	coding		HDB3		LINE_CODE_HDB3

	om_item $bn	$ds1	signalling	"PRI (none)"	PRI_NONE
	om_item $bn	$ds1	signalling	"PRI (ETSI)"	PRI_ETSI
	om_item $bn	$ds1	signalling	"PRI (MSA)"	PRI_MSA

	om_item $bn	$ds1	clocking	"Loop"		TIMING_LOOP

	om_item $bn	$ds1	jitter_atten	"Tx"		JITTER_TX
	om_item $bn	$ds1	jitter_atten	"Rx"		JITTER_RX
	om_item $bn	$ds1	jitter_atten	"Auto"		JITTER_AUTO
	om_item $bn	$ds1	jitter_atten	"None"		JITTER_NONE
}

#
#	Initialize the associative array with all of the
#	option names and values for bri adapter
#
proc init_bri_om { bn } {

	set bri bri_config

	#
	#	struct	member		label		value_variable
	#
	om_item $bn	$bri	switch_type		"None"		BRI_NONE
	om_item $bn	$bri	switch_type		"NI1"		BRI_NI1
	om_item $bn	$bri	switch_type		"5ESS"		BRI_5ESS
	om_item $bn	$bri	switch_type		"DMS100"		BRI_DMS100
	om_item $bn	$bri	switch_type		"ETSI"		BRI_ETSI
	om_item $bn	$bri	switch_type		"INS-64"		BRI_JAPAN
#	om_item $bn	$bri	auto_spid		"Disabled"		BRI_AUTOSPID_OFF
#	om_item $bn	$bri	auto_spid		"Enabled"		BRI_AUTOSPID_ON
	om_item $bn	$bri	msn		"Disabled"		BRI_MSN_OFF
	om_item $bn	$bri	msn		"Enabled"		BRI_MSN_ON
	om_item $bn	$bri	autoalert		"Disabled"		BRI_AUTOALERT_OFF
	om_item $bn	$bri	autoalert		"Enabled"		BRI_AUTOALERT_ON


}

# Initialize the associative array with the option pairs for serial
# configuration for the Acceleport Xp cards. 

proc init_serial_config_om { bn } {

    set sc serial_config
    set rs serial_rs485_config

		#  struct member		label		value_variable
    om_item	$bn	$sc	port_type	"RS232"		\
	    SERIAL_PORTTYPE_RS232
    om_item	$bn	$sc	port_type	"RS422"		\
	    SERIAL_PORTTYPE_RS422
    om_item	$bn	$sc	port_type	"RS485 2-wire, endpoint" \
	    SERIAL_PORTTYPE_RS485_2_E
    om_item	$bn	$sc	port_type	"RS485 2-wire, midpoint" \
	    SERIAL_PORTTYPE_RS485_2_M
    om_item	$bn	$sc	port_type	"RS485 4-wire, endpoint" \
	    SERIAL_PORTTYPE_RS485_4_E
    om_item	$bn	$sc	port_type	"RS485 4-wire, midpoint" \
	    SERIAL_PORTTYPE_RS485_4_M

    om_item	$bn	$sc	alt_pin		"On"	SERIAL_ALTPIN_ON
    om_item	$bn	$sc	alt_pin		"Off"	SERIAL_ALTPIN_OFF

    om_item	$bn	$rs	tx_enable	"Automatic"	\
	    RS485_TXENABLE_AUTO
    om_item	$bn	$rs	tx_enable	"Follows RTS"	\
	    RS485_TXENABLE_RTS

    om_item	$bn	$rs	rx_enable	"Automatic"	\
	    RS485_RXENABLE_AUTO
    om_item	$bn	$rs	rx_enable	"Always"	\
	    RS485_RXENABLE_ALWAYS

}



    

#
#	Initialize the associative array with all of the
#	option names and values for modem nationality
#
proc init_mn_om { bn } {
	global	adapter_info

	set ai $adapter_info($bn)

	#
	#	Set the valid modem Country codes for this adapter
	#

	set adapter_type [ adapter_info_t_board_type_get $ai ]

	for { set i 0 } { [ get_cc_id $adapter_type $i ] != 0 } { incr i +1 } {

		set country_id [ get_cc_id $adapter_type $i ]
		set country_name [ cc2name $country_id ]

		om_item2 $bn modem_nationality country_code $country_name $country_id

	}
}

#
#	Process all PRI fields affected by other PRI fields
#
proc cascade_pri_settings { bn pn } {
	global om
	global board_type
	global ds1_config
	upvar #0 ds1_config init

	set w .dm_config.boards_nb.nbframe.board$bn.config_nb.nbframe

	#
	#	Determine current setting for signalling on port 0
	#

	set val [ ds1_config_t_signalling_get $init($bn,0) ]
	set lblval0 ""

	if { [ info exists om($bn,lbl,ds1_config,signalling,$val) ] } {
		set lblval0 $om($bn,lbl,ds1_config,signalling,$val)
	}

	switch -exact -- $board_type($bn) {
		"dfdpt1" -
		"dfrpt1" {

			switch -glob -- $lblval0 {
				PRI* {
					set 24_state disabled
					set wink_state disabled
				}

				Wink* {
					set 24_state normal
					set wink_state normal
				}

				default {
					set 24_state normal
					set wink_state disabled
				}

			}

			if { [ winfo exists $w.port$pn.tbl.wink_digits ] } {
				$w.port$pn.tbl.wink_digits config -state $wink_state
			}

			if { [ winfo exists $w.channels.tbl.leased_mask0 ] } {
				[ $w.channels.tbl.leased_mask0 subwidget bit24 ] config -state $24_state
			}
		}

		"dfdpt2" -
		"dfrpt2" {

			#
			#	If init(bn,1) doesn't yet exist, return.
			#	We'll get called again for port 1
			#	

			if { ( [ info exists init($bn,1) ] != 1 ) } {
				return
			}

			#
			#	Determine current setting for signalling on port 1
			#

			set val [ ds1_config_t_signalling_get $init($bn,1) ]
			set lblval1 ""

			if { [ info exists om($bn,lbl,ds1_config,signalling,$val) ] } {
				set lblval1 $om($bn,lbl,ds1_config,signalling,$val)
			}

			switch -glob -- $lblval0 {

				PRI* {
					set pri0 1
					set wink_state0 disabled
				}

				Wink* {
					set pri0 0
					set wink_state0 normal
				}

				default {
					set pri0 0
					set wink_state0 disabled
				}
			}

			switch -glob -- $lblval1 {

				PRI* {
					set pri1 1
					set wink_state1 disabled
				}

				Wink* {
					set pri1 0
					set wink_state1 normal
				}

				default {
					set pri1 0
					set wink_state1 disabled
				}
			}

			#
			#	Determine current setting for nfas_master_port
			#	Be sure to clear the 0x80000000 bit, if set
			#

			set master [ ds1_config_t_nfas_master_port_get $ds1_config($bn,0) ]
			set master [ expr $master & 2147483647 ]

			set nfas_master_port ""

			if { [ info exists om($bn,lbl,ds1_config,nfas_master_port,$master) ] } {
				set nfas_master_port $om($bn,lbl,ds1_config,nfas_master_port,$master)
			}

			if { ($pri0 == 1) && ($pri1 == 1) } {

				set nfas_master_state normal

				if { $master == 0 } {
					set nfas_id_state disabled
				} else {
					set nfas_id_state normal
				}
			} else {
				set nfas_master_state disabled
				set nfas_id_state disabled
			}

			#
			# Set state for all NFAS widgets
			#

			if { [ winfo exists $w.channels.tbl.nfas_master_port ] } {
				$w.channels.tbl.nfas_master_port config -state $nfas_master_state
			}

			if { [ winfo exists $w.channels.tbl.nfas_interface_id0 ] } {
				$w.channels.tbl.nfas_interface_id0 config -state $nfas_id_state
			}

			if { [ winfo exists $w.channels.tbl.nfas_interface_id1 ] } {
				$w.channels.tbl.nfas_interface_id1 config -state $nfas_id_state
			}

			#
			# Determine time slot 24 leased_mask state
			#

			if { $nfas_master_state == "normal" } {

				switch -exact -- $nfas_master_port {
					"Port 1" {
						set 24_port0_state disabled
						set 24_port1_state normal
					}

					"Port 2" {
						set 24_port0_state normal
						set 24_port1_state disabled
					}

					"None" {
						set 24_port0_state disabled
						set 24_port1_state disabled
					}
				}

			} else {

				if { $pri0 == 1 } {
					set 24_port0_state disabled
				} else {
					set 24_port0_state normal
				}

				if { $pri1 == 1 } {
					set 24_port1_state disabled
				} else {
					set 24_port1_state normal
				}
			}

			#
			# Set time slot 24 leased mask widget state
			#

			if { [ winfo exists $w.channels.tbl.leased_mask0 ] } {
				[ $w.channels.tbl.leased_mask0 subwidget bit24 ] config -state $24_port0_state
			}

			if { [ winfo exists $w.channels.tbl.leased_mask1 ] } {
				[ $w.channels.tbl.leased_mask1 subwidget bit24 ] config -state $24_port1_state
			}

			#
			# Set wink_digit widget state
			#

			if { [ winfo exists $w.port0.tbl.wink_digits ] } {
				$w.port0.tbl.wink_digits config -state $wink_state0
			}

			if { [ winfo exists $w.port1.tbl.wink_digits ] } {
				$w.port1.tbl.wink_digits config -state $wink_state1
			}

		}

		# dfrpe1, dfrpe2, dfdpe1 and dfdpe2 are currently ignored since
		# signalling doesn't affect other fields

	}

}

#
#	Process all BRI fields affected by other BRI fields
#
proc cascade_bri_settings { w bn pn } {
	global om
	global MSN_MAX
	global bri_config
	upvar #0 bri_config init

	#
	#	Determine current setting for switch_type on this port
	#

	set val [ bri_config_t_switch_type_get $init($bn,$pn) ]
	set switchval ""

	if { [ info exists om($bn,lbl,bri_config,switch_type,$val) ] } {
		set switchval $om($bn,lbl,bri_config,switch_type,$val)
	}

	set dir_count [ bri_config_t_dir_count_get $init($bn,$pn) ]
	set dir_count [ expr $dir_count & 2147483647 ]

	switch -exact -- $switchval {

		"ETSI" {

			set sub_address_state normal
			set spid_state disabled
			set msn_state normal
			set val [ bri_config_t_msn_get $init($bn,$pn) ]
			set msnval ""

			if { [ info exists om($bn,lbl,bri_config,msn,$val) ] } {
				set msnval $om($bn,lbl,bri_config,msn,$val)
			}

			if { $msnval == "Enabled" } {
				set dir_count_state normal
				set dir_count_max $MSN_MAX
			} else {
				set dir_count_max 1
				set dir_count_state disabled
			}
		}

		"INS-64" {
			set dir_count_max 1
			set dir_count_state disabled
			set sub_address_state normal
			set spid_state disabled
			set msn_state disabled
		}

		"5ESS" -
		"None" -
		"NI1" -
		"DMS100" -
		default {
			set dir_count_max 2
			set dir_count_state normal
			set sub_address_state disabled
			set spid_state normal
			set msn_state disabled
		}
	}

	if { [ winfo exists $w.msn ] } {
		$w.msn config -state $msn_state
	}

	if { [ winfo exists $w.dir_count ] } {
		$w.dir_count config -state $dir_count_state
		$w.dir_count config -max $dir_count_max

		if { $dir_count > $dir_count_max } {
			set dir_count 1
			$w.dir_count config -value 1
		}
	}

	for { set i 1 } { $i < [ expr $MSN_MAX + 1 ] } { incr i +1 } {

		if { $i > $dir_count } {

			if { [ winfo exists $w.number$i ] } {
				$w.number$i config -state disabled
			}

			if { [ winfo exists $w.sub_address$i ] } {
				$w.sub_address$i config -state disabled
			}

			if { [ winfo exists $w.spid$i ] } {
				$w.spid$i config -state disabled
			}

		} else {

			if { [ winfo exists $w.number$i ] } {
				$w.number$i config -state normal
			}

			if { [ winfo exists $w.sub_address$i ] } {
				$w.sub_address$i config -state $sub_address_state
			}

			if { [ winfo exists $w.spid$i ] } {
				$w.spid$i config -state $spid_state
			}
		}
	}
}


#
#	Process all serial config fields affected by other serial config fields
#
proc cascade_serial_settings { w bn pn } {
	global om
	global serial_config

	#
	#	Determine current setting for switch_type on this port
	#

	set val [ serial_config_t_port_type_get $serial_config($bn,$pn) ]
	set switchval ""

	if { [ info exists om($bn,lbl,serial_config,port_type,$val) ] } {
		set switchval $om($bn,lbl,serial_config,port_type,$val)
	}

	switch -glob -- $switchval {
		RS485* {
			set tx_enable_state normal
			set rx_enable_state normal
		}

		default {
			set tx_enable_state disabled
			set rx_enable_state disabled
		}
	}

	if { [ winfo exists $w.tx_enable ] } {
		$w.tx_enable configure -state $tx_enable_state
	}

	if { [ winfo exists $w.rx_enable ] } {
		$w.rx_enable configure -state $rx_enable_state
	}
}


#
#	Handle change to a number field
#
proc set_new_number_field { w bn pn struct member label } {
	global	no_port

	upvar #0 $struct init

	#
	#	If we really want to set 0, the 0x80000000 bit must be set
	#

	if { $label == 0 } {
		set label [ expr 1 << 31 ]
	}

	if { $pn == $no_port } {
		global adapter_change_$bn

		#
		#	Don't do anything if we're not initialized
		#

		if { [ set adapter_change_$bn ] == 0 } {
			return
		}

		#
		#	Indicate that a change has happened for this adapter
		#

		set adapter_change_$bn 2

		${struct}_t_${member}_set $init($bn) $label
	} else {
		global port_change_${bn}_${pn}

		#
		#	Don't do anything if we're not initialized
		#

		if { [ set port_change_${bn}_${pn} ] == 0 } {
			return
		}

		#
		#	Indicate that a change has happened for this adapter/port
		#

		set port_change_${bn}_${pn} 2

		${struct}_t_${member}_set $init($bn,$pn) $label
	}
}

#
#	Process any unreported changes to directory number, spid or
#	sub-address.  Return Error if invalid length
#
proc check_for_errors { w } {
	global MSN_MAX
	global DN_LIMIT
	global nboards
	global board_family
	global port_count
	global bri_config
	upvar #0 bri_config init

	for {set bn 0} {$bn < $nboards} {incr bn} {
		if {$board_family($bn) == "dfrb"} {
			for {set pn 0} {$pn < $port_count($bn)} {incr pn} {
				for {set i 1} {$i < [ expr $MSN_MAX + 1 ]} {incr i} {
					global number_${bn}_${pn}_$i
					global sub_address_${bn}_${pn}_$i
					global spid_${bn}_${pn}_$i
					global port_change_${bn}_${pn}

					if { [ set number_${bn}_${pn}_$i ] != \
					[ bri_config_t_directory_var_get $init($bn,$pn) "number" \
					[ expr $i - 1] ] } {

						set len [ string length [ set number_${bn}_${pn}_$i ] ]

						if { $len > $DN_LIMIT } {

							notifier .number_error "Invalid Directory number" \
							 "number:\n\"[ set number_${bn}_${pn}_$i ]\"\nhas length $len, max length is $DN_LIMIT"

							set number_${bn}_${pn}_$i [ bri_config_t_directory_var_get $init($bn,$pn) "number" [ expr $i - 1] ]

							return "Error"
						}

						#
						#	Indicate that a change has happened for this
						#	adapter/port
						#

						set port_change_${bn}_${pn} 2

						bri_config_t_directory_var_set $init($bn,$pn) "number" [ expr $i - 1 ] [ set number_${bn}_${pn}_$i ]

					}

					if { [ set sub_address_${bn}_${pn}_$i ] != \
					[ bri_config_t_directory_var_get $init($bn,$pn) "sub_address" \
					[ expr $i - 1] ] } {

						set len [ string length [ set sub_address_${bn}_${pn}_$i ] ]

						if { $len > $DN_LIMIT } {

							notifier .sub_address_error "Invalid Directory sub-address" \
							 "sub-address:\n\"[ set sub_address_${bn}_${pn}_$i ]\"\nhas length $len, max length is $DN_LIMIT"

							set sub_address_${bn}_${pn}_$i [ bri_config_t_directory_var_get $init($bn,$pn) "sub_address" [ expr $i - 1] ]

							return "Error"
						}

						#
						#	Indicate that a change has happened for this
						#	adapter/port
						#

						set port_change_${bn}_${pn} 2

						bri_config_t_directory_var_set $init($bn,$pn) "sub_address" [ expr $i - 1 ] [ set sub_address_${bn}_${pn}_$i ]

					}

					if { [ set spid_${bn}_${pn}_$i ] != \
					[ bri_config_t_directory_var_get $init($bn,$pn) "spid" \
					[ expr $i - 1] ] } {

						set len [ string length [ set spid_${bn}_${pn}_$i ] ]

						if { $len > $DN_LIMIT } {

							notifier .spid_error "Invalid Directory spid" \
							 "spid:\n\"[ set spid_${bn}_${pn}_$i ]\"\nhas length $len, max length is $DN_LIMIT"

							set spid_${bn}_${pn}_$i [ bri_config_t_directory_var_get $init($bn,$pn) "spid" [ expr $i - 1] ]

							return "Error"
						}

						#
						#	Indicate that a change has happened for this
						#	adapter/port
						#

						set port_change_${bn}_${pn} 2

						bri_config_t_directory_var_set $init($bn,$pn) "spid" [ expr $i - 1 ] [ set spid_${bn}_${pn}_$i ]

					}
				}
			}
		}
	}

	return "Good"
}

#
#	Handle change to a bri directory number
#
proc update_directory_number { w bn pn i } {
	global	number_${bn}_${pn}_$i
	global	DN_LIMIT
	global	bri_config
	global port_change_${bn}_${pn}

	upvar #0 bri_config init

	#
	#	Don't do anything if we're not initialized
	#

	if { [ set port_change_${bn}_${pn} ] == 0 } {
		return
	}

	set len [ string length [ set number_${bn}_${pn}_$i ] ]

	if { $len > $DN_LIMIT } {

		notifier .number_error "Invalid Directory number" \
		 "number:\n\"[ set number_${bn}_${pn}_$i ]\"\nhas length $len, max length is $DN_LIMIT"

		set number_${bn}_${pn}_$i [ bri_config_t_directory_var_get $init($bn,$pn) "number" [ expr $i - 1] ]

		return
	}

	if { [ set number_${bn}_${pn}_$i ] == \
	[ bri_config_t_directory_var_get $init($bn,$pn) "number" \
	[ expr $i - 1] ] } {
		return
	}

	#
	#	Indicate that a change has happened for this adapter/port
	#

	set port_change_${bn}_${pn} 2

	bri_config_t_directory_var_set $init($bn,$pn) "number" [ expr $i - 1 ] [ set number_${bn}_${pn}_$i ]

}

#
#	Handle change to a bri directory sub_address
#
proc update_directory_sub_address { w bn pn i } {
	global	sub_address_${bn}_${pn}_$i
	global	DN_LIMIT
	global	bri_config
	global	port_change_${bn}_${pn}
	upvar #0 bri_config init

	#
	#	Don't do anything if we're not initialized
	#

	if { [ set port_change_${bn}_${pn} ] == 0 } {
		return
	}

	set len [ string length [ set sub_address_${bn}_${pn}_$i ] ]

	if { $len > $DN_LIMIT } {

		notifier .sub_address_error "Invalid Directory sub_address" \
		 "sub_address:\n\"[ set sub_address_${bn}_${pn}_$i ]\"\nhas length $len, max length is $DN_LIMIT"

		set sub_address_${bn}_${pn}_$i [ bri_config_t_directory_var_get $init($bn,$pn) "sub_address" [ expr $i - 1] ]

		return
	}

	if { [ set sub_address_${bn}_${pn}_$i ] == \
	[ bri_config_t_directory_var_get $init($bn,$pn) "sub_address" \
	[ expr $i - 1] ] } {
		return
	}

	#
	#	Indicate that a change has happened for this adapter/port
	#

	set port_change_${bn}_${pn} 2

	bri_config_t_directory_var_set $init($bn,$pn) "sub_address" [ expr $i - 1 ] [ set sub_address_${bn}_${pn}_$i ]

}

#
#	Handle change to a bri directory spid
#
proc update_directory_spid { w bn pn i } {
	global	spid_${bn}_${pn}_$i
	global	DN_LIMIT
	global	bri_config
	global	port_change_${bn}_${pn}
	upvar #0 bri_config init

	#
	#	Don't do anything if we're not initialized
	#

	if { [ set port_change_${bn}_${pn} ] == 0 } {
		return
	}

	set len [ string length [ set spid_${bn}_${pn}_$i ] ]

	if { $len > $DN_LIMIT } {

		notifier .spid_error "Invalid Directory spid" \
		 "spid:\n\"[ set spid_${bn}_${pn}_$i ]\"\nhas length $len, max length is $DN_LIMIT"

		set spid_${bn}_${pn}_$i [ bri_config_t_directory_var_get $init($bn,$pn) "spid" [ expr $i - 1] ]

		return
	}

	if { [ set spid_${bn}_${pn}_$i ] == \
	[ bri_config_t_directory_var_get $init($bn,$pn) "spid" \
	[ expr $i - 1] ] } {
		return
	}

	#
	#	Indicate that a change has happened for this adapter/port
	#

	set port_change_${bn}_${pn} 2

	bri_config_t_directory_var_set $init($bn,$pn) "spid" [ expr $i - 1 ] [ set spid_${bn}_${pn}_$i ]

}

#
#	Handle change to leased_mask
#
proc set_new_leased_mask { w bn pn struct bit } {
	global	adapter_info
	global	port_change_${bn}_${pn}
	upvar	#0 $struct init

	#
	#	Don't do anything if we're not initialized
	#

	if { [ set port_change_${bn}_${pn} ] == 0 } {
		return
	}

	#
	#	Indicate that a change has happened for this adapter/port
	#

	set port_change_${bn}_${pn} 2

	set ai $adapter_info($bn)

	set cpp [adapter_info_t_channels_per_port_get $ai]

	#
	#	Get the current leased mask for this port
	#	Clear the 0x80000000 bit
	#	XOR with the changed bit
	#	Set the 0x80000000 bit if it becomes zero
	#	Set the new value in the $struct structure
	#

	set leased_mask [ ${struct}_t_leased_mask_get $init($bn,$pn) ]
	set leased_mask [ expr $leased_mask & 2147483647 ]

	set leased_mask [ expr $leased_mask ^ [ expr 1 << ( $bit - 1 ) ] ]

	if { $leased_mask == 0 } {
		set leased_mask [ expr $leased_mask | 2147483648 ]
	}

	${struct}_t_leased_mask_set $init($bn,$pn) $leased_mask

	set value ""

	for {set i $cpp} { $i > 0 } {incr i -1} {

		if { ( $leased_mask & [ expr 1 << ( $i - 1 ) ] ) != 0 } {
			set value "$value bit$i"
		}

	}

	#
	#	Now update the settings for all of the bit<i> widgets
	#

	$w config -value $value

}

#
#	Handle a new nfas_master_port
#
proc set_new_nfas_master_port { w bn pn struct member label } {
	global	port_change_${bn}_${pn}
	upvar #0 $struct init

	#
	#	Don't do anything if we're not initialized
	#

	if { [ set port_change_${bn}_${pn} ] == 0 } {
		return
	}

	#
	#	Indicate that a change has happened for this adapter/port
	#

	set port_change_${bn}_${pn} 2

	set_new_option_value  $w $bn 0 $struct $member $label

	set_new_option_value  $w $bn 1 $struct $member $label

	cascade_pri_settings $bn $pn

}

#
#	Handle a new signalling value
#
proc set_new_signalling { w bn pn struct member label } {
	global om
	global	port_change_${bn}_${pn}
	upvar #0 $struct init

	#
	#	Don't do anything if we're not initialized
	#

	if { [ set port_change_${bn}_${pn} ] == 0 } {
		return
	}

	#
	#	Indicate that a change has happened for this adapter/port
	#

	set port_change_${bn}_${pn} 2

	set_new_option_value $w $bn $pn $struct $member $label

	#
	# Update widgets affected by signalling
	#

	cascade_pri_settings $bn $pn
}

#
#	Handle a new if_type
#
proc set_new_if_type { w bn pn struct member label } {
	global om
	global	port_change_${bn}_${pn}
	upvar #0 $struct init

	#
	#	Don't do anything if we're not initialized
	#

	if { [ set port_change_${bn}_${pn} ] == 0 } {
		return
	}

	#
	#	Indicate that a change has happened for this adapter/port
	#

	set port_change_${bn}_${pn} 2

	set_new_option_value $w $bn $pn $struct $member $label

	#
	#	Determine current setting for if_type
	#

	set val [ ds1_config_t_if_type_get $init($bn,$pn) ]
	set lblval ""

	if { [ info exists om($bn,lbl,ds1_config,if_type,$val) ] } {
		set lblval $om($bn,lbl,ds1_config,if_type,$val)
	}

	switch -glob -- $lblval {

		Short* {

			if { [ winfo exists $w.long_haul_lbo ] } {
				$w.long_haul_lbo configure -state disabled
			}

			if { [ winfo exists $w.short_haul_len ] } {
				$w.short_haul_len configure -state normal
			}

			if { [ winfo exists $w.eq_gain_limit ] } {
				$w.eq_gain_limit configure -state disabled
			}

		}

		Long* {

			if { [ winfo exists $w.long_haul_lbo ] } {
				$w.long_haul_lbo configure -state normal
			}

			if { [ winfo exists $w.short_haul_len ] } {
				$w.short_haul_len configure -state disabled
			}

			if { [ winfo exists $w.eq_gain_limit ] } {
				$w.eq_gain_limit configure -state normal
			}

		}

		Loop* {

			if { [ winfo exists $w.long_haul_lbo ] } {
				$w.long_haul_lbo configure -state disabled
			}

			if { [ winfo exists $w.short_haul_len ] } {
				$w.short_haul_len configure -state disabled
			}

			if { [ winfo exists $w.eq_gain_limit ] } {
				$w.eq_gain_limit configure -state disabled
			}

		}

	}
}

#
#	Handle a new switch_type value
#
proc set_new_switch_type { w bn pn struct member label } {
	global om
	global MSN_MAX
	global	port_change_${bn}_${pn}
	upvar #0 $struct init

	#
	#	Don't do anything if we're not initialized
	#

	if { [ set port_change_${bn}_${pn} ] == 0 } {
		return
	}

	#
	#	Indicate that a change has happened for this adapter/port
	#

	set port_change_${bn}_${pn} 2

	set_new_option_value $w $bn $pn $struct $member $label

	#
	#	When switch type changes, force dir_count to 1
	#

	if { [ winfo exists $w.dir_count ] } {
		$w.dir_count config -value 1
	}

	#
	#	When switch type changes, force msn to Disabled
	#

	if { [ winfo exists $w.msn ] } {
		$w.msn config -value Disabled

	}

	#
	#	Set things based on the new switch type
	#

	switch -exact -- $label {

		"ETSI" {

			set sub_address_state normal
			set spid_state disabled
			set msn_state normal
			set dir_count_state disabled
			set dir_count_max 1
		}

		"INS-64" {
			set dir_count_state disabled
			set dir_count_max 1
			set sub_address_state normal
			set spid_state disabled
			set msn_state disabled
		}

		"5ESS" -
		"None" -
		"NI1" -
		"DMS100" -
		default {
			set dir_count_state normal
			set dir_count_max 2
			set sub_address_state disabled
			set spid_state normal
			set msn_state disabled
		}
	}

	if { [ winfo exists $w.msn ] } {
		$w.msn config -state $msn_state
	}

	if { [ winfo exists $w.dir_count ] } {
		$w.dir_count config -max $dir_count_max
		$w.dir_count config -state $dir_count_state
	}

	#
	# Make the directory stuff match dir_count of 1
	#

	if { [ winfo exists $w.number1 ] } {
		$w.number1 config -state normal
	}

	if { [ winfo exists $w.sub_address1 ] } {
		$w.sub_address1 config -state $sub_address_state
	}

	if { [ winfo exists $w.spid1 ] } {
		$w.spid1 config -state $spid_state
	}

	for { set i 2 } { $i < [ expr $MSN_MAX + 1 ] } { incr i +1 } {

		if { [ winfo exists $w.number$i ] } {
			$w.number$i config -state disabled
		}

		if { [ winfo exists $w.sub_address$i ] } {
			$w.sub_address$i config -state disabled
		}

		if { [ winfo exists $w.spid$i ] } {
			$w.spid$i config -state disabled
		}
	}

}

#
#	Handle a new msn value
#
proc set_new_msn { w bn pn struct member label } {
	global om
	global MSN_MAX
	global	port_change_${bn}_${pn}
	upvar #0 $struct init

	#
	#	Don't do anything if we're not initialized
	#

	if { [ set port_change_${bn}_${pn} ] == 0 } {
		return
	}

	#
	#	Indicate that a change has happened for this adapter/port
	#

	set port_change_${bn}_${pn} 2

	set_new_option_value $w $bn $pn $struct $member $label

	#
	# Force directory count to 1
	#

	$w.dir_count config -value 1

	#
	# Set new state and max value for dir_count
	#

	if { $label == "Enabled" } {

		$w.dir_count config -max $MSN_MAX
		$w.dir_count config -state normal

	} else {

		$w.dir_count config -max 1
		$w.dir_count config -state disabled

	}
}

#
#	Handle a new serial port type value
#
proc set_new_port_type { w bn pn struct member label } {
	global om
	global	port_change_${bn}_${pn}
	upvar #0 $struct init

	#
	#	Don't do anything if we're not initialized
	#

	if { [ set port_change_${bn}_${pn} ] == 0 } {
		return
	}

	#
	#	Indicate that a change has happened for this adapter/port
	#

	set port_change_${bn}_${pn} 2

	set_new_option_value $w $bn $pn $struct $member $label

	cascade_serial_settings $w $bn $pn
}

#
#	Handle change to dir_count
#
proc set_new_dir_count { w bn pn struct member val } {
	global MSN_MAX
	global om
	global	port_change_${bn}_${pn}
	upvar #0 $struct init

	#
	#	Don't do anything if we're not initialized
	#

	if { [ set port_change_${bn}_${pn} ] == 0 } {
		return
	}

	#
	#	Indicate that a change has happened for this adapter/port
	#

	set port_change_${bn}_${pn} 2

	${struct}_t_${member}_set $init($bn,$pn) $val

	#
	#	Determine current setting for switch_type on this port
	#

	set tmpval [ bri_config_t_switch_type_get $init($bn,$pn) ]
	set switchval ""

	if { [ info exists om($bn,lbl,bri_config,switch_type,$tmpval) ] } {
		set switchval $om($bn,lbl,bri_config,switch_type,$tmpval)
	}

	switch -exact -- $switchval {

		"ETSI" -
		"INS-64" {
			set sub_address_state normal
			set spid_state disabled
		}

		"5ESS" -
		"None" -
		"NI1" -
		"DMS100" {
			set sub_address_state disabled
			set spid_state normal
		}

		default {
			puts "Unknown switch_type $switchval"
			set sub_address_state disabled
			set spid_state disabled
		}
	}

	for { set i 1 } { $i < [ expr $MSN_MAX + 1 ] } { incr i +1 } {

		if { $i > $val } {

			if { [ winfo exists $w.number$i ] } {
				$w.number$i config -state disabled
			}

			if { [ winfo exists $w.sub_address$i ] } {
				$w.sub_address$i config -state disabled
			}

			if { [ winfo exists $w.spid$i ] } {
				$w.spid$i config -state disabled
			}

		} else {

			if { [ winfo exists $w.number$i ] } {
				$w.number$i config -state normal
			}

			if { [ winfo exists $w.sub_address$i ] } {
				$w.sub_address$i config -state $sub_address_state
			}

			if { [ winfo exists $w.spid$i ] } {
				$w.spid$i config -state $spid_state
			}
		}
	}
}

#
#	Set a new option value into the ioctl structure
#
proc set_new_option_value { w bn pn struct member label } {
	global	no_port
	global om
	upvar #0 $struct init

	set num $om($bn,num,$struct,$member,$label)

	#
	#	If we really want to set 0, the 0x80000000 bit must be set
	#

	if { $num == 0 } {
		set num [ expr 1 << 31 ]
	}

	if { $pn == $no_port } {
		global adapter_change_$bn

		#
		#	Don't do anything if we're not initialized
		#

		if { [ set adapter_change_$bn ] == 0 } {
			return
		}

		#
		#	Indicate that a change has happened for this adapter
		#

		set adapter_change_$bn 2

		${struct}_t_${member}_set $init($bn) $num
	} else {
		global port_change_${bn}_${pn}

		#
		#	Don't do anything if we're not initialized
		#

		if { [ set port_change_${bn}_${pn} ] == 0 } {
			return
		}

		#
		#	Indicate that a change has happened for this adapter/port
		#

		set port_change_${bn}_${pn} 2

		${struct}_t_${member}_set $init($bn,$pn) $num
	}
}

#
#	Create a single option widget
#
proc create_option_widget {w bn pn struct member lwidth bwidth } {
	global no_port
	global om
	upvar #0 $struct init

	#
	#	Get the initial option menu value from the structure
	#

	if { $pn == $no_port } {
		set val [ ${struct}_t_${member}_get $init($bn) ]
	} else {
		set val [ ${struct}_t_${member}_get $init($bn,$pn) ]
	}

	set lblval ""
	if { [ info exists om($bn,lbl,$struct,$member,$val) ] } {
		set lblval $om($bn,lbl,$struct,$member,$val)
	}

	switch -exact -- $member {

		"if_type" {

			set cmd set_new_if_type
		}

		"signalling" {

			set cmd set_new_signalling
		}

		"nfas_master_port" {

			set cmd set_new_nfas_master_port
		}

		"switch_type" {

			set cmd set_new_switch_type
		}

		"msn" {

			set cmd set_new_msn
		}

		"port_type" {

			set cmd set_new_port_type
		}

		default {

			set cmd set_new_option_value
		}
	}

	#
	#	Create an option menu widget
	#
	tixOptionMenu $w.$member -label "$member: " \
		-value "$lblval" \
		-command "$cmd $w $bn $pn $struct $member" \
		-options "
			label.width $lwidth
			label.anchor e
			menubutton.width $bwidth
		"

	#
	#	Get the list of option values for this widget,
	#	sort it, and add each one to the menu.
	#
	set olist [lsort -dictionary [array names om $bn,lbllbl,$struct,$member,* ]]

	foreach option $olist {
		$w.$member add command $om($option)
	}

	#
	#	Set the initial value from the structure
	#
	$w.$member configure -value $lblval

	pack $w.$member -side top -anchor w
}

#
#       Create a single adapter page
#
proc create_adapter {w bn} {
	global	mgmt
	global	modem_nationality
	global	ioc_modem_nationality
	global	GET_MODEM_NATIONALITY
	global	board_type
	global	board_family
	global	adapter_info
	global	om
	global	no_port
	global	adapter_change_$bn

	#
	#	Show that we aren't initialized yet
	#

	set adapter_change_$bn 0

	#
	#	Add new notebook page
	#
	$w add adapter -label "Adapter" -underline 0

	set f [$w subwidget adapter]

	set ai $adapter_info($bn)


	#
	#	ioctl the mgmt channel and get the modem_nationality structure
	#
	set ioc_modem_nationality($bn) [new_getset_modem_nationality_t]

	set modem_nationality($bn) \
		[getset_modem_nationality_t_nation_get $ioc_modem_nationality($bn) ]

	ioctl $mgmt($bn) $GET_MODEM_NATIONALITY $ioc_modem_nationality($bn)

	frame $f.tbl

	set lwidth 15

	create_option_widget $f.tbl $bn $no_port \
		 modem_nationality country_code $lwidth 20

	pack $f.tbl

	#
	#	Ready to accept changes
	#

	set adapter_change_$bn 1
}

#
#       Create a single port page
#
proc create_config_port {w bn pn} {
	global	board_family
	global	adapter_info
	global	port_change_${bn}_${pn}

	#
	#	Show that we aren't initialized yet
	#

	set port_change_${bn}_${pn} 0

	#
	#	Add new notebook page
	#

	$w add port$pn -label "Port [ expr $pn + 1 ]" -underline 5

	set f [$w subwidget port$pn]

	set ai $adapter_info($bn)

	frame $f.tbl

	#
	#	Do adapter specific stuff
	#

	if { ($board_family($bn) == "dfrp") || ($board_family($bn) == "dfdp") } {
		create_config_pri_port $f.tbl $bn $pn
	}

	if { $board_family($bn) == "dfrb" } {
		create_config_bri_port $f.tbl $bn $pn
	}

	pack $f.tbl
}


#
#	Create Configuration for Serial Port Tree details
#	Based on current selection, determines what should be displayed
#	in the "right hand" pane of the frame.
#
proc create_config_sp_details {path} {
	global channel_pane_w
	global channel_pane_w_frame
	global channel_pane_path

	# Grab the board number from the path
	regexp -- "board(\[0-9]*).*" $path path bn

	# If path hasn't changed, no need to update
	if { $channel_pane_path($bn) == $path } { return; }

	# If the detail frame already exists, kill it.
	if { [string length $channel_pane_w_frame($bn)] } {
		destroy $channel_pane_w_frame($bn)
	}

	set w $channel_pane_w($bn)
	set f [frame $w.frame -bg lightgray]
	set channel_pane_w_frame($bn) $f

	set channel_pane_path($bn) $path

	switch -regexp -- $path {
		"board\[0-9]*.port\[0-9]*" {
			regexp -- "board(\[0-9]*).port(\[0-9]*)" \
				$path path bn pn
			create_config_ap2k_port $f $bn [expr $pn - 1]
		}
		"board\[0-9]*" {
			regexp -- "board(\[0-9]*)" \
				$path path bn
			create_adapter_info_detail $f $bn
		}
	}

	pack $f -expand yes -fill both
}

#
#	Create Configuration for Serial Port Tree
#	fills in the structure of the initial tree on the left.
#	Does not fill in the tree on the right.
#
proc create_config_sp_tree {l_pane r_pane bn} {
	global dmlib
	global adapter_cn
	global port_count

	# Obtain configuration settings for all ports
	for { set port 0 } { $port < $port_count($bn) } { incr port } {
		get_serial_config $bn $port
		get_serial_rs485_config $bn $port
	}

	tixTree $l_pane.tree -browsecmd create_config_sp_details -options {
		hlist.itemType		imagetext
		hlist.drawBranch	True
		hlist.indent		14
		hlist.selectmode	single
	}

	pack $l_pane.tree -expand yes -fill both

	# Build icons for the tree
	image create photo adapter_icon -file \
		[file join $dmlib "tree-adapter.gif"]
	image create photo port_icon -file \
		[file join $dmlib "tree-port.gif"]

	# Start filling in the tree
	set ports_tree [$l_pane.tree subwidget hlist]
	$ports_tree add board${bn} -image adapter_icon -text $adapter_cn

	# Add all of the ports
	for { set port 1 } { $port <= $port_count($bn) } { incr port } {
		$ports_tree add board${bn}.port${port} \
			-image port_icon -text "Port $port"
	}

	$l_pane.tree autosetmode

	# Select the adapter view by default
	$ports_tree selection set board${bn}

	# Fill in rest of tree
	create_config_sp_details "board${bn}"
}

#
#       Create a page for all serial ports on an adapter
#       (At this point, an Xp only function... we make
#        the [possibly eventually incorrect] assumption
#        that for pure serial adapters, there is always
#        one channel per port, so we can just refer to
#        these by port number.)
#
proc create_config_serial_ports {w bn} {
	global	port_count

	global	channel_pane_w
	global	channel_pane_w_frame
	global	channel_pane_path

	#
	#	Show that we aren't initialized yet
	#
	for {set pn 0} {$pn < $port_count($bn)} {incr pn} {
		global	port_change_${bn}_${pn}
		set port_change_${bn}_${pn} 0
	}

	#
	#	Add new notebook page
	#

	$w add serial_ports -label "Serial Ports" -underline 0

	set f [$w subwidget serial_ports]

	frame $f.tbl

	#
	#	Create two panes for the "serial port" view.  The left
	#	pane will have ye olde scrolling "tree" listing of ports.
	#	The right pane will have the current configuration
	#	information for each port.
	#

	tixPanedWindow $f.tbl.p -orient horizontal

	$f.tbl.p add pane1 -size 240 -max 350
	$f.tbl.p add pane2 -size 350 -max 400

	set p1 [$f.tbl.p subwidget pane1]
	set p2 [$f.tbl.p subwidget pane2]

	#
	#	Initialize the variables used to maintain the
	#	tree, including:
	#		channel_pane_w    <-- widget for the pane
	#		channel_pane_w_frame <-- frame in this pane
	#		channel_pane_path <-- tree path last selected
	#
	set channel_pane_w($bn) $p2
	set channel_pane_w_frame($bn) ""
	set channel_pane_path($bn) ""

	# Setup the tree and associated update commands
	create_config_sp_tree $p1 $p2 $bn

	# Pack out the panes
	pack $f.tbl.p -expand yes -fill both

	# Pack the whole lot out
	pack $f.tbl -anchor w -expand yes -fill both
}

#
#       Create a single PRI port page
#
proc create_config_pri_port {w bn pn} {
	global	mgmt
	global	ds1_config
	global	ioc_ds1_config
	global	GET_DS1_CONFIG
	global	adapter_info
	global	om
	global	board_type

	#
	#	ioctl the mgmt channel and get the ds1_config structure
	#
	set ioc_ds1_config($bn,$pn) [new_getset_ds1_config_t]

	set ds1_config($bn,$pn) \
		[getset_ds1_config_t_config_get $ioc_ds1_config($bn,$pn) ]

	getset_ds1_config_t_port_set $ioc_ds1_config($bn,$pn) [expr $pn + 1]
	ioctl $mgmt($bn) $GET_DS1_CONFIG $ioc_ds1_config($bn,$pn)

	set lwidth 15

	create_option_widget $w $bn $pn \
		 ds1_config signalling $lwidth 20
	create_option_widget $w $bn $pn \
		 ds1_config framing $lwidth 20
	create_option_widget $w $bn $pn \
		 ds1_config coding $lwidth 20
	create_option_widget $w $bn $pn \
		 ds1_config clocking $lwidth 20
	create_option_widget $w $bn $pn \
		 ds1_config jitter_atten $lwidth 20

	if { ($board_type($bn) == "dfrpt1") || ($board_type($bn) == "dfrpt2") || \
	 ($board_type($bn) == "dfdpt1") || ($board_type($bn) == "dfdpt2") } {
		create_option_widget $w $bn $pn \
			 ds1_config eq_gain_limit $lwidth 20
		create_option_widget $w $bn $pn \
			 ds1_config if_type $lwidth 20
		create_option_widget $w $bn $pn \
			 ds1_config long_haul_lbo $lwidth 20
		create_option_widget $w $bn $pn \
			 ds1_config short_haul_len $lwidth 20

		#
		#	Determine current setting for if_type
		#	Enable/Disable haul choices as appropriate
		#

		set val [ ds1_config_t_if_type_get $ds1_config($bn,$pn) ]
		set lblval ""

		if { [ info exists om($bn,lbl,ds1_config,if_type,$val) ] } {
			set lblval $om($bn,lbl,ds1_config,if_type,$val)
		}

		switch -glob -- $lblval {

			Short* {

				$w.long_haul_lbo configure -state disabled
				$w.short_haul_len configure -state normal

			}

			Long* {

				$w.long_haul_lbo configure -state normal
				$w.short_haul_len configure -state disabled

			}

			Loop* {

				$w.long_haul_lbo configure -state disabled
				$w.short_haul_len configure -state disabled

			}

		}

		set val [ ds1_config_t_wink_digits_get $ds1_config($bn,$pn) ]

		tixControl $w.wink_digits -label "wink_digits: " \
			-disabledforeground darkgray \
			-integer true -min 1 -max 20 \
			-command "set_new_number_field $w $bn $pn ds1_config wink_digits" \
			-value $val \
			-options "
				label.width $lwidth
				label.anchor e
				entry.width 22
			"
		pack $w.wink_digits -side top -anchor w
	}
}


#
#	Create a single Ap2k port page
#
proc create_config_ap2k_port {w bn pn} {
    global board_type

    set lwidth 12

    if [string match *_mpi $board_type($bn)] {
	create_option_widget $w $bn $pn serial_config port_type $lwidth 23
    }
    create_option_widget $w $bn $pn serial_config alt_pin $lwidth 23

    # If an MPI board.
    if [string match *_mpi $board_type($bn)] {
	create_option_widget $w $bn $pn serial_rs485_config tx_enable \
		$lwidth 23
	create_option_widget $w $bn $pn serial_rs485_config rx_enable \
		$lwidth 23
    }

    cascade_serial_settings $w $bn $pn
}

#
#       Create a single BRI port page
#
proc create_config_bri_port {w bn pn} {
	global	mgmt
	global	bri_config
	global	ioc_bri_config
	global	GET_BRI_CONFIG
	global	MSN_MAX
	global	adapter_info
	global	om
	global	old_top_${bn}_${pn}

	for {set i 1} {$i < 9} {incr i} {
		global	number_${bn}_${pn}_${i}
		global	sub_address_${bn}_${pn}_${i}
		global	spid_${bn}_${pn}_${i}
	}

	#
	#	ioctl the mgmt channel and get the bri_config structure
	#
	set ioc_bri_config($bn,$pn) [new_getset_bri_config_t]

	set bri_config($bn,$pn) \
		[getset_bri_config_t_config_get $ioc_bri_config($bn,$pn) ]

	getset_bri_config_t_port_set $ioc_bri_config($bn,$pn) [expr $pn + 1]
	ioctl $mgmt($bn) $GET_BRI_CONFIG $ioc_bri_config($bn,$pn)

	scrollbar $w.yscroll -orient vertical -command "scroll_bri_port $w $bn $pn"
	pack $w.yscroll -side right -fill y

	$w.yscroll set 28 7 1 7

	set old_top_${bn}_${pn} 1

	set lwidth 27
	set ewidth 27
	set bwidth 5

	create_option_widget $w $bn $pn \
		 bri_config switch_type $lwidth 23

	#
	#	Autospid not yet implemented
	#
#	create_option_widget $w $bn $pn \
#		 bri_config auto_spid $lwidth 23

	create_option_widget $w $bn $pn \
		 bri_config autoalert $lwidth 23

	set dir_count [ bri_config_t_dir_count_get $bri_config($bn,$pn) ]
	set dir_count [ expr $dir_count & 2147483647 ]

	tixControl $w.dir_count -label "Number of Directories: " \
		-disabledforeground darkgray \
		-integer true -min 1 -max $MSN_MAX \
		-borderwidth $bwidth \
		-command "set_new_dir_count $w $bn $pn bri_config dir_count" \
		-value $dir_count \
		-options "
			label.width $lwidth
			label.anchor e
			entry.width 23
		"
	pack $w.dir_count -side top -anchor w

	create_option_widget $w $bn $pn \
		 bri_config msn $lwidth 23

	for { set i 1 } { $i < [ expr $MSN_MAX + 1 ] } { incr i +1 } {
		set number_${bn}_${pn}_$i [ bri_config_t_directory_var_get $bri_config($bn,$pn) "number" [ expr $i - 1 ] ]

		tixLabelEntry $w.number$i -label "Directory Number $i:" \
			-disabledforeground darkgray \
			-borderwidth $bwidth \
			-options "
				label.width $lwidth
				label.anchor e
				entry.width $ewidth
			"

		set esw [ $w.number$i subwidget entry ]
		bind $esw <FocusOut> " update_directory_number $esw $bn $pn $i "
		$esw config -textvariable number_${bn}_${pn}_$i

		if { $i == 1} {
			pack $w.number$i -side top -anchor w
		}

		set sub_address_${bn}_${pn}_$i [ bri_config_t_directory_var_get $bri_config($bn,$pn) "sub_address" [ expr $i - 1 ] ]

		tixLabelEntry $w.sub_address$i -label "Sub-Address $i:" \
			-disabledforeground darkgray \
			-borderwidth $bwidth \
			-options "
				label.width $lwidth
				label.anchor e
				entry.width $ewidth
			"

		set esw [ $w.sub_address$i subwidget entry ]
		bind $esw <FocusOut> " update_directory_sub_address $esw $bn $pn $i "
		$esw config -textvariable sub_address_${bn}_${pn}_$i

		if { $i == 1} {
			pack $w.sub_address$i -side top -anchor w
		}

		set spid_${bn}_${pn}_$i [ bri_config_t_directory_var_get $bri_config($bn,$pn) "spid" [ expr $i - 1 ] ]

		tixLabelEntry $w.spid$i -label "Spid $i:" \
			-disabledforeground darkgray \
			-borderwidth $bwidth \
			-options "
				label.width $lwidth
				label.anchor e
				entry.width $ewidth
			"

		set esw [ $w.spid$i subwidget entry ]
		bind $esw <FocusOut> " update_directory_spid $esw $bn $pn $i "
		$esw config -textvariable spid_${bn}_${pn}_$i

		if { $i == 1} {
			pack $w.spid$i -side top -anchor w
		}
	}

	cascade_bri_settings $w $bn $pn

}

#
#       Scroll the BRI port page
#
proc scroll_bri_port {w bn pn top} {
	global old_top_${bn}_${pn}
	global MSN_MAX

	if { $top < 1 } {
		set top 1
	}

	if { $top > 22 } {
		set top 22
	}

	set old_top [ set old_top_${bn}_${pn} ]
	set old_bottom [ expr $old_top + 6 ]

	#
	#	First adjust the scrollbar to its new position
	#

	$w.yscroll set 28 7 $top [ expr $top + 6 ]

	#
	#	Now tell the packer to forget everything
	#

	if { $old_top == 1 } {
		pack forget $w.switch_type
	}

	if { $old_top <= 2 } {
		pack forget $w.autoalert
	}

	if { $old_top <= 3 } {
		pack forget $w.dir_count
	}

	if { $old_top <= 4 } {
		pack forget $w.msn
	}

	for { set i 1 } { $i < [ expr $MSN_MAX + 1 ] } { incr i +1 } {

		set top_test [ expr 5 + ($i - 1) * 3 ]

		if { ($old_top <= $top_test) && ($old_bottom >= $top_test) } {
			pack forget $w.number$i
		}

		incr top_test +1

		if { ($old_top <= $top_test) && ($old_bottom >= $top_test) } {
			pack forget $w.sub_address$i
		}

		incr top_test +1

		if { ($old_top <= $top_test) && ($old_bottom >= $top_test) } {
			pack forget $w.spid$i
		}
	}

	if { $top == 1 } {
		pack $w.switch_type -side top -anchor w
	}

	if { $top <= 2 } {
		pack $w.autoalert -side top -anchor w
	}

	if { $top <= 3 } {
		pack $w.dir_count -side top -anchor w
	}

	if { $top <= 4 } {
		pack $w.msn -side top -anchor w
	}

	set new_bottom [ expr $top + 6 ]

	for { set i 1 } { $i < [ expr $MSN_MAX + 1 ] } { incr i +1 } {

		set top_test [ expr 5 + ($i - 1) * 3 ]

		if { ($top <= $top_test) && ($new_bottom >= $top_test) } {
			pack $w.number$i -side top -anchor w
		}

		incr top_test +1

		if { ($top <= $top_test) && ($new_bottom >= $top_test) } {
			pack $w.sub_address$i -side top -anchor w
		}

		incr top_test +1

		if { ($top <= $top_test) && ($new_bottom >= $top_test) } {
			pack $w.spid$i -side top -anchor w
		}
	}

	set old_top_${bn}_${pn} $top

}

#
#       Create the channels page for a single board
#
proc create_config_channels {w bn} {
	global	board_type
	global	port_count

	$w add channels -label "Channels" -underline 0

	set f [$w subwidget channels]

	set lwidth 30

	frame $f.tbl

	#
	# Fill in the channel page based on adapter type
	#

	switch -exact -- $board_type($bn) {

		"dfdpt1" -
		"dfrpt1" {
			config_dfrpt1_channels $f.tbl $bn
		}

		"dfdpt2" -
		"dfrpt2" {
			config_dfrpt2_channels $f.tbl $bn
		}

		"dfdpe1" -
		"dfrpe1" {
			config_dfrpe1_channels $f.tbl $bn
		}

		"dfdpe2" -
		"dfrpe2" {
			config_dfrpe2_channels $f.tbl $bn
		}

		"dfrb4u" {
			config_dfrb_channels $f.tbl $bn
		}

		"dfrb4st" {
			config_dfrb_channels $f.tbl $bn
		}
	}

	pack $f.tbl

}

#
# Fill the channels page for a dfrpt1/dfdpt1 board
#
proc config_dfrpt1_channels {w bn} {

	global	adapter_info
	global	ds1_config
	global	om
	upvar #0 ds1_config init

	set lwidth 30

	set ai $adapter_info($bn)

	set cpp [adapter_info_t_channels_per_port_get $ai]

	tixSelect $w.leased_mask0 -label "Port 1 Time Slot leased_mask: " \
		-allowzero true -radio false \
		-options "
			label.width $lwidth
			label.anchor e
		"
	pack $w.leased_mask0 -side top -anchor w

	#
	#	Get the current leased mask
	#	Clear the 0x80000000 bit
	#

	set leased_mask0 [ ds1_config_t_leased_mask_get $ds1_config($bn,0) ]
	set leased_mask0 [ expr $leased_mask0 & 2147483647 ]

	set value ""

	for {set i $cpp} { $i > 0 } {incr i -1} {

		if { ( $leased_mask0 & [ expr 1 << ( $i - 1 ) ] ) != 0 } {
			set value "$value bit$i"
		}

		$w.leased_mask0 add bit$i -text "$i" \
		-command "set_new_leased_mask $w.leased_mask0 $bn 0 ds1_config $i " \
		-font "-*-Courier-bold-r-normal-*-8-*-*-*-*-*-*-*" \
		-width 2 -padx 0
	}

	#
	#	Now initialize the settings for all of the bit<i> widgets
	#

	$w.leased_mask0 config -value $value

	#
	# Set widget state for all widgets affected by signalling
	#

	cascade_pri_settings $bn 0
}

#
# Fill the channels page for a dfrpt2/dfdpt2 board
#
proc config_dfrpt2_channels {w bn} {

	global	adapter_info
	global	ds1_config
	global	om
	upvar #0 ds1_config init

	set lwidth 30

	set ai $adapter_info($bn)

	set cpp [adapter_info_t_channels_per_port_get $ai]

	create_option_widget $w $bn 0 \
		 ds1_config nfas_master_port $lwidth 20

	tixSelect $w.leased_mask0 -label "Port 1 Time Slot leased_mask: " \
		-allowzero true -radio false \
		-options "
			label.width $lwidth
			label.anchor e
		"
	pack $w.leased_mask0 -side top -anchor w

	#
	#	Get the current leased mask for port 0
	#	Clear the 0x80000000 bit
	#

	set leased_mask0 [ ds1_config_t_leased_mask_get $ds1_config($bn,0) ]
	set leased_mask0 [ expr $leased_mask0 & 2147483647 ]

	set value ""

	for {set i $cpp} { $i > 0 } {incr i -1} {

		if { ( $leased_mask0 & [ expr 1 << ( $i - 1 ) ] ) != 0 } {
			set value "$value bit$i"
		}

		$w.leased_mask0 add bit$i -text "$i" \
		-command "set_new_leased_mask $w.leased_mask0 $bn 0 ds1_config $i" \
		-font "-*-Courier-bold-r-normal-*-8-*-*-*-*-*-*-*" \
		-width 2 -padx 0
	}

	#
	#	Now initialize the settings for all of the bit<i> widgets
	#

	$w.leased_mask0 config -value $value

	tixControl $w.nfas_interface_id0 -label "port 1 nfas_interface_id: " \
		-disabledforeground darkgray \
		-integer true -min 0 -max 127 \
		-command "set_new_number_field $w $bn 0 ds1_config nfas_interface_id" \
		-value [ ds1_config_t_nfas_interface_id_get $ds1_config($bn,0) ] \
		-options "
			label.width $lwidth
			label.anchor e
			entry.width 22
		"

	pack $w.nfas_interface_id0 -side top -anchor w

	tixSelect $w.leased_mask1 -label "Port 2 Time Slot leased_mask: " \
		-allowzero true -radio false \
		-options "
			label.width $lwidth
			label.anchor e
		"
	pack $w.leased_mask1 -side top -anchor w

	#
	#	Get the current leased mask for port 1
	#	Clear the 0x80000000 bit
	#

	set leased_mask1 [ ds1_config_t_leased_mask_get $ds1_config($bn,1) ]
	set leased_mask1 [ expr $leased_mask1 & 2147483647 ]

	set value ""

	for {set i $cpp} { $i > 0 } {incr i -1} {

		if { ( $leased_mask1 & [ expr 1 << ( $i - 1 ) ] ) != 0 } {
			set value "$value bit$i"
		}

		$w.leased_mask1 add bit$i -text "$i" \
		-command "set_new_leased_mask $w.leased_mask1 $bn 1 ds1_config $i " \
		-font "-*-Courier-bold-r-normal-*-8-*-*-*-*-*-*-*" \
		-width 2 -padx 0
	}

	#
	#	Now initialize the settings for all of the bit<i> widgets
	#

	$w.leased_mask1 config -value $value

	tixControl $w.nfas_interface_id1 -label "port 2 nfas_interface_id: " \
		-disabledforeground darkgray \
		-integer true -min 0 -max 127 \
		-command "set_new_number_field $w $bn 1 ds1_config nfas_interface_id" \
		-value [ ds1_config_t_nfas_interface_id_get $ds1_config($bn,1) ] \
		-options "
			label.width $lwidth
			label.anchor e
			entry.width 22
		"

	pack $w.nfas_interface_id1 -side top -anchor w

	#
	# Set widget state for all widgets affected by signalling
	#

	cascade_pri_settings $bn 0
	cascade_pri_settings $bn 1
}

#
# Fill the channels page for a dfrpe1/dfdpe1 board
#
proc config_dfrpe1_channels {w bn} {

	global	adapter_info
	global	ds1_config
	global	om
	upvar #0 ds1_config init

	set lwidth 30

	set ai $adapter_info($bn)

	set cpp [adapter_info_t_channels_per_port_get $ai]

	tixSelect $w.leased_mask0 -label "Port 1 Time Slot leased_mask: " \
		-allowzero true -radio false \
		-options "
			label.width $lwidth
			label.anchor e
		"
	pack $w.leased_mask0 -side top -anchor w

	#
	#	Get the current leased mask for port 0
	#	Clear the 0x80000000 bit
	#

	set leased_mask0 [ ds1_config_t_leased_mask_get $ds1_config($bn,0) ]
	set leased_mask0 [ expr $leased_mask0 & 2147483647 ]

	set value ""

	for {set i [ expr $cpp + 1 ]} { $i > 0 } {incr i -1} {

		if { $i == 16 } {
			set state disabled
		} else {
			set state normal
		}

		if { ( $leased_mask0 & [ expr 1 << ( $i - 1 ) ] ) != 0 } {
			set value "$value bit$i"
		}

		$w.leased_mask0 add bit$i -text "$i" -state "$state" \
		-command "set_new_leased_mask $w.leased_mask0 $bn 0 ds1_config $i" \
		-font "-*-Courier-bold-r-normal-*-8-*-*-*-*-*-*-*" \
		-width 2 -padx 0
	}

	#
	#	Now initialize the settings for all of the bit<i> widgets
	#

	$w.leased_mask0 config -value $value
}

#
# Fill the channels page for a dfrpe2/dfdpe2 board
#
proc config_dfrpe2_channels {w bn} {

	global	adapter_info
	global	ds1_config
	global	om
	upvar #0 ds1_config init

	set lwidth 30

	set ai $adapter_info($bn)

	set cpp [adapter_info_t_channels_per_port_get $ai]

	tixSelect $w.leased_mask0 -label "Port 1 Time Slot leased_mask: " \
		-allowzero true -radio false \
		-options "
			label.width $lwidth
			label.anchor e
		"
	pack $w.leased_mask0 -side top -anchor w

	#
	#	Get the current leased mask for port 0
	#	Clear the 0x80000000 bit
	#

	set leased_mask0 [ ds1_config_t_leased_mask_get $ds1_config($bn,0) ]
	set leased_mask0 [ expr $leased_mask0 & 2147483647 ]

	set value ""

	for {set i [ expr $cpp + 1 ] } { $i > 0 } {incr i -1} {

		if { $i == 16 } {
			set state disabled
		} else {
			set state normal
		}

		if { ( $leased_mask0 & [ expr 1 << ( $i - 1 ) ] ) != 0 } {
			set value "$value bit$i"
		}

		$w.leased_mask0 add bit$i -text "$i" -state "$state" \
		-command "set_new_leased_mask $w.leased_mask0 $bn 0 ds1_config $i" \
		-font "-*-Courier-bold-r-normal-*-8-*-*-*-*-*-*-*" \
		-width 2 -padx 0
	}

	#
	#	Now initialize the settings for all of the bit<i> widgets
	#

	$w.leased_mask0 config -value $value

	tixSelect $w.leased_mask1 -label "Port 2 Time Slot leased_mask: " \
		-allowzero true -radio false \
		-options "
			label.width $lwidth
			label.anchor e
		"
	pack $w.leased_mask1 -side top -anchor w

	#
	#	Get the current leased mask for port 1
	#	Clear the 0x80000000 bit
	#

	set leased_mask1 [ ds1_config_t_leased_mask_get $ds1_config($bn,1) ]
	set leased_mask1 [ expr $leased_mask1 & 2147483647 ]

	set value ""

	for {set i [ expr $cpp + 1 ] } { $i > 0 } {incr i -1} {

		if { $i == 16 } {
			set state disabled
		} else {
			set state normal
		}

		if { ( $leased_mask1 & [ expr 1 << ( $i - 1 ) ] ) != 0 } {
			set value "$value bit$i"
		}

		$w.leased_mask1 add bit$i -text "$i" -state "$state" \
		-command "set_new_leased_mask $w.leased_mask1 $bn 1 ds1_config $i" \
		-font "-*-Courier-bold-r-normal-*-8-*-*-*-*-*-*-*" \
		-width 2 -padx 0
	}

	#
	#	Now initialize the settings for all of the bit<i> widgets
	#

	$w.leased_mask1 config -value $value
}

#
# Fill the channels page for a dfrb boards
#
proc config_dfrb_channels {w bn} {

	global	adapter_info
	global	bri_config
	global	om
	upvar #0 bri_config init

	set lwidth 30

	set ai $adapter_info($bn)

	set pc [adapter_info_t_port_count_get $ai]
	set cpp [adapter_info_t_channels_per_port_get $ai]

	for { set j 0 } { $j < $pc } { incr j +1 } {

		tixSelect $w.leased_mask$j \
			-label "Port [ expr $j + 1 ] Time Slot leased_mask: " \
			-allowzero true -radio false \
			-options "
				label.width $lwidth
				label.anchor e
			"
		pack $w.leased_mask$j -side top -anchor w

		#
		#	Get the current leased mask for port $j
		#	Clear the 0x80000000 bit
		#

		set leased_mask [ bri_config_t_leased_mask_get $bri_config($bn,$j) ]
		set leased_mask [ expr $leased_mask & 2147483647 ]

		set value ""

		for {set i $cpp} { $i > 0 } {incr i -1} {

			if { ( $leased_mask & [ expr 1 << ( $i - 1 ) ] ) != 0 } {
				set value "$value bit$i"
			}

			$w.leased_mask$j add bit$i -text "$i" \
			-command "set_new_leased_mask $w.leased_mask$j $bn $j bri_config $i" \
			-font "-*-Courier-bold-r-normal-*-8-*-*-*-*-*-*-*" \
			-width 2 -padx 0
		}

		#
		#	Now initialize the settings for all of the bit<i> widgets
		#

		$w.leased_mask$j config -value $value

	}
}

#
#	Create the toplevel config page
#
proc create_config_toplevel {} {
	global	dmlib
	global  version
	global	max_num_boards
	global	DM_GUI_WIN_CONFIG
	global	DM_GUI_WIN_STATUS


	set w .dm_config
	toplevel $w
	wm title $w "Dense Modem Configuration $version"

	dm_gui_win_open $DM_GUI_WIN_CONFIG
	if { [ dm_gui_win_isopen $DM_GUI_WIN_STATUS ] } {
		frame $w.f
		label $w.f.errorlbl -justify "left" -text "\
You may not have the adapter configurator and status\n\
windows open simultaneously.  Please close the status\n\
window and try again.\n"
		pack $w.f.errorlbl
		button $w.f.dismiss -text "Dismiss" -command {
		                                  	dm_gui_win_close $DM_GUI_WIN_CONFIG
							destroy .dm_config
		                                  }
		pack $w.f.dismiss
		pack $w.f -padx 5 -pady 5

		return
	}

	global  nboards
	set nboards 0

	#
	#	Load the wrapper functions
	#
	load $dmlib/tcldgdm.so

	#
	# Create the Board notebook
	#
	tixNoteBook $w.boards_nb -ipadx 6 -ipady 6
	$w config -bg gray
	$w.boards_nb subwidget nbframe config -backpagecolor gray 

	#
	# Create the Board pages
	#
	for {set i 0} {$i < $max_num_boards} {incr i} {
		create_config_board $w.boards_nb $i
	}
	pack $w.boards_nb -expand yes -fill both -padx 5 -pady 5 -side top

	if {$nboards == 0} {
		label $w.noboards \
			-text "Sorry, Dense Modem GUI unable to access the driver.\n\
Possible reasons are:  No adapters installed;\n\
driver not loaded; driver devices not created;\n\
insufficient permissions on the management device.\n\
See man dm_driver for additional information."
		pack $w.noboards
	}

	#
	# Create the OK/Cancel buttons
	#
	frame $w.buttons -bg gray
	pack $w.buttons -side bottom -fill x -pady 2m

	button $w.buttons.ok -text "Ok" \
		-command "ok_config_toplevel $w"
	button $w.buttons.apply -text "Apply" \
		-command "apply_config_toplevel $w"
	button $w.buttons.help -text "Help" \
		-command "create_help_toplevel"
	button $w.buttons.cancel -text "Cancel" \
		-command "cancel_config_toplevel $w"

	pack $w.buttons.ok $w.buttons.apply $w.buttons.help $w.buttons.cancel \
		-side left -expand yes
}

#
#	OK:
#		do apply and then cancel
#
proc ok_config_toplevel {w} {

	#
	#	Do a last check for changes not yet reported
	#	Don't apply or exit if any were in error
	#

	if { [ check_for_errors $w ] == "Error" } {
		return
	}

	apply_config_toplevel $w
	cancel_config_toplevel $w
}

#
#	Apply:
#		ioctl the mgmt channel and save the configuration
#
proc apply_config_toplevel { w } {
    global nboards
    global mgmt
    global port_count
    global board_family
    global SET_DS1_CONFIG
    global ioc_ds1_config
    global SET_BRI_CONFIG
    global ioc_bri_config
    global bri_config
    global SET_MODEM_NATIONALITY
    global ioc_modem_nationality
    global ioc_modem_nationality

    global SET_SERIAL_CONFIG
    global serial_config
    global ioc_serial_config

    global SET_SERIAL_RS485_CONFIG
    global serial_rs485_config
    global ioc_serial_rs485_config
    
    #
    #	Do a last check for changes not yet reported
    #	Don't apply or exit if any were in error
    #
    
    if { [ check_for_errors $w ] == "Error" } {
	return
    }
    
    for {set bn 0} {$bn < $nboards} {incr bn} {
	global adapter_change_$bn
	
	if { ($board_family($bn) == "dfrp") || \
		($board_family($bn) == "dfdp") || \
		($board_family($bn) == "dfrb") || \
		($board_family($bn) == "apr") } {
	    
	    if { [ set adapter_change_$bn ] == 2 } {
		set adapter_change_$bn 1
		ioctl $mgmt($bn) $SET_MODEM_NATIONALITY \
			$ioc_modem_nationality($bn)
	    }
	}
	
	if { ($board_family($bn) == "dfrp") || \
		($board_family($bn) == "dfdp") || \
		($board_family($bn) == "dfrb") || \
		($board_family($bn) == "ap2k") } {
	    for {set pn 0} {$pn < $port_count($bn)} {incr pn} {
		global port_change_${bn}_${pn}
		
		if { ($board_family($bn) == "dfrp") || \
			($board_family($bn) == "dfdp") } {
		    
		    if { [ set port_change_${bn}_${pn} ] == 2 } {
			set port_change_${bn}_${pn} 1
			ioctl $mgmt($bn) $SET_DS1_CONFIG \
				$ioc_ds1_config($bn,$pn)
		    }
		}
		
		if { $board_family($bn) == "dfrb" } {
		    if { [ set port_change_${bn}_${pn} ] == 2 } {
			
			set port_change_${bn}_${pn} 1
			set i [ ioctl $mgmt($bn) $SET_BRI_CONFIG \
				$ioc_bri_config($bn,$pn) ]
		    }
		}
		
		if { $board_family($bn) == "ap2k" } {
		    if { [set port_change_${bn}_${pn} ] == 2 } {
			set port_change_${bn}_${pn} 1
			set i [ ioctl $mgmt($bn) \
				$SET_SERIAL_CONFIG \
				$ioc_serial_config($bn,$pn) ]
			set j [ ioctl $mgmt($bn) \
				$SET_SERIAL_RS485_CONFIG \
				$ioc_serial_rs485_config($bn,$pn) ]
			update_portreg $bn $pn
		    }
		}
	    }
	}
    }
}


#
#	Cancel:
#		Destroy the config toplevel and free all resources
#
proc cancel_config_toplevel {w} {
	global nboards
	global mgmt
	global port_count
	global ioc_adapter_info
	global ioc_ds1_config
	global ioc_bri_config
	global ioc_modem_nationality
	global board_type
	global board_family
	global DM_GUI_WIN_CONFIG

	for {set bn 0} {$bn < $nboards} {incr bn} {

		close_mgmt $bn

		delete_get_adapter_info_t $ioc_adapter_info($bn)

		if { ($board_family($bn) == "dfrp") || \
		 ($board_family($bn) == "dfdp") || \
		 ($board_family($bn) == "dfrb") || \
		 ($board_family($bn) == "apr") } {

			delete_getset_modem_nationality_t $ioc_modem_nationality($bn)
		}

		if { ($board_family($bn) == "dfrp") || \
		 ($board_family($bn) == "dfdp") || \
		 ($board_family($bn) == "dfrb") } {
			for {set pn 0} {$pn < $port_count($bn)} {incr pn} {

				if { ($board_family($bn) == "dfrp") || \
				 ($board_family($bn) == "dfdp") } {
					delete_getset_ds1_config_t \
						$ioc_ds1_config($bn,$pn)
				}

				if { $board_family($bn) == "dfrb" } {
					delete_getset_bri_config_t \
						$ioc_bri_config($bn,$pn)
				}

				if { $board_family($bn) == "ap2k" } {
					delete_getset_serial_config_t \
						$ioc_serial_config($bn,$pn)
					delete_getset_serial_rs485_config_t \
						$ioc_serial_rs485_config($bn,$pn)
				}
			}
		}
	}

	dm_gui_win_close $DM_GUI_WIN_CONFIG

	destroy $w
}
