
#
# Copyright 1999 by Digi International (www.digi.com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED; without even the 
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

#
#	$Id: dm_help.tcl,v 1.4 2000/11/03 07:11:29 jamesp Exp $
#
package provide dm_gui 1.0


proc create_help_toplevel {} {
	global	dmlib
	global	DIGI_PCIHOUSTON_FAMILY_NAME
	global	DIGI_CPCIHOUSTON_FAMILY_NAME
	global	DIGI_LONDON_FAMILY_NAME
	global	DIGI_CHASKA_FAMILY_NAME
	global	DIGI_AP2K_FAMILY_NAME

	set w .dm_help
	toplevel $w
	wm title $w "Dense Modem Help System"

	#
	# Load the wrapper functions
	#
	load $dmlib/tcldgdm.so

	#
	# Create the Adapter notebook
	#
	tixNoteBook $w.adapter_nb -ipadx 6 -ipady 6
	$w config -bg gray
	$w.adapter_nb subwidget nbframe config -backpagecolor gray

	#
	# Create the Adapter Family pages
	#

	create_adapter_family $w.adapter_nb dfrp "$DIGI_PCIHOUSTON_FAMILY_NAME"

	create_adapter_family $w.adapter_nb dfdp "$DIGI_CPCIHOUSTON_FAMILY_NAME"

	create_adapter_family $w.adapter_nb dfrb "$DIGI_LONDON_FAMILY_NAME"

	create_adapter_family $w.adapter_nb apr "$DIGI_CHASKA_FAMILY_NAME"

	create_adapter_family $w.adapter_nb ap2k "$DIGI_AP2K_FAMILY_NAME"

	pack $w.adapter_nb -expand yes -fill both -padx 5 -pady 5 -side top

	frame $w.buttons
	pack $w.buttons -side bottom -fill x -pady 2m

	button $w.buttons.dismiss -text "Dismiss" \
		-command "destroy $w"

	pack $w.buttons.dismiss -side left -expand yes
}

#
#	Create the notebook page for this family of adapters
#
proc create_adapter_family {w family family_name} {

	#
	#	Insert family info text
	#

	$w add fam_$family -label $family_name

	#
	#	Create the Adapter Info/Adapter/Ports/Channels notebook
	#

	set f [$w subwidget fam_$family]
	tixNoteBook $f.help_nb -ipadx 6 -ipady 6

	$f.help_nb subwidget nbframe config -backpagecolor gray


	message $f.help_nb.msg -aspect 300 -text [ family_info_$family ] \
	 -width 20c

	pack $f.help_nb.msg -expand yes -fill both -padx 5 -pady 5 -side top

	if { $family == "apr" } {

		frame $f.help_nb.buttons
		pack $f.help_nb.buttons -side bottom -fill x -pady 2m

		button $f.help_nb.buttons.adapter_help -text "Adapter Parm Help" \
		 -command "create_apr_adapter_help_toplevel"

		pack $f.help_nb.buttons.adapter_help -side left -expand yes

	}

	if { ( $family == "dfrp" ) || ( $family == "dfdp" ) } {

		frame $f.help_nb.buttons
		pack $f.help_nb.buttons -side bottom -fill x -pady 2m

		button $f.help_nb.buttons.adapter_help -text "Adapter Parm Help" \
		 -command "create_dfr_adapter_help_toplevel"

		pack $f.help_nb.buttons.adapter_help -side left -expand yes

		button $f.help_nb.buttons.dfrt1_port_help \
		 -text "T1 Port Parm Help" \
		 -command "create_dfrt1_port_help_toplevel"

		pack $f.help_nb.buttons.dfrt1_port_help -side left -expand yes

		button $f.help_nb.buttons.dfre1_port_help \
		 -text "E1 Port Parm Help" \
		 -command "create_dfre1_port_help_toplevel"

		pack $f.help_nb.buttons.dfre1_port_help -side left -expand yes

		button $f.help_nb.buttons.channel_help -text "Channel Parm Help" \
		 -command "create_dfrp_channel_help_toplevel"

		pack $f.help_nb.buttons.channel_help -side left -expand yes

	}

	if { $family == "dfrb" } {

		frame $f.help_nb.buttons
		pack $f.help_nb.buttons -side bottom -fill x -pady 2m

		button $f.help_nb.buttons.adapter_help -text "Adapter Parm Help" \
		 -command "create_dfrb_adapter_help_toplevel"

		pack $f.help_nb.buttons.adapter_help -side left -expand yes

		button $f.help_nb.buttons.port_help -text "Port Parm Help" \
		 -command "create_dfrb_port_help_toplevel"

		pack $f.help_nb.buttons.port_help -side left -expand yes

		button $f.help_nb.buttons.channel_help -text "Channel Parm Help" \
		 -command "create_dfrb_channel_help_toplevel"

		pack $f.help_nb.buttons.channel_help -side left -expand yes
	}

	if { $family == "ap2k" } {
	    
	    frame $f.help_nb.buttons
	    pack $f.help_nb.buttons -side bottom -fill x -pady 2m

	    button $f.help_nb.buttons.port_help -text "Channel Parm Help" \
		    -command "create_ap2k_channel_help_toplevel"

	    pack $f.help_nb.buttons.port_help -side left -expand yes
	}

	pack $f.help_nb -expand yes -fill both -padx 5 -pady 5 -side top

}

#
#	dfrp family info text
#

proc family_info_dfrp {} {
	global DIGI_PCIHOUSTON_FAMILY_NAME
	global DIGI_PCIHOUSTON24_NAME
	global DIGI_PCIHOUSTON48_NAME
	global DIGI_PCIHOUSTON30_NAME
	global DIGI_PCIHOUSTON60_NAME

	#
	#	Return the requested text
	#

	return "$DIGI_PCIHOUSTON_FAMILY_NAME Adapters:\
\n\n\t$DIGI_PCIHOUSTON24_NAME\
\n\t$DIGI_PCIHOUSTON48_NAME\
\n\t$DIGI_PCIHOUSTON30_NAME\
\n\t$DIGI_PCIHOUSTON60_NAME\
\n\n\nVisit the Digi International web site at www.digi.com for Adapter\n\
Provisioning sheets, the latest driver package and the latest driver\n\
package documentation."

}

#
#	dfdp family info text
#

proc family_info_dfdp {} {
	global DIGI_CPCIHOUSTON_FAMILY_NAME
	global DIGI_CPCIHOUSTON24_NAME
	global DIGI_CPCIHOUSTON48_NAME
	global DIGI_CPCIHOUSTON30_NAME
	global DIGI_CPCIHOUSTON60_NAME

	#
	#	Return the requested text
	#

	return "$DIGI_CPCIHOUSTON_FAMILY_NAME Adapters:\
\n\n\t$DIGI_CPCIHOUSTON24_NAME\
\n\t$DIGI_CPCIHOUSTON48_NAME\
\n\t$DIGI_CPCIHOUSTON30_NAME\
\n\t$DIGI_CPCIHOUSTON60_NAME\
\n\n\nVisit the Digi International web site at www.digi.com for Adapter\n\
Provisioning sheets, the latest driver package and the latest driver\n\
package documentation."

}

#
#	dfrb family info text
#

proc family_info_dfrb {} {
	global DIGI_LONDON_FAMILY_NAME
	global DIGI_LONDONU_NAME
	global DIGI_LONDONST_NAME

	#
	#	Return the requested text
	#

	return "$DIGI_LONDON_FAMILY_NAME Adapters:\
\n\n\t$DIGI_LONDONU_NAME\
\n\t$DIGI_LONDONST_NAME\
\n\n\nVisit the Digi International web site at www.digi.com for Adapter\n\
Provisioning sheets, the latest driver package and the latest driver package\n\
documentation."

}

#
#	apr family info text
#

proc family_info_apr {} {
	global DIGI_CHASKA_FAMILY_NAME
	global DIGI_CHASKA40_NAME
	global DIGI_CHASKA41_NAME
	global DIGI_CHASKA42_NAME
	global DIGI_CHASKA43_NAME
	global DIGI_CHASKA80_NAME
	global DIGI_CHASKA81_NAME
	global DIGI_CHASKA82_NAME
	global DIGI_CHASKA83_NAME

	#
	#	Return the requested text
	#

	return "$DIGI_CHASKA_FAMILY_NAME Adapters:\
\n\n\t$DIGI_CHASKA40_NAME\
\n\t$DIGI_CHASKA41_NAME\
\n\t$DIGI_CHASKA42_NAME\
\n\t$DIGI_CHASKA43_NAME\
\n\t$DIGI_CHASKA80_NAME\
\n\t$DIGI_CHASKA81_NAME\
\n\t$DIGI_CHASKA82_NAME\
\n\t$DIGI_CHASKA83_NAME\
\n\n\nVisit the Digi International web site at www.digi.com for the latest\n\
driver package and the latest driver package documentation."

}

proc family_info_ap2k {} {
	global DIGI_AP2K_FAMILY_NAME
	global DIGI_AP2K_NAME
	global DIGI_AP4K_NAME
	global DIGI_AP8K_NAME
	global DIGI_AP16K_NAME
	global DIGI_AP32K_NAME
	global DIGI_AP4K_MPI_NAME
	global DIGI_AP8K_MPI_NAME
	global DIGI_AP16K_MPI_NAME

	#
	#	Return the requested text
	#

	return "$DIGI_AP2K_FAMILY_NAME Adapters:\
\n\n\t$DIGI_AP2K_NAME\
\n\t$DIGI_AP4K_NAME\
\n\t$DIGI_AP8K_NAME\
\n\t$DIGI_AP16K_NAME\
\n\t$DIGI_AP32K_NAME\
\n\t$DIGI_AP4K_MPI_NAME\
\n\t$DIGI_AP8K_MPI_NAME\
\n\t$DIGI_AP16K_MPI_NAME\
\n\n\nVisit the Digi International web site at www.digi.com for the latest\n\
driver package and the latest driver package documentation."

}

proc create_apr_adapter_help_toplevel {} {
	global DIGI_CHASKA_FAMILY_NAME
	global DIGI_CHASKA40_NAME
	global DIGI_CHASKA80_NAME
	global DIGI_CHASKA41_NAME
	global DIGI_CHASKA81_NAME
	global DIGI_CHASKA42_NAME
	global DIGI_CHASKA82_NAME
	global DIGI_CHASKA43_NAME
	global DIGI_CHASKA83_NAME
	global ADAPTER_CHASKA80
	global ADAPTER_CHASKA81
	global ADAPTER_CHASKA82
	global ADAPTER_CHASKA83

	set daa0 ""

	for { set i 0 } { [ get_cc_id $ADAPTER_CHASKA80 $i ] != 0 } { incr i +1 } {

		set country_name [ cc2name [get_cc_id $ADAPTER_CHASKA80 $i ] ]

		if { $daa0 == "" } {
			set daa0 $country_name
		} else {
			set daa0 "$daa0, $country_name"
		}
	}

	set daa1 ""

	for { set i 0 } { [ get_cc_id $ADAPTER_CHASKA81 $i ] != 0 } { incr i +1 } {

		set country_name [ cc2name [get_cc_id $ADAPTER_CHASKA81 $i ] ]

		if { $daa1 == "" } {
			set daa1 $country_name
		} else {
			set daa1 "$daa1, $country_name"
		}
	}

	set daa2 ""

	for { set i 0 } { [ get_cc_id $ADAPTER_CHASKA82 $i ] != 0 } { incr i +1 } {

		set country_name [ cc2name [get_cc_id $ADAPTER_CHASKA82 $i ] ]

		if { $daa2 == "" } {
			set daa2 $country_name
		} else {
			set daa2 "$daa2, $country_name"
		}
	}

	set daa3 ""

	for { set i 0 } { [ get_cc_id $ADAPTER_CHASKA83 $i ] != 0 } { incr i +1 } {

		set country_name [ cc2name [get_cc_id $ADAPTER_CHASKA83 $i ] ]

		if { $daa3 == "" } {
			set daa3 $country_name
		} else {
			set daa3 "$daa3, $country_name"
		}
	}

	set daa_txt "$DIGI_CHASKA40_NAME\n\
$DIGI_CHASKA80_NAME:\n\n\
$daa0\n\n\
$DIGI_CHASKA41_NAME\n\
$DIGI_CHASKA81_NAME:\n\n\
$daa1\n\n\
$DIGI_CHASKA42_NAME\n\
$DIGI_CHASKA82_NAME:\n\n\
$daa2\n\n\
$DIGI_CHASKA43_NAME\n\
$DIGI_CHASKA83_NAME:\n\n\
$daa3\n\n"

	set txt "\nModem Nationality:\n\n\
Set the Modem nationality for an adapter.  All supported modem\n\
nationalities for an adapter are listed in the pick list for that adapter.\n\n\
$daa_txt"

	notifier .apr_adapter_help_toplevel \
	 "$DIGI_CHASKA_FAMILY_NAME Adapter Level Parameters" \
	 $txt

}

proc create_dfr_adapter_help_toplevel {} {
	global DIGI_PCIHOUSTON_FAMILY_NAME
	global DIGI_CPCIHOUSTON_FAMILY_NAME
	global DIGI_PCIHOUSTON24_NAME
	global DIGI_PCIHOUSTON48_NAME
	global DIGI_PCIHOUSTON30_NAME
	global DIGI_PCIHOUSTON60_NAME
	global DIGI_CPCIHOUSTON24_NAME
	global DIGI_CPCIHOUSTON48_NAME
	global DIGI_CPCIHOUSTON30_NAME
	global DIGI_CPCIHOUSTON60_NAME
	global ADAPTER_HOU30
	global ADAPTER_HOU24

	set t1 ""

	for { set i 0 } { [ get_cc_id $ADAPTER_HOU24 $i ] != 0 } { incr i +1 } {

		set country_name [ cc2name [get_cc_id $ADAPTER_HOU24 $i ] ]

		if { $t1 == "" } {
			set t1 $country_name
		} else {
			set t1 "$t1, $country_name"
		}
	}

	set e1 ""

	for { set i 0 } { [ get_cc_id $ADAPTER_HOU30 $i ] != 0 } { incr i +1 } {

		set country_name [ cc2name [get_cc_id $ADAPTER_HOU30 $i ] ]

		if { $e1 == "" } {
			set e1 $country_name
		} else {
			set e1 "$e1, $country_name"
		}
	}

	set x1_txt "$DIGI_PCIHOUSTON24_NAME\n$DIGI_PCIHOUSTON48_NAME\n\
$DIGI_CPCIHOUSTON24_NAME\n$DIGI_CPCIHOUSTON48_NAME:\n\n\
$t1\n\n\
$DIGI_PCIHOUSTON30_NAME\n$DIGI_PCIHOUSTON60_NAME\n\
$DIGI_CPCIHOUSTON30_NAME\n$DIGI_CPCIHOUSTON60_NAME:\n\n\
$e1\n\n"

	set txt "\nModem Nationality:\n\n\
Set the Modem nationality for an adapter.  All supported modem\n\
nationalities for an adapter are listed in the pick list for that adapter.\n\n\
$x1_txt"

	notifier .dfrp_adapter_help_toplevel \
	 "$DIGI_PCIHOUSTON_FAMILY_NAME/$DIGI_CPCIHOUSTON_FAMILY_NAME Adapter Level Parameters" \
	 $txt

}

proc create_dfrb_adapter_help_toplevel {} {
	global DIGI_LONDON_FAMILY_NAME
	global DIGI_LONDONU_NAME
	global DIGI_LONDONST_NAME
	global ADAPTER_LONU
	global ADAPTER_LONST

	set u ""

	for { set i 0 } { [ get_cc_id $ADAPTER_LONU $i ] != 0 } { incr i +1 } {

		set country_name [ cc2name [get_cc_id $ADAPTER_LONU $i ] ]

		if { $u == "" } {
			set u $country_name
		} else {
			set u "$u, $country_name"
		}
	}

	set st ""

	for { set i 0 } { [ get_cc_id $ADAPTER_LONST $i ] != 0 } { incr i +1 } {

		set country_name [ cc2name [get_cc_id $ADAPTER_LONST $i ] ]

		if { $st == "" } {
			set st $country_name
		} else {
			set st "$st, $country_name"
		}
	}

	set stu_txt "$DIGI_LONDONU_NAME:\n\n$u\n\n$DIGI_LONDONST_NAME:\n\n$st\n\n"

	set txt "\nModem Nationality:\n\n\
Set the Modem nationality for an adapter.  All supported modem\n\
nationalities for an adapter are listed in the pick list for that adapter.\n\n\
$stu_txt"

	notifier .dfrb_adapter_help_toplevel \
	 "$DIGI_LONDON_FAMILY_NAME Adapter Level Parameters" \
	 $txt

}

proc create_dfrt1_port_help_toplevel {} {
	global DIGI_PCIHOUSTON24_NAME
	global DIGI_PCIHOUSTON48_NAME
	global DIGI_CPCIHOUSTON24_NAME
	global DIGI_CPCIHOUSTON48_NAME

	set txt0 "For $DIGI_PCIHOUSTON24_NAME, $DIGI_PCIHOUSTON48_NAME,\n\
$DIGI_CPCIHOUSTON24_NAME and $DIGI_CPCIHOUSTON48_NAME Adapters"

	set txt1 "Signalling:\n\n\
Specify the signalling for this T1 port.  Choices for Channelized T1 ports\n\
are Loop Start, Wink Start, Ground Start and Immediate Start.  Choices\n\
for Primary Rate ISDN ports are PRI None, NI2, 5ESS, DMS100 and\n\
INS-1500.  Note that Signalling can affect the validity of other\n\
configuration fields on this and other pages.  Fields that are invalid are\n\
\"grayed-out\" and cannot be set or changed."

	set txt2 "Framing:\n\n\
Specify the framing for this T1 port.  Choices are ESF and D4 for\n\
channelized T1 ports.  ESF is the only valid choice for Primary Rate ISDN\n\
ports."

	set txt3 "Line Coding:\n\n\
Specify the line coding for this T1 port.  Choices are B8ZS and EMI for\n\
channelized T1 ports.  BZ8S is the only valid choice for Primary Rate\n\
ISDN ports"

	set txt4 "Line Clocking:\n\n\
Loop is the only value allowed by dm_gui.  See \"man dm_admin\" for\n\
additional, debug-only vaues for this field."

	set txt5 "Jitter Attenuation:\n\n\
Specify the jitter attenuation for this T1 port.  Choices are Auto\n\
Attenuation, Receive Attenuation, Transmit Attenuation and None."

	set txt6 "Equalization Gain Limit:\n\n\
Specify the equalization gain limit for this T1 port.  This field is enabled\n\
only if Interface Type is set to Long Haul.  Choices are Low and High."

	set txt7 "Interface Type:\n\n\
Specify the interface type for this T1 port.  Choices are Short Haul and\n\
Long Haul.  Note that Interface Type can affect the validity of other\n\
configuration fields on this page.  Fields that are invalid are\n\
\"grayed-out\" and cannot be set or changed."

	set txt8 "Long Haul Level:\n\n\
Valid and enabled only if Interface Type is set to Long Haul.  Specify the\n\
long haul level for this T1 port.  Choices are 00.0 dB, 07.5 dB, 15.0 dB\n\
and 22.5 dB"

	set txt9 "Short Haul Lenth:\n\n\
Valid and enabled only if Interface Type is set to Short Haul.  Specify the\n\
short haul length for this T1 port.  Choices are 0 - 133 ft., 134 - 266\n\
ft., 267 - 399 ft., 400 - 533 ft. and 534 - 655 ft."

	set txt10 "Wink Digits:\n\n\
Valid and enabled only if Signalling is set to Wink Start.  Specify the number of wink digits (1 - 20) for this port."

	set txt11 "Use the Digi Internation Provisioning Infomation Form to gather\n\
this information from your telephone company."

	set txt "\n$txt0\n\n$txt1\n\n$txt2\n\n$txt3\n\n$txt4\n\n$txt5\n\n$txt6\n\n$txt7\n\n$txt8\n\n$txt9\n\n$txt10\n\n$txt11\n"

	notifier .dfrt1_port_help_toplevel \
	 "T1 Port Level Parameters" \
	 $txt

}

proc create_dfre1_port_help_toplevel {} {
	global DIGI_PCIHOUSTON30_NAME
	global DIGI_PCIHOUSTON60_NAME
	global DIGI_CPCIHOUSTON30_NAME
	global DIGI_CPCIHOUSTON60_NAME

	set txt0 "For $DIGI_PCIHOUSTON30_NAME, $DIGI_PCIHOUSTON60_NAME,\n\
$DIGI_CPCIHOUSTON30_NAME and $DIGI_CPCIHOUSTON60_NAME Adapters"

	set txt1 "Signalling:\n\n\
Specify the signalling for this E1 port.  Choices are PRI None and ETSI."

	set txt2 "Framing:\n\n\
Specify the framing for this E1 port.  Choices are CEPT with CRC4 and\n\
CEPT without CRC4."

	set txt3 "Line Coding:\n\n\
HDB3 is the only value allowed."

	set txt4 "Line Clocking:\n\n\
Loop is the only value allowed by dm_gui.  See \"man dm_admin\" for\n\
additional, debug-only vaues for this field."

	set txt5 "Jitter Attenuation:\n\n\
Specify the jitter attenuation for this E1 port.  Choices are Auto\n\
Attenuation, Receive Attenuation, Transmit Attenuation and None."

	set txt6 "Use the Digi Internation Provisioning Infomation Form to gather\n\
this information from your telephone company."

	set txt "\n$txt0\n\n$txt1\n\n$txt2\n\n$txt3\n\n$txt4\n\n$txt5\n\n$txt6\n"

	notifier .dfre1_port_help_toplevel \
	 "E1 Port Level Parameters" \
	 $txt

}

proc create_dfrb_port_help_toplevel {} {
	global DIGI_LONDON_FAMILY_NAME

	set txt1 "Switch Type:\n\n\
Choose the switch type for this BRI port.  Choices are BRI None, NI1, 5ESS,\n\
DMS100, ETSI, and INS-64."

	set txt2 "Auto Alert:\n\n\
Select \"Enabled\" or \"Disabled\" from the pick list.  When Enabled, this\n\
allows the adapter to automatically generate alerts."

	set txt3 "Number of Directories:\n\n\
This option is valid and enabled ONLY if the maximum number of directories\n\
is geater than 1.  It allows the user to indicate the number of Directory\n\
Numbers and Sub-Address(es)/Spid(s) that will be specified.  If Switch\n\
Type is BRI None, NI1, 5ESS or DMS100, Number of Directories can\n\
range from 1 to 2.  If Switch Type is INS-64, the Number of Directories\n\
field is disabled and only one Directory Number/Sub-Address pair can be\n\
entered.  If Switch Type is ETSI and msn is disabled, the Number of\n\
Directories field is disabled and only one Directory Number/Sub-Address\n\
pair can be entered.  If Switch Type is ETSI and Multiple Subscriber\n\
Number Filtering (msn) is enabled, Number of Directories can range from\n\
1 to 8."

	set txt4 "Multiple Subscriber Number Filtering (msn):\n\n\
This option is valid and enabled ONLY if Switch Type is ETSI.  If Enabled,\n\
it allows Number of Directories to range from 1 to 8.  If Disabled, the\n\
Number of Directories field is disabled."

	set txt5 "Directory Number < 1 - 1 | 2 | 8 >:\n\n\
The number of Directory Number fields that are enabled for input is based\n\
on the selected Switch Type and the Number of Directories setting.  This\n\
field cannot exceed 20 characters in length."

	set txt6 "Sub-Address < 1 - 1 | 8 >:\n\n\
This field is valid only if Switch Type is ETSI or INS-64.  The number of\n\
Sub-Address fields that are enabled is based on the Number of Directories\n\
field.  This field cannot exceed 20 characters in length."

	set txt7 "SPID < 1 - 2 >:\n\n\
This field is valid only if Switch Type is BRI None, NI1, 5ESS or DMS-100.\n\
The number of SPID fields that are enabled is based on the Number of\n\
Directories field.  This field cannot exceed 20 characters in length."

	set txtp "Use the Digi Internation BRI Provisioning Infomation Form to\n\
gather this information from your telephone company."

	set txt "\n$txt1\n\n$txt2\n\n$txt3\n\n$txt4\n\n$txt5\n\n$txt6\n\n$txt7\n\n$txtp\n"

	notifier .dfrb_port_help_toplevel \
	 "$DIGI_LONDON_FAMILY_NAME Port Level Parameters" \
	 $txt


}

proc create_dfrp_channel_help_toplevel {} {
	global DIGI_PCIHOUSTON_FAMILY_NAME
	global DIGI_CPCIHOUSTON_FAMILY_NAME
	global DIGI_PCIHOUSTON24_NAME
	global DIGI_PCIHOUSTON48_NAME
	global DIGI_PCIHOUSTON30_NAME
	global DIGI_PCIHOUSTON60_NAME
	global DIGI_CPCIHOUSTON24_NAME
	global DIGI_CPCIHOUSTON48_NAME
	global DIGI_CPCIHOUSTON30_NAME
	global DIGI_CPCIHOUSTON60_NAME

	set txt1 "Port < 1 - 2 > Time Slot leased_mask:\n\n\
Set the leased mask for port time slots.\n\n\
Push the time slot button in to indicate \"leased\", time slot button out\n\
to indicate \"not leased\".  Buttons for Time Slots which are not available\n\
as leased are disabled."

	set txt2 "For $DIGI_PCIHOUSTON24_NAME, $DIGI_PCIHOUSTON48_NAME,\n\
$DIGI_CPCIHOUSTON24_NAME, and $DIGI_CPCIHOUSTON48_NAME, the Time Slot\n\
24 button is disabled whenever PRI switching is specified."

	set txt3 "$DIGI_PCIHOUSTON30_NAME, $DIGI_PCIHOUSTON60_NAME,\n\
$DIGI_CPCIHOUSTON30_NAME, and $DIGI_CPCIHOUSTON60_NAME adapters have\n\
32 time slots per port, ranging from 0 to 31.  Time slot 0 is reserved for line\n\
management and is not displayed, time slot 1 is the first B-channel, time\n\
slot 15 is the 15th B-channel, time slot 16 is the D-channel and is always\n\
dispayed as disabled, time slot 17 is the 16th B-channel and time slot 31\n\
is the 30th B-channel."

	set txt4 "NFAS Master Port:\n\n\
NFAS (Non-Facility Associated Signalling) options are displayed only for\n\
$DIGI_PCIHOUSTON48_NAME and $DIGI_CPCIHOUSTON48_NAME adapters when both\n\
ports are configured for PRI signalling.  Choices for this field are \"Disabled\",\n\
\"Port 1\", or \"Port 2\".  If \"Disabled\" is selected, each port on the\n\
adapter does its own signalling, leaving 23 time slots per port for data\n\
traffic.  If \"Port 1\" or \"Port 2\" is selected, the selected port does\n\
signalling for both ports, freeing up time slot 24 on the non-selcted port\n\
for data traffic."

	set txt5 "Port <1 - 2> nfas_interface_id:\n\n\
Valid and enabled only if \"NFAS Master Port\" is set to \"Port 1\" or\n\
\"Port 2\".  This parameter can range from 0 to 127 and the Port 1 ID must\n\
not be the same as the Port 2 ID."

	set txt6 "Use the Digi International Provisioning Infomation Forms to gather this\n\
information from your telephone company."

	set txt "\n$txt1\n\n$txt2\n\n$txt3\n\n$txt4\n\n$txt5\n\n$txt6\n"

	notifier .dfrp_channel_help_toplevel \
	 "$DIGI_PCIHOUSTON_FAMILY_NAME/$DIGI_CPCIHOUSTON_FAMILY_NAME Channel Level Parameters" \
	 $txt


}

proc create_dfrb_channel_help_toplevel {} {
	global DIGI_LONDON_FAMILY_NAME

	set txt "Port < 1 - 4 > Time Slot leased_mask:\n\n\
Set the leased mask for the two time slots for each of the four BRI ports on\n\
this adapter.\n\nPush the time slot button in to indicate \"leased\", time\n\
slot button out to indicate \"not leased\".\n\n"

	notifier .dfrb_channel_help_toplevel \
	 "$DIGI_LONDON_FAMILY_NAME Channel Level Parameters" \
	 $txt

}

proc create_ap2k_channel_help_toplevel {} {
	global DIGI_AP2K_FAMILY_NAME

	set txt1 "\
The following parameter is common across all AccelePort Xp\n\
adapters:"

	set txt2 "\
\     Alt-Pin"

	set txt3 "\
\          This feature is typically only used in configurations\n\
\          with RJ-45 cabling with 8 or fewer pins.  When enabled,\n\
\          the DCD and DSR signals are swapped, bringing the DCD\n\
\          signal to one of the inner 8 pins on an RJ-45 connector."

	set txt4 "\
The following parameters are specific to the multi-electrical\n\
interface flavors of the AccelePort Xp family:"

	set txt5 "\
\     Port Type"

	set txt6 "\
\          Determines the electrical interface used for an individual\n\
\          port."

	set txt7 "\
\     TX Enable"

	set txt8 "\
\          Determines the behavior of the transmitter for channels in\n\
\          RS-485 mode.  The \"automatic\" setting enables the trans-\n\
\          mitter automatically when sending data, and turns it off\n\
\          afterwards.  The \"follow RTS\" setting enables the trans-\n\
\          mitter whenever the RTS signal is high, and disables it\n\
\          otherwise."

	set txt9 "\
\     RX Enable"

	set txt10 "\
\          Determines the behavior of the receiver for channels in\n\
\          RS-485 mode.  The \"automatic\" setting disables the\n\
\          receiver when transmitting to avoid transmission echo.\n\
\          The \"always\" setting keeps the receiver enabled at all\n\
\          times."

	set txt "\n$txt1\n\n$txt2\n\n$txt3\n\n$txt4\n\n$txt5\n\n$txt6\n\n$txt7\n\n$txt8\n\n$txt9\n\n$txt10\n"

notifier .ap2k_channel_help_toplevel \
	"$DIGI_AP2K_FAMILY_NAME Channel Level Parameters" \
	$txt

}


