
/*
 Copyright 1999-2000 by Digi International (www.digi.com)

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED; without even the 
 implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*****************************************************************************
    tracelog.h

    Common Digi trace definitions

    Written by:
    Thomas Pitzen
    Digi International
    Copyright (C) 1995-96, Digi International, Inc.

    $Archive: /Components/Common/Traces/tracelog.h $    
    $Author: emillan $  
    $Date: 2000/01/10 21:25:41 $     
    $Revision: 1.4 $

******************************************************************************
Description:

******************************************************************************

                   CHANGE LOG

--date---------------------------change---------------------------------------

------------------------------------------------------------------------------

*****************************************************************************/

/*****************************************************************************
 * Copyright (c) 1988-99 Digi International Inc., All Rights Reserved
 * 
 * This  software  contains  proprietary  and  confidential   information   of
 * Digi International  Inc.   By  accepting  transfer  of this copy, Recipient
 * agrees to retain this software in  confidence,  to  prevent  disclosure  to
 * others,  and  to make no  use of this software other than that for which it
 * was delivered.  This is an unpublished copyright work  Digi   International
 * Inc.    Except   as   permitted   by  federal  law,  17 USC 117, copying is
 * strictly prohibited.
 * 
 * Digi  International  Inc.  CONFIDENTIAL  -   (Digi    International    Inc.
 * Confidential  when   combined   with   the   aggregated  modules  for  this
 * product) OBJECT CODE ONLY SOURCE MATERIALS
 * 
 * (C) COPYRIGHT Digi International Inc. 1999
 * 
 * All Rights Reserved
 * 
 * US   Government   Users   Restricted   Rights   -   Use,   duplication   or
 * disclosure  restricted by GSA ADP Schedule Contract with Digi International
 * Inc.
 * 
 *****************************************************************************/

#ifndef _tracelog_h         /* Do not reinclude */
#define _tracelog_h

/* Set pack option to 1 */
/* We may need compiler specific ifdef's here */
/*#ifdef _WATCOMC_ */

#ifndef UW7
#pragma pack(1)
#endif

/* Current trace file format version */
#define TLF_FILE_VERSION 0x0100

/*************************************************************/
/* These definitions are from dgtypes.h */
#define 	uint8			unsigned char
#define 	dxb_uint8		uint8
#define 	shared_uint8_le		uint8
#define 	host_uint8_le		uint8
#define 	board_uint8_le		uint8
#define 	UINT8			unsigned char
#define		uchar			unsigned char
#define		BYTE			unsigned char
#define		INT8			char

#define		int16			short
#define		uint16			unsigned short
#define 	dxb_uint16		uint16
#define 	shared_uint16_le	uint16
#define 	host_uint16_le		uint16
#define 	board_uint16_le		uint16
#define		UINT16			unsigned short
#define		WORD			unsigned short
#define		INT16			short

#define 	eshort			uint16

#define		uint32			unsigned long
#define 	dxb_uint32		uint32
#define 	shared_uint32_le	uint32
#define 	host_uint32_le		uint32
#define 	board_uint32_le		uint32
#define		UINT32			unsigned long
#define		LONG			unsigned long
#define		DWORD			unsigned long
#define		INT32			long

#define 	elong			uint32

#define		host_uint32		unsigned long
#define		host_uint16		unsigned short
#define		host_uint8		unsigned char
#define		host_uchar		unsigned char

#define		shared_uint32	volatile uint32
#define		shared_uint16	volatile uint16
#define		shared_uint8	volatile uint8

#define		board_uint32	volatile const ulong
#define		board_uint16	volatile const ushort
#define		board_uint8		volatile const char
#define     	board_uchar		uint8
#define		pad_32			uint32
/*************************************************************/

/* Length type */
typedef UINT32 TLF_LENGTH;

/* Time structure */
typedef struct
{
    UINT8 Hour;
    UINT8 Minute;
    UINT8 Second;
    UINT8 CentiSecond;
} TLF_TIME;

/*
** Log file header
*/
typedef struct _TLF_LOG_FILE_HEADER
{
    char    Title[48]; 
#define TLF_LOG_TITLE "Digi Intl Binary Trace File %.2d/%.2d/%.2d %.2d:%.2d:%.2d\n\x1a"
#define TLF_LOG_TITLE_CHECK_SIZE 28 /* bytes to check to validate file        */
    UINT16  TraceFileVersion;       /* File format version number             */
    UINT16  TraceEnginePlatform;    /* OS used to capture trace               */
#define TLF_OS_UNKNOWN    0x0000
#define TLF_OS_DOS      0x0001
#define TLF_OS_NETWARE  0x0002
#define TLF_OS_WINDOWS  0x0003
#define TLF_OS_UNIX     0x0004
} TLF_LOG_FILE_HEADER;


/*
** Log file record header
*/
typedef struct
{
    TLF_LENGTH  Len;                    /* Length of this record              */
    UINT16      TypeId;                 /* Type of record entry               */
#define TLF_TYPE_ASCII  0x0001          /* Generic ASCII entry                */
#define TLF_TYPE_HEX    0x0002          /* Generic hex entry                  */
#define TLF_TYPE_DCH    0x0003          /* D-Channel trace entry              */
#define TLF_TYPE_BCH    0x0004          /* B-Channel trace entry              */
#define TLF_TYPE_PPP    0x0005          /* PPP trace entry                    */
#define TLF_TYPE_X25    0x0006          /* X25 trace entry                    */
    UINT16      SeqNum;                 /* Sequence number of entry           */
#define TLF_SEQ_MAX     0xffff          /* Largest sequence number            */
    TLF_TIME    TimeStamp;              /* Time of trace                      */
    UINT8       DeviceId;               /* Board index                        */
    UINT8       LineId;                 /* Line index                         */
    UINT8       Unused[2];              /* Padding                            */
} TLF_HEADER;   


/*
** ASCII record
*/
typedef struct _TLF_ASCII_RECORD
{
    TLF_HEADER  Header;
    TLF_LENGTH  DataLen;
/*  UINT8       Data[]; */
/*  TLF_LENGTH  EndLen; */
} TLF_ASCII_RECORD;


/*
** Hex record
*/
typedef struct _TLF_HEX_RECORD
{
    TLF_HEADER  Header;
    TLF_LENGTH  DataLen;
/*  UINT8       Data[]; */
/*  TLF_LENGTH  EndLen; */
} TLF_HEX_RECORD;
    

/*
** D-Channel record
*/
typedef struct
{
    TLF_HEADER  Header;
    UINT8       IsdnStandard;               /* Line's switch protocol */
#define ISDN_STANDARD_UNKNOWN   0x00
#define ISDN_STANDARD_ATT       0x01
#define ISDN_STANDARD_NI1       0x02
#define ISDN_STANDARD_NTI       0x03
#define ISDN_STANDARD_NET3      0x04
#define ISDN_STANDARD_VN3       0x05
#define ISDN_STANDARD_INS64     0x06
#define ISDN_STANDARD_AUSTEL    0x07
#define ISDN_STANDARD_DEFINITY  0x08
#define ISDN_STANDARD_GENERIC   0x09
#define ISDN_STANDARD_1TR6      0x0a
#define ISDN_STANDARD_LEASED    0x0b
#define ISDN_STANDARD_TOTAL     0x0c

    UINT8       Flags;
#define DCH_FLAG_DIR_RX         0x01        /* Direction flag:                */
                                            /*   0 = Tx (To switch)           */
                                            /*   1 = Rx (From switch)         */
#define DCH_FLAG_TRUNCATED      0x02        /* Frame was truncated            */
#define DCH_FLAG_NT_SIDE        0x04        /* Device is acting as NT         */
    UINT8       Unused[2];                  /* Padding                        */
    TLF_LENGTH  DataLen;                    /* Length of data portion         */
/*  UINT8       Data[]; */
/*  TLF_LENGTH  EndLen; */
} TLF_DCH_RECORD;

/* This stuff used to be in isdnshow.h */
#define YES 1
#define NO  0
#define OK  0
#define ERR -1

/* flags for the isdn_dump flags argument */
#define IDF_MSG_HEADER    0x0001  /* Display column headings for message line */
#define IDF_HEX_DUMP      0x0002  /* Display hex dump of the entire ISDN frame*/
#define IDF_NO_ASCII_HEX_DUMP 0x0040 /* Suppress hex dump for TLF_TYPE_ASCII  */
                                     /*  records. */
#define IDF_INFO_HEADER   0x0004  /* Display column headings for information  */
                                  /*  elements (if IDF_INFO_ELEMENTS is set   */
                                  /*  and information elements are present)   */
#define IDF_INFO_ELEMENTS 0x0008  /* Display information elements for level 3 */
                                  /*  frames. */
#define IDF_WHITESPACE    0x0010  /* Adds a blank line to the end of output.  */
#define IDF_FILTER_L2     0x0020  /* This filter extracts layer 3 messages    */
                                  /*  from a full data link layer trace.  Data*/
                                  /*  link layer frames not containing layer 3*/
                                  /*  messages are discarded.   */
/* error returns */
#define ERR_NO_MEM  201 /* out_buf was not big enough to hold entire output */

extern int isdn_dump(TLF_HEADER *te,
             char *out_buf, long out_buf_size, int flags);

#define HEXDUMPLEN(datalen) (((((datalen) + 15) / 16) * 80) + 1)
extern int hex_dump(char *data, int datalen, char *outbuf, long outbuflen);

/* Return pack value back to default */

#ifndef UW7
#pragma pack()
#endif

#endif /* _tracelog_h */
