/* -*- mode: C++; c-basic-offset: 4; tab-width: 8 -*- */
/* ex:set ts=8 sw=4: */

/*
 Copyright 1999-2000 by Digi International (www.digi.com)

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED; without even the 
 implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/***********************************************************************
 * $Header: /eng/cvs/triboot/rap_hw.c,v 1.16 2000/05/26 21:40:50 randall Exp $
 * RAP2 specific
 */

#include "types.h"
#include "tribl.h"
#include "rap2mmap.h"

extern uint32 swap32(uint32, BOARD *);
extern uint32 swap_le32(uint32, BOARD*);

/*
 * I would rather use rap2.h to access these registers, unfortunately
 * it brings far more baggage than I want to deal with.
 */

#define PCI_RST	0x8158

#define CPU_INTP	(CPU_REGS_OFF + 0x04)
#define CPU_INTE	(CPU_REGS_OFF + 0x08)

#define DMAC_RXEN	(DMAC_REGS_OFF + 0x08)
#define DMAC_TXEN	(DMAC_REGS_OFF + 0x10)

#define DMAC_TXINTP	(DMAC_REGS_OFF + 0x30)
#define DMAC_RXINTP	(DMAC_REGS_OFF + 0x38)

#define DMAC_RXINTE	(DMAC_REGS_OFF + 0x18)
#define DMAC_TXINTE	(DMAC_REGS_OFF + 0x20)

#define MBOX_TXFIFO	(MBOX_REGS_OFF + 0x00)
#define MBOX_RXFIFO	(MBOX_REGS_OFF + 0x08)

#define MBOX_XPCI_CTRL	(MBOX_REGS_OFF + 0x10)
#   define MB_TX_EMPTY			(1 << 0)
#   define MB_TX_FULL			(1 << 2)
#   define MB_RX_EMPTY			(1 << 3)
#   define MB_RX_FULL			(1 << 5)
#   define MB_ENABLE_INT_TO_HOST	(1 << 12)

void
rap_reset(BOARD *brd, uint32 force)
{
    uint32 unmap;
    void *vaddr;
    uint32 *reg;

    if(brd->vreg == NULL)
    {
	unmap = 1;
	vaddr = (void *)brd->reg;
	brd->cb(MAPP2V, (tribl_uintptr_t *)&vaddr, &brd->reglen, NULL, NULL);
    }
    else
    {
	vaddr = brd->vreg;
	unmap = 0;
    }

    if(vaddr == 0)
	return;

    reg = (uint32 *) ((uint8 *) vaddr + PCI_RST);
    TRIBL_PCI_WRITE32(reg, swap32(0x60, brd), brd->id, TRIBL_REGS);

    if(unmap)
	brd->cb(UNMAP, (tribl_uintptr_t *)&vaddr, &brd->reglen, NULL, NULL);

    return;
}

void
rap_release_reset(BOARD *brd)
{
    uint32 unmap;
    void *vaddr;
    uint32 *reg;

    if(brd->vreg == NULL)
    {
	unmap = 1;
	vaddr = (void *)brd->reg;
	brd->cb(MAPP2V, (tribl_uintptr_t *)&vaddr, &brd->reglen, NULL, NULL);
    }
    else
    {
	vaddr = brd->vreg;
	unmap = 0;
    }

    if(vaddr == 0)
	return;

    reg = (uint32 *) ((uint8 *) vaddr + PCI_RST);
    TRIBL_PCI_WRITE32(reg, swap32(0x0, brd), brd->id, TRIBL_REGS);

    if(unmap)
	brd->cb(UNMAP, (tribl_uintptr_t *)&vaddr, &brd->reglen, NULL, NULL);

    return;
}

int32
rap_postmap(BOARD *brd)
{
#if 0
    rap_reset(brd, 0);
    return(0);
#else
    uint8 *gpd;
    void *vaddr;
    uint32 unmap;
    uint32 rc = 0;

    if(brd->vram == 0)
    {
	unmap = 1;
	vaddr = (void *)brd->ram;
	brd->cb(MAPP2V, (tribl_uintptr_t *)&vaddr, &brd->ramlen, NULL, NULL);
    }
    else
    {
	unmap = 0;
	vaddr = brd->vram;
    }

    if(vaddr == 0)
	return(0);

    gpd = (uint8 *) vaddr;
    gpd += swap_le32(TRIBL_PCI_READ32((uint8 *) vaddr + 0x1000,
				      brd->id, TRIBL_MEM), brd);

    if((gpd >= (uint8*) vaddr && gpd < (uint8*) vaddr + brd->ramlen) &&
       (swap_le32(TRIBL_PCI_READ32(&(((GLOBAL_POST_DATA *) gpd)->common.magic),
				   brd->id, TRIBL_MEM), brd)
	   == MAGIC_POST))
    {
	TRIBL_MEMCPY_FROM_PCI(&brd->post->pst, gpd, sizeof(GLOBAL_POST_DATA),
			      brd->id, TRIBL_MEM);

	/* Copy in the error report array */

	{
	    uint8 *rep;
	    int i;
	    int offset = brd->post->pst.summary.error_list_offset;

	    rep = (uint8 *) vaddr + swap_le32(offset, brd);

	    for(i = 0; i < POST_MAX_ENTRIES; i++)
	    {
		TRIBL_MEMCPY_FROM_PCI(&brd->post->err[i], rep,
				      sizeof(POST_ERROR_REPORT),
				      brd->id, TRIBL_MEM);

		if(((POST_ERROR_REPORT *) rep)->identifier == POST_ERROR_EOL)
		    break;

		rep += sizeof(POST_ERROR_REPORT);
	    }
	}

	rc = 1;
    }

    if(unmap)
	brd->cb(UNMAP, (tribl_uintptr_t *)&vaddr, &brd->ramlen, NULL, NULL);

    return(rc);
#endif
}

void
rap_start(BOARD *brd, uint32 address)
{
    uint32 unmap;
    void *vaddr;
    volatile uint32 status;
    uint32 handler;
    uint8 *memp;
    uint32 hihandler;
    uint32 lohandler;
    uint32 *rap_cpu_intp;
    uint32 *rap_cpu_inte;
    uint32 *rap_dmac_txintp;
    uint32 *rap_dmac_rxintp;

    if(brd->vram == NULL)
    {
	unmap = 1;
	vaddr = (void *)brd->ram;
	brd->cb(MAPP2V, (tribl_uintptr_t *)&vaddr, &brd->ramlen, NULL, NULL);
    }
    else
    {
	unmap = 0;
	vaddr = brd->vram;
    }

    if(vaddr == 0)
	return;

    handler = 0xa0000000 + address;
    memp = (uint8 *) vaddr + BASE_EXCEPTION_ADDRESS;

    hihandler = (uint32) handler >> 16 ;
    lohandler = (uint32) handler & 0xffff ;

    /* lui $k0 hi-half word */
    TRIBL_PCI_WRITE32(&memp[0], swap32(0x3c1a0000 | hihandler, brd), 
		      brd->id, TRIBL_MEM);

    /* ori $k0 lo-half word */
    TRIBL_PCI_WRITE32(&memp[4], swap32(0x375a0000 | lohandler, brd), 
		      brd->id, TRIBL_MEM);

    /* jr $k0 */
    TRIBL_PCI_WRITE32(&memp[8], swap32(0x03400008, brd), brd->id, TRIBL_MEM);

    /* nop */
    TRIBL_PCI_WRITE32(&memp[12], swap32(0, brd), brd->id, TRIBL_MEM);

    /* Undocumented 'feature', necessary to get beyond hang() */
    TRIBL_PCI_WRITE32(&memp[16], swap32(0, brd), brd->id, TRIBL_MEM);

    if(unmap)
	brd->cb(UNMAP, (tribl_uintptr_t *)&vaddr, &brd->ramlen, NULL, NULL);

    if(brd->vreg == NULL)
    {
	unmap = 1;
	vaddr = (void *)brd->reg;
	brd->cb(MAPP2V, (tribl_uintptr_t *)&vaddr, &brd->reglen, NULL, NULL);
    }
    else
    {
	unmap = 0;
	vaddr = brd->vreg;
    }

    if(vaddr == 0)
	return;

    rap_cpu_intp = (uint32 *) ((uint8 *) vaddr + CPU_INTP);
    rap_cpu_inte = (uint32 *) ((uint8 *) vaddr + CPU_INTE);

    rap_dmac_txintp = (uint32 *) ((uint8 *) vaddr + DMAC_TXINTP);
    rap_dmac_rxintp = (uint32 *) ((uint8 *) vaddr + DMAC_RXINTP);

    /* Inhibit ints */
    TRIBL_PCI_WRITE32(rap_cpu_inte, swap32(0, brd), brd->id, TRIBL_REGS);

    /* Clear Pending interrupts */
    TRIBL_PCI_WRITE32(rap_cpu_intp, swap32(0, brd), brd->id, TRIBL_REGS);

    /* Clear pending MCDMAC interrupts */
    status = TRIBL_PCI_READ32(rap_dmac_txintp, brd->id, TRIBL_REGS);
    status = TRIBL_PCI_READ32(rap_dmac_rxintp, brd->id, TRIBL_REGS);

    if(unmap)
	brd->cb(UNMAP, (tribl_uintptr_t *)&vaddr, &brd->reglen, NULL, NULL);

    /* release reset */
    rap_release_reset(brd);

    return ;
}

void
rap_post_board_int(BOARD *brd)
{
    void *vaddr;
    uint32 unmap;
    uint32 *reg;

    /*
     * This may look backwards, but the registers are named from the 
     * RAP's view of the world.  So the "rx_fifo" of the RAP is the 
     * things that we (the host driver) write into.
     */

    /*
     * This will write a 32bit word.  The value  isn't
     * important.  It's used only to generate an interrupt on the
     * card
     */

    if(brd->vreg == NULL)
    {
	unmap = 1;
	vaddr = (void *)brd->reg;
	brd->cb(MAPP2V, (tribl_uintptr_t *)&vaddr, &brd->reglen, NULL, NULL);
    }
    else
    {
	vaddr = brd->vreg;
	unmap = 0;
    }

    if(vaddr == 0)
	return;

    reg = (uint32 *) ((uint8 *) vaddr + MBOX_RXFIFO);

    TRIBL_PCI_WRITE32(reg, 0x12345678, brd->id, TRIBL_REGS);

    if(unmap)
	brd->cb(UNMAP, (tribl_uintptr_t *)&vaddr, &brd->reglen, NULL, NULL);

    return;
}


/* returns 0 if no interrupts were cleared, 1 if an interrupt was cleared. */
int
rap_clear_board_int(BOARD *brd)
{
    void *vaddr;
    uint32 unmap;
    volatile uint32 rap_mbox_intr_value;
    uint32 *ctrl_reg;
    uint32 *reg;

    /*
     * This may look backwards, but the registers are named from the 
     * RAP's view of the world.  So the "rx_fifo" of the RAP is the 
     * things that we (the host driver) write into.
     */

    if(brd->vreg == NULL)
    {
	unmap = 1;
	vaddr = (void *)brd->reg;
	brd->cb(MAPP2V, (tribl_uintptr_t *)&vaddr, &brd->reglen, NULL, NULL);
    }
    else
    {
	vaddr = brd->vreg;
	unmap = 0;
    }

    if(vaddr == 0)
	return(0);

    ctrl_reg = (uint32 *) ((uint8 *) vaddr + MBOX_XPCI_CTRL);
    reg = (uint32 *) ((uint8 *) vaddr + MBOX_TXFIFO);

    while((swap32(TRIBL_PCI_READ32(ctrl_reg, brd->id, TRIBL_REGS), brd) &
	   0x01) == 0)
    {
	/*
	 * This read a 32bit word.  By secret convention (easy when I'm
	 * both the FEP dude and the driver dude) the value read from
	 * fifo is actually the systime value of when the card posted
	 * interrupt.   This might change at my future whim, but it's
	 * a handy debugging aide until that time.
	 */

	rap_mbox_intr_value = TRIBL_PCI_READ32(reg, brd->id, TRIBL_REGS);

	brd->intr_count++;
    }

    if(unmap)
	brd->cb(UNMAP, (tribl_uintptr_t *)&vaddr, &brd->reglen, NULL, NULL);

    return(1);
}

void
rap_shutdown_board(BOARD *brd)
{
    void *vaddr;
    uint32 unmap;
    uint32 *rap_cpu_intp;
    uint32 *rap_cpu_inte;
    uint32 *rap_dmac_txen;
    uint32 *rap_dmac_rxen;

    rap_reset(brd, 0);

    if(brd->vreg == NULL)
    {
	unmap = 1;
	vaddr = (void *)brd->reg;
	brd->cb(MAPP2V, (tribl_uintptr_t *)&vaddr, &brd->reglen, NULL, NULL);
    }
    else
    {
	vaddr = brd->vreg;
	unmap = 0;
    }

    if(vaddr == 0)
	return;

    rap_cpu_intp = (uint32 *) ((uint8 *) vaddr + CPU_INTP);
    rap_cpu_inte = (uint32 *) ((uint8 *) vaddr + CPU_INTE);
    rap_dmac_rxen = (uint32 *) ((uint8 *) vaddr + DMAC_RXEN);
    rap_dmac_txen = (uint32 *) ((uint8 *) vaddr + DMAC_TXEN);

    if(vaddr)
    {
	/*
	 * FIXME: Only the RAP1 should really need the enables
	 * cleared in this way.   I think that RAP2 should probably
	 * simply poke sys reset.
	 */

	TRIBL_PCI_WRITE32(rap_cpu_inte, swap32(0, brd), brd->id, TRIBL_REGS);
	TRIBL_PCI_WRITE32(rap_cpu_intp, swap32(0, brd), brd->id, TRIBL_REGS);

	TRIBL_PCI_WRITE32(rap_dmac_rxen, 0, brd->id, TRIBL_REGS);
	TRIBL_PCI_WRITE32(rap_dmac_txen, 0, brd->id, TRIBL_REGS); 
    }

    if(unmap)
	brd->cb(UNMAP, (tribl_uintptr_t *)&vaddr, &brd->reglen, NULL, NULL);

    return;
}

