/* -*- mode: C++; c-basic-offset: 4; tab-width: 8 -*- */
/* ex:set ts=8 sw=4: */

/*
 Copyright 1999-2000 by Digi International (www.digi.com)

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED; without even the 
 implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/**********************************************************************
 * $Header: /eng/cvs/triboot/tb_dl.c,v 1.15 2001/01/16 22:17:14 randall Exp $
 */


#include "types.h"
#include "tribl.h"
#include "pci_ids.h"

#ifndef S2K_4PP_PCI_NAME
#include "s2k_pci_ids.h"
#endif

#include "rap2mmap.h"
// #include "mipsheader.hh"

extern void return_error(TRIBL_BOARD *, int);
extern void next_state(TRIBL_BOARD *, struct d_info *, BOARD *);
extern uint16 swap16(uint16, BOARD *);
extern uint32 swap32(uint32, BOARD *);
extern uint32 swap_le32(uint32, BOARD *);

extern void init_crc32(void);
extern uint32 crc32(uint32, uint8 *, uint32, BOARD *);

extern int nbrds;
extern struct d_info drv_info[NDEVS];

int
tb_download(TRIBL_BOARD *tb, BOARD *brd, struct d_info *dp)
{
    void *vaddr;
    int unmap;
    int i;

    dp->cur_state = tb->state;

    if(tb->sequence == 1)
    {
	/*
	 * if POST download was skipped or POST is board resident,
	 * we may still want to retrieve the POST result structure.
	 */
	if(brd->postmap != (int (*)(struct brd *))NULL &&
	   (dp->flags & POSTMAP) == 0)
	{
	    if(brd->postmap(brd))
		dp->flags |= POSTMAP;
	}
	if(tb->size == 0)
	{
	    /*
	     * need a mechanism to allow a wait for BOOT,POST
	     * or FEP to go to run state.
	     */

	    if(brd->wait_run > 0)
	    {
		tb->state = TRIBL_WAIT;
		tb->count = 1000;
		memset(tb->file_name, 0, FLEN);
		tb->pdata = 0;

		dp->wait_count = 1;
	    }
	    else
	    {
		tb->status = TRIBL_OK;

		next_state(tb, dp, brd);
	    }

	    return(0);;
	}
/*DBUstart: Parameterize Physical offset of Fep Load addr.
	dp->cur_down_addr = FEP_CODE_OFFSET;
*/
	dp->cur_down_addr = brd->fco;
/*DBUcont: this is the SECOND reset: useless delay for S2K 
	if(brd->reset_board)
*/
	if(brd->reset_board && (!is_s2k(brd->device)))
	{
	    brd->reset_board(brd, 0);

	    if(brd->cb)
	    {
		tribl_uintptr_t dtime = 20;
		brd->cb(DELAY, &dtime, NULL, NULL, NULL);
	    }
	}
    }

    /* if size is zero, the code has all been downloaded. */

    if(tb->size == 0)
    {
	uint32 crc;
	int ckoffset;
	uint8 *vcode;
	uint32 address;

/*DBUstart: S2k has no such hdr at start of image */
	if (is_s2k(brd->device))
	{
		TRIBL_PCI_WRITE32((uint8 *) brd->vram + brd->gho,
			  swap_le32((uint32) brd->ghd, brd), 
			  brd->id, TRIBL_MEM);
		goto skip2k;
	}
	/*
	 * Just in case, and before we use these values, let
	 * the SWAP? brethern decide if these are OK to use.
	 */

	dp->hdr.soft_version = swap16(dp->hdr.soft_version, brd);
	dp->hdr.lo_boot_rev = swap16(dp->hdr.lo_boot_rev, brd);
	dp->hdr.hi_boot_rev = swap16(dp->hdr.hi_boot_rev, brd);
	dp->hdr.load_segment = swap16(dp->hdr.load_segment, brd);
	dp->hdr.length = swap16(dp->hdr.length, brd);
	dp->hdr.exec_start = swap16(dp->hdr.exec_start, brd);
	dp->hdr.image_offset = swap16(dp->hdr.image_offset, brd);
	dp->hdr.startup_code[0] = swap32(dp->hdr.startup_code[0], brd);
	dp->hdr.startup_code[1] = swap32(dp->hdr.startup_code[1], brd);
	dp->hdr.checksum = swap32(dp->hdr.checksum, brd);

#	if DEBUG
	myprintf("%c%c %.2x %.2x %.2x %.2x %.2x %.2x %.2x %.4x %.4x %.4x\n",
		 (uint16) dp->hdr.type[0],
		 (uint16) dp->hdr.type[1],
		 (uint16) dp->hdr.soft_version,
		 (uint16) dp->hdr.lo_boot_rev,
		 (uint16) dp->hdr.hi_boot_rev,
		 (uint16) dp->hdr.load_segment,
		 (uint16) dp->hdr.length,
		 (uint16) dp->hdr.exec_start,
		 (uint16) dp->hdr.image_offset,
		 (uint32) dp->hdr.startup_code[0],
		 (uint32) dp->hdr.startup_code[1],
		 (uint32) dp->hdr.checksum);
#	endif

	address = dp->hdr.load_segment * 4096;

#	if DEBUG
	myprintf("address(%x,%x)\n", dp->hdr.load_segment, address);
#	endif

	/*
	 * FIX !!
	 * Should FEP ever be larger than 1/2 memory size??
	 * We only check to see if the write will be beyond
	 * the bounds of mapped memory.
	 */


	if(address + dp->hdr.length * 4096 > brd->ramlen)
	{
	    return_error(tb, TRIBL_CODE_TOO_BIG);
	    return(-1);
	}

	/*
	 * This is only meaningful if the FEP supports it, DXB fep does,
	 * so set it here, even if this will set it multiple times
	 */

	if(brd->vram == 0)
	{
	    unmap = 1;
	    vaddr = (void *)brd->ram;
	    brd->cb(MAPP2V, (tribl_uintptr_t *)&vaddr, &brd->ramlen, NULL, NULL);
	}
	else
	{
	    vaddr = brd->vram;
	    unmap = 0;
	}

	if(vaddr == 0)
	{
	    return_error(tb, TRIBL_NO_VADDR);
	    return(-1);
	}

    #ifdef HPUX
    /*Tell board HPUX is really big endian*/
           TRIBL_PCI_WRITE32(((uint8*) vaddr + GLOBAL_ENDIAN_OFFSET), 
                             GLOBAL_ENDIAN_BIG, brd->id, TRIBL_MEM);
    #else
     
    TRIBL_PCI_WRITE32(((uint8*) vaddr + GLOBAL_ENDIAN_OFFSET), 
                              HOST_ENDIAN, brd->id, TRIBL_MEM);
    #endif

/*DBUcont: Parameterize Physical offset of Fep Load addr.
	TRIBL_PCI_WRITE32((uint8 *) vaddr + GLOBAL_HOST_OFFSET,
*/
	TRIBL_PCI_WRITE32((uint8 *) vaddr + brd->gho,
			  swap_le32((uint32) brd->ghd, brd), 
			  brd->id, TRIBL_MEM);
/*DBUend*/
	ckoffset = OFFSETOF(struct digiheader, checksum);
	vcode = (uint8 *) vaddr + address;

	init_crc32();
	crc = crc32(0xffffffff, vcode, ckoffset, brd);
	crc = crc32(crc, vcode + ckoffset + sizeof(uint32),
		    dp->hdr.length * 4096 - ckoffset - sizeof(uint32), brd);

	if(unmap)
	    brd->cb(UNMAP, (tribl_uintptr_t *)&vaddr, &brd->ramlen, NULL, NULL);

	if(crc != dp->hdr.checksum)
	{
#	    if DEBUG
	    myprintf("crc(%.8x) hdr.checksum(%.8x)\n",
		     crc, (uint) dp->hdr.checksum);
#	    endif

	    return_error(tb, TRIBL_CRC_ERROR);
	    return(-1);
	}

	/* release reset */

	address += dp->hdr.exec_start;
skip2k:

	brd->start_board(brd, address);

	/* arrange to wait for code to run */

	tb->state = TRIBL_WAIT;
	tb->count = 1000;
	memset(tb->file_name, 0, FLEN);
	tb->pdata = 0;

	dp->wait_count = 1;

	return(0);
    }

    /*
     * in the midst of a download,
     *	1) are we still saving the header information
     *	2) copy the data to the board and increment counters
     */

/*DBUstart: Parameterize Physical offset of Fep Load addr.
    if((dp->cur_down_addr - FEP_CODE_OFFSET) < sizeof(struct digiheader))
*/
    if((dp->cur_down_addr - brd->fco) < sizeof(struct digiheader))
/*DBUend*/
    {
	uint8 *s;
	uint8 *d;
	int count = (dp->hp + tb->size > sizeof(struct digiheader))
	    ? sizeof(struct digiheader) - dp->hp : tb->size;

	d = (uint8 *) &dp->hdr + dp->hp;
	s = (uint8 *) tb->pdata;
	for(i = 0; i < count; i++, d++, s++)
	    *d = *s;

	dp->hp += count;

    }
    else if(dp->hp != sizeof(struct digiheader))
    {
	/*
	 * This is probably a mipsbin error, if you see
	 * it, take it to a fep guy.
	     */
	return_error(tb, TRIBL_HEADER_ERROR);
	return(-1);
    }

    if(brd->vram == 0)
    {
	unmap = 1;
	vaddr = (void *)brd->ram;
	brd->cb(MAPP2V, (tribl_uintptr_t *)&vaddr, &brd->ramlen, NULL, NULL);
    }
    else
    {
	vaddr = brd->vram;
	unmap = 0;
    }

    if(vaddr == 0)
    {
	return_error(tb, TRIBL_NO_VADDR);
	return(-1);
    }

    TRIBL_MEMCPY_TO_PCI((uint8 *) vaddr + dp->cur_down_addr,
			(uint8 *) tb->pdata, tb->size, brd->id, TRIBL_MEM);

    if(unmap)
	brd->cb(UNMAP, (tribl_uintptr_t *)&vaddr, &brd->ramlen, NULL, NULL);

    /* now increment addr for the next down-load segment */

    dp->cur_down_addr += tb->size;
    dp->cur_sequence = tb->sequence;

    tb->status = TRIBL_OK;

    return(0);
}
