/* -*- mode: C++; c-basic-offset: 4; tab-width: 8 -*- */
/* ex:set ts=8 sw=4: */

/*
 Copyright 1999-2000 by Digi International (www.digi.com)

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED; without even the 
 implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/**********************************************************************
 * $Header: /eng/cvs/triboot/tb_reg.c,v 1.22 2001/01/05 14:43:32 dickl Exp $
 */

#define NEED_NAMES

#include "types.h"
#include "tribl.h"
#include "pci_ids.h"

#ifndef S2K_4PP_PCI_NAME
#include "s2k_pci_ids.h"
#endif

#include "rap2mmap.h"
// #include "mipsheader.hh"

extern void hlc_unmask_board_int(BOARD *brd);
extern void hlc_mask_board_int(BOARD *brd);
extern void return_error(TRIBL_BOARD *, int);
extern int rap2_start;
extern int hlc2_start;
extern int s2k2_start;
extern struct d_info drv_info[NDEVS];

extern void rap_reset(BOARD*, uint32);
extern void rap_release_reset(BOARD*);
extern int32 rap_postmap(BOARD *);
extern void rap_start(BOARD *, uint32);
extern void rap_post_board_int(BOARD *);
extern int rap_clear_board_int(BOARD*);
extern void rap_shutdown_board(BOARD *);
extern int32 ramtest(void *, uint32, uint32, BOARD *);

extern void hlc_reset(BOARD*, uint32);
extern void hlc_release_reset(BOARD*);
extern int32 hlc_postmap(BOARD *);
extern void hlc_start(BOARD *, uint32);
extern void hlc_post_board_int(BOARD *);
extern int hlc_clear_board_int(BOARD*);
extern void hlc_shutdown_board(BOARD *);

/*DBUstart: s2k routines */
extern void s2k_post_board_int(BOARD *);
extern int s2k_clear_board_int(BOARD *);
extern void s2k_shutdown_board(BOARD *);
extern void s2k_start(BOARD *,uint32);
extern int32 s2k_postmap(BOARD *);
extern void s2k_reset(BOARD *, uint32);
extern void s2k_release_reset(BOARD *);
extern volatile long lbolt;
/*DBUend*/

int
tb_register(TRIBL_BOARD *tb, BOARD *brd, struct d_info *dp)
{
    PADDRS *paddr;
    uint32 nbars;
    uint32 i;
    uint32 needtype = 0;

    switch(tb->pci_vendor)
    {
    case PCI_DIGI:
	switch(tb->pci_device)
	{
	case DIGI_CHASKA_DID:
	case DIGI_PCIHOUSTON_DID:
	case DIGI_LONDON_DID:
	case DIGI_CPCIHOUSTON_DID:
	    for(paddr = (PADDRS *) tb->pdata, nbars = 0, i = 0;
		i < NBARS; i++, paddr++)
	    {
		switch(paddr->type)
		{
		case TRIBL_MEM:
		    if(needtype == 0)
		    {
			nbars += 1;
			brd->ram = (uint32)paddr->addr;
			if(paddr->size == 0)
			{
			    if(tb->pci_device == DIGI_CPCIHOUSTON_DID)
				paddr->size = 1024 * 1024 * 16;
			    else if(tb->pci_device == DIGI_PCIHOUSTON_DID)
				paddr->size = 1024 * 1024 * 8;
			    else
				paddr->size = 1024 * 1024 * 4;
			}

			brd->ramlen = paddr->size;
		    }
		    break;
		case TRIBL_REGS:
		    if(needtype == 0)
		    {
			nbars += 1;
			brd->reg = (uint32)paddr->addr;
			if(paddr->size == 0)
			    paddr->size = 0x10000;

			brd->reglen = paddr->size;
		    }
		    break;
		case TRIBL_FLASH:
		    if(needtype == 0)
		    {
			nbars += 1;
			brd->flash = (uint32)paddr->addr;
			if(paddr->size == 0)
			    paddr->size = 0x100000;

			brd->flashlen = paddr->size;
		    }
		    break;
		case TRIBL_RAWB:
		    if(nbars == 0 || needtype == 1)
		    {
			needtype = 1;
			switch(i)
			{
			case 0:	/* BAR1 => RAM */
			    nbars += 1;
			    brd->ram = (uint32)paddr->addr;
			    if(paddr->size == 0)
			    {
				if(tb->pci_device == DIGI_CPCIHOUSTON_DID)
				    paddr->size = 1024 * 1024 * 16;
				else if(tb->pci_device == DIGI_PCIHOUSTON_DID)
				    paddr->size = 1024 * 1024 * 8;
				else
				    paddr->size = 1024 * 1024 * 4;
			    }
			    brd->ramlen = paddr->size;
			    break;
			case 3:	/* BAR4 => FLASH memory */
			    nbars += 1;
			    brd->flash = (uint32)paddr->addr;
			    if(paddr->size == 0)
				paddr->size = 0x100000;
			    brd->flashlen = paddr->size;
			    break;
			case 4:	/* BAR5 => I/O space */
			    nbars += 1;
			    brd->reg = (uint32)paddr->addr;
			    if(paddr->size == 0)
				paddr->size = 0x10000;
			    brd->reglen = paddr->size;
			    break;
			default:
			    break;
			}
		    }
		    break;
		default:
		    break;
		}
	    }

	    /*
	     * HLC need 3 BARs mapped
	     */
	    if(nbars != 3)
	    {
		return_error(tb, TRIBL_BAR_ERROR);
		return(-1);
	    }

	    brd->device = tb->pci_device;

	    brd->post_board_int = hlc_post_board_int;
	    brd->clear_board_int = hlc_clear_board_int;
	    brd->unmask_board_int = (void (*)(struct brd *))hlc_unmask_board_int;
	    brd->mask_board_int = (void (*)(struct brd *))hlc_mask_board_int;

	    brd->shutdown_board = hlc_shutdown_board;
	    brd->start_board = hlc_start;
	    brd->postmap = hlc_postmap;

	    brd->reset_board = hlc_reset;
	    brd->release_board_reset = (void (*)(struct brd *))NULL;

	    brd->bin_names.fepname = hlc_binary_names;
	    brd->dl_start = hlc2_start;
	    brd->bigendian_board = 0;

	    brd->need_swap = (HOST_ENDIAN == GLOBAL_ENDIAN_LITTLE) ? 0 : 1;
/*DBUstart: parameterize GLOBAL_*_OFFSET */
		brd->gco = GLOBAL_DATA_OFFSET;
		brd->gho = GLOBAL_HOST_OFFSET;
		brd->fco = FEP_CODE_OFFSET;
/*DBUend*/
	    brd->ramtest_len = brd->ramlen - FEP_CODE_OFFSET
		- (512 * 1024) - 4;

	    brd->wait_run = 30;
	    break;

	case PCI_TIGGER8:
	    for(paddr = (PADDRS *) tb->pdata, nbars = 0, i = 0;
		i < NBARS; i++, paddr++)
	    {
		switch(paddr->type)
		{
		case TRIBL_MEM:
		    if(needtype == 0)
		    {
			nbars += 1;
			brd->ram = (uint32)paddr->addr;
			if(paddr->size == 0)
			    paddr->size = 1024 * 1024 * 2;

			brd->ramlen = paddr->size;
		    }
		    break;
		case TRIBL_REGS:
		    if(needtype == 0)
		    {
			nbars += 1;
			brd->reg = (uint32)paddr->addr;
			if(paddr->size == 0)
			    paddr->size = 0x10000;

			brd->reglen = paddr->size;
		    }
		    break;
		case TRIBL_RAWB:
		    if(nbars == 0 || needtype == 1)
		    {
			needtype = 1;
			switch(i)
			{
			case 0:	/* BAR1 => RAM */
			    nbars += 1;
			    brd->ram = (uint32)paddr->addr;
			    if(paddr->size == 0)
				paddr->size = 1024 * 1024 * 2;
			    brd->ramlen = paddr->size;
			    break;
			case 1:	/* BAR2 => I/O space */
			    nbars += 1;
			    brd->reg = (uint32)paddr->addr;
			    if(paddr->size == 0)
				paddr->size = 0x10000;
			    brd->reglen = paddr->size;
			    break;
			default:
			    break;
			}
		    }
		    break;
		default:
		    break;
		}
	    }

	    if(nbars != 2)
	    {
		return_error(tb, TRIBL_BAR_ERROR);
		return(-1);
	    }

	    brd->device = tb->pci_device;

	    brd->post_board_int = rap_post_board_int;
	    brd->clear_board_int = rap_clear_board_int;
	    brd->unmask_board_int = (void (*)(struct brd *))NULL;
	    brd->mask_board_int = (void (*)(struct brd *))NULL;

	    brd->shutdown_board = rap_shutdown_board;
	    brd->start_board = rap_start;
	    brd->postmap = rap_postmap;

	    brd->reset_board = rap_reset;
	    brd->release_board_reset = rap_release_reset;

	    brd->bin_names.fepname = rap2_binary_names;
	    brd->dl_start = rap2_start;
	    brd->bigendian_board = 1;
	    brd->need_swap = (HOST_ENDIAN == GLOBAL_ENDIAN_BIG)	? 0 : 1;
	    brd->ramtest_len = brd->ramlen - FEP_CODE_OFFSET - 4;
#if 1
	    /* BIG KLUDGE until all boards are 2 Meg */
	    brd->ramtest_len = (1024 * 1024) - FEP_CODE_OFFSET - 4;
#endif
/*DBUstart: parameterize GLOBAL_*_OFFSET */
		brd->gco = GLOBAL_DATA_OFFSET;
		brd->gho = GLOBAL_HOST_OFFSET;
		brd->fco = FEP_CODE_OFFSET;
/*DBUend*/
	    brd->wait_run = 90;
	    break;

/* DBUstart: S2K: assign board-related function pointers: Clue for next time:
 			if the set of functions req'd for talking to board is a structure
 			then we dont have to enumerate these thusly in this error-prone
 			way here.

 			Re. PADDRs: useless to dicker with them here: all the TYPES of
 			RAM/FLASH/etc have to be MAPPED before they are of any meaning
 			to me anyway.
*/
	CASE_S2K_DID :

	    brd->device = 			tb->pci_device;

	    brd->post_board_int =   s2k_post_board_int;
	    brd->clear_board_int =  s2k_clear_board_int;
	    brd->unmask_board_int = (void (*)(struct brd *))NULL;
	    brd->mask_board_int =   (void (*)(struct brd *))NULL;
	    brd->shutdown_board = 	s2k_shutdown_board;
	    brd->start_board = 		s2k_start;
	    brd->postmap = 			s2k_postmap;
	    brd->reset_board = 		s2k_reset;
	    brd->release_board_reset=s2k_release_reset;

	    brd->bin_names.fepname=	s2k_binary_names;
	    brd->dl_start = 		s2k2_start;

	    brd->bigendian_board = 0;
	    brd->need_swap = (HOST_ENDIAN == GLOBAL_ENDIAN_LITTLE) ? 0 : 1;
	    brd->ramtest_len = 0; /* To be set AFTER mapping */

		brd->gco = 0x2004;
		brd->gho = 0x2000;
		brd->fco = 0x4000;

	    brd->wait_run = 30;

	    nbars = 4;
	    break;
/* DBUend*/

	default:
	    return_error(tb, TRIBL_DEVICE_ERROR);
	    return(-1);
	}
	break;

    default:
	return_error(tb, TRIBL_VENDOR_ERROR);
	return(-1);
    }

    /*
     * new convention
     *		paddr[0] is RAM
     *		paddr[1] is REG
     *		paddr[2] is FLASH
     *		paddr[3] is ROM
     */

    if(needtype == 0)
    {
	for(paddr = (PADDRS *) tb->pdata, i = 0; i < nbars; i++, paddr++)
	    switch(i)
	    {
	    case 0:
		paddr->type = TRIBL_MEM;
		break;
	    case 1:
		paddr->type = TRIBL_REGS;
		break;
	    case 2:
		paddr->type = TRIBL_FLASH;
		break;
	    default:
		break;
	    }
    }
    else
    {
	/*
	 * if boards are added which need more BARS mapped,
	 * this will need to be extended.
	 */
	for(paddr = (PADDRS *) tb->pdata, i = 0; i < NBARS; i++, paddr++)
	    switch(i)
	    {
	    case 0:
		paddr->type = TRIBL_MEM;
		paddr->addr = (tribl_uintptr_t)brd->ram;
		paddr->size = brd->ramlen;
		break;
	    case 1:
		paddr->type = TRIBL_REGS;
		paddr->addr = (tribl_uintptr_t)brd->reg;
		paddr->size = brd->reglen;
		break;
	    case 2:
		switch(tb->pci_device)
		{
		case DIGI_CHASKA_DID:
		case DIGI_PCIHOUSTON_DID:
		case DIGI_LONDON_DID:
		    paddr->type = TRIBL_FLASH;
		    paddr->addr = (tribl_uintptr_t)brd->flash;
		    paddr->size = brd->flashlen;
		    break;
		default:	/* HERCULES */
		    paddr->type = 0;
		    paddr->addr = 0;
		    paddr->size = 0;
		    break;
		}
		break;
	    default:	/* zero out the extras which are now moved */
		paddr->type = 0;
		paddr->addr = 0;
		paddr->size = 0;
		break;
	    }
    }

    /*
     * the driver will provide the functions necessary to
     * map a PCI physical address to a virtual address.
     */
    dp->cur_state = tb->state;
    dp->pci_vendor = tb->pci_vendor;
    dp->pci_device = tb->pci_device;
    dp->intr_index = tb->int_index;
    dp->cur_sequence = 0;

    brd->cb = tb->tri_cb;

    tb->state = TRIBL_MAP_ADDRS;
    tb->status = TRIBL_OK;
    tb->count = 0;
    tb->sequence = 0;

    return(0);
}
