/* -*- mode: C++; c-basic-offset: 4; tab-width: 8 -*- */
/* ex:set ts=8 sw=4: */

/*
 Copyright 1999-2000 by Digi International (www.digi.com)

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED; without even the 
 implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/**********************************************************************
 * $Header: /eng/cvs/triboot/tb_wdone.c,v 1.17 2001/01/05 08:05:34 dickl Exp $
 */

#include "types.h"
#include "tribl.h"
#include "pci_ids.h"

#ifndef S2K_4PP_PCI_NAME
#include "s2k_pci_ids.h"
#endif

#include "rap2mmap.h"
extern int dgdxb_flashread(uint32_t );
// #include "mipsheader.hh"

extern void return_error(TRIBL_BOARD *, int);
extern void next_state(TRIBL_BOARD *, struct d_info *, BOARD *);
extern uint16 swap16(uint16, BOARD *);
extern uint32 swap32(uint32, BOARD *);
extern uint32 swap_le32(uint32, BOARD *);

extern void init_crc32(void);
extern uint32 crc32(uint32, uint8 *, uint32, BOARD *);

extern int32 ramtest(void *, uint32, uint32, BOARD *);

extern struct d_info drv_info[NDEVS];

int
tb_wait_done(TRIBL_BOARD *tb, BOARD *brd, struct d_info *dp)
{
    void *vaddr;
 caddr_t there;
    int unmap;
    int rc;
    int ok;
    int i;

    if(brd->vram == NULL)
    {
	unmap = 1;
	vaddr = (void *) brd->ram;
	brd->cb(MAPP2V, (tribl_uintptr_t *)&vaddr, &brd->ramlen, NULL, NULL);
    }
    else
    {
	unmap = 0;
	vaddr = brd->vram;
    }

    if(vaddr == 0)
    {
	return_error(tb, TRIBL_NO_VADDR);
	return(-1);
    }

    switch(dp->code)
    {
    case DL_BOOT:
	{
	    uint8 *gbd;

	    gbd = (uint8 *) vaddr;

/*DBUstart: parameterize GLOBAL_DATA_OFFSET 
	    gbd += swap_le32(TRIBL_PCI_READ32((uint8 *) vaddr + GLOBAL_DATA_OFFSET,
					      brd->id, TRIBL_MEM), brd);
*/
	    gbd += swap_le32(TRIBL_PCI_READ32((uint8 *) vaddr + brd->gco,
					      brd->id, TRIBL_MEM), brd);
/*DBUend*/

	    /*
	     * Check condition of BOOT, if it was successful
	     * ask for the next stage in the program
	     * else notify the driver of the error.
	     */

	    if(gbd < (uint8 *) vaddr || gbd >= (uint8*) vaddr + brd->ramlen)
	    {
		return_error(tb, TRIBL_BOOT_ERROR);
		goto errrtn;
	    }

	    if(!((swap_le32(TRIBL_PCI_READ32(&(((GLOBAL_BOOT_DATA*) gbd)->common.magic),
					     brd->id, TRIBL_MEM), brd)
		  == MAGIC_BOOT)
		 && (swap_le32(TRIBL_PCI_READ32(&(((GLOBAL_BOOT_DATA*) gbd)->common.status),
						brd->id, TRIBL_MEM), brd)
		     == STATUS_RUN)))
	    {
		if(dp->wait_count++ < ((brd->wait_run > 0)
				       ? brd->wait_run : WAIT_TIME))
		{
		    tb->state = TRIBL_WAIT;
		    tb->count = 1000;
		    goto rtn;
		}

#		if DEBUG
		myprintf("gbd(%x) magic(%x) status(%x)\n",
			 (uint) gbd,
			 (uint) TRIBL_PCI_READ32(&(((GLOBAL_BOOT_DATA*) gbd)->common.magic), brd->id, TRIBL_MEM),
			 (uint) TRIBL_PCI_READ32(&(((GLOBAL_BOOT_DATA*) gbd)->common.status), brd->id, TRIBL_MEM));
#		endif
		return_error(tb, TRIBL_BOOT_ERROR);
		goto errrtn;
	    }

	    next_state(tb, dp, brd);
	}
	break;

    case DL_POST:
	{
	    uint8 *gpd;
	    int ok;

	    gpd = (uint8 *) vaddr;

/*DBUstart: parameterize GLOBAL_DATA_OFFSET 
	    gpd += swap_le32(TRIBL_PCI_READ32(((uint8 *) vaddr+GLOBAL_DATA_OFFSET), 
										  brd->id, TRIBL_MEM), brd);
*/
		if (is_s2k(brd->device))
			gpd += 0x2e00;
	    else
	    	gpd += swap_le32(TRIBL_PCI_READ32(((uint8 *)vaddr + brd->gco),
										  brd->id, TRIBL_MEM), brd);
	    if(gpd < (uint8 *) vaddr || (gpd >= ((uint8 *)vaddr + brd->ramlen)))
	    {
			return_error(tb, TRIBL_POST_ERROR);
			goto errrtn;
	    }

	    /*
	     * Check condition of POST, if it was successful
	     * ask for the next stage in the program
	     * else notify the driver of the error.
	     */

/*DBUcont: S2K happiness after reset does not look like this at all:  I have
           tried to make the compound if..&&... a bit more readable for those
           of us limited to 80 col screens..  Sorry if I offend.
*/
		if (is_s2k(brd->device))
		{
#define	S2K_POST	MKMAGIC('D','O','N','E')
			there = ((caddr_t)brd->vflash) + 0x40;	/* POST reports here */
			ok = ((i=swap_le32(TRIBL_PCI_READ32(
								there,
				     			brd->id, 
				     			TRIBL_REGS), 
				     			brd)) == S2K_POST);
		}
		else
		{
			ok = ((swap_le32(TRIBL_PCI_READ32(
								&(((GLOBAL_POST_DATA*)gpd)->common.magic),
				     			brd->id, 
				     			TRIBL_MEM), 
				     			brd) == MAGIC_POST)
		 		&& (swap_le32(TRIBL_PCI_READ32(
		 						&(((GLOBAL_POST_DATA*)gpd)->common.status),
								brd->id,
								TRIBL_MEM),
								brd) == STATUS_RUN) );
		}

	    if (!ok)
	    {
			if(dp->wait_count++ < ((brd->wait_run > 0)
				       ? brd->wait_run : WAIT_TIME))
			{
		    	tb->state = TRIBL_WAIT;
		    	tb->count = 1000;
		    	goto rtn;
			}
/* S2K has no elaborate POST structures to gather data from */
			else if (is_s2k(brd->device))
			{
				dp->flags |= POSTMAP;
				return_error(tb, TRIBL_POST_ERROR);
				goto errrtn;
			}
/*DBUend*/
#		if DEBUG
		myprintf("gpd(%x) magic(%x) status(%x)\n",
			 (uint) gpd,
			 (uint) TRIBL_PCI_READ32(&(((GLOBAL_POST_DATA*) gpd)->common.magic), 
						 brd->id, TRIBL_MEM),
			 (uint) TRIBL_PCI_READ32(&(((GLOBAL_POST_DATA*) gpd)->common.status), 
						 brd->id, TRIBL_MEM));
#		endif

		/*
		 * save a copy of the POST results, good or bad
		 */

		TRIBL_MEMCPY_FROM_PCI(&brd->post->pst, gpd,
				      sizeof(GLOBAL_POST_DATA), 
				      brd->id, TRIBL_MEM);
		{
		    uint8 *rep;
		    int i;
		    int offset = brd->post->pst.summary.error_list_offset;

		    rep = (uint8 *) vaddr + swap_le32(offset, brd);

		    for(i = 0; i < POST_MAX_ENTRIES; i++)
		    {
			TRIBL_MEMCPY_FROM_PCI(&brd->post->err[i], rep,
					      sizeof(POST_ERROR_REPORT),
					      brd->id, TRIBL_MEM);

			if(((POST_ERROR_REPORT *) rep)->identifier
			   == POST_ERROR_EOL)
			    break;

			rep += sizeof(POST_ERROR_REPORT);
		    }
		}

		dp->flags |= POSTMAP;

		return_error(tb, TRIBL_POST_ERROR);
		goto errrtn;
	    }
	    
	    /* Ramtest, originally from tb_mdone.c.  Moved here to avoid
	       interfering with a possibly running POST */
	    /*
	     * Can have 4Meg or 8Meg, if 8 then test the first 4
	     * leaving the running POST area out.
	     * Then test all of the remaining 4 meg.
	     */
	    switch(tb->pci_device) {
	    case PCI_TIGGER8:
		rc = 0 ; /* We've already done this */
		break;
	    case DIGI_CHASKA_DID:
	    case DIGI_PCIHOUSTON_DID:
	    case DIGI_LONDON_DID:
	    case DIGI_CPCIHOUSTON_DID:
		if(brd->ramtest_len <= 1024 * 1024 * 4) {
		    /* Test from 128K to RAMSIZE-512K. */
		    rc = ramtest((uint8 *) vaddr + 0x20000, 0,
				 brd->ramtest_len - 0x20000 - 512 * 1024,
				 brd);
		} else
		    {
			/* Test from 128K to 4M-512K. */
			rc = ramtest((uint8*) vaddr, 0x20000,
				     4096 * 1024 - 0x20000 - 512 * 1024, 
				     brd);
			/* Test from 4M to RAMSIZE. */
			rc += ramtest((uint8*) vaddr, 4096*1024,
				      brd->ramtest_len - 4096 * 1024, brd);
		    }
		break;
/*DBUstart*/
	    CASE_S2K_DID :
	    rc = 0;
		break;
/*DBUend*/
	    default:
		rc = 1;
		break;
	    }

	    if (rc != 0) {
		return_error(tb, TRIBL_MEMORY_ERROR);
		goto errrtn;
	    }

/*DBUstart: again, S2k has no such data structure... */
		if (!is_s2k(tb->pci_device))
		{
	    /* The good case */
	    TRIBL_MEMCPY_FROM_PCI(brd->post, gpd,
				  sizeof(GLOBAL_POST_DATA),
				  brd->id, TRIBL_MEM);
	    {
		uint8 *rep;
		int i;
		int offset = brd->post->pst.summary.error_list_offset;

		rep = (uint8 *) vaddr + swap_le32(offset, brd);

		for(i = 0; i < POST_MAX_ENTRIES; i++)
		{
		    TRIBL_MEMCPY_FROM_PCI(&brd->post->err[i], rep,
					  sizeof(POST_ERROR_REPORT),
					  brd->id, TRIBL_MEM);

		    if(((POST_ERROR_REPORT *) rep)->identifier
		       == POST_ERROR_EOL)
			break;

		    rep += sizeof(POST_ERROR_REPORT);
		}
	    }
	    }

	    dp->flags |= POSTMAP;
	    next_state(tb, dp, brd);
	}
	break;

    case DL_FEP:
	{
	    uint8 *gfd;

	    gfd = (uint8 *) vaddr;
/*DBUstart: parameterize GLOBAL_DATA_OFFSET 
	    gfd += swap_le32(TRIBL_PCI_READ32((uint8 *) vaddr + GLOBAL_DATA_OFFSET,
					      brd->id, TRIBL_MEM), brd);
*/
		if (is_s2k(brd->device))
			gfd += 0x2e00;
		else
	    	gfd += swap_le32(TRIBL_PCI_READ32((uint8 *) vaddr + brd->gco,
					      brd->id, TRIBL_MEM), brd);
/*DBUend*/

	    if(gfd < (uint8 *) vaddr || gfd >= (uint8 *) vaddr + brd->ramlen)
	    {
		return_error(tb, TRIBL_FEP_ERROR);
		goto errrtn;
	    }

	    /*
	     * ask for the next stage in the program
	     */
/* DBUstart: see comment above re. POST.  test for success is different! */
		if (is_s2k(brd->device))
		{
/* Its really xbus_init.board_rsp, but why introduce a whole data struct. when
   all thats needed is a single offset to a single byte?
*/
	    	ok = TRIBL_PCI_READ8((gfd + 0x51), brd->id, TRIBL_MEM);
	    }
	    else
	    {
	    	ok = ((swap_le32(TRIBL_PCI_READ32(
	    							&(((GLOBAL_FEP_DATA*) gfd)->common.magic),
					     			brd->id, 
					     			TRIBL_MEM), 
					     			brd) == MAGIC_FEP)
		 		&& (swap_le32(TRIBL_PCI_READ32(
		 							&(((GLOBAL_FEP_DATA*) gfd)->common.status),
									brd->id, 
									TRIBL_MEM), 
									brd) == STATUS_RUN));
	    }
	    if (!ok)
	    {
			if(dp->wait_count++ < ((brd->wait_run > 0)
				       	? brd->wait_run : WAIT_TIME))
			{
		    	tb->state = TRIBL_WAIT;
		    	tb->count = 1000;
		    	goto rtn;
			}

#		if DEBUG
			myprintf("gfd(%x) magic(%x) status(%x)\n",
			 (uint) gfd,
			 (uint) TRIBL_PCI_READ32(&(((GLOBAL_FEP_DATA *) gfd)->common.magic),
					 	brd->id, TRIBL_MEM),
			 (uint)TRIBL_PCI_READ32(&(((GLOBAL_FEP_DATA *) gfd)->common.status),
					 	brd->id, TRIBL_MEM));
#		endif

			return_error(tb, TRIBL_FEP_ERROR);
			goto rtn;
	    }

	    next_state(tb, dp, brd);
	}
	break;
    }

 rtn:
    if(unmap)
	brd->cb(UNMAP, (tribl_uintptr_t *)&vaddr, &brd->ramlen, NULL, NULL);

    return(0);

 errrtn:
    if(unmap)
	brd->cb(UNMAP, (tribl_uintptr_t *)&vaddr, &brd->ramlen, NULL, NULL);

    return(-1);
}
