/* -*- mode: C; c-basic-offset: 4; tab-width: 8 -*- */

/*
 Copyright 1999-2000 by Digi International (www.digi.com)

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED; without even the 
 implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/**********************************************************************
 * $Header: /eng/cvs/triboot/types.h,v 1.19 2000/12/08 15:41:50 kurte Exp $
 */

#if !defined (_TYPES_H_INCLUDED)
#define _TYPES_H_INCLUDED

#if defined(LINUX)
#include <linux/string.h>
#endif

#if defined(SOLARIS)
#	include <sys/types.h>
#endif

#if defined(HPUX)
#include "dgdxb_tribl.h"
#endif

/************************************************************************/
/*									*/
/*	Types and type-related functions that work for both C and C++	*/
/*									*/
/************************************************************************/

/*
 *	The latest rage in trendy NULL definitions.
 */

#ifndef NULL
#define	NULL	((void *) 0)
#endif

typedef unsigned char		uchar;
#if !defined(LINUX) && !defined(SOLARIS) && !defined(HPUX)
typedef unsigned short		ushort;
#endif
#if !defined(NWDRV) && !defined(SOLARIS) && !defined(_AIX) && !defined(HPUX)
typedef unsigned short		eshort;
typedef unsigned long		elong;
#endif

#ifdef _GNUC_
typedef unsigned long long	ulonglong;
#endif
#if !defined(LINUX) && !defined(SOLARIS)
typedef unsigned long		ulong;
typedef unsigned char		unchar;

#if !defined(HPUX)
typedef unsigned int		uint;
#endif
#endif

#if !defined( _SIZE_T_DEFINED_) && !defined(LINUX) && !defined(SOLARIS) && !defined(HPUX)
typedef unsigned int		size_t;
#define _SIZE_T_DEFINED_
#endif

typedef void (*label) (void);

#if !defined(LINUX) && !defined(SOLARIS) && !defined(HPUX)
typedef char *			caddr_t;
#endif
typedef unsigned char *		bufp_t;

/* 
 *  Use these judiciously in system headers when you need something of a 
 *  known size.   
 */

typedef unsigned char		uint8;
typedef char			int8;
typedef unsigned short		uint16;
typedef short			int16;
typedef unsigned int		uint32;
typedef int			int32;
#ifdef _GNUC_
typedef unsigned long long	uint64;
typedef long long		int64;
#endif

#if defined(SOLARIS) || defined(HPUX)
/* These are used in struct digiheader, which represents
   the structure of the POST/FEP-file header. */
typedef uint16 eshort;
typedef uint32 elong;
#endif


/*
 *	The usual macros: min/max/asizeof/offsetof
 */

#define max(a,b)                ((a) < (b) ? (b) : (a))
#define min(a,b)                ((a) > (b) ? (b) : (a))
#define	asizeof(array)		(sizeof(array) / sizeof(array[0]))

#if !defined(offsetof)
# define	offsetof(parent, offset) ((int) &(((parent *) 0)->offset))
#endif
#define	OFFSETOF(parent, offset) ((int) &(((parent *) 0)->offset))


/************************************************************************/
/*									*/
/*	Global types for dual port memory				*/
/*									*/
/************************************************************************/

/*
 *	C versions.  The onus is on the program to byte-swap.
 */
typedef uint32			board_uint32;
typedef uint32			board_uint32_le;
typedef ushort			board_uint16;
typedef ushort			board_uint16_le;
typedef uchar			board_uint8;
typedef uchar			board_uchar;
typedef volatile uint32		host_uint32;
typedef volatile uint32		host_uint32_le;
typedef	volatile unsigned char	host_uchar;
typedef volatile uint32		shared_uint32;
typedef volatile uint32		shared_uint32_le;
typedef volatile uchar		shared_uint8;
typedef uint32			pad_32;

/*
 *	Types for data moved in messages.  From the FEP viewpoint,
 *	these are the same as board_* types (i.e. read/write,
 *	not constant, and not volatile.  The name for this type was,
 *	perhaps, a bad choice, implying that it is product specific.
 */

typedef board_uint32		dxb_uint32;
typedef board_uint16		dxb_uint16;
typedef board_uint8		dxb_uint8;


#if defined(SOLARIS)
#include "dgdxb_tribl.h"
#endif

#endif /* defined (_TYPES_H_INCLUDED) */












